/* topbar.c generated by valac 0.18.0.79-04df6, the Vala compiler
 * generated from topbar.vala, do not modify */

/* This file is part of GNOME Boxes. License: LGPLv2+*/

#include <glib.h>
#include <glib-object.h>
#include <clutter/clutter.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <clutter-gtk/clutter-gtk.h>
#include <libgd/gd.h>
#include <glib/gi18n-lib.h>
#include <gio/gio.h>


#define BOXES_TYPE_TOPBAR_PAGE (boxes_topbar_page_get_type ())

#define BOXES_TYPE_UI (boxes_ui_get_type ())
#define BOXES_UI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_UI, BoxesUI))
#define BOXES_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_UI, BoxesUIClass))
#define BOXES_IS_UI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_UI))
#define BOXES_IS_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_UI))
#define BOXES_UI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_UI, BoxesUIClass))

typedef struct _BoxesUI BoxesUI;
typedef struct _BoxesUIClass BoxesUIClass;
typedef struct _BoxesUIPrivate BoxesUIPrivate;

#define BOXES_TYPE_UI_STATE (boxes_ui_state_get_type ())

#define BOXES_TYPE_TOPBAR (boxes_topbar_get_type ())
#define BOXES_TOPBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_TOPBAR, BoxesTopbar))
#define BOXES_TOPBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_TOPBAR, BoxesTopbarClass))
#define BOXES_IS_TOPBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_TOPBAR))
#define BOXES_IS_TOPBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_TOPBAR))
#define BOXES_TOPBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_TOPBAR, BoxesTopbarClass))

typedef struct _BoxesTopbar BoxesTopbar;
typedef struct _BoxesTopbarClass BoxesTopbarClass;
typedef struct _BoxesTopbarPrivate BoxesTopbarPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

#define BOXES_TYPE_APP (boxes_app_get_type ())
#define BOXES_APP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_APP, BoxesApp))
#define BOXES_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_APP, BoxesAppClass))
#define BOXES_IS_APP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_APP))
#define BOXES_IS_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_APP))
#define BOXES_APP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_APP, BoxesAppClass))

typedef struct _BoxesApp BoxesApp;
typedef struct _BoxesAppClass BoxesAppClass;
typedef struct _BoxesAppPrivate BoxesAppPrivate;

#define BOXES_TYPE_CLUTTER_WIDGET (boxes_clutter_widget_get_type ())
#define BOXES_CLUTTER_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidget))
#define BOXES_CLUTTER_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidgetClass))
#define BOXES_IS_CLUTTER_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_CLUTTER_WIDGET))
#define BOXES_IS_CLUTTER_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_CLUTTER_WIDGET))
#define BOXES_CLUTTER_WIDGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidgetClass))

typedef struct _BoxesClutterWidget BoxesClutterWidget;
typedef struct _BoxesClutterWidgetClass BoxesClutterWidgetClass;

#define BOXES_TYPE_COLLECTION_ITEM (boxes_collection_item_get_type ())
#define BOXES_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItem))
#define BOXES_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))
#define BOXES_IS_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_IS_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_COLLECTION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))

typedef struct _BoxesCollectionItem BoxesCollectionItem;
typedef struct _BoxesCollectionItemClass BoxesCollectionItemClass;

#define BOXES_TYPE_SEARCHBAR (boxes_searchbar_get_type ())
#define BOXES_SEARCHBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SEARCHBAR, BoxesSearchbar))
#define BOXES_SEARCHBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SEARCHBAR, BoxesSearchbarClass))
#define BOXES_IS_SEARCHBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SEARCHBAR))
#define BOXES_IS_SEARCHBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SEARCHBAR))
#define BOXES_SEARCHBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SEARCHBAR, BoxesSearchbarClass))

typedef struct _BoxesSearchbar BoxesSearchbar;
typedef struct _BoxesSearchbarClass BoxesSearchbarClass;

#define BOXES_TYPE_NOTIFICATIONBAR (boxes_notificationbar_get_type ())
#define BOXES_NOTIFICATIONBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbar))
#define BOXES_NOTIFICATIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbarClass))
#define BOXES_IS_NOTIFICATIONBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_NOTIFICATIONBAR))
#define BOXES_IS_NOTIFICATIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_NOTIFICATIONBAR))
#define BOXES_NOTIFICATIONBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbarClass))

typedef struct _BoxesNotificationbar BoxesNotificationbar;
typedef struct _BoxesNotificationbarClass BoxesNotificationbarClass;

#define BOXES_TYPE_REVEALER (boxes_revealer_get_type ())
#define BOXES_REVEALER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_REVEALER, BoxesRevealer))
#define BOXES_REVEALER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_REVEALER, BoxesRevealerClass))
#define BOXES_IS_REVEALER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_REVEALER))
#define BOXES_IS_REVEALER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_REVEALER))
#define BOXES_REVEALER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_REVEALER, BoxesRevealerClass))

typedef struct _BoxesRevealer BoxesRevealer;
typedef struct _BoxesRevealerClass BoxesRevealerClass;

#define BOXES_TYPE_SIDEBAR (boxes_sidebar_get_type ())
#define BOXES_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SIDEBAR, BoxesSidebar))
#define BOXES_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SIDEBAR, BoxesSidebarClass))
#define BOXES_IS_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SIDEBAR))
#define BOXES_IS_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SIDEBAR))
#define BOXES_SIDEBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SIDEBAR, BoxesSidebarClass))

typedef struct _BoxesSidebar BoxesSidebar;
typedef struct _BoxesSidebarClass BoxesSidebarClass;

#define BOXES_TYPE_SELECTIONBAR (boxes_selectionbar_get_type ())
#define BOXES_SELECTIONBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbar))
#define BOXES_SELECTIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbarClass))
#define BOXES_IS_SELECTIONBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SELECTIONBAR))
#define BOXES_IS_SELECTIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SELECTIONBAR))
#define BOXES_SELECTIONBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbarClass))

typedef struct _BoxesSelectionbar BoxesSelectionbar;
typedef struct _BoxesSelectionbarClass BoxesSelectionbarClass;

#define BOXES_TYPE_WIZARD (boxes_wizard_get_type ())
#define BOXES_WIZARD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_WIZARD, BoxesWizard))
#define BOXES_WIZARD_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_WIZARD, BoxesWizardClass))
#define BOXES_IS_WIZARD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_WIZARD))
#define BOXES_IS_WIZARD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_WIZARD))
#define BOXES_WIZARD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_WIZARD, BoxesWizardClass))

typedef struct _BoxesWizard BoxesWizard;
typedef struct _BoxesWizardClass BoxesWizardClass;

#define BOXES_TYPE_PROPERTIES (boxes_properties_get_type ())
#define BOXES_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_PROPERTIES, BoxesProperties))
#define BOXES_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_PROPERTIES, BoxesPropertiesClass))
#define BOXES_IS_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_PROPERTIES))
#define BOXES_IS_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_PROPERTIES))
#define BOXES_PROPERTIES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_PROPERTIES, BoxesPropertiesClass))

typedef struct _BoxesProperties BoxesProperties;
typedef struct _BoxesPropertiesClass BoxesPropertiesClass;

#define BOXES_TYPE_DISPLAY_PAGE (boxes_display_page_get_type ())
#define BOXES_DISPLAY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPage))
#define BOXES_DISPLAY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPageClass))
#define BOXES_IS_DISPLAY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_DISPLAY_PAGE))
#define BOXES_IS_DISPLAY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_DISPLAY_PAGE))
#define BOXES_DISPLAY_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPageClass))

typedef struct _BoxesDisplayPage BoxesDisplayPage;
typedef struct _BoxesDisplayPageClass BoxesDisplayPageClass;

#define BOXES_TYPE_EMPTY_BOXES (boxes_empty_boxes_get_type ())
#define BOXES_EMPTY_BOXES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxes))
#define BOXES_EMPTY_BOXES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxesClass))
#define BOXES_IS_EMPTY_BOXES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_EMPTY_BOXES))
#define BOXES_IS_EMPTY_BOXES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_EMPTY_BOXES))
#define BOXES_EMPTY_BOXES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxesClass))

typedef struct _BoxesEmptyBoxes BoxesEmptyBoxes;
typedef struct _BoxesEmptyBoxesClass BoxesEmptyBoxesClass;

#define BOXES_TYPE_COLLECTION (boxes_collection_get_type ())
#define BOXES_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION, BoxesCollection))
#define BOXES_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION, BoxesCollectionClass))
#define BOXES_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION))
#define BOXES_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION))
#define BOXES_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION, BoxesCollectionClass))

typedef struct _BoxesCollection BoxesCollection;
typedef struct _BoxesCollectionClass BoxesCollectionClass;

#define BOXES_TYPE_COLLECTION_FILTER (boxes_collection_filter_get_type ())
#define BOXES_COLLECTION_FILTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilter))
#define BOXES_COLLECTION_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilterClass))
#define BOXES_IS_COLLECTION_FILTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_FILTER))
#define BOXES_IS_COLLECTION_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_FILTER))
#define BOXES_COLLECTION_FILTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilterClass))

typedef struct _BoxesCollectionFilter BoxesCollectionFilter;
typedef struct _BoxesCollectionFilterClass BoxesCollectionFilterClass;

#define BOXES_TYPE_COLLECTION_VIEW (boxes_collection_view_get_type ())
#define BOXES_COLLECTION_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionView))
#define BOXES_COLLECTION_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionViewClass))
#define BOXES_IS_COLLECTION_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_VIEW))
#define BOXES_IS_COLLECTION_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_VIEW))
#define BOXES_COLLECTION_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionViewClass))

typedef struct _BoxesCollectionView BoxesCollectionView;
typedef struct _BoxesCollectionViewClass BoxesCollectionViewClass;
typedef struct _BoxesCollectionPrivate BoxesCollectionPrivate;
#define __g_list_free__g_object_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_object_unref0_ (var), NULL)))

typedef enum  {
	BOXES_TOPBAR_PAGE_COLLECTION,
	BOXES_TOPBAR_PAGE_SELECTION,
	BOXES_TOPBAR_PAGE_WIZARD,
	BOXES_TOPBAR_PAGE_PROPERTIES
} BoxesTopbarPage;

typedef enum  {
	BOXES_UI_STATE_NONE,
	BOXES_UI_STATE_COLLECTION,
	BOXES_UI_STATE_CREDS,
	BOXES_UI_STATE_DISPLAY,
	BOXES_UI_STATE_SETTINGS,
	BOXES_UI_STATE_WIZARD,
	BOXES_UI_STATE_PROPERTIES
} BoxesUIState;

struct _BoxesUI {
	GObject parent_instance;
	BoxesUIPrivate * priv;
	BoxesUIState previous_ui_state;
};

struct _BoxesUIClass {
	GObjectClass parent_class;
	void (*ui_state_changed) (BoxesUI* self);
	ClutterActor* (*get_actor) (BoxesUI* self);
};

struct _BoxesTopbar {
	BoxesUI parent_instance;
	BoxesTopbarPrivate * priv;
	GtkNotebook* notebook;
	gchar* _status;
};

struct _BoxesTopbarClass {
	BoxesUIClass parent_class;
};

struct _BoxesTopbarPrivate {
	GtkClutterActor* gtk_actor;
	GtkSpinner* spinner;
	GtkToggleButton* search_btn;
	GtkToggleButton* search2_btn;
	GtkButton* select_btn;
	GtkButton* done_btn;
	GtkButton* back_btn;
	GtkButton* new_btn;
	GdMainToolbar* selection_toolbar;
	GdMainToolbar* collection_toolbar;
};

struct _BoxesApp {
	BoxesUI parent_instance;
	BoxesAppPrivate * priv;
	GtkApplicationWindow* window;
	GtkNotebook* notebook;
	BoxesClutterWidget* embed;
	ClutterStage* stage;
	ClutterBinLayout* stage_bin;
	ClutterActor* overlay_bin_actor;
	ClutterBinLayout* overlay_bin;
	BoxesCollectionItem* current_item;
	BoxesSearchbar* searchbar;
	BoxesTopbar* topbar;
	BoxesNotificationbar* notificationbar;
	BoxesRevealer* sidebar_revealer;
	BoxesRevealer* searchbar_revealer;
	BoxesSidebar* sidebar;
	BoxesSelectionbar* selectionbar;
	guint duration;
	GSettings* settings;
	BoxesWizard* wizard;
	BoxesProperties* properties;
	BoxesDisplayPage* display_page;
	BoxesEmptyBoxes* empty_boxes;
	BoxesCollection* collection;
	BoxesCollectionFilter* filter;
	BoxesCollectionView* view;
};

struct _BoxesAppClass {
	BoxesUIClass parent_class;
};

struct _BoxesCollection {
	GObject parent_instance;
	BoxesCollectionPrivate * priv;
	GPtrArray* items;
};

struct _BoxesCollectionClass {
	GObjectClass parent_class;
};


static gpointer boxes_topbar_parent_class = NULL;
extern BoxesApp* boxes_app_app;

GType boxes_topbar_page_get_type (void) G_GNUC_CONST;
GType boxes_ui_get_type (void) G_GNUC_CONST;
GType boxes_ui_state_get_type (void) G_GNUC_CONST;
GType boxes_topbar_get_type (void) G_GNUC_CONST;
#define BOXES_TOPBAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), BOXES_TYPE_TOPBAR, BoxesTopbarPrivate))
enum  {
	BOXES_TOPBAR_DUMMY_PROPERTY,
	BOXES_TOPBAR_ACTOR,
	BOXES_TOPBAR_STATUS
};
BoxesTopbar* boxes_topbar_new (void);
BoxesTopbar* boxes_topbar_construct (GType object_type);
BoxesUI* boxes_ui_construct (GType object_type);
static void boxes_topbar_setup_topbar (BoxesTopbar* self);
GType boxes_app_get_type (void) G_GNUC_CONST;
static void __lambda40_ (BoxesTopbar* self);
static void boxes_topbar_update_selection_label (BoxesTopbar* self);
static void ___lambda40__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static void __lambda34_ (BoxesTopbar* self, GtkButton* button);
void boxes_ui_set_ui_state (BoxesUI* self, BoxesUIState value);
static void ___lambda34__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void __lambda35_ (BoxesTopbar* self, GtkButton* button);
static void ___lambda35__gtk_button_clicked (GtkButton* _sender, gpointer self);
GType boxes_clutter_widget_get_type (void) G_GNUC_CONST;
GType boxes_collection_item_get_type (void) G_GNUC_CONST;
GType boxes_searchbar_get_type (void) G_GNUC_CONST;
GType boxes_notificationbar_get_type (void) G_GNUC_CONST;
GType boxes_revealer_get_type (void) G_GNUC_CONST;
GType boxes_sidebar_get_type (void) G_GNUC_CONST;
GType boxes_selectionbar_get_type (void) G_GNUC_CONST;
GType boxes_wizard_get_type (void) G_GNUC_CONST;
GType boxes_properties_get_type (void) G_GNUC_CONST;
GType boxes_display_page_get_type (void) G_GNUC_CONST;
GType boxes_empty_boxes_get_type (void) G_GNUC_CONST;
GType boxes_collection_get_type (void) G_GNUC_CONST;
GType boxes_collection_filter_get_type (void) G_GNUC_CONST;
GType boxes_collection_view_get_type (void) G_GNUC_CONST;
static void __lambda36_ (BoxesTopbar* self);
void boxes_app_set_selection_mode (BoxesApp* self, gboolean value);
static void ___lambda36__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void __lambda37_ (BoxesTopbar* self);
gboolean boxes_app_get_selection_mode (BoxesApp* self);
static void ___lambda37__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static void boxes_topbar_update_select_btn (BoxesTopbar* self);
static void _boxes_topbar_update_select_btn_boxes_collection_item_added (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self);
static void _boxes_topbar_update_select_btn_boxes_collection_item_removed (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self);
static void __lambda39_ (BoxesTopbar* self);
static void ___lambda39__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void boxes_topbar_update_search_btn (BoxesTopbar* self);
static void _boxes_topbar_update_search_btn_boxes_collection_item_added (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self);
static void _boxes_topbar_update_search_btn_boxes_collection_item_removed (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self);
GList* boxes_app_get_selected_items (BoxesApp* self);
static void _g_object_unref0_ (gpointer var);
static void _g_list_free__g_object_unref0_ (GList* self);
static void boxes_topbar_real_ui_state_changed (BoxesUI* base);
BoxesUIState boxes_ui_get_ui_state (BoxesUI* self);
const gchar* boxes_topbar_get_status (BoxesTopbar* self);
void boxes_topbar_set_status (BoxesTopbar* self, const gchar* value);
static void boxes_topbar_finalize (GObject* obj);
ClutterActor* boxes_ui_get_actor (BoxesUI* self);
static void _vala_boxes_topbar_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_boxes_topbar_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


GType boxes_topbar_page_get_type (void) {
	static volatile gsize boxes_topbar_page_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_topbar_page_type_id__volatile)) {
		static const GEnumValue values[] = {{BOXES_TOPBAR_PAGE_COLLECTION, "BOXES_TOPBAR_PAGE_COLLECTION", "collection"}, {BOXES_TOPBAR_PAGE_SELECTION, "BOXES_TOPBAR_PAGE_SELECTION", "selection"}, {BOXES_TOPBAR_PAGE_WIZARD, "BOXES_TOPBAR_PAGE_WIZARD", "wizard"}, {BOXES_TOPBAR_PAGE_PROPERTIES, "BOXES_TOPBAR_PAGE_PROPERTIES", "properties"}, {0, NULL, NULL}};
		GType boxes_topbar_page_type_id;
		boxes_topbar_page_type_id = g_enum_register_static ("BoxesTopbarPage", values);
		g_once_init_leave (&boxes_topbar_page_type_id__volatile, boxes_topbar_page_type_id);
	}
	return boxes_topbar_page_type_id__volatile;
}


static void __lambda40_ (BoxesTopbar* self) {
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_update_selection_label (self);
#line 379 "topbar.c"
}


static void ___lambda40__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 37 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	__lambda40_ (self);
#line 386 "topbar.c"
}


BoxesTopbar* boxes_topbar_construct (GType object_type) {
	BoxesTopbar * self = NULL;
	BoxesApp* _tmp0_;
#line 34 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self = (BoxesTopbar*) boxes_ui_construct (object_type);
#line 35 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_setup_topbar (self);
#line 37 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = boxes_app_app;
#line 37 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object ((GObject*) _tmp0_, "notify::selected-items", (GCallback) ___lambda40__g_object_notify, self, 0);
#line 34 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	return self;
#line 403 "topbar.c"
}


BoxesTopbar* boxes_topbar_new (void) {
#line 34 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	return boxes_topbar_construct (BOXES_TYPE_TOPBAR);
#line 410 "topbar.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 52 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	return self ? g_object_ref (self) : NULL;
#line 417 "topbar.c"
}


static void __lambda34_ (BoxesTopbar* self, GtkButton* button) {
	BoxesApp* _tmp0_;
#line 58 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_return_if_fail (button != NULL);
#line 58 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = boxes_app_app;
#line 58 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_ui_set_ui_state ((BoxesUI*) _tmp0_, BOXES_UI_STATE_WIZARD);
#line 429 "topbar.c"
}


static void ___lambda34__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 58 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	__lambda34_ (self, _sender);
#line 436 "topbar.c"
}


static void __lambda35_ (BoxesTopbar* self, GtkButton* button) {
	BoxesApp* _tmp0_;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_return_if_fail (button != NULL);
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = boxes_app_app;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_ui_set_ui_state ((BoxesUI*) _tmp0_, BOXES_UI_STATE_COLLECTION);
#line 448 "topbar.c"
}


static void ___lambda35__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	__lambda35_ (self, _sender);
#line 455 "topbar.c"
}


static void __lambda36_ (BoxesTopbar* self) {
	BoxesApp* _tmp0_;
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = boxes_app_app;
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_app_set_selection_mode (_tmp0_, TRUE);
#line 465 "topbar.c"
}


static void ___lambda36__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 73 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	__lambda36_ (self);
#line 472 "topbar.c"
}


static void __lambda37_ (BoxesTopbar* self) {
	gint _tmp0_ = 0;
	BoxesApp* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	GtkNotebook* _tmp5_;
	gint _tmp6_;
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp1_ = boxes_app_app;
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp2_ = boxes_app_get_selection_mode (_tmp1_);
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp3_ = _tmp2_;
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	if (_tmp3_) {
#line 78 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp0_ = (gint) BOXES_TOPBAR_PAGE_SELECTION;
#line 493 "topbar.c"
	} else {
		GtkNotebook* _tmp4_;
#line 78 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp4_ = self->notebook;
#line 78 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		g_object_set (_tmp4_, "page", (gint) BOXES_TOPBAR_PAGE_COLLECTION, NULL);
#line 78 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp0_ = (gint) BOXES_TOPBAR_PAGE_COLLECTION;
#line 502 "topbar.c"
	}
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp5_ = self->notebook;
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp6_ = _tmp0_;
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_set (_tmp5_, "page", _tmp6_, NULL);
#line 510 "topbar.c"
}


static void ___lambda37__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 76 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	__lambda37_ (self);
#line 517 "topbar.c"
}


static void _boxes_topbar_update_select_btn_boxes_collection_item_added (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self) {
#line 81 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_update_select_btn (self);
#line 524 "topbar.c"
}


static void _boxes_topbar_update_select_btn_boxes_collection_item_removed (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self) {
#line 82 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_update_select_btn (self);
#line 531 "topbar.c"
}


static void __lambda39_ (BoxesTopbar* self) {
	BoxesApp* _tmp0_;
#line 108 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = boxes_app_app;
#line 108 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_app_set_selection_mode (_tmp0_, FALSE);
#line 541 "topbar.c"
}


static void ___lambda39__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 107 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	__lambda39_ (self);
#line 548 "topbar.c"
}


static void _boxes_topbar_update_search_btn_boxes_collection_item_added (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self) {
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_update_search_btn (self);
#line 555 "topbar.c"
}


static void _boxes_topbar_update_search_btn_boxes_collection_item_removed (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self) {
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_update_search_btn (self);
#line 562 "topbar.c"
}


static void boxes_topbar_setup_topbar (BoxesTopbar* self) {
	GtkNotebook* _tmp0_;
	GtkNotebook* _tmp1_;
	GtkClutterActor* _tmp2_;
	GtkClutterActor* _tmp3_;
	GtkBox* _tmp4_;
	GtkBox* hbox;
	GtkNotebook* _tmp5_;
	GtkBox* _tmp6_;
	GdMainToolbar* _tmp7_;
	GdMainToolbar* toolbar;
	GdMainToolbar* _tmp8_;
	GtkStyleContext* _tmp9_ = NULL;
	GtkBox* _tmp10_;
	const gchar* _tmp11_ = NULL;
	GtkWidget* _tmp12_ = NULL;
	GtkButton* _tmp13_;
	GtkButton* _tmp14_;
	GtkButton* _tmp15_;
	GtkWidget* _tmp16_ = NULL;
	GtkButton* _tmp17_;
	GtkButton* _tmp18_;
	GtkSpinner* _tmp19_;
	GtkSpinner* _tmp20_;
	GtkSpinner* _tmp21_;
	GtkSpinner* _tmp22_;
	GtkSpinner* _tmp23_;
	GtkWidget* _tmp24_ = NULL;
	GtkToggleButton* _tmp25_;
	GtkToggleButton* _tmp26_;
	BoxesApp* _tmp27_;
	BoxesSearchbar* _tmp28_;
	GtkWidget* _tmp29_ = NULL;
	GtkButton* _tmp30_;
	GtkButton* _tmp31_;
	BoxesApp* _tmp32_;
	BoxesApp* _tmp33_;
	BoxesCollection* _tmp34_;
	BoxesApp* _tmp35_;
	BoxesCollection* _tmp36_;
	GtkBox* _tmp37_;
	GtkNotebook* _tmp38_;
	GtkBox* _tmp39_;
	GdMainToolbar* _tmp40_;
	GdMainToolbar* _tmp41_;
	GtkStyleContext* _tmp42_ = NULL;
	GdMainToolbar* _tmp43_;
	GtkStyleContext* _tmp44_ = NULL;
	GMenu* _tmp45_;
	GMenu* menu;
	const gchar* _tmp46_ = NULL;
	const gchar* _tmp47_ = NULL;
	const gchar* _tmp48_ = NULL;
	GdMainToolbar* _tmp49_;
	GtkBox* _tmp50_;
	GdMainToolbar* _tmp51_;
	GdMainToolbar* _tmp52_;
	GtkWidget* _tmp53_ = NULL;
	GtkToggleButton* _tmp54_;
	GtkToggleButton* _tmp55_;
	BoxesApp* _tmp56_;
	BoxesSearchbar* _tmp57_;
	GdMainToolbar* _tmp58_;
	const gchar* _tmp59_ = NULL;
	GtkWidget* _tmp60_ = NULL;
	GtkButton* _tmp61_;
	GtkButton* _tmp62_;
	GtkStyleContext* _tmp63_ = NULL;
	GtkButton* _tmp64_;
	GtkButton* _tmp65_;
	GtkBox* _tmp66_;
	GtkNotebook* _tmp67_;
	GtkBox* _tmp68_;
	GtkBox* _tmp69_;
	GtkNotebook* _tmp70_;
	GtkBox* _tmp71_;
	BoxesApp* _tmp72_;
	BoxesCollection* _tmp73_;
	BoxesApp* _tmp74_;
	BoxesCollection* _tmp75_;
	GtkNotebook* _tmp76_;
	GtkNotebook* _tmp77_;
#line 42 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_return_if_fail (self != NULL);
#line 43 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = (GtkNotebook*) gtk_notebook_new ();
#line 43 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_ref_sink (_tmp0_);
#line 43 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->notebook);
#line 43 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->notebook = _tmp0_;
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp1_ = self->notebook;
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp2_ = (GtkClutterActor*) gtk_clutter_actor_new_with_contents ((GtkWidget*) _tmp1_);
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_ref_sink (_tmp2_);
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->gtk_actor);
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->gtk_actor = _tmp2_;
#line 45 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp3_ = self->priv->gtk_actor;
#line 45 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	clutter_actor_set_name ((ClutterActor*) _tmp3_, "topbar");
#line 48 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp4_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
#line 48 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_ref_sink (_tmp4_);
#line 48 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	hbox = _tmp4_;
#line 49 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp5_ = self->notebook;
#line 49 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp6_ = hbox;
#line 49 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_notebook_append_page (_tmp5_, (GtkWidget*) _tmp6_, NULL);
#line 51 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp7_ = (GdMainToolbar*) gd_main_toolbar_new ();
#line 51 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_ref_sink (_tmp7_);
#line 51 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	toolbar = _tmp7_;
#line 52 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp8_ = _g_object_ref0 (toolbar);
#line 52 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->collection_toolbar);
#line 52 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->collection_toolbar = _tmp8_;
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp9_ = gtk_widget_get_style_context ((GtkWidget*) toolbar);
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_style_context_add_class (_tmp9_, GTK_STYLE_CLASS_MENUBAR);
#line 54 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp10_ = hbox;
#line 54 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_box_pack_start (_tmp10_, (GtkWidget*) toolbar, TRUE, TRUE, (guint) 0);
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp11_ = _ ("New");
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp12_ = gd_main_toolbar_add_button (toolbar, NULL, _tmp11_, TRUE);
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp13_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp12_, GTK_TYPE_BUTTON) ? ((GtkButton*) _tmp12_) : NULL);
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->new_btn);
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->new_btn = _tmp13_;
#line 57 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp14_ = self->priv->new_btn;
#line 57 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_widget_set_size_request ((GtkWidget*) _tmp14_, 70, -1);
#line 58 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp15_ = self->priv->new_btn;
#line 58 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object (_tmp15_, "clicked", (GCallback) ___lambda34__gtk_button_clicked, self, 0);
#line 60 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp16_ = gd_main_toolbar_add_button (toolbar, "go-previous-symbolic", NULL, TRUE);
#line 60 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp17_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp16_, GTK_TYPE_BUTTON) ? ((GtkButton*) _tmp16_) : NULL);
#line 60 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->back_btn);
#line 60 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->back_btn = _tmp17_;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp18_ = self->priv->back_btn;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object (_tmp18_, "clicked", (GCallback) ___lambda35__gtk_button_clicked, self, 0);
#line 63 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp19_ = (GtkSpinner*) gtk_spinner_new ();
#line 63 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_ref_sink (_tmp19_);
#line 63 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->spinner);
#line 63 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->spinner = _tmp19_;
#line 64 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp20_ = self->priv->spinner;
#line 64 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_spinner_start (_tmp20_);
#line 65 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp21_ = self->priv->spinner;
#line 65 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_widget_set_hexpand ((GtkWidget*) _tmp21_, TRUE);
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp22_ = self->priv->spinner;
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_widget_set_vexpand ((GtkWidget*) _tmp22_, TRUE);
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp23_ = self->priv->spinner;
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gd_main_toolbar_add_widget (toolbar, (GtkWidget*) _tmp23_, FALSE);
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp24_ = gd_main_toolbar_add_toggle (toolbar, "edit-find-symbolic", NULL, FALSE);
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp25_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp24_, GTK_TYPE_TOGGLE_BUTTON) ? ((GtkToggleButton*) _tmp24_) : NULL);
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->search_btn);
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->search_btn = _tmp25_;
#line 70 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp26_ = self->priv->search_btn;
#line 70 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp27_ = boxes_app_app;
#line 70 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp28_ = _tmp27_->searchbar;
#line 70 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_bind_property_with_closures ((GObject*) _tmp26_, "active", (GObject*) _tmp28_, "visible", G_BINDING_BIDIRECTIONAL, (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)), (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)));
#line 72 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp29_ = gd_main_toolbar_add_button (toolbar, "object-select-symbolic", NULL, FALSE);
#line 72 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp30_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp29_, GTK_TYPE_BUTTON) ? ((GtkButton*) _tmp29_) : NULL);
#line 72 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->select_btn);
#line 72 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->select_btn = _tmp30_;
#line 73 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp31_ = self->priv->select_btn;
#line 73 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object (_tmp31_, "clicked", (GCallback) ___lambda36__gtk_button_clicked, self, 0);
#line 76 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp32_ = boxes_app_app;
#line 76 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object ((GObject*) _tmp32_, "notify::selection-mode", (GCallback) ___lambda37__g_object_notify, self, 0);
#line 80 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_update_select_btn (self);
#line 81 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp33_ = boxes_app_app;
#line 81 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp34_ = _tmp33_->collection;
#line 81 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object (_tmp34_, "item-added", (GCallback) _boxes_topbar_update_select_btn_boxes_collection_item_added, self, 0);
#line 82 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp35_ = boxes_app_app;
#line 82 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp36_ = _tmp35_->collection;
#line 82 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object (_tmp36_, "item-removed", (GCallback) _boxes_topbar_update_select_btn_boxes_collection_item_removed, self, 0);
#line 85 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp37_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
#line 85 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_ref_sink (_tmp37_);
#line 85 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (hbox);
#line 85 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	hbox = _tmp37_;
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp38_ = self->notebook;
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp39_ = hbox;
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_notebook_append_page (_tmp38_, (GtkWidget*) _tmp39_, NULL);
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp40_ = (GdMainToolbar*) gd_main_toolbar_new ();
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_ref_sink (_tmp40_);
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->selection_toolbar);
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->selection_toolbar = _tmp40_;
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp41_ = self->priv->selection_toolbar;
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp42_ = gtk_widget_get_style_context ((GtkWidget*) _tmp41_);
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_style_context_add_class (_tmp42_, "selection-mode");
#line 89 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp43_ = self->priv->selection_toolbar;
#line 89 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp44_ = gtk_widget_get_style_context ((GtkWidget*) _tmp43_);
#line 89 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_style_context_add_class (_tmp44_, GTK_STYLE_CLASS_MENUBAR);
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp45_ = g_menu_new ();
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	menu = _tmp45_;
#line 92 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp46_ = _ ("Select All");
#line 92 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_menu_append (menu, _tmp46_, "app.select-all");
#line 93 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp47_ = _ ("Select Running");
#line 93 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_menu_append (menu, _tmp47_, "app.select-running");
#line 94 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp48_ = _ ("Select None");
#line 94 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_menu_append (menu, _tmp48_, "app.select-none");
#line 96 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp49_ = self->priv->selection_toolbar;
#line 96 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gd_main_toolbar_set_labels_menu (_tmp49_, (GMenuModel*) menu);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp50_ = hbox;
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp51_ = self->priv->selection_toolbar;
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_box_pack_start (_tmp50_, (GtkWidget*) _tmp51_, TRUE, TRUE, (guint) 0);
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_update_selection_label (self);
#line 101 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp52_ = self->priv->selection_toolbar;
#line 101 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp53_ = gd_main_toolbar_add_toggle (_tmp52_, "edit-find-symbolic", NULL, FALSE);
#line 101 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp54_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp53_, GTK_TYPE_TOGGLE_BUTTON) ? ((GtkToggleButton*) _tmp53_) : NULL);
#line 101 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->search2_btn);
#line 101 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->search2_btn = _tmp54_;
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp55_ = self->priv->search2_btn;
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp56_ = boxes_app_app;
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp57_ = _tmp56_->searchbar;
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_bind_property_with_closures ((GObject*) _tmp55_, "active", (GObject*) _tmp57_, "visible", G_BINDING_BIDIRECTIONAL, (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)), (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)));
#line 104 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp58_ = self->priv->selection_toolbar;
#line 104 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp59_ = _ ("_Done");
#line 104 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp60_ = gd_main_toolbar_add_button (_tmp58_, NULL, _tmp59_, FALSE);
#line 104 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp61_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp60_, GTK_TYPE_BUTTON) ? ((GtkButton*) _tmp60_) : NULL);
#line 104 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->done_btn);
#line 104 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv->done_btn = _tmp61_;
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp62_ = self->priv->done_btn;
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp63_ = gtk_widget_get_style_context ((GtkWidget*) _tmp62_);
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_style_context_add_class (_tmp63_, "suggested-action");
#line 106 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp64_ = self->priv->done_btn;
#line 106 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_button_set_use_stock (_tmp64_, TRUE);
#line 107 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp65_ = self->priv->done_btn;
#line 107 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object (_tmp65_, "clicked", (GCallback) ___lambda39__gtk_button_clicked, self, 0);
#line 112 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp66_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
#line 112 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_ref_sink (_tmp66_);
#line 112 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (hbox);
#line 112 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	hbox = _tmp66_;
#line 113 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp67_ = self->notebook;
#line 113 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp68_ = hbox;
#line 113 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_notebook_append_page (_tmp67_, (GtkWidget*) _tmp68_, NULL);
#line 116 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp69_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
#line 116 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_ref_sink (_tmp69_);
#line 116 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (hbox);
#line 116 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	hbox = _tmp69_;
#line 117 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp70_ = self->notebook;
#line 117 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp71_ = hbox;
#line 117 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_notebook_append_page (_tmp70_, (GtkWidget*) _tmp71_, NULL);
#line 120 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_update_search_btn (self);
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp72_ = boxes_app_app;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp73_ = _tmp72_->collection;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object (_tmp73_, "item-added", (GCallback) _boxes_topbar_update_search_btn_boxes_collection_item_added, self, 0);
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp74_ = boxes_app_app;
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp75_ = _tmp74_->collection;
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_signal_connect_object (_tmp75_, "item-removed", (GCallback) _boxes_topbar_update_search_btn_boxes_collection_item_removed, self, 0);
#line 124 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp76_ = self->notebook;
#line 124 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_notebook_set_show_tabs (_tmp76_, FALSE);
#line 125 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp77_ = self->notebook;
#line 125 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp77_);
#line 42 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (menu);
#line 42 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (toolbar);
#line 42 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (hbox);
#line 966 "topbar.c"
}


static gint g_ptr_array_get_length (GPtrArray* self) {
	gint result;
	guint _tmp0_;
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, 0);
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	_tmp0_ = self->len;
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	result = (gint) _tmp0_;
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	return result;
#line 981 "topbar.c"
}


static void g_ptr_array_set_length (GPtrArray* self, gint value) {
	gint _tmp0_;
#line 4081 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_return_if_fail (self != NULL);
#line 4081 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	_tmp0_ = value;
#line 4081 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_ptr_array_set_size (self, _tmp0_);
#line 993 "topbar.c"
}


static void boxes_topbar_update_search_btn (BoxesTopbar* self) {
	GtkToggleButton* _tmp0_;
	BoxesApp* _tmp1_;
	BoxesCollection* _tmp2_;
	GPtrArray* _tmp3_;
	gint _tmp4_;
	gint _tmp5_;
	GtkToggleButton* _tmp6_;
	BoxesApp* _tmp7_;
	BoxesCollection* _tmp8_;
	GPtrArray* _tmp9_;
	gint _tmp10_;
	gint _tmp11_;
#line 128 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_return_if_fail (self != NULL);
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = self->priv->search_btn;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp1_ = boxes_app_app;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp2_ = _tmp1_->collection;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp3_ = _tmp2_->items;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp4_ = g_ptr_array_get_length (_tmp3_);
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp5_ = _tmp4_;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_widget_set_sensitive ((GtkWidget*) _tmp0_, _tmp5_ != 0);
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp6_ = self->priv->search2_btn;
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp7_ = boxes_app_app;
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp8_ = _tmp7_->collection;
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp9_ = _tmp8_->items;
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp10_ = g_ptr_array_get_length (_tmp9_);
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp11_ = _tmp10_;
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_widget_set_sensitive ((GtkWidget*) _tmp6_, _tmp11_ != 0);
#line 1040 "topbar.c"
}


static void boxes_topbar_update_select_btn (BoxesTopbar* self) {
	GtkButton* _tmp0_;
	BoxesApp* _tmp1_;
	BoxesCollection* _tmp2_;
	GPtrArray* _tmp3_;
	gint _tmp4_;
	gint _tmp5_;
#line 133 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_return_if_fail (self != NULL);
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = self->priv->select_btn;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp1_ = boxes_app_app;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp2_ = _tmp1_->collection;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp3_ = _tmp2_->items;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp4_ = g_ptr_array_get_length (_tmp3_);
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp5_ = _tmp4_;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gtk_widget_set_sensitive ((GtkWidget*) _tmp0_, _tmp5_ != 0);
#line 1067 "topbar.c"
}


static void _g_object_unref0_ (gpointer var) {
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
#line 1074 "topbar.c"
}


static void _g_list_free__g_object_unref0_ (GList* self) {
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_list_foreach (self, (GFunc) _g_object_unref0_, NULL);
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_list_free (self);
#line 1083 "topbar.c"
}


static void boxes_topbar_update_selection_label (BoxesTopbar* self) {
	BoxesApp* _tmp0_;
	GList* _tmp1_;
	GList* _tmp2_;
	GList* _tmp3_;
	guint _tmp4_ = 0U;
	guint _tmp5_;
	guint items;
	guint _tmp6_;
#line 137 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_return_if_fail (self != NULL);
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = boxes_app_app;
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp1_ = boxes_app_get_selected_items (_tmp0_);
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp2_ = _tmp1_;
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp3_ = _tmp2_;
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp4_ = g_list_length (_tmp3_);
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp5_ = _tmp4_;
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	__g_list_free__g_object_unref0_0 (_tmp3_);
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	items = _tmp5_;
#line 139 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp6_ = items;
#line 139 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	if (_tmp6_ > ((guint) 0)) {
#line 1118 "topbar.c"
		GdMainToolbar* _tmp7_;
		guint _tmp8_;
		const gchar* _tmp9_ = NULL;
		guint _tmp10_;
		gchar* _tmp11_ = NULL;
		gchar* _tmp12_;
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp7_ = self->priv->selection_toolbar;
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp8_ = items;
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp9_ = ngettext ("%d selected", "%d selected", (gulong) _tmp8_);
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp10_ = items;
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp11_ = g_strdup_printf (_tmp9_, _tmp10_);
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp12_ = _tmp11_;
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		gd_main_toolbar_set_labels (_tmp7_, _tmp12_, NULL);
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_g_free0 (_tmp12_);
#line 1141 "topbar.c"
	} else {
		GdMainToolbar* _tmp13_;
		const gchar* _tmp14_ = NULL;
#line 144 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp13_ = self->priv->selection_toolbar;
#line 144 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		_tmp14_ = _ ("(Click on items to select them)");
#line 144 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		gd_main_toolbar_set_labels (_tmp13_, NULL, _tmp14_);
#line 1151 "topbar.c"
	}
}


static void boxes_topbar_real_ui_state_changed (BoxesUI* base) {
	BoxesTopbar * self;
	BoxesUIState _tmp0_;
	BoxesUIState _tmp1_;
#line 147 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self = (BoxesTopbar*) base;
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp1_ = _tmp0_;
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	switch (_tmp1_) {
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		case BOXES_UI_STATE_COLLECTION:
#line 1170 "topbar.c"
		{
			GtkNotebook* _tmp2_;
			GtkButton* _tmp3_;
			GtkSpinner* _tmp4_;
			GtkButton* _tmp5_;
			GtkToggleButton* _tmp6_;
			GtkButton* _tmp7_;
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp2_ = self->notebook;
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			g_object_set (_tmp2_, "page", (gint) BOXES_TOPBAR_PAGE_COLLECTION, NULL);
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp3_ = self->priv->back_btn;
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_hide ((GtkWidget*) _tmp3_);
#line 152 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp4_ = self->priv->spinner;
#line 152 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_hide ((GtkWidget*) _tmp4_);
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp5_ = self->priv->select_btn;
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_show ((GtkWidget*) _tmp5_);
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp6_ = self->priv->search_btn;
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_show ((GtkWidget*) _tmp6_);
#line 155 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp7_ = self->priv->new_btn;
#line 155 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_show ((GtkWidget*) _tmp7_);
#line 156 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			break;
#line 1204 "topbar.c"
		}
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		case BOXES_UI_STATE_CREDS:
#line 1208 "topbar.c"
		{
			GtkNotebook* _tmp8_;
			GtkButton* _tmp9_;
			GtkButton* _tmp10_;
			GtkSpinner* _tmp11_;
			GtkButton* _tmp12_;
			GtkToggleButton* _tmp13_;
#line 159 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp8_ = self->notebook;
#line 159 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			g_object_set (_tmp8_, "page", (gint) BOXES_TOPBAR_PAGE_COLLECTION, NULL);
#line 160 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp9_ = self->priv->new_btn;
#line 160 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_hide ((GtkWidget*) _tmp9_);
#line 161 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp10_ = self->priv->back_btn;
#line 161 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_show ((GtkWidget*) _tmp10_);
#line 162 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp11_ = self->priv->spinner;
#line 162 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_show ((GtkWidget*) _tmp11_);
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp12_ = self->priv->select_btn;
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_hide ((GtkWidget*) _tmp12_);
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp13_ = self->priv->search_btn;
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			gtk_widget_hide ((GtkWidget*) _tmp13_);
#line 165 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			break;
#line 1242 "topbar.c"
		}
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		case BOXES_UI_STATE_DISPLAY:
#line 1246 "topbar.c"
		{
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			break;
#line 1250 "topbar.c"
		}
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		case BOXES_UI_STATE_PROPERTIES:
#line 1254 "topbar.c"
		{
			GtkNotebook* _tmp14_;
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp14_ = self->notebook;
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			g_object_set (_tmp14_, "page", (gint) BOXES_TOPBAR_PAGE_PROPERTIES, NULL);
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			break;
#line 1263 "topbar.c"
		}
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		case BOXES_UI_STATE_WIZARD:
#line 1267 "topbar.c"
		{
			GtkNotebook* _tmp15_;
#line 175 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			_tmp15_ = self->notebook;
#line 175 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			g_object_set (_tmp15_, "page", (gint) BOXES_TOPBAR_PAGE_WIZARD, NULL);
#line 176 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			break;
#line 1276 "topbar.c"
		}
		default:
		{
#line 179 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
			break;
#line 1282 "topbar.c"
		}
	}
}


static ClutterActor* boxes_topbar_real_get_actor (BoxesUI* base) {
	ClutterActor* result;
	BoxesTopbar* self;
	GtkClutterActor* _tmp0_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self = (BoxesTopbar*) base;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = self->priv->gtk_actor;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	result = (ClutterActor*) _tmp0_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	return result;
#line 1300 "topbar.c"
}


const gchar* boxes_topbar_get_status (BoxesTopbar* self) {
	const gchar* result;
	const gchar* _tmp0_;
#line 30 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 30 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = self->_status;
#line 30 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	result = _tmp0_;
#line 30 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	return result;
#line 1315 "topbar.c"
}


void boxes_topbar_set_status (BoxesTopbar* self, const gchar* value) {
	const gchar* _tmp0_;
	gchar* _tmp1_;
	GdMainToolbar* _tmp2_;
	const gchar* _tmp3_;
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_return_if_fail (self != NULL);
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp0_ = value;
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_free0 (self->_status);
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->_status = _tmp1_;
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp2_ = self->priv->collection_toolbar;
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_tmp3_ = self->_status;
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	gd_main_toolbar_set_labels (_tmp2_, _tmp3_, NULL);
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_notify ((GObject *) self, "status");
#line 1342 "topbar.c"
}


static void boxes_topbar_class_init (BoxesTopbarClass * klass) {
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	boxes_topbar_parent_class = g_type_class_peek_parent (klass);
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_type_class_add_private (klass, sizeof (BoxesTopbarPrivate));
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	BOXES_UI_CLASS (klass)->ui_state_changed = boxes_topbar_real_ui_state_changed;
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	BOXES_UI_CLASS (klass)->get_actor = boxes_topbar_real_get_actor;
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_boxes_topbar_get_property;
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_boxes_topbar_set_property;
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	G_OBJECT_CLASS (klass)->finalize = boxes_topbar_finalize;
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_TOPBAR_ACTOR, g_param_spec_object ("actor", "actor", "actor", clutter_actor_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_TOPBAR_STATUS, g_param_spec_string ("status", "status", "status", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 1365 "topbar.c"
}


static void boxes_topbar_instance_init (BoxesTopbar * self) {
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self->priv = BOXES_TOPBAR_GET_PRIVATE (self);
#line 1372 "topbar.c"
}


static void boxes_topbar_finalize (GObject* obj) {
	BoxesTopbar * self;
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BOXES_TYPE_TOPBAR, BoxesTopbar);
#line 15 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->gtk_actor);
#line 16 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->notebook);
#line 18 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->spinner);
#line 19 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->search_btn);
#line 20 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->search2_btn);
#line 21 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->select_btn);
#line 22 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->done_btn);
#line 23 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->back_btn);
#line 24 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->new_btn);
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->selection_toolbar);
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_object_unref0 (self->priv->collection_toolbar);
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	_g_free0 (self->_status);
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	G_OBJECT_CLASS (boxes_topbar_parent_class)->finalize (obj);
#line 1406 "topbar.c"
}


GType boxes_topbar_get_type (void) {
	static volatile gsize boxes_topbar_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_topbar_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (BoxesTopbarClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) boxes_topbar_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BoxesTopbar), 0, (GInstanceInitFunc) boxes_topbar_instance_init, NULL };
		GType boxes_topbar_type_id;
		boxes_topbar_type_id = g_type_register_static (BOXES_TYPE_UI, "BoxesTopbar", &g_define_type_info, 0);
		g_once_init_leave (&boxes_topbar_type_id__volatile, boxes_topbar_type_id);
	}
	return boxes_topbar_type_id__volatile;
}


static void _vala_boxes_topbar_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	BoxesTopbar * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_TOPBAR, BoxesTopbar);
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	switch (property_id) {
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		case BOXES_TOPBAR_ACTOR:
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		g_value_set_object (value, boxes_ui_get_actor ((BoxesUI*) self));
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		break;
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		case BOXES_TOPBAR_STATUS:
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		g_value_set_string (value, boxes_topbar_get_status (self));
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		break;
#line 1439 "topbar.c"
		default:
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		break;
#line 1445 "topbar.c"
	}
}


static void _vala_boxes_topbar_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	BoxesTopbar * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_TOPBAR, BoxesTopbar);
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
	switch (property_id) {
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		case BOXES_TOPBAR_STATUS:
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		boxes_topbar_set_status (self, g_value_get_string (value));
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		break;
#line 1461 "topbar.c"
		default:
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/topbar.vala"
		break;
#line 1467 "topbar.c"
	}
}



