/* libvirt-machine.c generated by valac 0.18.0.79-04df6, the Vala compiler
 * generated from libvirt-machine.vala, do not modify */

/* This file is part of GNOME Boxes. License: LGPLv2+*/

#include <glib.h>
#include <glib-object.h>
#include <clutter/clutter.h>
#include <gio/gio.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <libcommon.h>
#include <float.h>
#include <math.h>
#include <libvirt-gobject/libvirt-gobject.h>
#include <libvirt-gconfig/libvirt-gconfig.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <clutter-gtk/clutter-gtk.h>
#include <glib/gi18n-lib.h>


#define BOXES_TYPE_UI (boxes_ui_get_type ())
#define BOXES_UI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_UI, BoxesUI))
#define BOXES_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_UI, BoxesUIClass))
#define BOXES_IS_UI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_UI))
#define BOXES_IS_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_UI))
#define BOXES_UI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_UI, BoxesUIClass))

typedef struct _BoxesUI BoxesUI;
typedef struct _BoxesUIClass BoxesUIClass;
typedef struct _BoxesUIPrivate BoxesUIPrivate;

#define BOXES_TYPE_UI_STATE (boxes_ui_state_get_type ())

#define BOXES_TYPE_COLLECTION_ITEM (boxes_collection_item_get_type ())
#define BOXES_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItem))
#define BOXES_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))
#define BOXES_IS_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_IS_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_COLLECTION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))

typedef struct _BoxesCollectionItem BoxesCollectionItem;
typedef struct _BoxesCollectionItemClass BoxesCollectionItemClass;
typedef struct _BoxesCollectionItemPrivate BoxesCollectionItemPrivate;

#define BOXES_TYPE_IPROPERTIES_PROVIDER (boxes_iproperties_provider_get_type ())
#define BOXES_IPROPERTIES_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER, BoxesIPropertiesProvider))
#define BOXES_IS_IPROPERTIES_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER))
#define BOXES_IPROPERTIES_PROVIDER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER, BoxesIPropertiesProviderIface))

typedef struct _BoxesIPropertiesProvider BoxesIPropertiesProvider;
typedef struct _BoxesIPropertiesProviderIface BoxesIPropertiesProviderIface;

#define BOXES_TYPE_PROPERTIES_PAGE (boxes_properties_page_get_type ())

#define TYPE_PROPERTY_CREATION_FLAG (property_creation_flag_get_type ())

#define BOXES_TYPE_PROPERTY (boxes_property_get_type ())
#define BOXES_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_PROPERTY, BoxesProperty))
#define BOXES_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_PROPERTY, BoxesPropertyClass))
#define BOXES_IS_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_PROPERTY))
#define BOXES_IS_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_PROPERTY))
#define BOXES_PROPERTY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_PROPERTY, BoxesPropertyClass))

typedef struct _BoxesProperty BoxesProperty;
typedef struct _BoxesPropertyClass BoxesPropertyClass;

#define BOXES_TYPE_MACHINE (boxes_machine_get_type ())
#define BOXES_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_MACHINE, BoxesMachine))
#define BOXES_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_MACHINE, BoxesMachineClass))
#define BOXES_IS_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_MACHINE))
#define BOXES_IS_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_MACHINE))
#define BOXES_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_MACHINE, BoxesMachineClass))

typedef struct _BoxesMachine BoxesMachine;
typedef struct _BoxesMachineClass BoxesMachineClass;
typedef struct _BoxesMachinePrivate BoxesMachinePrivate;

#define BOXES_TYPE_LIBVIRT_MACHINE (boxes_libvirt_machine_get_type ())
#define BOXES_LIBVIRT_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachine))
#define BOXES_LIBVIRT_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachineClass))
#define BOXES_IS_LIBVIRT_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_LIBVIRT_MACHINE))
#define BOXES_IS_LIBVIRT_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_LIBVIRT_MACHINE))
#define BOXES_LIBVIRT_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachineClass))

typedef struct _BoxesLibvirtMachine BoxesLibvirtMachine;
typedef struct _BoxesLibvirtMachineClass BoxesLibvirtMachineClass;
typedef struct _BoxesLibvirtMachinePrivate BoxesLibvirtMachinePrivate;

#define BOXES_TYPE_VM_CREATOR (boxes_vm_creator_get_type ())
#define BOXES_VM_CREATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_VM_CREATOR, BoxesVMCreator))
#define BOXES_VM_CREATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_VM_CREATOR, BoxesVMCreatorClass))
#define BOXES_IS_VM_CREATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_VM_CREATOR))
#define BOXES_IS_VM_CREATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_VM_CREATOR))
#define BOXES_VM_CREATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_VM_CREATOR, BoxesVMCreatorClass))

typedef struct _BoxesVMCreator BoxesVMCreator;
typedef struct _BoxesVMCreatorClass BoxesVMCreatorClass;

#define BOXES_TYPE_LIBVIRT_MACHINE_PROPERTIES (boxes_libvirt_machine_properties_get_type ())
#define BOXES_LIBVIRT_MACHINE_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_LIBVIRT_MACHINE_PROPERTIES, BoxesLibvirtMachineProperties))
#define BOXES_LIBVIRT_MACHINE_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_LIBVIRT_MACHINE_PROPERTIES, BoxesLibvirtMachinePropertiesClass))
#define BOXES_IS_LIBVIRT_MACHINE_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_LIBVIRT_MACHINE_PROPERTIES))
#define BOXES_IS_LIBVIRT_MACHINE_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_LIBVIRT_MACHINE_PROPERTIES))
#define BOXES_LIBVIRT_MACHINE_PROPERTIES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_LIBVIRT_MACHINE_PROPERTIES, BoxesLibvirtMachinePropertiesClass))

typedef struct _BoxesLibvirtMachineProperties BoxesLibvirtMachineProperties;
typedef struct _BoxesLibvirtMachinePropertiesClass BoxesLibvirtMachinePropertiesClass;

#define BOXES_LIBVIRT_MACHINE_TYPE_MACHINE_STAT (boxes_libvirt_machine_machine_stat_get_type ())
typedef struct _BoxesLibvirtMachineMachineStat BoxesLibvirtMachineMachineStat;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _boxes_vm_creator_unref0(var) ((var == NULL) ? NULL : (var = (boxes_vm_creator_unref (var), NULL)))

#define BOXES_TYPE_DISPLAY (boxes_display_get_type ())
#define BOXES_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_DISPLAY, BoxesDisplay))
#define BOXES_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_DISPLAY, BoxesDisplayClass))
#define BOXES_IS_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_DISPLAY))
#define BOXES_IS_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_DISPLAY))
#define BOXES_DISPLAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_DISPLAY, BoxesDisplayClass))

typedef struct _BoxesDisplay BoxesDisplay;
typedef struct _BoxesDisplayClass BoxesDisplayClass;
typedef struct _BoxesLibvirtMachineConnectDisplayData BoxesLibvirtMachineConnectDisplayData;
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define __vala_GVirDomainInfo_free0(var) ((var == NULL) ? NULL : (var = (_vala_GVirDomainInfo_free (var), NULL)))

#define BOXES_MACHINE_TYPE_MACHINE_STATE (boxes_machine_machine_state_get_type ())
typedef struct _Block18Data Block18Data;
typedef struct _Block19Data Block19Data;
#define __vala_GVirDomainDiskStats_free0(var) ((var == NULL) ? NULL : (var = (_vala_GVirDomainDiskStats_free (var), NULL)))
typedef struct _BoxesLibvirtMachineUpdateIoStatData BoxesLibvirtMachineUpdateIoStatData;
typedef struct _Block20Data Block20Data;
typedef struct _Block21Data Block21Data;
#define __vala_GVirDomainInterfaceStats_free0(var) ((var == NULL) ? NULL : (var = (_vala_GVirDomainInterfaceStats_free (var), NULL)))
typedef struct _BoxesLibvirtMachineUpdateNetStatData BoxesLibvirtMachineUpdateNetStatData;
typedef struct _BoxesLibvirtMachineUpdateStatsData BoxesLibvirtMachineUpdateStatsData;
typedef struct _Block22Data Block22Data;
#define __g_list_free__g_object_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_object_unref0_ (var), NULL)))

#define BOXES_TYPE_SPICE_DISPLAY (boxes_spice_display_get_type ())
#define BOXES_SPICE_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SPICE_DISPLAY, BoxesSpiceDisplay))
#define BOXES_SPICE_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SPICE_DISPLAY, BoxesSpiceDisplayClass))
#define BOXES_IS_SPICE_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SPICE_DISPLAY))
#define BOXES_IS_SPICE_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SPICE_DISPLAY))
#define BOXES_SPICE_DISPLAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SPICE_DISPLAY, BoxesSpiceDisplayClass))

typedef struct _BoxesSpiceDisplay BoxesSpiceDisplay;
typedef struct _BoxesSpiceDisplayClass BoxesSpiceDisplayClass;

#define BOXES_TYPE_VNC_DISPLAY (boxes_vnc_display_get_type ())
#define BOXES_VNC_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_VNC_DISPLAY, BoxesVncDisplay))
#define BOXES_VNC_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_VNC_DISPLAY, BoxesVncDisplayClass))
#define BOXES_IS_VNC_DISPLAY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_VNC_DISPLAY))
#define BOXES_IS_VNC_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_VNC_DISPLAY))
#define BOXES_VNC_DISPLAY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_VNC_DISPLAY, BoxesVncDisplayClass))

typedef struct _BoxesVncDisplay BoxesVncDisplay;
typedef struct _BoxesVncDisplayClass BoxesVncDisplayClass;
typedef struct _Block23Data Block23Data;
typedef struct _BoxesLibvirtMachineTakeScreenshotData BoxesLibvirtMachineTakeScreenshotData;
typedef struct _Block24Data Block24Data;
typedef struct _BoxesLibvirtMachineSaveRealData BoxesLibvirtMachineSaveRealData;
typedef struct _BoxesLibvirtMachineSuspendData BoxesLibvirtMachineSuspendData;

#define BOXES_TYPE_APP (boxes_app_get_type ())
#define BOXES_APP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_APP, BoxesApp))
#define BOXES_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_APP, BoxesAppClass))
#define BOXES_IS_APP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_APP))
#define BOXES_IS_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_APP))
#define BOXES_APP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_APP, BoxesAppClass))

typedef struct _BoxesApp BoxesApp;
typedef struct _BoxesAppClass BoxesAppClass;
typedef struct _BoxesAppPrivate BoxesAppPrivate;

#define BOXES_TYPE_CLUTTER_WIDGET (boxes_clutter_widget_get_type ())
#define BOXES_CLUTTER_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidget))
#define BOXES_CLUTTER_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidgetClass))
#define BOXES_IS_CLUTTER_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_CLUTTER_WIDGET))
#define BOXES_IS_CLUTTER_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_CLUTTER_WIDGET))
#define BOXES_CLUTTER_WIDGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidgetClass))

typedef struct _BoxesClutterWidget BoxesClutterWidget;
typedef struct _BoxesClutterWidgetClass BoxesClutterWidgetClass;

#define BOXES_TYPE_SEARCHBAR (boxes_searchbar_get_type ())
#define BOXES_SEARCHBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SEARCHBAR, BoxesSearchbar))
#define BOXES_SEARCHBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SEARCHBAR, BoxesSearchbarClass))
#define BOXES_IS_SEARCHBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SEARCHBAR))
#define BOXES_IS_SEARCHBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SEARCHBAR))
#define BOXES_SEARCHBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SEARCHBAR, BoxesSearchbarClass))

typedef struct _BoxesSearchbar BoxesSearchbar;
typedef struct _BoxesSearchbarClass BoxesSearchbarClass;

#define BOXES_TYPE_TOPBAR (boxes_topbar_get_type ())
#define BOXES_TOPBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_TOPBAR, BoxesTopbar))
#define BOXES_TOPBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_TOPBAR, BoxesTopbarClass))
#define BOXES_IS_TOPBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_TOPBAR))
#define BOXES_IS_TOPBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_TOPBAR))
#define BOXES_TOPBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_TOPBAR, BoxesTopbarClass))

typedef struct _BoxesTopbar BoxesTopbar;
typedef struct _BoxesTopbarClass BoxesTopbarClass;

#define BOXES_TYPE_NOTIFICATIONBAR (boxes_notificationbar_get_type ())
#define BOXES_NOTIFICATIONBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbar))
#define BOXES_NOTIFICATIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbarClass))
#define BOXES_IS_NOTIFICATIONBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_NOTIFICATIONBAR))
#define BOXES_IS_NOTIFICATIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_NOTIFICATIONBAR))
#define BOXES_NOTIFICATIONBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbarClass))

typedef struct _BoxesNotificationbar BoxesNotificationbar;
typedef struct _BoxesNotificationbarClass BoxesNotificationbarClass;

#define BOXES_TYPE_REVEALER (boxes_revealer_get_type ())
#define BOXES_REVEALER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_REVEALER, BoxesRevealer))
#define BOXES_REVEALER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_REVEALER, BoxesRevealerClass))
#define BOXES_IS_REVEALER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_REVEALER))
#define BOXES_IS_REVEALER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_REVEALER))
#define BOXES_REVEALER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_REVEALER, BoxesRevealerClass))

typedef struct _BoxesRevealer BoxesRevealer;
typedef struct _BoxesRevealerClass BoxesRevealerClass;

#define BOXES_TYPE_SIDEBAR (boxes_sidebar_get_type ())
#define BOXES_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SIDEBAR, BoxesSidebar))
#define BOXES_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SIDEBAR, BoxesSidebarClass))
#define BOXES_IS_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SIDEBAR))
#define BOXES_IS_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SIDEBAR))
#define BOXES_SIDEBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SIDEBAR, BoxesSidebarClass))

typedef struct _BoxesSidebar BoxesSidebar;
typedef struct _BoxesSidebarClass BoxesSidebarClass;

#define BOXES_TYPE_SELECTIONBAR (boxes_selectionbar_get_type ())
#define BOXES_SELECTIONBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbar))
#define BOXES_SELECTIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbarClass))
#define BOXES_IS_SELECTIONBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SELECTIONBAR))
#define BOXES_IS_SELECTIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SELECTIONBAR))
#define BOXES_SELECTIONBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbarClass))

typedef struct _BoxesSelectionbar BoxesSelectionbar;
typedef struct _BoxesSelectionbarClass BoxesSelectionbarClass;

#define BOXES_TYPE_WIZARD (boxes_wizard_get_type ())
#define BOXES_WIZARD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_WIZARD, BoxesWizard))
#define BOXES_WIZARD_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_WIZARD, BoxesWizardClass))
#define BOXES_IS_WIZARD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_WIZARD))
#define BOXES_IS_WIZARD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_WIZARD))
#define BOXES_WIZARD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_WIZARD, BoxesWizardClass))

typedef struct _BoxesWizard BoxesWizard;
typedef struct _BoxesWizardClass BoxesWizardClass;

#define BOXES_TYPE_PROPERTIES (boxes_properties_get_type ())
#define BOXES_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_PROPERTIES, BoxesProperties))
#define BOXES_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_PROPERTIES, BoxesPropertiesClass))
#define BOXES_IS_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_PROPERTIES))
#define BOXES_IS_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_PROPERTIES))
#define BOXES_PROPERTIES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_PROPERTIES, BoxesPropertiesClass))

typedef struct _BoxesProperties BoxesProperties;
typedef struct _BoxesPropertiesClass BoxesPropertiesClass;

#define BOXES_TYPE_DISPLAY_PAGE (boxes_display_page_get_type ())
#define BOXES_DISPLAY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPage))
#define BOXES_DISPLAY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPageClass))
#define BOXES_IS_DISPLAY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_DISPLAY_PAGE))
#define BOXES_IS_DISPLAY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_DISPLAY_PAGE))
#define BOXES_DISPLAY_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPageClass))

typedef struct _BoxesDisplayPage BoxesDisplayPage;
typedef struct _BoxesDisplayPageClass BoxesDisplayPageClass;

#define BOXES_TYPE_EMPTY_BOXES (boxes_empty_boxes_get_type ())
#define BOXES_EMPTY_BOXES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxes))
#define BOXES_EMPTY_BOXES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxesClass))
#define BOXES_IS_EMPTY_BOXES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_EMPTY_BOXES))
#define BOXES_IS_EMPTY_BOXES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_EMPTY_BOXES))
#define BOXES_EMPTY_BOXES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxesClass))

typedef struct _BoxesEmptyBoxes BoxesEmptyBoxes;
typedef struct _BoxesEmptyBoxesClass BoxesEmptyBoxesClass;

#define BOXES_TYPE_COLLECTION (boxes_collection_get_type ())
#define BOXES_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION, BoxesCollection))
#define BOXES_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION, BoxesCollectionClass))
#define BOXES_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION))
#define BOXES_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION))
#define BOXES_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION, BoxesCollectionClass))

typedef struct _BoxesCollection BoxesCollection;
typedef struct _BoxesCollectionClass BoxesCollectionClass;

#define BOXES_TYPE_COLLECTION_FILTER (boxes_collection_filter_get_type ())
#define BOXES_COLLECTION_FILTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilter))
#define BOXES_COLLECTION_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilterClass))
#define BOXES_IS_COLLECTION_FILTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_FILTER))
#define BOXES_IS_COLLECTION_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_FILTER))
#define BOXES_COLLECTION_FILTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilterClass))

typedef struct _BoxesCollectionFilter BoxesCollectionFilter;
typedef struct _BoxesCollectionFilterClass BoxesCollectionFilterClass;

#define BOXES_TYPE_COLLECTION_VIEW (boxes_collection_view_get_type ())
#define BOXES_COLLECTION_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionView))
#define BOXES_COLLECTION_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionViewClass))
#define BOXES_IS_COLLECTION_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_VIEW))
#define BOXES_IS_COLLECTION_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_VIEW))
#define BOXES_COLLECTION_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionViewClass))

typedef struct _BoxesCollectionView BoxesCollectionView;
typedef struct _BoxesCollectionViewClass BoxesCollectionViewClass;
typedef struct _BoxesLibvirtMachineStartData BoxesLibvirtMachineStartData;

typedef enum  {
	BOXES_UI_STATE_NONE,
	BOXES_UI_STATE_COLLECTION,
	BOXES_UI_STATE_CREDS,
	BOXES_UI_STATE_DISPLAY,
	BOXES_UI_STATE_SETTINGS,
	BOXES_UI_STATE_WIZARD,
	BOXES_UI_STATE_PROPERTIES
} BoxesUIState;

struct _BoxesUI {
	GObject parent_instance;
	BoxesUIPrivate * priv;
	BoxesUIState previous_ui_state;
};

struct _BoxesUIClass {
	GObjectClass parent_class;
	void (*ui_state_changed) (BoxesUI* self);
	ClutterActor* (*get_actor) (BoxesUI* self);
};

struct _BoxesCollectionItem {
	BoxesUI parent_instance;
	BoxesCollectionItemPrivate * priv;
};

struct _BoxesCollectionItemClass {
	BoxesUIClass parent_class;
};

typedef enum  {
	BOXES_PROPERTIES_PAGE_LOGIN,
	BOXES_PROPERTIES_PAGE_SYSTEM,
	BOXES_PROPERTIES_PAGE_DISPLAY,
	BOXES_PROPERTIES_PAGE_DEVICES,
	BOXES_PROPERTIES_PAGE_LAST
} BoxesPropertiesPage;

typedef enum  {
	PROPERTY_CREATION_FLAG_NONE = 0,
	PROPERTY_CREATION_FLAG_NO_USB = 1 << 0
} PropertyCreationFlag;

struct _BoxesIPropertiesProviderIface {
	GTypeInterface parent_iface;
	GList* (*get_properties) (BoxesIPropertiesProvider* self, BoxesPropertiesPage page, PropertyCreationFlag flags);
};

struct _BoxesMachine {
	BoxesCollectionItem parent_instance;
	BoxesMachinePrivate * priv;
	BoxesCollectionSource* source;
	BoxesBoxConfig* config;
	gboolean stay_on_display;
	gboolean suspend_at_exit;
	gboolean deleted;
};

struct _BoxesMachineClass {
	BoxesCollectionItemClass parent_class;
	void (*save_real) (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*save_real_finish) (BoxesMachine* self, GAsyncResult* _res_, GError** error);
	void (*take_screenshot) (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GdkPixbuf* (*take_screenshot_finish) (BoxesMachine* self, GAsyncResult* _res_, GError** error);
	GList* (*get_properties) (BoxesMachine* self, BoxesPropertiesPage page, PropertyCreationFlag flags);
	void (*connect_display) (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*connect_display_finish) (BoxesMachine* self, GAsyncResult* _res_, GError** error);
	void (*disconnect_display) (BoxesMachine* self);
	void (*delete) (BoxesMachine* self, gboolean by_user);
};

struct _BoxesLibvirtMachine {
	BoxesMachine parent_instance;
	BoxesLibvirtMachinePrivate * priv;
	gdouble* cpu_stats;
	gint cpu_stats_length1;
	gint _cpu_stats_size_;
	gdouble* io_stats;
	gint io_stats_length1;
	gint _io_stats_size_;
	gdouble* net_stats;
	gint net_stats_length1;
	gint _net_stats_size_;
};

struct _BoxesLibvirtMachineClass {
	BoxesMachineClass parent_class;
};

struct _BoxesLibvirtMachineMachineStat {
	gint64 timestamp;
	gdouble cpu_time;
	gdouble cpu_time_abs;
	gdouble cpu_guest_percent;
	gdouble memory_percent;
	GVirDomainDiskStats* disk;
	gdouble disk_read;
	gdouble disk_write;
	GVirDomainInterfaceStats* net;
	gdouble net_read;
	gdouble net_write;
};

struct _BoxesLibvirtMachinePrivate {
	GVirDomain* _domain;
	GVirConfigDomain* _domain_config;
	GVirConnection* _connection;
	GVirStorageVol* _storage_volume;
	BoxesVMCreator* _vm_creator;
	BoxesLibvirtMachineProperties* properties;
	guint shutdown_timeout;
	guint stats_update_timeout;
	GCancellable* stats_cancellable;
	BoxesLibvirtMachineMachineStat* stats;
	gint stats_length1;
	gint _stats_size_;
	gboolean force_stopped;
};

struct _BoxesLibvirtMachineConnectDisplayData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesLibvirtMachine* self;
	gboolean _tmp0_;
	gboolean _tmp1_;
	BoxesDisplay* _tmp2_;
	BoxesDisplay* _tmp3_;
	BoxesDisplay* _tmp4_;
	BoxesDisplay* _tmp5_;
	GError * _inner_error_;
};

typedef enum  {
	BOXES_MACHINE_MACHINE_STATE_UNKNOWN,
	BOXES_MACHINE_MACHINE_STATE_STOPPED,
	BOXES_MACHINE_MACHINE_STATE_FORCE_STOPPED,
	BOXES_MACHINE_MACHINE_STATE_RUNNING,
	BOXES_MACHINE_MACHINE_STATE_PAUSED,
	BOXES_MACHINE_MACHINE_STATE_SAVED,
	BOXES_MACHINE_MACHINE_STATE_SLEEPING
} BoxesMachineMachineState;

struct _Block18Data {
	int _ref_count_;
	BoxesLibvirtMachine * self;
	BoxesLibvirtMachineMachineStat* stat;
	gpointer _async_data_;
};

struct _Block19Data {
	int _ref_count_;
	Block18Data * _data18_;
	GVirDomainDisk* disk;
};

struct _BoxesLibvirtMachineUpdateIoStatData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesLibvirtMachine* self;
	GVirDomainInfo* info;
	BoxesLibvirtMachineMachineStat* stat;
	Block18Data* _data18_;
	BoxesLibvirtMachineMachineStat* _tmp0_;
	GVirDomainInfo* _tmp1_;
	GVirDomainState _tmp2_;
	Block19Data* _data19_;
	GVirDomainDisk* _tmp3_;
	GVirDomainDisk* _tmp4_;
	BoxesLibvirtMachineMachineStat* _tmp5_;
	gint _tmp5__length1;
	BoxesLibvirtMachineMachineStat _tmp6_;
	BoxesLibvirtMachineMachineStat _tmp7_;
	BoxesLibvirtMachineMachineStat prev;
	BoxesLibvirtMachineMachineStat _tmp8_;
	GVirDomainDiskStats* _tmp9_;
	BoxesLibvirtMachineMachineStat* _tmp10_;
	BoxesLibvirtMachineMachineStat* _tmp11_;
	GVirDomainDiskStats* _tmp12_;
	gint64 _tmp13_;
	BoxesLibvirtMachineMachineStat _tmp14_;
	GVirDomainDiskStats* _tmp15_;
	gint64 _tmp16_;
	BoxesLibvirtMachineMachineStat* _tmp17_;
	BoxesLibvirtMachineMachineStat* _tmp18_;
	GVirDomainDiskStats* _tmp19_;
	gint64 _tmp20_;
	BoxesLibvirtMachineMachineStat _tmp21_;
	GVirDomainDiskStats* _tmp22_;
	gint64 _tmp23_;
	GError* err;
	const gchar* _tmp24_;
	const gchar* _tmp25_;
	GError* _tmp26_;
	const gchar* _tmp27_;
	GError * _inner_error_;
};

struct _Block20Data {
	int _ref_count_;
	BoxesLibvirtMachine * self;
	BoxesLibvirtMachineMachineStat* stat;
	gpointer _async_data_;
};

struct _Block21Data {
	int _ref_count_;
	Block20Data * _data20_;
	GVirDomainInterface* net;
};

struct _BoxesLibvirtMachineUpdateNetStatData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesLibvirtMachine* self;
	GVirDomainInfo* info;
	BoxesLibvirtMachineMachineStat* stat;
	Block20Data* _data20_;
	BoxesLibvirtMachineMachineStat* _tmp0_;
	GVirDomainInfo* _tmp1_;
	GVirDomainState _tmp2_;
	Block21Data* _data21_;
	GVirDomainInterface* _tmp3_;
	GVirDomainInterface* _tmp4_;
	BoxesLibvirtMachineMachineStat* _tmp5_;
	gint _tmp5__length1;
	BoxesLibvirtMachineMachineStat _tmp6_;
	BoxesLibvirtMachineMachineStat _tmp7_;
	BoxesLibvirtMachineMachineStat prev;
	BoxesLibvirtMachineMachineStat _tmp8_;
	GVirDomainInterfaceStats* _tmp9_;
	BoxesLibvirtMachineMachineStat* _tmp10_;
	BoxesLibvirtMachineMachineStat* _tmp11_;
	GVirDomainInterfaceStats* _tmp12_;
	gint64 _tmp13_;
	BoxesLibvirtMachineMachineStat _tmp14_;
	GVirDomainInterfaceStats* _tmp15_;
	gint64 _tmp16_;
	BoxesLibvirtMachineMachineStat* _tmp17_;
	BoxesLibvirtMachineMachineStat* _tmp18_;
	GVirDomainInterfaceStats* _tmp19_;
	gint64 _tmp20_;
	BoxesLibvirtMachineMachineStat _tmp21_;
	GVirDomainInterfaceStats* _tmp22_;
	gint64 _tmp23_;
	GError* err;
	const gchar* _tmp24_;
	const gchar* _tmp25_;
	GError* _tmp26_;
	const gchar* _tmp27_;
	GError * _inner_error_;
};

struct _BoxesLibvirtMachineUpdateStatsData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesLibvirtMachine* self;
	gint64 _tmp0_;
	gint64 now;
	gint64 _tmp1_;
	BoxesLibvirtMachineMachineStat _tmp2_;
	BoxesLibvirtMachineMachineStat stat;
	GVirDomain* _tmp3_;
	GCancellable* _tmp4_;
	GVirDomainInfo* _tmp5_;
	GVirDomainInfo* info;
	GVirDomainInfo* _tmp6_;
	GVirDomainInfo* _tmp7_;
	GVirDomainInfo* _tmp8_;
	GVirDomainInfo* _tmp9_;
	BoxesLibvirtMachineMachineStat* _tmp10_;
	gint _tmp10__length1;
	BoxesLibvirtMachineMachineStat* _tmp11_;
	gint _tmp11__length1;
	BoxesLibvirtMachineMachineStat* _tmp12_;
	gint _tmp12__length1;
	BoxesLibvirtMachineMachineStat _tmp13_;
	BoxesLibvirtMachineMachineStat _tmp14_;
	GError* err;
	GError* _vala1_err;
	GError* _tmp15_;
	const gchar* _tmp16_;
	gdouble* _tmp17_;
	gdouble* _tmp18_;
	gdouble* _tmp19_;
	BoxesLibvirtMachineMachineStat* _tmp20_;
	gint _tmp20__length1;
	BoxesLibvirtMachineMachineStat* stat_collection;
	gint stat_collection_length1;
	gint _stat_collection_size_;
	gint stat_it;
	BoxesLibvirtMachineMachineStat _tmp21_;
	BoxesLibvirtMachineMachineStat _vala1_stat;
	gdouble* _tmp22_;
	gint _tmp22__length1;
	BoxesLibvirtMachineMachineStat _tmp23_;
	gdouble _tmp24_;
	gdouble* _tmp25_;
	gint _tmp25__length1;
	BoxesLibvirtMachineMachineStat _tmp26_;
	gdouble _tmp27_;
	BoxesLibvirtMachineMachineStat _tmp28_;
	gdouble _tmp29_;
	gdouble* _tmp30_;
	gint _tmp30__length1;
	BoxesLibvirtMachineMachineStat _tmp31_;
	gdouble _tmp32_;
	BoxesLibvirtMachineMachineStat _tmp33_;
	gdouble _tmp34_;
	GError * _inner_error_;
};

struct _Block22Data {
	int _ref_count_;
	BoxesLibvirtMachine * self;
	gboolean stats_updating;
};

struct _Block23Data {
	int _ref_count_;
	BoxesLibvirtMachine * self;
	GVirStream* stream;
	gpointer _async_data_;
};

struct _BoxesLibvirtMachineTakeScreenshotData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesLibvirtMachine* self;
	GdkPixbuf* result;
	Block23Data* _data23_;
	GVirDomainState state;
	GVirDomain* _tmp0_;
	GVirDomainInfo* _tmp1_;
	GVirDomainInfo* _tmp2_;
	GVirDomainInfo* _tmp3_;
	GVirDomainState _tmp4_;
	GError* _error_;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	gboolean _tmp7_;
	gboolean _tmp8_;
	GVirDomainState _tmp9_;
	GVirDomainState _tmp10_;
	gboolean _tmp11_;
	GVirDomainState _tmp12_;
	gboolean _tmp13_;
	GVirConnection* _tmp14_;
	GVirStream* _tmp15_;
	GdkPixbufLoader* _tmp16_;
	GdkPixbufLoader* loader;
	GVirStream* _tmp17_;
	GInputStream* _tmp18_;
	GInputStream* _tmp19_;
	GInputStream* input_stream;
	guint8* _tmp20_;
	guint8* buffer;
	gint buffer_length1;
	gint _buffer_size_;
	gssize length;
	gboolean _tmp21_;
	gboolean _tmp22_;
	gssize _tmp23_;
	GInputStream* _tmp24_;
	guint8* _tmp25_;
	gint _tmp25__length1;
	gssize _tmp26_;
	gssize _tmp27_;
	GdkPixbufLoader* _tmp28_;
	guint8* _tmp29_;
	gint _tmp29__length1;
	gssize _tmp30_;
	GdkPixbufLoader* _tmp31_;
	GdkPixbufLoader* _tmp32_;
	GdkPixbuf* _tmp33_;
	GdkPixbuf* _tmp34_;
	GError * _inner_error_;
};

struct _Block24Data {
	int _ref_count_;
	BoxesLibvirtMachine * self;
	GVirDomain* domain;
};

struct _BoxesLibvirtMachineSaveRealData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesLibvirtMachine* self;
	GVirDomain* _tmp0_;
	GError * _inner_error_;
};

struct _BoxesLibvirtMachineSuspendData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesLibvirtMachine* self;
	gboolean _tmp0_;
	gboolean _tmp1_;
	GVirDomain* _tmp2_;
	GError * _inner_error_;
};

struct _BoxesApp {
	BoxesUI parent_instance;
	BoxesAppPrivate * priv;
	GtkApplicationWindow* window;
	GtkNotebook* notebook;
	BoxesClutterWidget* embed;
	ClutterStage* stage;
	ClutterBinLayout* stage_bin;
	ClutterActor* overlay_bin_actor;
	ClutterBinLayout* overlay_bin;
	BoxesCollectionItem* current_item;
	BoxesSearchbar* searchbar;
	BoxesTopbar* topbar;
	BoxesNotificationbar* notificationbar;
	BoxesRevealer* sidebar_revealer;
	BoxesRevealer* searchbar_revealer;
	BoxesSidebar* sidebar;
	BoxesSelectionbar* selectionbar;
	guint duration;
	GSettings* settings;
	BoxesWizard* wizard;
	BoxesProperties* properties;
	BoxesDisplayPage* display_page;
	BoxesEmptyBoxes* empty_boxes;
	BoxesCollection* collection;
	BoxesCollectionFilter* filter;
	BoxesCollectionView* view;
};

struct _BoxesAppClass {
	BoxesUIClass parent_class;
};

struct _BoxesLibvirtMachineStartData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesLibvirtMachine* self;
	BoxesMachineMachineState _tmp0_;
	BoxesMachineMachineState _tmp1_;
	BoxesMachineMachineState _tmp2_;
	BoxesMachineMachineState _tmp3_;
	GVirDomain* _tmp4_;
	BoxesMachineMachineState _tmp5_;
	BoxesMachineMachineState _tmp6_;
	GVirDomain* _tmp7_;
	GVirDomain* _tmp8_;
	gboolean _tmp9_;
	const gchar* _tmp10_;
	const gchar* _tmp11_;
	const gchar* _tmp12_;
	gchar* _tmp13_;
	gchar* _tmp14_;
	const gchar* _tmp15_;
	const gchar* _tmp16_;
	const gchar* _tmp17_;
	gchar* _tmp18_;
	gchar* _tmp19_;
	GVirDomain* _tmp20_;
	GError * _inner_error_;
};


static gpointer boxes_libvirt_machine_parent_class = NULL;
extern BoxesApp* boxes_app_app;

GType boxes_ui_get_type (void) G_GNUC_CONST;
GType boxes_ui_state_get_type (void) G_GNUC_CONST;
GType boxes_collection_item_get_type (void) G_GNUC_CONST;
GType boxes_properties_page_get_type (void) G_GNUC_CONST;
GType property_creation_flag_get_type (void) G_GNUC_CONST;
GType boxes_property_get_type (void) G_GNUC_CONST;
GType boxes_iproperties_provider_get_type (void) G_GNUC_CONST;
GType boxes_machine_get_type (void) G_GNUC_CONST;
GType boxes_libvirt_machine_get_type (void) G_GNUC_CONST;
gpointer boxes_vm_creator_ref (gpointer instance);
void boxes_vm_creator_unref (gpointer instance);
GParamSpec* boxes_param_spec_vm_creator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void boxes_value_set_vm_creator (GValue* value, gpointer v_object);
void boxes_value_take_vm_creator (GValue* value, gpointer v_object);
gpointer boxes_value_get_vm_creator (const GValue* value);
GType boxes_vm_creator_get_type (void) G_GNUC_CONST;
GType boxes_libvirt_machine_properties_get_type (void) G_GNUC_CONST;
static GType boxes_libvirt_machine_machine_stat_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static BoxesLibvirtMachineMachineStat* boxes_libvirt_machine_machine_stat_dup (const BoxesLibvirtMachineMachineStat* self);
static void boxes_libvirt_machine_machine_stat_free (BoxesLibvirtMachineMachineStat* self);
static void boxes_libvirt_machine_machine_stat_copy (const BoxesLibvirtMachineMachineStat* self, BoxesLibvirtMachineMachineStat* dest);
static void boxes_libvirt_machine_machine_stat_destroy (BoxesLibvirtMachineMachineStat* self);
#define BOXES_LIBVIRT_MACHINE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachinePrivate))
enum  {
	BOXES_LIBVIRT_MACHINE_DUMMY_PROPERTY,
	BOXES_LIBVIRT_MACHINE_DOMAIN,
	BOXES_LIBVIRT_MACHINE_DOMAIN_CONFIG,
	BOXES_LIBVIRT_MACHINE_CONNECTION,
	BOXES_LIBVIRT_MACHINE_STORAGE_VOLUME,
	BOXES_LIBVIRT_MACHINE_VM_CREATOR,
	BOXES_LIBVIRT_MACHINE_SAVE_ON_QUIT
};
static void _vala_BoxesLibvirtMachineMachineStat_array_free (BoxesLibvirtMachineMachineStat* array, gint array_length);
#define BOXES_LIBVIRT_MACHINE_STATS_SIZE 20
static void boxes_libvirt_machine_real_disconnect_display (BoxesMachine* base);
void boxes_machine_disconnect_display (BoxesMachine* self);
static void boxes_libvirt_machine_real_connect_display_data_free (gpointer _data);
static void boxes_libvirt_machine_real_connect_display (BoxesMachine* base, GAsyncReadyCallback _callback_, gpointer _user_data_);
static gboolean boxes_libvirt_machine_real_connect_display_co (BoxesLibvirtMachineConnectDisplayData* _data_);
void boxes_libvirt_machine_start (BoxesLibvirtMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_libvirt_machine_start_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_, GError** error);
static void boxes_libvirt_machine_connect_display_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
gboolean boxes_libvirt_machine_update_display (BoxesLibvirtMachine* self, GError** error);
GType boxes_display_get_type (void) G_GNUC_CONST;
BoxesDisplay* boxes_machine_get_display (BoxesMachine* self);
void boxes_display_connect_it (BoxesDisplay* self, GError** error);
void boxes_machine_show_display (BoxesMachine* self);
void boxes_display_set_enable_audio (BoxesDisplay* self, gboolean enable);
void boxes_libvirt_machine_update_domain_config (BoxesLibvirtMachine* self);
GVirDomain* boxes_libvirt_machine_get_domain (BoxesLibvirtMachine* self);
void boxes_libvirt_machine_set_domain_config (BoxesLibvirtMachine* self, GVirConfigDomain* value);
GVirStorageVol* boxes_get_storage_volume (GVirConnection* connection, GVirDomain* domain);
GVirConnection* boxes_libvirt_machine_get_connection (BoxesLibvirtMachine* self);
void boxes_libvirt_machine_set_storage_volume (BoxesLibvirtMachine* self, GVirStorageVol* value);
static void boxes_libvirt_machine_reconnect_display (BoxesLibvirtMachine* self);
void boxes_machine_connect_display (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_machine_connect_display_finish (BoxesMachine* self, GAsyncResult* _res_, GError** error);
BoxesLibvirtMachine* boxes_libvirt_machine_new (BoxesCollectionSource* source, GVirConnection* connection, GVirDomain* domain, GError** error);
BoxesLibvirtMachine* boxes_libvirt_machine_construct (GType object_type, BoxesCollectionSource* source, GVirConnection* connection, GVirDomain* domain, GError** error);
BoxesMachine* boxes_machine_construct (GType object_type, BoxesCollectionSource* source, const gchar* name);
void boxes_machine_create_display_config (BoxesMachine* self, const gchar* uuid);
void boxes_libvirt_machine_set_connection (BoxesLibvirtMachine* self, GVirConnection* value);
void boxes_libvirt_machine_set_domain (BoxesLibvirtMachine* self, GVirDomain* value);
BoxesLibvirtMachineProperties* boxes_libvirt_machine_properties_new (BoxesLibvirtMachine* machine);
BoxesLibvirtMachineProperties* boxes_libvirt_machine_properties_construct (GType object_type, BoxesLibvirtMachine* machine);
static void _vala_GVirDomainInfo_free (GVirDomainInfo* self);
GType boxes_machine_machine_state_get_type (void) G_GNUC_CONST;
void boxes_machine_set_state (BoxesMachine* self, BoxesMachineMachineState value);
static void boxes_libvirt_machine_set_stats_enable (BoxesLibvirtMachine* self, gboolean enable);
static void __lambda119_ (BoxesLibvirtMachine* self);
static void ___lambda119__gvir_domain_started (GVirDomain* _sender, gpointer self);
static void __lambda120_ (BoxesLibvirtMachine* self);
static void ___lambda120__gvir_domain_suspended (GVirDomain* _sender, gpointer self);
static void __lambda121_ (BoxesLibvirtMachine* self);
static void ___lambda121__gvir_domain_resumed (GVirDomain* _sender, gpointer self);
static void __lambda122_ (BoxesLibvirtMachine* self);
static void ___lambda122__gvir_domain_stopped (GVirDomain* _sender, gpointer self);
static void __lambda123_ (BoxesLibvirtMachine* self);
static void ___lambda123__gvir_domain_pmsuspended (GVirDomain* _sender, gpointer self);
static void __lambda124_ (BoxesLibvirtMachine* self);
BoxesMachineMachineState boxes_machine_get_state (BoxesMachine* self);
static void ___lambda124__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static void _boxes_libvirt_machine_update_domain_config_gvir_domain_updated (GVirDomain* _sender, gpointer self);
void boxes_machine_load_screenshot (BoxesMachine* self);
void boxes_machine_set_screenshot_enable (BoxesMachine* self, gboolean enable);
static void boxes_libvirt_machine_update_cpu_stat (BoxesLibvirtMachine* self, GVirDomainInfo* info, BoxesLibvirtMachineMachineStat* stat);
static void boxes_libvirt_machine_update_mem_stat (BoxesLibvirtMachine* self, GVirDomainInfo* info, BoxesLibvirtMachineMachineStat* stat);
static void boxes_libvirt_machine_update_io_stat_data_free (gpointer _data);
static GVirDomainInfo* _vala_GVirDomainInfo_copy (GVirDomainInfo* self);
static void boxes_libvirt_machine_update_io_stat (BoxesLibvirtMachine* self, GVirDomainInfo* info, BoxesLibvirtMachineMachineStat* stat, GAsyncReadyCallback _callback_, gpointer _user_data_);
static void boxes_libvirt_machine_update_io_stat_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_);
static gboolean boxes_libvirt_machine_update_io_stat_co (BoxesLibvirtMachineUpdateIoStatData* _data_);
static Block18Data* block18_data_ref (Block18Data* _data18_);
static void block18_data_unref (void * _userdata_);
static Block19Data* block19_data_ref (Block19Data* _data19_);
static void block19_data_unref (void * _userdata_);
GVirDomainDisk* boxes_libvirt_machine_get_domain_disk (BoxesLibvirtMachine* self, GError** error);
static void ___lambda116_ (Block19Data* _data19_, GError** error);
static void _vala_GVirDomainDiskStats_free (GVirDomainDiskStats* self);
static void ____lambda116__boxes_run_in_thread_func (gpointer self, GError** error);
static void boxes_libvirt_machine_update_io_stat_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
const gchar* boxes_collection_item_get_name (BoxesCollectionItem* self);
static void boxes_libvirt_machine_update_net_stat_data_free (gpointer _data);
static void boxes_libvirt_machine_update_net_stat (BoxesLibvirtMachine* self, GVirDomainInfo* info, BoxesLibvirtMachineMachineStat* stat, GAsyncReadyCallback _callback_, gpointer _user_data_);
static void boxes_libvirt_machine_update_net_stat_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_);
static gboolean boxes_libvirt_machine_update_net_stat_co (BoxesLibvirtMachineUpdateNetStatData* _data_);
static Block20Data* block20_data_ref (Block20Data* _data20_);
static void block20_data_unref (void * _userdata_);
static Block21Data* block21_data_ref (Block21Data* _data21_);
static void block21_data_unref (void * _userdata_);
static GVirDomainInterface* boxes_libvirt_machine_get_domain_network_interface (BoxesLibvirtMachine* self, GError** error);
static void ___lambda117_ (Block21Data* _data21_, GError** error);
static void _vala_GVirDomainInterfaceStats_free (GVirDomainInterfaceStats* self);
static void ____lambda117__boxes_run_in_thread_func (gpointer self, GError** error);
static void boxes_libvirt_machine_update_net_stat_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static void boxes_libvirt_machine_update_stats_data_free (gpointer _data);
static void boxes_libvirt_machine_update_stats (BoxesLibvirtMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
static void boxes_libvirt_machine_update_stats_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_);
static gboolean boxes_libvirt_machine_update_stats_co (BoxesLibvirtMachineUpdateStatsData* _data_);
static void boxes_libvirt_machine_update_stats_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static BoxesLibvirtMachineMachineStat* _vala_array_dup1 (BoxesLibvirtMachineMachineStat* self, int length);
static void _vala_array_add2 (BoxesLibvirtMachineMachineStat** array, int* length, int* size, const BoxesLibvirtMachineMachineStat* value);
static void _vala_array_add3 (gdouble** array, int* length, int* size, gdouble value);
static void _vala_array_add4 (gdouble** array, int* length, int* size, gdouble value);
static void _vala_array_add5 (gdouble** array, int* length, int* size, gdouble value);
static Block22Data* block22_data_ref (Block22Data* _data22_);
static void block22_data_unref (void * _userdata_);
static gboolean ___lambda115_ (Block22Data* _data22_);
static void __lambda118_ (Block22Data* _data22_);
static void ___lambda118__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self);
static gboolean ____lambda115__gsource_func (gpointer self);
static GList* boxes_libvirt_machine_real_get_properties (BoxesMachine* base, BoxesPropertiesPage page, PropertyCreationFlag flags);
GList* boxes_iproperties_provider_get_properties (BoxesIPropertiesProvider* self, BoxesPropertiesPage page, PropertyCreationFlag flags);
GList* boxes_display_get_properties (BoxesDisplay* self, BoxesPropertiesPage page, PropertyCreationFlag flags);
static void _g_object_unref0_ (gpointer var);
static void _g_list_free__g_object_unref0_ (GList* self);
static BoxesDisplay* boxes_libvirt_machine_create_display (BoxesLibvirtMachine* self, GError** error);
void boxes_machine_set_display (BoxesMachine* self, BoxesDisplay* value);
GVirConfigDomain* boxes_libvirt_machine_get_domain_config (BoxesLibvirtMachine* self);
gchar* boxes_extract_xpath (const gchar* xmldoc, const gchar* xpath, gboolean required, GError** error);
BoxesSpiceDisplay* boxes_spice_display_new (BoxesBoxConfig* config, const gchar* host, gint port, gint tls_port);
BoxesSpiceDisplay* boxes_spice_display_construct (GType object_type, BoxesBoxConfig* config, const gchar* host, gint port, gint tls_port);
GType boxes_spice_display_get_type (void) G_GNUC_CONST;
BoxesVncDisplay* boxes_vnc_display_new (BoxesBoxConfig* config, const gchar* host, gint port);
BoxesVncDisplay* boxes_vnc_display_construct (GType object_type, BoxesBoxConfig* config, const gchar* host, gint port);
GType boxes_vnc_display_get_type (void) G_GNUC_CONST;
static void boxes_libvirt_machine_real_take_screenshot_data_free (gpointer _data);
static void boxes_libvirt_machine_real_take_screenshot (BoxesMachine* base, GAsyncReadyCallback _callback_, gpointer _user_data_);
static gboolean boxes_libvirt_machine_real_take_screenshot_co (BoxesLibvirtMachineTakeScreenshotData* _data_);
static Block23Data* block23_data_ref (Block23Data* _data23_);
static void block23_data_unref (void * _userdata_);
static void boxes_libvirt_machine_take_screenshot_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static void __lambda125_ (Block23Data* _data23_, GError** error);
static void ___lambda125__boxes_run_in_thread_func (gpointer self, GError** error);
static void boxes_libvirt_machine_real_delete (BoxesMachine* base, gboolean by_user);
void boxes_machine_delete (BoxesMachine* self, gboolean by_user);
static Block24Data* block24_data_ref (Block24Data* _data24_);
static void block24_data_unref (void * _userdata_);
static void ___lambda126_ (Block24Data* _data24_, GError** error);
GVirStorageVol* boxes_libvirt_machine_get_storage_volume (BoxesLibvirtMachine* self);
static void ____lambda126__boxes_run_in_thread_func (gpointer self, GError** error);
static void boxes_libvirt_machine_real_save_real_data_free (gpointer _data);
static void boxes_libvirt_machine_real_save_real (BoxesMachine* base, GAsyncReadyCallback _callback_, gpointer _user_data_);
static gboolean boxes_libvirt_machine_real_save_real_co (BoxesLibvirtMachineSaveRealData* _data_);
static void boxes_libvirt_machine_save_real_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static void boxes_libvirt_machine_suspend_data_free (gpointer _data);
void boxes_libvirt_machine_suspend (BoxesLibvirtMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_libvirt_machine_suspend_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_, GError** error);
static gboolean boxes_libvirt_machine_suspend_co (BoxesLibvirtMachineSuspendData* _data_);
gboolean boxes_libvirt_machine_get_save_on_quit (BoxesLibvirtMachine* self);
void boxes_machine_save (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_machine_save_finish (BoxesMachine* self, GAsyncResult* _res_, GError** error);
static void boxes_libvirt_machine_suspend_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
void boxes_libvirt_machine_force_shutdown (BoxesLibvirtMachine* self, gboolean confirm);
GType boxes_app_get_type (void) G_GNUC_CONST;
GType boxes_clutter_widget_get_type (void) G_GNUC_CONST;
GType boxes_searchbar_get_type (void) G_GNUC_CONST;
GType boxes_topbar_get_type (void) G_GNUC_CONST;
GType boxes_notificationbar_get_type (void) G_GNUC_CONST;
GType boxes_revealer_get_type (void) G_GNUC_CONST;
GType boxes_sidebar_get_type (void) G_GNUC_CONST;
GType boxes_selectionbar_get_type (void) G_GNUC_CONST;
GType boxes_wizard_get_type (void) G_GNUC_CONST;
GType boxes_properties_get_type (void) G_GNUC_CONST;
GType boxes_display_page_get_type (void) G_GNUC_CONST;
GType boxes_empty_boxes_get_type (void) G_GNUC_CONST;
GType boxes_collection_get_type (void) G_GNUC_CONST;
GType boxes_collection_filter_get_type (void) G_GNUC_CONST;
GType boxes_collection_view_get_type (void) G_GNUC_CONST;
void boxes_libvirt_machine_try_shutdown (BoxesLibvirtMachine* self);
static void boxes_libvirt_machine_start_data_free (gpointer _data);
static gboolean boxes_libvirt_machine_start_co (BoxesLibvirtMachineStartData* _data_);
static void boxes_libvirt_machine_start_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
void boxes_machine_set_status (BoxesMachine* self, const gchar* value);
BoxesVMCreator* boxes_libvirt_machine_get_vm_creator (BoxesLibvirtMachine* self);
void boxes_libvirt_machine_set_vm_creator (BoxesLibvirtMachine* self, BoxesVMCreator* value);
void boxes_libvirt_machine_set_save_on_quit (BoxesLibvirtMachine* self, gboolean value);
static GObject * boxes_libvirt_machine_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
void boxes_machine_set_can_save (BoxesMachine* self, gboolean value);
static GVirDomainDiskStats* _vala_GVirDomainDiskStats_copy (GVirDomainDiskStats* self);
static GVirDomainInterfaceStats* _vala_GVirDomainInterfaceStats_copy (GVirDomainInterfaceStats* self);
static void boxes_libvirt_machine_finalize (GObject* obj);
static void _vala_boxes_libvirt_machine_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_boxes_libvirt_machine_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


static void _vala_BoxesLibvirtMachineMachineStat_array_free (BoxesLibvirtMachineMachineStat* array, gint array_length) {
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (array != NULL) {
#line 1014 "libvirt-machine.c"
		int i;
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		for (i = 0; i < array_length; i = i + 1) {
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			boxes_libvirt_machine_machine_stat_destroy (&array[i]);
#line 1020 "libvirt-machine.c"
		}
	}
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_free (array);
#line 1025 "libvirt-machine.c"
}


static void boxes_libvirt_machine_real_disconnect_display (BoxesMachine* base) {
	BoxesLibvirtMachine * self;
#line 19 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = (BoxesLibvirtMachine*) base;
#line 20 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	((BoxesMachine*) self)->stay_on_display = FALSE;
#line 22 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (boxes_libvirt_machine_parent_class)->disconnect_display (G_TYPE_CHECK_INSTANCE_CAST (self, BOXES_TYPE_MACHINE, BoxesMachine));
#line 1037 "libvirt-machine.c"
}


static void boxes_libvirt_machine_real_connect_display_data_free (gpointer _data) {
	BoxesLibvirtMachineConnectDisplayData* _data_;
	_data_ = _data;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_slice_free (BoxesLibvirtMachineConnectDisplayData, _data_);
#line 1048 "libvirt-machine.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return self ? g_object_ref (self) : NULL;
#line 1055 "libvirt-machine.c"
}


static void boxes_libvirt_machine_real_connect_display (BoxesMachine* base, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesLibvirtMachine * self;
	BoxesLibvirtMachineConnectDisplayData* _data_;
	BoxesLibvirtMachine* _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = (BoxesLibvirtMachine*) base;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_slice_new0 (BoxesLibvirtMachineConnectDisplayData);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_libvirt_machine_real_connect_display);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_libvirt_machine_real_connect_display_data_free);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self = _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_real_connect_display_co (_data_);
#line 1077 "libvirt-machine.c"
}


static void boxes_libvirt_machine_real_connect_display_finish (BoxesMachine* base, GAsyncResult* _res_, GError** error) {
	BoxesLibvirtMachineConnectDisplayData* _data_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (_res_), error)) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 1087 "libvirt-machine.c"
	}
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 1091 "libvirt-machine.c"
}


static void boxes_libvirt_machine_connect_display_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesLibvirtMachineConnectDisplayData* _data_;
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = _user_data_;
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_source_object_ = source_object;
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_res_ = _res_;
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_real_connect_display_co (_data_);
#line 1105 "libvirt-machine.c"
}


static gboolean boxes_libvirt_machine_real_connect_display_co (BoxesLibvirtMachineConnectDisplayData* _data_) {
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (_data_->_state_) {
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 0:
#line 1114 "libvirt-machine.c"
		goto _state_0;
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 1:
#line 1118 "libvirt-machine.c"
		goto _state_1;
		default:
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_assert_not_reached ();
#line 1123 "libvirt-machine.c"
	}
	_state_0:
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_state_ = 1;
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_start (_data_->self, boxes_libvirt_machine_connect_display_ready, _data_);
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 1132 "libvirt-machine.c"
	_state_1:
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_start_finish (_data_->self, _data_->_res_, &_data_->_inner_error_);
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_inner_error_ != NULL) {
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_error_free (_data_->_inner_error_);
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 1146 "libvirt-machine.c"
		} else {
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 1150 "libvirt-machine.c"
		}
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 1156 "libvirt-machine.c"
	}
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp0_ = FALSE;
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp0_ = boxes_libvirt_machine_update_display (_data_->self, &_data_->_inner_error_);
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp1_ = _data_->_tmp0_;
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_inner_error_ != NULL) {
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_error_free (_data_->_inner_error_);
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 1174 "libvirt-machine.c"
		} else {
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 1178 "libvirt-machine.c"
		}
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 1184 "libvirt-machine.c"
	}
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_tmp1_) {
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp2_ = boxes_machine_get_display ((BoxesMachine*) _data_->self);
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp3_ = _data_->_tmp2_;
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_display_connect_it (_data_->_tmp3_, &_data_->_inner_error_);
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_error_free (_data_->_inner_error_);
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_state_ == 0) {
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 1204 "libvirt-machine.c"
			} else {
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete (_data_->_async_result);
#line 1208 "libvirt-machine.c"
			}
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_object_unref (_data_->_async_result);
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 1214 "libvirt-machine.c"
		}
	} else {
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_machine_show_display ((BoxesMachine*) _data_->self);
#line 32 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp4_ = boxes_machine_get_display ((BoxesMachine*) _data_->self);
#line 32 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp5_ = _data_->_tmp4_;
#line 32 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_display_set_enable_audio (_data_->_tmp5_, TRUE);
#line 1225 "libvirt-machine.c"
	}
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_state_ == 0) {
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 1231 "libvirt-machine.c"
	} else {
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 1235 "libvirt-machine.c"
	}
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_unref (_data_->_async_result);
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 1241 "libvirt-machine.c"
}


void boxes_libvirt_machine_update_domain_config (BoxesLibvirtMachine* self) {
	GError * _inner_error_ = NULL;
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 1249 "libvirt-machine.c"
	{
		GVirDomain* _tmp0_;
		GVirConfigDomain* _tmp1_ = NULL;
		GVirConfigDomain* _tmp2_;
		GVirConfigDomain* _tmp3_;
		GVirConnection* _tmp4_;
		GVirDomain* _tmp5_;
		GVirStorageVol* _tmp6_ = NULL;
		GVirStorageVol* _tmp7_;
#line 68 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp0_ = self->priv->_domain;
#line 68 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp1_ = gvir_domain_get_config (_tmp0_, (guint) GVIR_DOMAIN_XML_NONE, &_inner_error_);
#line 68 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp2_ = _tmp1_;
#line 68 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_ != NULL) {
#line 1267 "libvirt-machine.c"
			goto __catch12_g_error;
		}
#line 68 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp3_ = _tmp2_;
#line 68 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_domain_config (self, _tmp3_);
#line 68 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (_tmp3_);
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp4_ = self->priv->_connection;
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp5_ = self->priv->_domain;
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp6_ = boxes_get_storage_volume (_tmp4_, _tmp5_);
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp7_ = _tmp6_;
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_storage_volume (self, _tmp7_);
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (_tmp7_);
#line 1288 "libvirt-machine.c"
	}
	goto __finally12;
	__catch12_g_error:
	{
		GError* _error_ = NULL;
		GVirDomain* _tmp8_;
		const gchar* _tmp9_ = NULL;
		GError* _tmp10_;
		const gchar* _tmp11_;
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_error_ = _inner_error_;
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_inner_error_ = NULL;
#line 71 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp8_ = self->priv->_domain;
#line 71 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp9_ = gvir_domain_get_name (_tmp8_);
#line 71 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp10_ = _error_;
#line 71 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp11_ = _tmp10_->message;
#line 71 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_critical ("libvirt-machine.vala:71: Failed to fetch configuration for domain '%s'" \
": %s", _tmp9_, _tmp11_);
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (_error_);
#line 1314 "libvirt-machine.c"
	}
	__finally12:
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_clear_error (&_inner_error_);
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 1325 "libvirt-machine.c"
	}
}


static void boxes_libvirt_machine_reconnect_display (BoxesLibvirtMachine* self) {
	BoxesDisplay* _tmp0_;
	BoxesDisplay* _tmp1_;
#line 75 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = boxes_machine_get_display ((BoxesMachine*) self);
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _tmp0_;
#line 77 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp1_ == NULL) {
#line 78 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 1343 "libvirt-machine.c"
	}
#line 80 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_machine_disconnect_display ((BoxesMachine*) self);
#line 81 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_machine_connect_display ((BoxesMachine*) self, NULL, NULL);
#line 1349 "libvirt-machine.c"
}


static void _vala_GVirDomainInfo_free (GVirDomainInfo* self) {
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_boxed_free (gvir_domain_info_get_type (), self);
#line 1356 "libvirt-machine.c"
}


static void __lambda119_ (BoxesLibvirtMachine* self) {
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_RUNNING);
#line 1363 "libvirt-machine.c"
}


static void ___lambda119__gvir_domain_started (GVirDomain* _sender, gpointer self) {
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__lambda119_ (self);
#line 1370 "libvirt-machine.c"
}


static void __lambda120_ (BoxesLibvirtMachine* self) {
#line 131 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_PAUSED);
#line 1377 "libvirt-machine.c"
}


static void ___lambda120__gvir_domain_suspended (GVirDomain* _sender, gpointer self) {
#line 131 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__lambda120_ (self);
#line 1384 "libvirt-machine.c"
}


static void __lambda121_ (BoxesLibvirtMachine* self) {
#line 132 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_RUNNING);
#line 1391 "libvirt-machine.c"
}


static void ___lambda121__gvir_domain_resumed (GVirDomain* _sender, gpointer self) {
#line 132 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__lambda121_ (self);
#line 1398 "libvirt-machine.c"
}


static void __lambda122_ (BoxesLibvirtMachine* self) {
	GVirDomain* _tmp0_;
	GSignalInvocationHint* _tmp1_ = NULL;
	GQuark _tmp2_;
	GQuark _tmp3_ = 0U;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->_domain;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = g_signal_get_invocation_hint (_tmp0_);
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = (*_tmp1_).detail;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = g_quark_from_string ("saved");
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp2_ == _tmp3_) {
#line 135 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_SAVED);
#line 1419 "libvirt-machine.c"
	} else {
		gboolean _tmp4_;
#line 136 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp4_ = self->priv->force_stopped;
#line 136 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_tmp4_) {
#line 137 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			self->priv->force_stopped = FALSE;
#line 138 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_FORCE_STOPPED);
#line 1430 "libvirt-machine.c"
		} else {
#line 140 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_STOPPED);
#line 1434 "libvirt-machine.c"
		}
	}
}


static void ___lambda122__gvir_domain_stopped (GVirDomain* _sender, gpointer self) {
#line 133 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__lambda122_ (self);
#line 1443 "libvirt-machine.c"
}


static void __lambda123_ (BoxesLibvirtMachine* self) {
#line 143 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_SLEEPING);
#line 1450 "libvirt-machine.c"
}


static void ___lambda123__gvir_domain_pmsuspended (GVirDomain* _sender, gpointer self) {
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__lambda123_ (self);
#line 1457 "libvirt-machine.c"
}


static void __lambda124_ (BoxesLibvirtMachine* self) {
	BoxesMachineMachineState _tmp0_;
	BoxesMachineMachineState _tmp1_;
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = boxes_machine_get_state ((BoxesMachine*) self);
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _tmp0_;
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp1_ == BOXES_MACHINE_MACHINE_STATE_RUNNING) {
#line 147 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_reconnect_display (self);
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_stats_enable (self, TRUE);
#line 1474 "libvirt-machine.c"
	} else {
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_stats_enable (self, FALSE);
#line 1478 "libvirt-machine.c"
	}
}


static void ___lambda124__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 145 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__lambda124_ (self);
#line 1486 "libvirt-machine.c"
}


static void _boxes_libvirt_machine_update_domain_config_gvir_domain_updated (GVirDomain* _sender, gpointer self) {
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_domain_config (self);
#line 1493 "libvirt-machine.c"
}


BoxesLibvirtMachine* boxes_libvirt_machine_construct (GType object_type, BoxesCollectionSource* source, GVirConnection* connection, GVirDomain* domain, GError** error) {
	BoxesLibvirtMachine * self = NULL;
	GVirDomain* _tmp0_;
	GVirConfigDomain* _tmp1_ = NULL;
	GVirConfigDomain* config;
	GVirConfigDomain* _tmp2_;
	const gchar* _tmp3_ = NULL;
	gchar* _tmp4_;
	gchar* _tmp5_;
	const gchar* _tmp6_;
	const gchar* _tmp10_;
	gchar* _tmp11_;
	gchar* item_name;
	BoxesCollectionSource* _tmp12_;
	const gchar* _tmp13_;
	GVirDomain* _tmp14_;
	const gchar* _tmp15_ = NULL;
	gchar* _tmp16_;
	gchar* _tmp17_;
	GVirDomain* _tmp18_;
	const gchar* _tmp19_ = NULL;
	GVirConnection* _tmp20_;
	GVirDomain* _tmp21_;
	BoxesLibvirtMachineProperties* _tmp22_;
	GVirDomain* _tmp32_;
	GVirDomain* _tmp33_;
	GVirDomain* _tmp34_;
	GVirDomain* _tmp35_;
	GVirDomain* _tmp36_;
	GVirDomain* _tmp37_;
	BoxesMachineMachineState _tmp38_;
	BoxesMachineMachineState _tmp39_;
	GError * _inner_error_ = NULL;
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (source != NULL, NULL);
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (connection != NULL, NULL);
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (domain != NULL, NULL);
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = domain;
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = gvir_domain_get_config (_tmp0_, (guint) GVIR_DOMAIN_XML_INACTIVE, &_inner_error_);
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	config = _tmp1_;
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_propagate_error (error, _inner_error_);
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (self);
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return NULL;
#line 1550 "libvirt-machine.c"
	}
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = config;
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = gvir_config_domain_get_title (_tmp2_);
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp4_ = g_strdup (_tmp3_);
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp5_ = _tmp4_;
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp6_ = _tmp5_;
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp6_ == NULL) {
#line 1564 "libvirt-machine.c"
		GVirDomain* _tmp7_;
		const gchar* _tmp8_ = NULL;
		gchar* _tmp9_;
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp7_ = domain;
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp8_ = gvir_domain_get_name (_tmp7_);
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp9_ = g_strdup (_tmp8_);
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_free0 (_tmp5_);
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp5_ = _tmp9_;
#line 1578 "libvirt-machine.c"
	}
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp10_ = _tmp5_;
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp11_ = g_strdup (_tmp10_);
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	item_name = _tmp11_;
#line 89 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp12_ = source;
#line 89 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp13_ = item_name;
#line 89 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = (BoxesLibvirtMachine*) boxes_machine_construct (object_type, _tmp12_, _tmp13_);
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp14_ = domain;
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp15_ = gvir_domain_get_name (_tmp14_);
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp16_ = g_strconcat ("new libvirt machine: ", _tmp15_, NULL);
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp17_ = _tmp16_;
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_debug ("libvirt-machine.vala:91: %s", _tmp17_);
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (_tmp17_);
#line 92 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp18_ = domain;
#line 92 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp19_ = gvir_domain_get_uuid (_tmp18_);
#line 92 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_machine_create_display_config ((BoxesMachine*) self, _tmp19_);
#line 93 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp20_ = connection;
#line 93 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_set_connection (self, _tmp20_);
#line 94 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp21_ = domain;
#line 94 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_set_domain (self, _tmp21_);
#line 95 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp22_ = boxes_libvirt_machine_properties_new (self);
#line 95 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->properties);
#line 95 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->properties = _tmp22_;
#line 1624 "libvirt-machine.c"
	{
		GVirDomain* _tmp23_;
		GVirDomainInfo* _tmp24_ = NULL;
		GVirDomainInfo* _tmp25_;
		GVirDomainInfo* _tmp26_;
		GVirDomainState _tmp27_;
		GVirDomainState _tmp28_;
		GVirDomainState s;
		GVirDomainState _tmp29_;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp23_ = domain;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp24_ = gvir_domain_get_info (_tmp23_, &_inner_error_);
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp25_ = _tmp24_;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_ != NULL) {
#line 1642 "libvirt-machine.c"
			goto __catch13_g_error;
		}
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp26_ = _tmp25_;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp27_ = _tmp26_->state;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp28_ = _tmp27_;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		__vala_GVirDomainInfo_free0 (_tmp26_);
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		s = _tmp28_;
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp29_ = s;
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		switch (_tmp29_) {
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			case GVIR_DOMAIN_STATE_RUNNING:
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			case GVIR_DOMAIN_STATE_BLOCKED:
#line 1663 "libvirt-machine.c"
			{
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_RUNNING);
#line 103 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				boxes_libvirt_machine_set_stats_enable (self, TRUE);
#line 104 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				break;
#line 1671 "libvirt-machine.c"
			}
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			case GVIR_DOMAIN_STATE_PAUSED:
#line 1675 "libvirt-machine.c"
			{
#line 106 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_PAUSED);
#line 107 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				break;
#line 1681 "libvirt-machine.c"
			}
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			case GVIR_DOMAIN_STATE_SHUTDOWN:
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			case GVIR_DOMAIN_STATE_SHUTOFF:
#line 1687 "libvirt-machine.c"
			{
				GVirDomain* _tmp30_;
				gboolean _tmp31_ = FALSE;
#line 110 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp30_ = domain;
#line 110 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp31_ = gvir_domain_get_saved (_tmp30_);
#line 110 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				if (_tmp31_) {
#line 111 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_SAVED);
#line 1699 "libvirt-machine.c"
				} else {
#line 113 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_STOPPED);
#line 1703 "libvirt-machine.c"
				}
#line 114 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				break;
#line 1707 "libvirt-machine.c"
			}
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			case GVIR_DOMAIN_STATE_CRASHED:
#line 1711 "libvirt-machine.c"
			{
#line 116 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_STOPPED);
#line 117 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				break;
#line 1717 "libvirt-machine.c"
			}
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			case GVIR_DOMAIN_STATE_PMSUSPENDED:
#line 1721 "libvirt-machine.c"
			{
#line 119 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_SLEEPING);
#line 120 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				break;
#line 1727 "libvirt-machine.c"
			}
			default:
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			case GVIR_DOMAIN_STATE_NONE:
#line 1732 "libvirt-machine.c"
			{
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_UNKNOWN);
#line 124 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				break;
#line 1738 "libvirt-machine.c"
			}
		}
	}
	goto __finally13;
	__catch13_g_error:
	{
		GError* _error_ = NULL;
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_error_ = _inner_error_;
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_inner_error_ = NULL;
#line 127 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_machine_set_state ((BoxesMachine*) self, BOXES_MACHINE_MACHINE_STATE_UNKNOWN);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (_error_);
#line 1754 "libvirt-machine.c"
	}
	__finally13:
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_propagate_error (error, _inner_error_);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_free0 (item_name);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_free0 (_tmp5_);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (config);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (self);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return NULL;
#line 1771 "libvirt-machine.c"
	}
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp32_ = domain;
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_signal_connect_object (_tmp32_, "started", (GCallback) ___lambda119__gvir_domain_started, self, 0);
#line 131 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp33_ = domain;
#line 131 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_signal_connect_object (_tmp33_, "suspended", (GCallback) ___lambda120__gvir_domain_suspended, self, 0);
#line 132 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp34_ = domain;
#line 132 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_signal_connect_object (_tmp34_, "resumed", (GCallback) ___lambda121__gvir_domain_resumed, self, 0);
#line 133 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp35_ = domain;
#line 133 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_signal_connect_object (_tmp35_, "stopped", (GCallback) ___lambda122__gvir_domain_stopped, self, 0);
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp36_ = domain;
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_signal_connect_object (_tmp36_, "pmsuspended", (GCallback) ___lambda123__gvir_domain_pmsuspended, self, 0);
#line 145 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_signal_connect_object ((GObject*) self, "notify::state", (GCallback) ___lambda124__g_object_notify, self, 0);
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_domain_config (self);
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp37_ = domain;
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_signal_connect_object (_tmp37_, "updated", (GCallback) _boxes_libvirt_machine_update_domain_config_gvir_domain_updated, self, 0);
#line 156 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp38_ = boxes_machine_get_state ((BoxesMachine*) self);
#line 156 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp39_ = _tmp38_;
#line 156 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp39_ != BOXES_MACHINE_MACHINE_STATE_STOPPED) {
#line 157 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_machine_load_screenshot ((BoxesMachine*) self);
#line 1809 "libvirt-machine.c"
	}
#line 158 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_machine_set_screenshot_enable ((BoxesMachine*) self, TRUE);
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (item_name);
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (_tmp5_);
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (config);
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return self;
#line 1821 "libvirt-machine.c"
}


BoxesLibvirtMachine* boxes_libvirt_machine_new (BoxesCollectionSource* source, GVirConnection* connection, GVirDomain* domain, GError** error) {
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return boxes_libvirt_machine_construct (BOXES_TYPE_LIBVIRT_MACHINE, source, connection, domain, error);
#line 1828 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_cpu_stat (BoxesLibvirtMachine* self, GVirDomainInfo* info, BoxesLibvirtMachineMachineStat* stat) {
	BoxesLibvirtMachineMachineStat* _tmp0_;
	gint _tmp0__length1;
	BoxesLibvirtMachineMachineStat _tmp1_;
	BoxesLibvirtMachineMachineStat _tmp2_ = {0};
	BoxesLibvirtMachineMachineStat prev;
	gboolean _tmp3_ = FALSE;
	GVirDomainInfo* _tmp4_;
	GVirDomainState _tmp5_;
	gboolean _tmp8_;
	GVirDomainInfo* _tmp9_;
	guint64 _tmp10_;
	BoxesLibvirtMachineMachineStat _tmp11_;
	gdouble _tmp12_;
	GVirDomainInfo* _tmp13_;
	guint64 _tmp14_;
	BoxesLibvirtMachineMachineStat _tmp15_;
	gint64 _tmp16_;
	BoxesLibvirtMachineMachineStat _tmp17_;
	gint64 _tmp18_;
	gint64 dt;
	BoxesLibvirtMachineMachineStat _tmp19_;
	gdouble _tmp20_;
	gint64 _tmp21_;
	gdouble percent;
	gdouble _tmp22_;
	GVirDomainInfo* _tmp23_;
	guint16 _tmp24_;
	gdouble _tmp25_;
	gdouble _tmp26_ = 0.0;
#line 161 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 161 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (info != NULL);
#line 161 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (stat != NULL);
#line 162 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->stats;
#line 162 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0__length1 = self->priv->stats_length1;
#line 162 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _tmp0_[BOXES_LIBVIRT_MACHINE_STATS_SIZE - 1];
#line 162 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_machine_stat_copy (&_tmp1_, &_tmp2_);
#line 162 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	prev = _tmp2_;
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp4_ = info;
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp5_ = _tmp4_->state;
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp5_ == GVIR_DOMAIN_STATE_CRASHED) {
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp3_ = TRUE;
#line 1886 "libvirt-machine.c"
	} else {
		GVirDomainInfo* _tmp6_;
		GVirDomainState _tmp7_;
#line 165 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp6_ = info;
#line 165 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp7_ = _tmp6_->state;
#line 165 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp3_ = _tmp7_ == GVIR_DOMAIN_STATE_SHUTOFF;
#line 1896 "libvirt-machine.c"
	}
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp8_ = _tmp3_;
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp8_) {
#line 166 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_machine_stat_destroy (&prev);
#line 166 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 1906 "libvirt-machine.c"
	}
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp9_ = info;
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp10_ = _tmp9_->cpuTime;
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp11_ = prev;
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp12_ = _tmp11_.cpu_time_abs;
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*stat).cpu_time = _tmp10_ - _tmp12_;
#line 169 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp13_ = info;
#line 169 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp14_ = _tmp13_->cpuTime;
#line 169 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*stat).cpu_time_abs = (gdouble) _tmp14_;
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp15_ = *stat;
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp16_ = _tmp15_.timestamp;
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp17_ = prev;
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp18_ = _tmp17_.timestamp;
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	dt = (_tmp16_ - _tmp18_) * 10;
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp19_ = *stat;
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp20_ = _tmp19_.cpu_time;
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp21_ = dt;
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	percent = _tmp20_ / _tmp21_;
#line 173 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp22_ = percent;
#line 173 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp23_ = info;
#line 173 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp24_ = _tmp23_->nrVirtCpu;
#line 173 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	percent = _tmp22_ / _tmp24_;
#line 174 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp25_ = percent;
#line 174 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp26_ = CLAMP (_tmp25_, (gdouble) 0, (gdouble) 100);
#line 174 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*stat).cpu_guest_percent = _tmp26_;
#line 161 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_machine_stat_destroy (&prev);
#line 1958 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_mem_stat (BoxesLibvirtMachine* self, GVirDomainInfo* info, BoxesLibvirtMachineMachineStat* stat) {
	GVirDomainInfo* _tmp0_;
	GVirDomainState _tmp1_;
	GVirDomainInfo* _tmp2_;
	guint64 _tmp3_;
	GVirDomainInfo* _tmp4_;
	guint64 _tmp5_;
#line 177 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 177 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (info != NULL);
#line 177 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (stat != NULL);
#line 178 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = info;
#line 178 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _tmp0_->state;
#line 178 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp1_ != GVIR_DOMAIN_STATE_RUNNING) {
#line 179 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 1983 "libvirt-machine.c"
	}
#line 181 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = info;
#line 181 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = _tmp2_->memory;
#line 181 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp4_ = info;
#line 181 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp5_ = _tmp4_->maxMem;
#line 181 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*stat).memory_percent = (_tmp3_ * 100.0) / _tmp5_;
#line 1995 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_io_stat_data_free (gpointer _data) {
	BoxesLibvirtMachineUpdateIoStatData* _data_;
	_data_ = _data;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainInfo_free0 (_data_->info);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_slice_free (BoxesLibvirtMachineUpdateIoStatData, _data_);
#line 2008 "libvirt-machine.c"
}


static GVirDomainInfo* _vala_GVirDomainInfo_copy (GVirDomainInfo* self) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return g_boxed_copy (gvir_domain_info_get_type (), self);
#line 2015 "libvirt-machine.c"
}


static gpointer __vala_GVirDomainInfo_copy0 (gpointer self) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return self ? _vala_GVirDomainInfo_copy (self) : NULL;
#line 2022 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_io_stat (BoxesLibvirtMachine* self, GVirDomainInfo* info, BoxesLibvirtMachineMachineStat* stat, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesLibvirtMachineUpdateIoStatData* _data_;
	BoxesLibvirtMachine* _tmp0_;
	GVirDomainInfo* _tmp1_;
	GVirDomainInfo* _tmp2_;
	BoxesLibvirtMachineMachineStat* _tmp3_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_slice_new0 (BoxesLibvirtMachineUpdateIoStatData);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_libvirt_machine_update_io_stat);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_libvirt_machine_update_io_stat_data_free);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self = _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = info;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = __vala_GVirDomainInfo_copy0 (_tmp1_);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainInfo_free0 (_data_->info);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->info = _tmp2_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = stat;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->stat = _tmp3_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_io_stat_co (_data_);
#line 2056 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_io_stat_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_) {
	BoxesLibvirtMachineUpdateIoStatData* _data_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 2064 "libvirt-machine.c"
}


static Block18Data* block18_data_ref (Block18Data* _data18_) {
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_atomic_int_inc (&_data18_->_ref_count_);
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return _data18_;
#line 2073 "libvirt-machine.c"
}


static void block18_data_unref (void * _userdata_) {
	Block18Data* _data18_;
	_data18_ = (Block18Data*) _userdata_;
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_atomic_int_dec_and_test (&_data18_->_ref_count_)) {
#line 2082 "libvirt-machine.c"
		BoxesLibvirtMachine * self;
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self = _data18_->self;
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (self);
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_slice_free (Block18Data, _data18_);
#line 2090 "libvirt-machine.c"
	}
}


static Block19Data* block19_data_ref (Block19Data* _data19_) {
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_atomic_int_inc (&_data19_->_ref_count_);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return _data19_;
#line 2100 "libvirt-machine.c"
}


static void block19_data_unref (void * _userdata_) {
	Block19Data* _data19_;
	_data19_ = (Block19Data*) _userdata_;
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_atomic_int_dec_and_test (&_data19_->_ref_count_)) {
#line 2109 "libvirt-machine.c"
		BoxesLibvirtMachine * self;
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self = _data19_->_data18_->self;
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (_data19_->disk);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block18_data_unref (_data19_->_data18_);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data19_->_data18_ = NULL;
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_slice_free (Block19Data, _data19_);
#line 2121 "libvirt-machine.c"
	}
}


static void _vala_GVirDomainDiskStats_free (GVirDomainDiskStats* self) {
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_boxed_free (gvir_domain_disk_stats_get_type (), self);
#line 2129 "libvirt-machine.c"
}


static void ___lambda116_ (Block19Data* _data19_, GError** error) {
	Block18Data* _data18_;
	BoxesLibvirtMachine * self;
	GVirDomainDisk* _tmp0_;
	GVirDomainDiskStats* _tmp1_ = NULL;
	GVirDomainDiskStats* _tmp2_;
	BoxesLibvirtMachineMachineStat* _tmp3_;
	GError * _inner_error_ = NULL;
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data18_ = _data19_->_data18_;
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = _data18_->self;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _data19_->disk;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = gvir_domain_disk_get_stats (_tmp0_, &_inner_error_);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = _tmp1_;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_propagate_error (error, _inner_error_);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 2157 "libvirt-machine.c"
	}
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = _data18_->stat;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainDiskStats_free0 ((*_tmp3_).disk);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*_tmp3_).disk = _tmp2_;
#line 2165 "libvirt-machine.c"
}


static void ____lambda116__boxes_run_in_thread_func (gpointer self, GError** error) {
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	___lambda116_ (self, error);
#line 2172 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_io_stat_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesLibvirtMachineUpdateIoStatData* _data_;
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = _user_data_;
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_source_object_ = source_object;
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_res_ = _res_;
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_io_stat_co (_data_);
#line 2186 "libvirt-machine.c"
}


static gboolean boxes_libvirt_machine_update_io_stat_co (BoxesLibvirtMachineUpdateIoStatData* _data_) {
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (_data_->_state_) {
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 0:
#line 2195 "libvirt-machine.c"
		goto _state_0;
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 1:
#line 2199 "libvirt-machine.c"
		goto _state_1;
		default:
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_assert_not_reached ();
#line 2204 "libvirt-machine.c"
	}
	_state_0:
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data18_ = g_slice_new0 (Block18Data);
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data18_->_ref_count_ = 1;
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data18_->self = g_object_ref (_data_->self);
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp0_ = _data_->stat;
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data18_->stat = _data_->_tmp0_;
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data18_->_async_data_ = _data_;
#line 185 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp1_ = _data_->info;
#line 185 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp2_ = _data_->_tmp1_->state;
#line 185 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_tmp2_ != GVIR_DOMAIN_STATE_RUNNING) {
#line 186 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block18_data_unref (_data_->_data18_);
#line 186 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data18_ = NULL;
#line 186 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 186 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 2233 "libvirt-machine.c"
		} else {
#line 186 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 2237 "libvirt-machine.c"
		}
#line 186 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 186 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 2243 "libvirt-machine.c"
	}
	{
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data19_ = g_slice_new0 (Block19Data);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data19_->_ref_count_ = 1;
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data19_->_data18_ = block18_data_ref (_data_->_data18_);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp3_ = NULL;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp3_ = boxes_libvirt_machine_get_domain_disk (_data_->self, &_data_->_inner_error_);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data19_->disk = _data_->_tmp3_;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			block19_data_unref (_data_->_data19_);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_data19_ = NULL;
#line 2264 "libvirt-machine.c"
			goto __catch14_g_error;
		}
#line 190 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp4_ = _data_->_data19_->disk;
#line 190 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_tmp4_ == NULL) {
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			block19_data_unref (_data_->_data19_);
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_data19_ = NULL;
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			block18_data_unref (_data_->_data18_);
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_data18_ = NULL;
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_state_ == 0) {
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 2283 "libvirt-machine.c"
			} else {
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete (_data_->_async_result);
#line 2287 "libvirt-machine.c"
			}
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_object_unref (_data_->_async_result);
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 2293 "libvirt-machine.c"
		}
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_state_ = 1;
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_run_in_thread (____lambda116__boxes_run_in_thread_func, block19_data_ref (_data_->_data19_), block19_data_unref, NULL, boxes_libvirt_machine_update_io_stat_ready, _data_);
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 2301 "libvirt-machine.c"
		_state_1:
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_run_in_thread_finish (_data_->_res_, &_data_->_inner_error_);
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			block19_data_unref (_data_->_data19_);
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_data19_ = NULL;
#line 2311 "libvirt-machine.c"
			goto __catch14_g_error;
		}
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp5_ = _data_->self->priv->stats;
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp5__length1 = _data_->self->priv->stats_length1;
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp6_ = _data_->_tmp5_[BOXES_LIBVIRT_MACHINE_STATS_SIZE - 1];
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		memset (&_data_->_tmp7_, 0, sizeof (BoxesLibvirtMachineMachineStat));
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_machine_stat_copy (&_data_->_tmp6_, &_data_->_tmp7_);
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->prev = _data_->_tmp7_;
#line 197 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp8_ = _data_->prev;
#line 197 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp9_ = _data_->_tmp8_.disk;
#line 197 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_tmp9_ != NULL) {
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp10_ = _data_->_data18_->stat;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp11_ = _data_->_data18_->stat;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp12_ = (*_data_->_tmp11_).disk;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp13_ = _data_->_tmp12_->rd_bytes;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp14_ = _data_->prev;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp15_ = _data_->_tmp14_.disk;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp16_ = _data_->_tmp15_->rd_bytes;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			(*_data_->_tmp10_).disk_read = (gdouble) (_data_->_tmp13_ - _data_->_tmp16_);
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp17_ = _data_->_data18_->stat;
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp18_ = _data_->_data18_->stat;
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp19_ = (*_data_->_tmp18_).disk;
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp20_ = _data_->_tmp19_->wr_bytes;
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp21_ = _data_->prev;
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp22_ = _data_->_tmp21_.disk;
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp23_ = _data_->_tmp22_->wr_bytes;
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			(*_data_->_tmp17_).disk_write = (gdouble) (_data_->_tmp20_ - _data_->_tmp23_);
#line 2364 "libvirt-machine.c"
		}
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_machine_stat_destroy (&_data_->prev);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block19_data_unref (_data_->_data19_);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data19_ = NULL;
#line 2372 "libvirt-machine.c"
	}
	goto __finally14;
	__catch14_g_error:
	{
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->err = _data_->_inner_error_;
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_inner_error_ = NULL;
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp24_ = boxes_collection_item_get_name ((BoxesCollectionItem*) _data_->self);
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp25_ = _data_->_tmp24_;
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp26_ = _data_->err;
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp27_ = _data_->_tmp26_->message;
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_warning ("libvirt-machine.vala:202: Failed to fetch I/O statistics for %s: %s", _data_->_tmp25_, _data_->_tmp27_);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (_data_->err);
#line 2393 "libvirt-machine.c"
	}
	__finally14:
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_inner_error_ != NULL) {
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block18_data_unref (_data_->_data18_);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data18_ = NULL;
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_clear_error (&_data_->_inner_error_);
#line 188 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 2408 "libvirt-machine.c"
	}
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	block18_data_unref (_data_->_data18_);
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data18_ = NULL;
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_state_ == 0) {
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 2418 "libvirt-machine.c"
	} else {
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 2422 "libvirt-machine.c"
	}
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_unref (_data_->_async_result);
#line 184 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 2428 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_net_stat_data_free (gpointer _data) {
	BoxesLibvirtMachineUpdateNetStatData* _data_;
	_data_ = _data;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainInfo_free0 (_data_->info);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_slice_free (BoxesLibvirtMachineUpdateNetStatData, _data_);
#line 2441 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_net_stat (BoxesLibvirtMachine* self, GVirDomainInfo* info, BoxesLibvirtMachineMachineStat* stat, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesLibvirtMachineUpdateNetStatData* _data_;
	BoxesLibvirtMachine* _tmp0_;
	GVirDomainInfo* _tmp1_;
	GVirDomainInfo* _tmp2_;
	BoxesLibvirtMachineMachineStat* _tmp3_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_slice_new0 (BoxesLibvirtMachineUpdateNetStatData);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_libvirt_machine_update_net_stat);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_libvirt_machine_update_net_stat_data_free);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self = _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = info;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = __vala_GVirDomainInfo_copy0 (_tmp1_);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainInfo_free0 (_data_->info);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->info = _tmp2_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = stat;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->stat = _tmp3_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_net_stat_co (_data_);
#line 2475 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_net_stat_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_) {
	BoxesLibvirtMachineUpdateNetStatData* _data_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 2483 "libvirt-machine.c"
}


static Block20Data* block20_data_ref (Block20Data* _data20_) {
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_atomic_int_inc (&_data20_->_ref_count_);
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return _data20_;
#line 2492 "libvirt-machine.c"
}


static void block20_data_unref (void * _userdata_) {
	Block20Data* _data20_;
	_data20_ = (Block20Data*) _userdata_;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_atomic_int_dec_and_test (&_data20_->_ref_count_)) {
#line 2501 "libvirt-machine.c"
		BoxesLibvirtMachine * self;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self = _data20_->self;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (self);
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_slice_free (Block20Data, _data20_);
#line 2509 "libvirt-machine.c"
	}
}


static Block21Data* block21_data_ref (Block21Data* _data21_) {
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_atomic_int_inc (&_data21_->_ref_count_);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return _data21_;
#line 2519 "libvirt-machine.c"
}


static void block21_data_unref (void * _userdata_) {
	Block21Data* _data21_;
	_data21_ = (Block21Data*) _userdata_;
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_atomic_int_dec_and_test (&_data21_->_ref_count_)) {
#line 2528 "libvirt-machine.c"
		BoxesLibvirtMachine * self;
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self = _data21_->_data20_->self;
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (_data21_->net);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block20_data_unref (_data21_->_data20_);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data21_->_data20_ = NULL;
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_slice_free (Block21Data, _data21_);
#line 2540 "libvirt-machine.c"
	}
}


static void _vala_GVirDomainInterfaceStats_free (GVirDomainInterfaceStats* self) {
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_boxed_free (gvir_domain_interface_stats_get_type (), self);
#line 2548 "libvirt-machine.c"
}


static void ___lambda117_ (Block21Data* _data21_, GError** error) {
	Block20Data* _data20_;
	BoxesLibvirtMachine * self;
	GVirDomainInterface* _tmp0_;
	GVirDomainInterfaceStats* _tmp1_ = NULL;
	GVirDomainInterfaceStats* _tmp2_;
	BoxesLibvirtMachineMachineStat* _tmp3_;
	GError * _inner_error_ = NULL;
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data20_ = _data21_->_data20_;
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = _data20_->self;
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _data21_->net;
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = gvir_domain_interface_get_stats (_tmp0_, &_inner_error_);
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = _tmp1_;
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_propagate_error (error, _inner_error_);
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 2576 "libvirt-machine.c"
	}
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = _data20_->stat;
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainInterfaceStats_free0 ((*_tmp3_).net);
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*_tmp3_).net = _tmp2_;
#line 2584 "libvirt-machine.c"
}


static void ____lambda117__boxes_run_in_thread_func (gpointer self, GError** error) {
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	___lambda117_ (self, error);
#line 2591 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_net_stat_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesLibvirtMachineUpdateNetStatData* _data_;
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = _user_data_;
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_source_object_ = source_object;
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_res_ = _res_;
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_net_stat_co (_data_);
#line 2605 "libvirt-machine.c"
}


static gboolean boxes_libvirt_machine_update_net_stat_co (BoxesLibvirtMachineUpdateNetStatData* _data_) {
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (_data_->_state_) {
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 0:
#line 2614 "libvirt-machine.c"
		goto _state_0;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 1:
#line 2618 "libvirt-machine.c"
		goto _state_1;
		default:
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_assert_not_reached ();
#line 2623 "libvirt-machine.c"
	}
	_state_0:
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data20_ = g_slice_new0 (Block20Data);
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data20_->_ref_count_ = 1;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data20_->self = g_object_ref (_data_->self);
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp0_ = _data_->stat;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data20_->stat = _data_->_tmp0_;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data20_->_async_data_ = _data_;
#line 207 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp1_ = _data_->info;
#line 207 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp2_ = _data_->_tmp1_->state;
#line 207 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_tmp2_ != GVIR_DOMAIN_STATE_RUNNING) {
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block20_data_unref (_data_->_data20_);
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data20_ = NULL;
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 2652 "libvirt-machine.c"
		} else {
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 2656 "libvirt-machine.c"
		}
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 2662 "libvirt-machine.c"
	}
	{
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data21_ = g_slice_new0 (Block21Data);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data21_->_ref_count_ = 1;
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data21_->_data20_ = block20_data_ref (_data_->_data20_);
#line 211 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp3_ = NULL;
#line 211 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp3_ = boxes_libvirt_machine_get_domain_network_interface (_data_->self, &_data_->_inner_error_);
#line 211 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data21_->net = _data_->_tmp3_;
#line 211 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 211 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			block21_data_unref (_data_->_data21_);
#line 211 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_data21_ = NULL;
#line 2683 "libvirt-machine.c"
			goto __catch15_g_error;
		}
#line 212 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp4_ = _data_->_data21_->net;
#line 212 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_tmp4_ == NULL) {
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			block21_data_unref (_data_->_data21_);
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_data21_ = NULL;
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			block20_data_unref (_data_->_data20_);
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_data20_ = NULL;
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_state_ == 0) {
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 2702 "libvirt-machine.c"
			} else {
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete (_data_->_async_result);
#line 2706 "libvirt-machine.c"
			}
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_object_unref (_data_->_async_result);
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 2712 "libvirt-machine.c"
		}
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_state_ = 1;
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_run_in_thread (____lambda117__boxes_run_in_thread_func, block21_data_ref (_data_->_data21_), block21_data_unref, NULL, boxes_libvirt_machine_update_net_stat_ready, _data_);
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 2720 "libvirt-machine.c"
		_state_1:
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_run_in_thread_finish (_data_->_res_, &_data_->_inner_error_);
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			block21_data_unref (_data_->_data21_);
#line 215 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_data21_ = NULL;
#line 2730 "libvirt-machine.c"
			goto __catch15_g_error;
		}
#line 218 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp5_ = _data_->self->priv->stats;
#line 218 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp5__length1 = _data_->self->priv->stats_length1;
#line 218 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp6_ = _data_->_tmp5_[BOXES_LIBVIRT_MACHINE_STATS_SIZE - 1];
#line 218 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		memset (&_data_->_tmp7_, 0, sizeof (BoxesLibvirtMachineMachineStat));
#line 218 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_machine_stat_copy (&_data_->_tmp6_, &_data_->_tmp7_);
#line 218 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->prev = _data_->_tmp7_;
#line 219 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp8_ = _data_->prev;
#line 219 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp9_ = _data_->_tmp8_.net;
#line 219 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_tmp9_ != NULL) {
#line 220 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp10_ = _data_->_data20_->stat;
#line 220 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp11_ = _data_->_data20_->stat;
#line 220 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp12_ = (*_data_->_tmp11_).net;
#line 220 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp13_ = _data_->_tmp12_->rx_bytes;
#line 220 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp14_ = _data_->prev;
#line 220 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp15_ = _data_->_tmp14_.net;
#line 220 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp16_ = _data_->_tmp15_->rx_bytes;
#line 220 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			(*_data_->_tmp10_).net_read = (gdouble) (_data_->_tmp13_ - _data_->_tmp16_);
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp17_ = _data_->_data20_->stat;
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp18_ = _data_->_data20_->stat;
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp19_ = (*_data_->_tmp18_).net;
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp20_ = _data_->_tmp19_->tx_bytes;
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp21_ = _data_->prev;
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp22_ = _data_->_tmp21_.net;
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp23_ = _data_->_tmp22_->tx_bytes;
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			(*_data_->_tmp17_).net_write = (gdouble) (_data_->_tmp20_ - _data_->_tmp23_);
#line 2783 "libvirt-machine.c"
		}
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_machine_stat_destroy (&_data_->prev);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block21_data_unref (_data_->_data21_);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data21_ = NULL;
#line 2791 "libvirt-machine.c"
	}
	goto __finally15;
	__catch15_g_error:
	{
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->err = _data_->_inner_error_;
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_inner_error_ = NULL;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp24_ = boxes_collection_item_get_name ((BoxesCollectionItem*) _data_->self);
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp25_ = _data_->_tmp24_;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp26_ = _data_->err;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp27_ = _data_->_tmp26_->message;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_warning ("libvirt-machine.vala:224: Failed to fetch network statistics for %s: %" \
"s", _data_->_tmp25_, _data_->_tmp27_);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (_data_->err);
#line 2812 "libvirt-machine.c"
	}
	__finally15:
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_inner_error_ != NULL) {
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block20_data_unref (_data_->_data20_);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data20_ = NULL;
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_clear_error (&_data_->_inner_error_);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 2827 "libvirt-machine.c"
	}
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	block20_data_unref (_data_->_data20_);
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data20_ = NULL;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_state_ == 0) {
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 2837 "libvirt-machine.c"
	} else {
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 2841 "libvirt-machine.c"
	}
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_unref (_data_->_async_result);
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 2847 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_stats_data_free (gpointer _data) {
	BoxesLibvirtMachineUpdateStatsData* _data_;
	_data_ = _data;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_slice_free (BoxesLibvirtMachineUpdateStatsData, _data_);
#line 2858 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_stats (BoxesLibvirtMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesLibvirtMachineUpdateStatsData* _data_;
	BoxesLibvirtMachine* _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_slice_new0 (BoxesLibvirtMachineUpdateStatsData);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_libvirt_machine_update_stats);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_libvirt_machine_update_stats_data_free);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self = _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_stats_co (_data_);
#line 2877 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_stats_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_) {
	BoxesLibvirtMachineUpdateStatsData* _data_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 2885 "libvirt-machine.c"
}


static void boxes_libvirt_machine_update_stats_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesLibvirtMachineUpdateStatsData* _data_;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = _user_data_;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_source_object_ = source_object;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_res_ = _res_;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_stats_co (_data_);
#line 2899 "libvirt-machine.c"
}


static BoxesLibvirtMachineMachineStat* _vala_array_dup1 (BoxesLibvirtMachineMachineStat* self, int length) {
	BoxesLibvirtMachineMachineStat* result;
	int i;
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = g_new0 (BoxesLibvirtMachineMachineStat, length);
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	for (i = 0; i < length; i++) {
#line 2910 "libvirt-machine.c"
		BoxesLibvirtMachineMachineStat _tmp0_ = {0};
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_machine_stat_copy (&self[i], &_tmp0_);
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		result[i] = _tmp0_;
#line 2916 "libvirt-machine.c"
	}
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 2920 "libvirt-machine.c"
}


static void _vala_array_add2 (BoxesLibvirtMachineMachineStat** array, int* length, int* size, const BoxesLibvirtMachineMachineStat* value) {
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if ((*length) == (*size)) {
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		*array = g_renew (BoxesLibvirtMachineMachineStat, *array, *size);
#line 2931 "libvirt-machine.c"
	}
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*array)[(*length)++] = *value;
#line 2935 "libvirt-machine.c"
}


static void _vala_array_add3 (gdouble** array, int* length, int* size, gdouble value) {
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if ((*length) == (*size)) {
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		*array = g_renew (gdouble, *array, *size);
#line 2946 "libvirt-machine.c"
	}
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*array)[(*length)++] = value;
#line 2950 "libvirt-machine.c"
}


static void _vala_array_add4 (gdouble** array, int* length, int* size, gdouble value) {
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if ((*length) == (*size)) {
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		*array = g_renew (gdouble, *array, *size);
#line 2961 "libvirt-machine.c"
	}
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*array)[(*length)++] = value;
#line 2965 "libvirt-machine.c"
}


static void _vala_array_add5 (gdouble** array, int* length, int* size, gdouble value) {
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if ((*length) == (*size)) {
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		*size = (*size) ? (2 * (*size)) : 4;
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		*array = g_renew (gdouble, *array, *size);
#line 2976 "libvirt-machine.c"
	}
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*array)[(*length)++] = value;
#line 2980 "libvirt-machine.c"
}


static gboolean boxes_libvirt_machine_update_stats_co (BoxesLibvirtMachineUpdateStatsData* _data_) {
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (_data_->_state_) {
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 0:
#line 2989 "libvirt-machine.c"
		goto _state_0;
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 1:
#line 2993 "libvirt-machine.c"
		goto _state_1;
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 2:
#line 2997 "libvirt-machine.c"
		goto _state_2;
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 3:
#line 3001 "libvirt-machine.c"
		goto _state_3;
		default:
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_assert_not_reached ();
#line 3006 "libvirt-machine.c"
	}
	_state_0:
	{
#line 235 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp0_ = 0LL;
#line 235 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp0_ = g_get_monotonic_time ();
#line 235 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->now = _data_->_tmp0_;
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp1_ = _data_->now;
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		memset (&_data_->_tmp2_, 0, sizeof (BoxesLibvirtMachineMachineStat));
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		memset (&_data_->_tmp2_, 0, sizeof (BoxesLibvirtMachineMachineStat));
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp2_.timestamp = _data_->_tmp1_;
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->stat = _data_->_tmp2_;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp3_ = _data_->self->priv->_domain;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp4_ = _data_->self->priv->stats_cancellable;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_state_ = 1;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gvir_domain_get_info_async (_data_->_tmp3_, _data_->_tmp4_, boxes_libvirt_machine_update_stats_ready, _data_);
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 3036 "libvirt-machine.c"
		_state_1:
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp5_ = NULL;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp5_ = gvir_domain_get_info_finish (_data_->_tmp3_, _data_->_res_, &_data_->_inner_error_);
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->info = _data_->_tmp5_;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			boxes_libvirt_machine_machine_stat_destroy (&_data_->stat);
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (g_error_matches (_data_->_inner_error_, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
#line 3050 "libvirt-machine.c"
				goto __catch16_g_io_error_cancelled;
			}
			goto __catch16_g_error;
		}
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp6_ = _data_->info;
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_update_cpu_stat (_data_->self, _data_->_tmp6_, &_data_->stat);
#line 240 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp7_ = _data_->info;
#line 240 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_update_mem_stat (_data_->self, _data_->_tmp7_, &_data_->stat);
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp8_ = _data_->info;
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_state_ = 2;
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_update_io_stat (_data_->self, _data_->_tmp8_, &_data_->stat, boxes_libvirt_machine_update_stats_ready, _data_);
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 3071 "libvirt-machine.c"
		_state_2:
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_update_io_stat_finish (_data_->self, _data_->_res_);
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp9_ = _data_->info;
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_state_ = 3;
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_update_net_stat (_data_->self, _data_->_tmp9_, &_data_->stat, boxes_libvirt_machine_update_stats_ready, _data_);
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 3083 "libvirt-machine.c"
		_state_3:
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_update_net_stat_finish (_data_->self, _data_->_res_);
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp10_ = _data_->self->priv->stats;
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp10__length1 = _data_->self->priv->stats_length1;
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp11_ = ((_data_->_tmp10_ + 1) != NULL) ? _vala_array_dup1 (_data_->_tmp10_ + 1, BOXES_LIBVIRT_MACHINE_STATS_SIZE - 1) : ((gpointer) (_data_->_tmp10_ + 1));
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp11__length1 = BOXES_LIBVIRT_MACHINE_STATS_SIZE - 1;
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->self->priv->stats = (_vala_BoxesLibvirtMachineMachineStat_array_free (_data_->self->priv->stats, _data_->self->priv->stats_length1), NULL);
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->self->priv->stats = _data_->_tmp11_;
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->self->priv->stats_length1 = _data_->_tmp11__length1;
#line 244 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->self->priv->_stats_size_ = _data_->self->priv->stats_length1;
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp12_ = _data_->self->priv->stats;
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp12__length1 = _data_->self->priv->stats_length1;
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp13_ = _data_->stat;
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		memset (&_data_->_tmp14_, 0, sizeof (BoxesLibvirtMachineMachineStat));
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_machine_stat_copy (&_data_->_tmp13_, &_data_->_tmp14_);
#line 245 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_vala_array_add2 (&_data_->self->priv->stats, &_data_->self->priv->stats_length1, &_data_->self->priv->_stats_size_, &_data_->_tmp14_);
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		__vala_GVirDomainInfo_free0 (_data_->info);
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_machine_stat_destroy (&_data_->stat);
#line 3119 "libvirt-machine.c"
	}
	goto __finally16;
	__catch16_g_io_error_cancelled:
	{
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->err = _data_->_inner_error_;
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_inner_error_ = NULL;
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (_data_->err);
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 3134 "libvirt-machine.c"
		} else {
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 3138 "libvirt-machine.c"
		}
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 3144 "libvirt-machine.c"
	}
	goto __finally16;
	__catch16_g_error:
	{
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_vala1_err = _data_->_inner_error_;
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_inner_error_ = NULL;
#line 250 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp15_ = _data_->_vala1_err;
#line 250 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp16_ = _data_->_tmp15_->message;
#line 250 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_warning ("libvirt-machine.vala:250: %s", _data_->_tmp16_);
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (_data_->_vala1_err);
#line 3161 "libvirt-machine.c"
	}
	__finally16:
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_inner_error_ != NULL) {
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_clear_error (&_data_->_inner_error_);
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 3172 "libvirt-machine.c"
	}
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp17_ = NULL;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp17_ = g_new0 (gdouble, 0);
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->cpu_stats = (g_free (_data_->self->cpu_stats), NULL);
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->cpu_stats = _data_->_tmp17_;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->cpu_stats_length1 = 0;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->_cpu_stats_size_ = _data_->self->cpu_stats_length1;
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp18_ = NULL;
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp18_ = g_new0 (gdouble, 0);
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->io_stats = (g_free (_data_->self->io_stats), NULL);
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->io_stats = _data_->_tmp18_;
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->io_stats_length1 = 0;
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->_io_stats_size_ = _data_->self->io_stats_length1;
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp19_ = NULL;
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp19_ = g_new0 (gdouble, 0);
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->net_stats = (g_free (_data_->self->net_stats), NULL);
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->net_stats = _data_->_tmp19_;
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->net_stats_length1 = 0;
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self->_net_stats_size_ = _data_->self->net_stats_length1;
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp20_ = _data_->self->priv->stats;
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp20__length1 = _data_->self->priv->stats_length1;
#line 3214 "libvirt-machine.c"
	{
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->stat_collection_length1 = 0;
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_stat_collection_size_ = 0;
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->stat_collection = _data_->_tmp20_;
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->stat_collection_length1 = _data_->_tmp20__length1;
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		for (_data_->stat_it = 0; _data_->stat_it < _data_->_tmp20__length1; _data_->stat_it = _data_->stat_it + 1) {
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			memset (&_data_->_tmp21_, 0, sizeof (BoxesLibvirtMachineMachineStat));
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			boxes_libvirt_machine_machine_stat_copy (&_data_->stat_collection[_data_->stat_it], &_data_->_tmp21_);
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_vala1_stat = _data_->_tmp21_;
#line 3232 "libvirt-machine.c"
			{
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp22_ = _data_->self->cpu_stats;
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp22__length1 = _data_->self->cpu_stats_length1;
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp23_ = _data_->_vala1_stat;
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp24_ = _data_->_tmp23_.cpu_guest_percent;
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_vala_array_add3 (&_data_->self->cpu_stats, &_data_->self->cpu_stats_length1, &_data_->self->_cpu_stats_size_, _data_->_tmp24_);
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp25_ = _data_->self->net_stats;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp25__length1 = _data_->self->net_stats_length1;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp26_ = _data_->_vala1_stat;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp27_ = _data_->_tmp26_.net_read;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp28_ = _data_->_vala1_stat;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp29_ = _data_->_tmp28_.net_write;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_vala_array_add4 (&_data_->self->net_stats, &_data_->self->net_stats_length1, &_data_->self->_net_stats_size_, _data_->_tmp27_ + _data_->_tmp29_);
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp30_ = _data_->self->io_stats;
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp30__length1 = _data_->self->io_stats_length1;
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp31_ = _data_->_vala1_stat;
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp32_ = _data_->_tmp31_.disk_read;
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp33_ = _data_->_vala1_stat;
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp34_ = _data_->_tmp33_.disk_write;
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_vala_array_add5 (&_data_->self->io_stats, &_data_->self->io_stats_length1, &_data_->self->_io_stats_size_, _data_->_tmp32_ + _data_->_tmp34_);
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				boxes_libvirt_machine_machine_stat_destroy (&_data_->_vala1_stat);
#line 3274 "libvirt-machine.c"
			}
		}
	}
#line 263 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_signal_emit_by_name (_data_->self, "stats-updated");
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_state_ == 0) {
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 3284 "libvirt-machine.c"
	} else {
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 3288 "libvirt-machine.c"
	}
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_unref (_data_->_async_result);
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 3294 "libvirt-machine.c"
}


static Block22Data* block22_data_ref (Block22Data* _data22_) {
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_atomic_int_inc (&_data22_->_ref_count_);
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return _data22_;
#line 3303 "libvirt-machine.c"
}


static void block22_data_unref (void * _userdata_) {
	Block22Data* _data22_;
	_data22_ = (Block22Data*) _userdata_;
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_atomic_int_dec_and_test (&_data22_->_ref_count_)) {
#line 3312 "libvirt-machine.c"
		BoxesLibvirtMachine * self;
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self = _data22_->self;
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (self);
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_slice_free (Block22Data, _data22_);
#line 3320 "libvirt-machine.c"
	}
}


static void __lambda118_ (Block22Data* _data22_) {
	BoxesLibvirtMachine * self;
#line 282 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = _data22_->self;
#line 282 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data22_->stats_updating = FALSE;
#line 3331 "libvirt-machine.c"
}


static void ___lambda118__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self) {
#line 282 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__lambda118_ (self);
#line 282 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	block22_data_unref (self);
#line 3340 "libvirt-machine.c"
}


static gboolean ___lambda115_ (Block22Data* _data22_) {
	BoxesLibvirtMachine * self;
	gboolean result = FALSE;
	gboolean _tmp0_;
#line 274 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = _data22_->self;
#line 275 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _data22_->stats_updating;
#line 275 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp0_) {
#line 3354 "libvirt-machine.c"
		const gchar* _tmp1_;
		const gchar* _tmp2_;
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp1_ = boxes_collection_item_get_name ((BoxesCollectionItem*) self);
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp2_ = _tmp1_;
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_warning ("libvirt-machine.vala:276: Fetching of stats for '%s' is taking too lon" \
"g. Probably a libvirt bug.", _tmp2_);
#line 278 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		result = TRUE;
#line 278 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return result;
#line 3367 "libvirt-machine.c"
	}
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data22_->stats_updating = TRUE;
#line 282 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_stats (self, ___lambda118__gasync_ready_callback, block22_data_ref (_data22_));
#line 284 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = TRUE;
#line 284 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 3377 "libvirt-machine.c"
}


static gboolean ____lambda115__gsource_func (gpointer self) {
	gboolean result;
	result = ___lambda115_ (self);
#line 274 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 3386 "libvirt-machine.c"
}


static void boxes_libvirt_machine_set_stats_enable (BoxesLibvirtMachine* self, gboolean enable) {
	gboolean _tmp0_;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = enable;
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp0_) {
#line 3398 "libvirt-machine.c"
		Block22Data* _data22_;
		const gchar* _tmp1_;
		const gchar* _tmp2_;
		gchar* _tmp3_;
		gchar* _tmp4_;
		guint _tmp5_;
		GCancellable* _tmp6_;
		guint _tmp7_ = 0U;
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data22_ = g_slice_new0 (Block22Data);
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data22_->_ref_count_ = 1;
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data22_->self = g_object_ref (self);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp1_ = boxes_collection_item_get_name ((BoxesCollectionItem*) self);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp2_ = _tmp1_;
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp3_ = g_strconcat ("enable statistics for ", _tmp2_, NULL);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp4_ = _tmp3_;
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_debug ("libvirt-machine.vala:268: %s", _tmp4_);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_free0 (_tmp4_);
#line 269 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp5_ = self->priv->stats_update_timeout;
#line 269 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_tmp5_ != ((guint) 0)) {
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			block22_data_unref (_data22_);
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data22_ = NULL;
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return;
#line 3435 "libvirt-machine.c"
		}
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp6_ = self->priv->stats_cancellable;
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_cancellable_reset (_tmp6_);
#line 273 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data22_->stats_updating = FALSE;
#line 274 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp7_ = g_timeout_add_seconds_full (G_PRIORITY_DEFAULT, (guint) 1, ____lambda115__gsource_func, block22_data_ref (_data22_), block22_data_unref);
#line 274 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self->priv->stats_update_timeout = _tmp7_;
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block22_data_unref (_data22_);
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data22_ = NULL;
#line 3451 "libvirt-machine.c"
	} else {
		const gchar* _tmp8_;
		const gchar* _tmp9_;
		gchar* _tmp10_;
		gchar* _tmp11_;
		guint _tmp12_;
#line 287 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp8_ = boxes_collection_item_get_name ((BoxesCollectionItem*) self);
#line 287 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp9_ = _tmp8_;
#line 287 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp10_ = g_strconcat ("disable statistics for ", _tmp9_, NULL);
#line 287 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp11_ = _tmp10_;
#line 287 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_debug ("libvirt-machine.vala:287: %s", _tmp11_);
#line 287 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_free0 (_tmp11_);
#line 288 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp12_ = self->priv->stats_update_timeout;
#line 288 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_tmp12_ != ((guint) 0)) {
#line 3474 "libvirt-machine.c"
			GCancellable* _tmp13_;
			guint _tmp14_;
#line 289 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_tmp13_ = self->priv->stats_cancellable;
#line 289 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_cancellable_cancel (_tmp13_);
#line 290 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_tmp14_ = self->priv->stats_update_timeout;
#line 290 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_source_remove (_tmp14_);
#line 3485 "libvirt-machine.c"
		}
#line 292 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self->priv->stats_update_timeout = (guint) 0;
#line 3489 "libvirt-machine.c"
	}
}


static void _g_object_unref0_ (gpointer var) {
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
#line 3497 "libvirt-machine.c"
}


static void _g_list_free__g_object_unref0_ (GList* self) {
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_list_foreach (self, (GFunc) _g_object_unref0_, NULL);
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_list_free (self);
#line 3506 "libvirt-machine.c"
}


static GList* boxes_libvirt_machine_real_get_properties (BoxesMachine* base, BoxesPropertiesPage page, PropertyCreationFlag flags) {
	BoxesLibvirtMachine * self;
	GList* result = NULL;
	BoxesLibvirtMachineProperties* _tmp0_;
	BoxesPropertiesPage _tmp1_;
	PropertyCreationFlag _tmp2_;
	GList* _tmp3_ = NULL;
	GList* list;
	BoxesDisplay* _tmp4_;
	BoxesDisplay* _tmp5_;
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = (BoxesLibvirtMachine*) base;
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->properties;
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = page;
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = flags;
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = boxes_iproperties_provider_get_properties ((BoxesIPropertiesProvider*) _tmp0_, _tmp1_, _tmp2_);
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	list = _tmp3_;
#line 299 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp4_ = boxes_machine_get_display ((BoxesMachine*) self);
#line 299 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp5_ = _tmp4_;
#line 299 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp5_ != NULL) {
#line 3538 "libvirt-machine.c"
		BoxesDisplay* _tmp6_;
		BoxesDisplay* _tmp7_;
		BoxesPropertiesPage _tmp8_;
		PropertyCreationFlag _tmp9_;
		GList* _tmp10_ = NULL;
#line 300 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp6_ = boxes_machine_get_display ((BoxesMachine*) self);
#line 300 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp7_ = _tmp6_;
#line 300 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp8_ = page;
#line 300 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp9_ = flags;
#line 300 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp10_ = boxes_display_get_properties (_tmp7_, _tmp8_, _tmp9_);
#line 300 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		list = g_list_concat (list, _tmp10_);
#line 3556 "libvirt-machine.c"
	}
#line 303 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = list;
#line 303 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 3562 "libvirt-machine.c"
}


gboolean boxes_libvirt_machine_update_display (BoxesLibvirtMachine* self, GError** error) {
	gboolean result = FALSE;
	BoxesDisplay* _tmp0_;
	BoxesDisplay* _tmp1_;
	gboolean created_display;
	BoxesDisplay* _tmp2_;
	BoxesDisplay* _tmp3_;
	GError * _inner_error_ = NULL;
#line 306 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 307 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_update_domain_config (self);
#line 309 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = boxes_machine_get_display ((BoxesMachine*) self);
#line 309 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _tmp0_;
#line 309 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	created_display = _tmp1_ == NULL;
#line 310 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = boxes_machine_get_display ((BoxesMachine*) self);
#line 310 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = _tmp2_;
#line 310 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp3_ == NULL) {
#line 3590 "libvirt-machine.c"
		BoxesDisplay* _tmp4_ = NULL;
		BoxesDisplay* _tmp5_;
		BoxesDisplay* _tmp6_;
#line 311 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp4_ = boxes_libvirt_machine_create_display (self, &_inner_error_);
#line 311 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp5_ = _tmp4_;
#line 311 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_ != NULL) {
#line 311 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_propagate_error (error, _inner_error_);
#line 311 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 3604 "libvirt-machine.c"
		}
#line 311 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp6_ = _tmp5_;
#line 311 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_machine_set_display ((BoxesMachine*) self, _tmp6_);
#line 311 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (_tmp6_);
#line 3612 "libvirt-machine.c"
	}
#line 312 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = created_display;
#line 312 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 3618 "libvirt-machine.c"
}


static const gchar* string_to_string (const gchar* self) {
	const gchar* result = NULL;
#line 1264 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, NULL);
#line 1265 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	result = self;
#line 1265 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	return result;
#line 3630 "libvirt-machine.c"
}


static BoxesDisplay* boxes_libvirt_machine_create_display (BoxesLibvirtMachine* self, GError** error) {
	BoxesDisplay* result = NULL;
	gchar* type = NULL;
	gchar* port = NULL;
	gchar* socket = NULL;
	gchar* host = NULL;
	GVirConfigDomain* _tmp0_;
	gchar* _tmp1_ = NULL;
	gchar* xmldoc;
	const gchar* _tmp2_;
	gchar* _tmp3_ = NULL;
	gchar* _tmp4_;
	const gchar* _tmp5_;
	const gchar* _tmp6_;
	const gchar* _tmp7_ = NULL;
	gchar* _tmp8_ = NULL;
	gchar* _tmp9_;
	gchar* _tmp10_ = NULL;
	gchar* _tmp11_;
	gchar* _tmp12_;
	const gchar* _tmp13_;
	const gchar* _tmp14_;
	const gchar* _tmp15_ = NULL;
	gchar* _tmp16_ = NULL;
	gchar* _tmp17_;
	gchar* _tmp18_ = NULL;
	gchar* _tmp19_;
	gchar* _tmp20_;
	const gchar* _tmp21_;
	const gchar* _tmp22_;
	const gchar* _tmp23_ = NULL;
	gchar* _tmp24_ = NULL;
	gchar* _tmp25_;
	gchar* _tmp26_ = NULL;
	gchar* _tmp27_;
	gchar* _tmp28_;
	gboolean _tmp29_ = FALSE;
	const gchar* _tmp30_;
	gboolean _tmp32_;
	const gchar* _tmp34_;
	const gchar* _tmp35_;
	GQuark _tmp37_ = 0U;
#line 327 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	static GQuark _tmp36_label0 = 0;
#line 327 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	static GQuark _tmp36_label1 = 0;
#line 3680 "libvirt-machine.c"
	GError * _inner_error_ = NULL;
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 318 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->_domain_config;
#line 318 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = gvir_config_object_to_xml ((GVirConfigObject*) _tmp0_);
#line 318 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	xmldoc = _tmp1_;
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = xmldoc;
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = boxes_extract_xpath (_tmp2_, "string(/domain/devices/graphics/@type)", TRUE, &_inner_error_);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp4_ = _tmp3_;
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_->domain == BOXES_ERROR) {
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_propagate_error (error, _inner_error_);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (xmldoc);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (host);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (socket);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (port);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (type);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return NULL;
#line 3714 "libvirt-machine.c"
		} else {
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (xmldoc);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (host);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (socket);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (port);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (type);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_clear_error (&_inner_error_);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return NULL;
#line 3732 "libvirt-machine.c"
		}
	}
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (type);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	type = _tmp4_;
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp5_ = xmldoc;
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp6_ = type;
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp7_ = string_to_string (_tmp6_);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp8_ = g_strconcat ("string(/domain/devices/graphics[@type='", _tmp7_, "']/@port)", NULL);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp9_ = _tmp8_;
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp10_ = boxes_extract_xpath (_tmp5_, _tmp9_, FALSE, &_inner_error_);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp11_ = _tmp10_;
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (_tmp9_);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp12_ = _tmp11_;
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_->domain == BOXES_ERROR) {
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_propagate_error (error, _inner_error_);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (xmldoc);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (host);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (socket);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (port);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (type);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return NULL;
#line 3775 "libvirt-machine.c"
		} else {
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (xmldoc);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (host);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (socket);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (port);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (type);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_clear_error (&_inner_error_);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return NULL;
#line 3793 "libvirt-machine.c"
		}
	}
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (port);
#line 320 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	port = _tmp12_;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp13_ = xmldoc;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp14_ = type;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp15_ = string_to_string (_tmp14_);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp16_ = g_strconcat ("string(/domain/devices/graphics[@type='", _tmp15_, "']/@socket)", NULL);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp17_ = _tmp16_;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp18_ = boxes_extract_xpath (_tmp13_, _tmp17_, FALSE, &_inner_error_);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp19_ = _tmp18_;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (_tmp17_);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp20_ = _tmp19_;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_->domain == BOXES_ERROR) {
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_propagate_error (error, _inner_error_);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (xmldoc);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (host);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (socket);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (port);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (type);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return NULL;
#line 3836 "libvirt-machine.c"
		} else {
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (xmldoc);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (host);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (socket);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (port);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (type);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_clear_error (&_inner_error_);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return NULL;
#line 3854 "libvirt-machine.c"
		}
	}
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (socket);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	socket = _tmp20_;
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp21_ = xmldoc;
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp22_ = type;
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp23_ = string_to_string (_tmp22_);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp24_ = g_strconcat ("string(/domain/devices/graphics[@type='", _tmp23_, "']/@listen)", NULL);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp25_ = _tmp24_;
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp26_ = boxes_extract_xpath (_tmp21_, _tmp25_, FALSE, &_inner_error_);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp27_ = _tmp26_;
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (_tmp25_);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp28_ = _tmp27_;
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_->domain == BOXES_ERROR) {
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_propagate_error (error, _inner_error_);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (xmldoc);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (host);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (socket);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (port);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (type);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return NULL;
#line 3897 "libvirt-machine.c"
		} else {
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (xmldoc);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (host);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (socket);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (port);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_free0 (type);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_clear_error (&_inner_error_);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return NULL;
#line 3915 "libvirt-machine.c"
		}
	}
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (host);
#line 322 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	host = _tmp28_;
#line 324 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp30_ = host;
#line 324 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp30_ == NULL) {
#line 324 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp29_ = TRUE;
#line 3928 "libvirt-machine.c"
	} else {
		const gchar* _tmp31_;
#line 324 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp31_ = host;
#line 324 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp29_ = g_strcmp0 (_tmp31_, "") == 0;
#line 3935 "libvirt-machine.c"
	}
#line 324 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp32_ = _tmp29_;
#line 324 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp32_) {
#line 3941 "libvirt-machine.c"
		gchar* _tmp33_;
#line 325 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp33_ = g_strdup ("localhost");
#line 325 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_free0 (host);
#line 325 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		host = _tmp33_;
#line 3949 "libvirt-machine.c"
	}
#line 327 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp34_ = type;
#line 327 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp35_ = _tmp34_;
#line 327 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp37_ = (NULL == _tmp35_) ? 0 : g_quark_from_string (_tmp35_);
#line 327 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp37_ == ((0 != _tmp36_label0) ? _tmp36_label0 : (_tmp36_label0 = g_quark_from_static_string ("spice")))) {
#line 327 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		switch (0) {
#line 3961 "libvirt-machine.c"
			default:
			{
				BoxesBoxConfig* _tmp38_;
				const gchar* _tmp39_;
				const gchar* _tmp40_;
				gint _tmp41_ = 0;
				BoxesSpiceDisplay* _tmp42_;
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp38_ = ((BoxesMachine*) self)->config;
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp39_ = host;
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp40_ = port;
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp41_ = atoi (_tmp40_);
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp42_ = boxes_spice_display_new (_tmp38_, _tmp39_, _tmp41_, 0);
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				result = (BoxesDisplay*) _tmp42_;
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (xmldoc);
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (host);
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (socket);
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (port);
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (type);
#line 329 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				return result;
#line 3993 "libvirt-machine.c"
			}
		}
	} else if (_tmp37_ == ((0 != _tmp36_label1) ? _tmp36_label1 : (_tmp36_label1 = g_quark_from_static_string ("vnc")))) {
#line 327 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		switch (0) {
#line 3999 "libvirt-machine.c"
			default:
			{
				BoxesBoxConfig* _tmp43_;
				const gchar* _tmp44_;
				const gchar* _tmp45_;
				gint _tmp46_ = 0;
				BoxesVncDisplay* _tmp47_;
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp43_ = ((BoxesMachine*) self)->config;
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp44_ = host;
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp45_ = port;
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp46_ = atoi (_tmp45_);
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp47_ = boxes_vnc_display_new (_tmp43_, _tmp44_, _tmp46_);
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				result = (BoxesDisplay*) _tmp47_;
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (xmldoc);
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (host);
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (socket);
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (port);
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (type);
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				return result;
#line 4031 "libvirt-machine.c"
			}
		}
	} else {
#line 327 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		switch (0) {
#line 4037 "libvirt-machine.c"
			default:
			{
				const gchar* _tmp48_;
				gchar* _tmp49_;
				gchar* _tmp50_;
				GError* _tmp51_;
				GError* _tmp52_;
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp48_ = type;
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp49_ = g_strconcat ("unsupported display of type ", _tmp48_, NULL);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp50_ = _tmp49_;
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp51_ = g_error_new_literal (BOXES_ERROR, BOXES_ERROR_INVALID, _tmp50_);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp52_ = _tmp51_;
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (_tmp50_);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_inner_error_ = _tmp52_;
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				if (_inner_error_->domain == BOXES_ERROR) {
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_propagate_error (error, _inner_error_);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (xmldoc);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (host);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (socket);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (port);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (type);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					return NULL;
#line 4075 "libvirt-machine.c"
				} else {
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (xmldoc);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (host);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (socket);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (port);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (type);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_clear_error (&_inner_error_);
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					return NULL;
#line 4093 "libvirt-machine.c"
				}
			}
		}
	}
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (xmldoc);
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (host);
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (socket);
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (port);
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (type);
#line 4108 "libvirt-machine.c"
}


static void boxes_libvirt_machine_real_take_screenshot_data_free (gpointer _data) {
	BoxesLibvirtMachineTakeScreenshotData* _data_;
	_data_ = _data;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->result);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_slice_free (BoxesLibvirtMachineTakeScreenshotData, _data_);
#line 4121 "libvirt-machine.c"
}


static void boxes_libvirt_machine_real_take_screenshot (BoxesMachine* base, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesLibvirtMachine * self;
	BoxesLibvirtMachineTakeScreenshotData* _data_;
	BoxesLibvirtMachine* _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = (BoxesLibvirtMachine*) base;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_slice_new0 (BoxesLibvirtMachineTakeScreenshotData);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_libvirt_machine_real_take_screenshot);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_libvirt_machine_real_take_screenshot_data_free);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self = _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_real_take_screenshot_co (_data_);
#line 4143 "libvirt-machine.c"
}


static GdkPixbuf* boxes_libvirt_machine_real_take_screenshot_finish (BoxesMachine* base, GAsyncResult* _res_, GError** error) {
	GdkPixbuf* result;
	BoxesLibvirtMachineTakeScreenshotData* _data_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (_res_), error)) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return NULL;
#line 4154 "libvirt-machine.c"
	}
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = _data_->result;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->result = NULL;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 4164 "libvirt-machine.c"
}


static Block23Data* block23_data_ref (Block23Data* _data23_) {
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_atomic_int_inc (&_data23_->_ref_count_);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return _data23_;
#line 4173 "libvirt-machine.c"
}


static void block23_data_unref (void * _userdata_) {
	Block23Data* _data23_;
	_data23_ = (Block23Data*) _userdata_;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_atomic_int_dec_and_test (&_data23_->_ref_count_)) {
#line 4182 "libvirt-machine.c"
		BoxesLibvirtMachine * self;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self = _data23_->self;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (_data23_->stream);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (self);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_slice_free (Block23Data, _data23_);
#line 4192 "libvirt-machine.c"
	}
}


static void boxes_libvirt_machine_take_screenshot_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesLibvirtMachineTakeScreenshotData* _data_;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = _user_data_;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_source_object_ = source_object;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_res_ = _res_;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_real_take_screenshot_co (_data_);
#line 4207 "libvirt-machine.c"
}


static void __lambda125_ (Block23Data* _data23_, GError** error) {
	BoxesLibvirtMachine * self;
	GVirDomain* _tmp0_;
	GVirStream* _tmp1_;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_;
	GError * _inner_error_ = NULL;
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = _data23_->self;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->_domain;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _data23_->stream;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = gvir_domain_screenshot (_tmp0_, _tmp1_, (guint) 0, (guint) 0, &_inner_error_);
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = _tmp2_;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (_tmp3_);
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_propagate_error (error, _inner_error_);
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 4236 "libvirt-machine.c"
	}
}


static void ___lambda125__boxes_run_in_thread_func (gpointer self, GError** error) {
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__lambda125_ (self, error);
#line 4244 "libvirt-machine.c"
}


static gboolean boxes_libvirt_machine_real_take_screenshot_co (BoxesLibvirtMachineTakeScreenshotData* _data_) {
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (_data_->_state_) {
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 0:
#line 4253 "libvirt-machine.c"
		goto _state_0;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 1:
#line 4257 "libvirt-machine.c"
		goto _state_1;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 2:
#line 4261 "libvirt-machine.c"
		goto _state_2;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 3:
#line 4265 "libvirt-machine.c"
		goto _state_3;
		default:
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_assert_not_reached ();
#line 4270 "libvirt-machine.c"
	}
	_state_0:
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data23_ = g_slice_new0 (Block23Data);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data23_->_ref_count_ = 1;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data23_->self = g_object_ref (_data_->self);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data23_->_async_data_ = _data_;
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->state = GVIR_DOMAIN_STATE_NONE;
#line 4283 "libvirt-machine.c"
	{
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp0_ = _data_->self->priv->_domain;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_state_ = 1;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gvir_domain_get_info_async (_data_->_tmp0_, NULL, boxes_libvirt_machine_take_screenshot_ready, _data_);
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 4293 "libvirt-machine.c"
		_state_1:
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp1_ = NULL;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp1_ = gvir_domain_get_info_finish (_data_->_tmp0_, _data_->_res_, &_data_->_inner_error_);
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp2_ = _data_->_tmp1_;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 4303 "libvirt-machine.c"
			goto __catch17_g_error;
		}
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp3_ = _data_->_tmp2_;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp4_ = _data_->_tmp3_->state;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->state = _data_->_tmp4_;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		__vala_GVirDomainInfo_free0 (_data_->_tmp3_);
#line 4314 "libvirt-machine.c"
	}
	goto __finally17;
	__catch17_g_error:
	{
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_error_ = _data_->_inner_error_;
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_inner_error_ = NULL;
#line 344 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp5_ = boxes_collection_item_get_name ((BoxesCollectionItem*) _data_->self);
#line 344 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp6_ = _data_->_tmp5_;
#line 344 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_warning ("libvirt-machine.vala:344: Failed to get information on '%s'", _data_->_tmp6_);
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (_data_->_error_);
#line 4331 "libvirt-machine.c"
	}
	__finally17:
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_inner_error_ != NULL) {
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_error_free (_data_->_inner_error_);
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block23_data_unref (_data_->_data23_);
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data23_ = NULL;
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 4348 "libvirt-machine.c"
		} else {
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 4352 "libvirt-machine.c"
		}
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 341 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 4358 "libvirt-machine.c"
	}
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp9_ = _data_->state;
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_tmp9_ != GVIR_DOMAIN_STATE_RUNNING) {
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp10_ = _data_->state;
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp8_ = _data_->_tmp10_ != GVIR_DOMAIN_STATE_PAUSED;
#line 4368 "libvirt-machine.c"
	} else {
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp8_ = FALSE;
#line 4372 "libvirt-machine.c"
	}
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp11_ = _data_->_tmp8_;
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_tmp11_) {
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp12_ = _data_->state;
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp7_ = _data_->_tmp12_ != GVIR_DOMAIN_STATE_PMSUSPENDED;
#line 4382 "libvirt-machine.c"
	} else {
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp7_ = FALSE;
#line 4386 "libvirt-machine.c"
	}
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp13_ = _data_->_tmp7_;
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_tmp13_) {
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->result = NULL;
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block23_data_unref (_data_->_data23_);
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data23_ = NULL;
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 4402 "libvirt-machine.c"
		} else {
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 4406 "libvirt-machine.c"
		}
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 4412 "libvirt-machine.c"
	}
#line 350 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp14_ = _data_->self->priv->_connection;
#line 350 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp15_ = NULL;
#line 350 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp15_ = gvir_connection_get_stream (_data_->_tmp14_, (guint) 0);
#line 350 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data23_->stream = _data_->_tmp15_;
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_state_ = 2;
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_run_in_thread (___lambda125__boxes_run_in_thread_func, block23_data_ref (_data_->_data23_), block23_data_unref, NULL, boxes_libvirt_machine_take_screenshot_ready, _data_);
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 4428 "libvirt-machine.c"
	_state_2:
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_run_in_thread_finish (_data_->_res_, &_data_->_inner_error_);
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_inner_error_ != NULL) {
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_error_free (_data_->_inner_error_);
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block23_data_unref (_data_->_data23_);
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data23_ = NULL;
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 4446 "libvirt-machine.c"
		} else {
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 4450 "libvirt-machine.c"
		}
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 4456 "libvirt-machine.c"
	}
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp16_ = gdk_pixbuf_loader_new ();
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->loader = _data_->_tmp16_;
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp17_ = _data_->_data23_->stream;
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp18_ = NULL;
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp18_ = g_io_stream_get_input_stream ((GIOStream*) _data_->_tmp17_);
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp19_ = _g_object_ref0 (_data_->_tmp18_);
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->input_stream = _data_->_tmp19_;
#line 357 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp20_ = NULL;
#line 357 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp20_ = g_new0 (guint8, 65535);
#line 357 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->buffer = _data_->_tmp20_;
#line 357 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->buffer_length1 = 65535;
#line 357 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_buffer_size_ = _data_->buffer_length1;
#line 358 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->length = (gssize) 0;
#line 4484 "libvirt-machine.c"
	{
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp21_ = TRUE;
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		while (TRUE) {
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp22_ = _data_->_tmp21_;
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (!_data_->_tmp22_) {
#line 362 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp23_ = _data_->length;
#line 362 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				if (!(_data_->_tmp23_ > ((gssize) 0))) {
#line 362 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					break;
#line 4500 "libvirt-machine.c"
				}
			}
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp21_ = FALSE;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp24_ = _data_->input_stream;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp25_ = _data_->buffer;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp25__length1 = _data_->buffer_length1;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_state_ = 3;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_input_stream_read_async (_data_->_tmp24_, (void*) _data_->_tmp25_, (gsize) _data_->_tmp25__length1, G_PRIORITY_DEFAULT, NULL, boxes_libvirt_machine_take_screenshot_ready, _data_);
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 4517 "libvirt-machine.c"
			_state_3:
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp26_ = 0L;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp26_ = g_input_stream_read_finish (_data_->_tmp24_, _data_->_res_, &_data_->_inner_error_);
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp27_ = _data_->_tmp26_;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_inner_error_ != NULL) {
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_error_free (_data_->_inner_error_);
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->buffer = (g_free (_data_->buffer), NULL);
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_object_unref0 (_data_->input_stream);
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_object_unref0 (_data_->loader);
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				block23_data_unref (_data_->_data23_);
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_data23_ = NULL;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				if (_data_->_state_ == 0) {
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 4545 "libvirt-machine.c"
				} else {
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_simple_async_result_complete (_data_->_async_result);
#line 4549 "libvirt-machine.c"
				}
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_object_unref (_data_->_async_result);
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				return FALSE;
#line 4555 "libvirt-machine.c"
			}
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->length = _data_->_tmp27_;
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp28_ = _data_->loader;
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp29_ = _data_->buffer;
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp29__length1 = _data_->buffer_length1;
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp30_ = _data_->length;
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			gdk_pixbuf_loader_write (_data_->_tmp28_, _data_->_tmp29_ + 0, (gsize) (_data_->_tmp30_ - 0), &_data_->_inner_error_);
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_inner_error_ != NULL) {
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_error_free (_data_->_inner_error_);
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->buffer = (g_free (_data_->buffer), NULL);
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_object_unref0 (_data_->input_stream);
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_object_unref0 (_data_->loader);
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				block23_data_unref (_data_->_data23_);
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_data23_ = NULL;
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				if (_data_->_state_ == 0) {
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 4589 "libvirt-machine.c"
				} else {
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_simple_async_result_complete (_data_->_async_result);
#line 4593 "libvirt-machine.c"
				}
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_object_unref (_data_->_async_result);
#line 361 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				return FALSE;
#line 4599 "libvirt-machine.c"
			}
		}
	}
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp31_ = _data_->loader;
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	gdk_pixbuf_loader_close (_data_->_tmp31_, &_data_->_inner_error_);
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_inner_error_ != NULL) {
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_error_free (_data_->_inner_error_);
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->buffer = (g_free (_data_->buffer), NULL);
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (_data_->input_stream);
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (_data_->loader);
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block23_data_unref (_data_->_data23_);
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_data23_ = NULL;
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 4627 "libvirt-machine.c"
		} else {
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 4631 "libvirt-machine.c"
		}
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 4637 "libvirt-machine.c"
	}
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp32_ = _data_->loader;
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp33_ = NULL;
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp33_ = gdk_pixbuf_loader_get_pixbuf (_data_->_tmp32_);
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp34_ = _g_object_ref0 (_data_->_tmp33_);
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->result = _data_->_tmp34_;
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->buffer = (g_free (_data_->buffer), NULL);
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->input_stream);
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->loader);
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	block23_data_unref (_data_->_data23_);
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data23_ = NULL;
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_state_ == 0) {
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 4663 "libvirt-machine.c"
	} else {
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 4667 "libvirt-machine.c"
	}
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_unref (_data_->_async_result);
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->buffer = (g_free (_data_->buffer), NULL);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->input_stream);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->loader);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	block23_data_unref (_data_->_data23_);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_data23_ = NULL;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_state_ == 0) {
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 4687 "libvirt-machine.c"
	} else {
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 4691 "libvirt-machine.c"
	}
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_unref (_data_->_async_result);
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 4697 "libvirt-machine.c"
}


static Block24Data* block24_data_ref (Block24Data* _data24_) {
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_atomic_int_inc (&_data24_->_ref_count_);
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return _data24_;
#line 4706 "libvirt-machine.c"
}


static void block24_data_unref (void * _userdata_) {
	Block24Data* _data24_;
	_data24_ = (Block24Data*) _userdata_;
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_atomic_int_dec_and_test (&_data24_->_ref_count_)) {
#line 4715 "libvirt-machine.c"
		BoxesLibvirtMachine * self;
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self = _data24_->self;
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (_data24_->domain);
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (self);
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_slice_free (Block24Data, _data24_);
#line 4725 "libvirt-machine.c"
	}
}


static void ___lambda126_ (Block24Data* _data24_, GError** error) {
	BoxesLibvirtMachine * self;
	GVirStorageVol* _tmp6_;
	GError * _inner_error_ = NULL;
#line 383 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = _data24_->self;
#line 4736 "libvirt-machine.c"
	{
		GVirDomain* _tmp0_;
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp0_ = _data24_->domain;
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gvir_domain_delete (_tmp0_, (guint) GVIR_DOMAIN_DELETE_SAVED_STATE, &_inner_error_);
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_ != NULL) {
#line 4745 "libvirt-machine.c"
			goto __catch18_g_error;
		}
	}
	goto __finally18;
	__catch18_g_error:
	{
		GError* err = NULL;
		GError* _tmp1_;
		const gchar* _tmp2_;
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		err = _inner_error_;
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_inner_error_ = NULL;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp1_ = err;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp2_ = _tmp1_->message;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_warning ("libvirt-machine.vala:388: %s", _tmp2_);
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (err);
#line 4767 "libvirt-machine.c"
	}
	__finally18:
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_propagate_error (error, _inner_error_);
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 4776 "libvirt-machine.c"
	}
	{
		GVirDomain* _tmp3_;
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp3_ = _data24_->domain;
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gvir_domain_stop (_tmp3_, (guint) 0, &_inner_error_);
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_ != NULL) {
#line 4786 "libvirt-machine.c"
			goto __catch19_g_error;
		}
	}
	goto __finally19;
	__catch19_g_error:
	{
		GError* err = NULL;
		GError* _tmp4_;
		const gchar* _tmp5_;
#line 391 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		err = _inner_error_;
#line 391 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_inner_error_ = NULL;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp4_ = err;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp5_ = _tmp4_->message;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_debug ("libvirt-machine.vala:395: %s", _tmp5_);
#line 391 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (err);
#line 4808 "libvirt-machine.c"
	}
	__finally19:
#line 391 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 391 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_propagate_error (error, _inner_error_);
#line 391 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 4817 "libvirt-machine.c"
	}
#line 399 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp6_ = self->priv->_storage_volume;
#line 399 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp6_ != NULL) {
#line 4823 "libvirt-machine.c"
		{
			GVirStorageVol* _tmp7_;
#line 401 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_tmp7_ = self->priv->_storage_volume;
#line 401 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			gvir_storage_vol_delete (_tmp7_, (guint) 0, &_inner_error_);
#line 401 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_inner_error_ != NULL) {
#line 4832 "libvirt-machine.c"
				goto __catch20_g_error;
			}
		}
		goto __finally20;
		__catch20_g_error:
		{
			GError* err = NULL;
			GError* _tmp8_;
			const gchar* _tmp9_;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			err = _inner_error_;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_inner_error_ = NULL;
#line 403 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_tmp8_ = err;
#line 403 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_tmp9_ = _tmp8_->message;
#line 403 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_warning ("libvirt-machine.vala:403: %s", _tmp9_);
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_error_free0 (err);
#line 4854 "libvirt-machine.c"
		}
		__finally20:
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_ != NULL) {
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_propagate_error (error, _inner_error_);
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return;
#line 4863 "libvirt-machine.c"
		}
	}
}


static void ____lambda126__boxes_run_in_thread_func (gpointer self, GError** error) {
#line 383 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	___lambda126_ (self, error);
#line 4872 "libvirt-machine.c"
}


static void boxes_libvirt_machine_real_delete (BoxesMachine* base, gboolean by_user) {
	BoxesLibvirtMachine * self;
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gboolean _tmp4_;
	guint _tmp5_;
	gboolean _tmp7_;
#line 368 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = (BoxesLibvirtMachine*) base;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = boxes_collection_item_get_name ((BoxesCollectionItem*) self);
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _tmp0_;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = g_strconcat ("delete libvirt machine: ", _tmp1_, NULL);
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = _tmp2_;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_debug ("libvirt-machine.vala:369: %s", _tmp3_);
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_free0 (_tmp3_);
#line 370 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp4_ = by_user;
#line 370 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (boxes_libvirt_machine_parent_class)->delete (G_TYPE_CHECK_INSTANCE_CAST (self, BOXES_TYPE_MACHINE, BoxesMachine), _tmp4_);
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_set_stats_enable (self, FALSE);
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp5_ = self->priv->shutdown_timeout;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp5_ != ((guint) 0)) {
#line 4909 "libvirt-machine.c"
		guint _tmp6_;
#line 374 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp6_ = self->priv->shutdown_timeout;
#line 374 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_source_remove (_tmp6_);
#line 375 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self->priv->shutdown_timeout = (guint) 0;
#line 4917 "libvirt-machine.c"
	}
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp7_ = by_user;
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp7_) {
#line 4923 "libvirt-machine.c"
		Block24Data* _data24_;
		GVirDomain* _tmp8_;
		GVirDomain* _tmp9_;
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data24_ = g_slice_new0 (Block24Data);
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data24_->_ref_count_ = 1;
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data24_->self = g_object_ref (self);
#line 379 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp8_ = self->priv->_domain;
#line 379 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp9_ = _g_object_ref0 (_tmp8_);
#line 379 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data24_->domain = _tmp9_;
#line 383 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_run_in_thread (____lambda126__boxes_run_in_thread_func, block24_data_ref (_data24_), block24_data_unref, NULL, NULL, NULL);
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		block24_data_unref (_data24_);
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data24_ = NULL;
#line 4945 "libvirt-machine.c"
	}
}


static void boxes_libvirt_machine_real_save_real_data_free (gpointer _data) {
	BoxesLibvirtMachineSaveRealData* _data_;
	_data_ = _data;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_slice_free (BoxesLibvirtMachineSaveRealData, _data_);
#line 4957 "libvirt-machine.c"
}


static void boxes_libvirt_machine_real_save_real (BoxesMachine* base, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesLibvirtMachine * self;
	BoxesLibvirtMachineSaveRealData* _data_;
	BoxesLibvirtMachine* _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = (BoxesLibvirtMachine*) base;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_slice_new0 (BoxesLibvirtMachineSaveRealData);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_libvirt_machine_real_save_real);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_libvirt_machine_real_save_real_data_free);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self = _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_real_save_real_co (_data_);
#line 4979 "libvirt-machine.c"
}


static void boxes_libvirt_machine_real_save_real_finish (BoxesMachine* base, GAsyncResult* _res_, GError** error) {
	BoxesLibvirtMachineSaveRealData* _data_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (_res_), error)) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 4989 "libvirt-machine.c"
	}
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 4993 "libvirt-machine.c"
}


static void boxes_libvirt_machine_save_real_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesLibvirtMachineSaveRealData* _data_;
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = _user_data_;
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_source_object_ = source_object;
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_res_ = _res_;
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_real_save_real_co (_data_);
#line 5007 "libvirt-machine.c"
}


static gboolean boxes_libvirt_machine_real_save_real_co (BoxesLibvirtMachineSaveRealData* _data_) {
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (_data_->_state_) {
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 0:
#line 5016 "libvirt-machine.c"
		goto _state_0;
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 1:
#line 5020 "libvirt-machine.c"
		goto _state_1;
		default:
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_assert_not_reached ();
#line 5025 "libvirt-machine.c"
	}
	_state_0:
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp0_ = _data_->self->priv->_domain;
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_state_ = 1;
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	gvir_domain_save_async (_data_->_tmp0_, (guint) 0, NULL, boxes_libvirt_machine_save_real_ready, _data_);
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 5036 "libvirt-machine.c"
	_state_1:
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	gvir_domain_save_finish (_data_->_tmp0_, _data_->_res_, &_data_->_inner_error_);
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_inner_error_ != NULL) {
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_error_free (_data_->_inner_error_);
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5050 "libvirt-machine.c"
		} else {
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 5054 "libvirt-machine.c"
		}
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 5060 "libvirt-machine.c"
	}
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_state_ == 0) {
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5066 "libvirt-machine.c"
	} else {
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 5070 "libvirt-machine.c"
	}
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_unref (_data_->_async_result);
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 5076 "libvirt-machine.c"
}


static void boxes_libvirt_machine_suspend_data_free (gpointer _data) {
	BoxesLibvirtMachineSuspendData* _data_;
	_data_ = _data;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_slice_free (BoxesLibvirtMachineSuspendData, _data_);
#line 5087 "libvirt-machine.c"
}


void boxes_libvirt_machine_suspend (BoxesLibvirtMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesLibvirtMachineSuspendData* _data_;
	BoxesLibvirtMachine* _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_slice_new0 (BoxesLibvirtMachineSuspendData);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_libvirt_machine_suspend);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_libvirt_machine_suspend_data_free);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self = _tmp0_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_suspend_co (_data_);
#line 5106 "libvirt-machine.c"
}


void boxes_libvirt_machine_suspend_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_, GError** error) {
	BoxesLibvirtMachineSuspendData* _data_;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (_res_), error)) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 5116 "libvirt-machine.c"
	}
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 5120 "libvirt-machine.c"
}


static void boxes_libvirt_machine_suspend_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesLibvirtMachineSuspendData* _data_;
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = _user_data_;
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_source_object_ = source_object;
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_res_ = _res_;
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_suspend_co (_data_);
#line 5134 "libvirt-machine.c"
}


static gboolean boxes_libvirt_machine_suspend_co (BoxesLibvirtMachineSuspendData* _data_) {
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (_data_->_state_) {
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 0:
#line 5143 "libvirt-machine.c"
		goto _state_0;
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 1:
#line 5147 "libvirt-machine.c"
		goto _state_1;
		default:
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_assert_not_reached ();
#line 5152 "libvirt-machine.c"
	}
	_state_0:
#line 414 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp0_ = boxes_libvirt_machine_get_save_on_quit (_data_->self);
#line 414 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp1_ = _data_->_tmp0_;
#line 414 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_tmp1_) {
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_state_ = 1;
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_machine_save ((BoxesMachine*) _data_->self, boxes_libvirt_machine_suspend_ready, _data_);
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 5167 "libvirt-machine.c"
		_state_1:
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_machine_save_finish ((BoxesMachine*) _data_->self, _data_->_res_, &_data_->_inner_error_);
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_error_free (_data_->_inner_error_);
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_state_ == 0) {
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5181 "libvirt-machine.c"
			} else {
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete (_data_->_async_result);
#line 5185 "libvirt-machine.c"
			}
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_object_unref (_data_->_async_result);
#line 415 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 5191 "libvirt-machine.c"
		}
	} else {
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp2_ = _data_->self->priv->_domain;
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gvir_domain_suspend (_data_->_tmp2_, &_data_->_inner_error_);
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_error_free (_data_->_inner_error_);
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_state_ == 0) {
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5208 "libvirt-machine.c"
			} else {
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete (_data_->_async_result);
#line 5212 "libvirt-machine.c"
			}
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_object_unref (_data_->_async_result);
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 5218 "libvirt-machine.c"
		}
	}
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_state_ == 0) {
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5225 "libvirt-machine.c"
	} else {
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 5229 "libvirt-machine.c"
	}
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_unref (_data_->_async_result);
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 5235 "libvirt-machine.c"
}


void boxes_libvirt_machine_force_shutdown (BoxesLibvirtMachine* self, gboolean confirm) {
	gboolean _tmp0_;
	const gchar* _tmp9_;
	const gchar* _tmp10_;
	GError * _inner_error_ = NULL;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 421 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = confirm;
#line 421 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_tmp0_) {
#line 5250 "libvirt-machine.c"
		BoxesApp* _tmp1_;
		GtkApplicationWindow* _tmp2_;
		const gchar* _tmp3_ = NULL;
		GtkMessageDialog* _tmp4_;
		GtkMessageDialog* dialog;
		GtkMessageDialog* _tmp5_;
		gint _tmp6_ = 0;
		gint response;
		GtkMessageDialog* _tmp7_;
		gint _tmp8_;
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp1_ = boxes_app_app;
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp2_ = _tmp1_->window;
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp3_ = _ ("When you force shutdown, the box may lose data.");
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp4_ = (GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) _tmp2_, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK_CANCEL, _tmp3_);
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_ref_sink (_tmp4_);
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		dialog = _tmp4_;
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp5_ = dialog;
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp6_ = gtk_dialog_run ((GtkDialog*) _tmp5_);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		response = _tmp6_;
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp7_ = dialog;
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gtk_widget_destroy ((GtkWidget*) _tmp7_);
#line 430 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp8_ = response;
#line 430 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_tmp8_ != ((gint) GTK_RESPONSE_OK)) {
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_g_object_unref0 (dialog);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return;
#line 5291 "libvirt-machine.c"
		}
#line 421 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_object_unref0 (dialog);
#line 5295 "libvirt-machine.c"
	}
#line 434 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp9_ = boxes_collection_item_get_name ((BoxesCollectionItem*) self);
#line 434 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp10_ = _tmp9_;
#line 434 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_debug ("libvirt-machine.vala:434: Force shutdown '%s'..", _tmp10_);
#line 5303 "libvirt-machine.c"
	{
		GVirDomain* _tmp11_;
#line 436 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		self->priv->force_stopped = TRUE;
#line 437 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp11_ = self->priv->_domain;
#line 437 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gvir_domain_stop (_tmp11_, (guint) 0, &_inner_error_);
#line 437 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_ != NULL) {
#line 5314 "libvirt-machine.c"
			goto __catch21_g_error;
		}
	}
	goto __finally21;
	__catch21_g_error:
	{
		GError* _error_ = NULL;
		GVirDomain* _tmp12_;
		const gchar* _tmp13_ = NULL;
		GError* _tmp14_;
		const gchar* _tmp15_;
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_error_ = _inner_error_;
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_inner_error_ = NULL;
#line 439 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp12_ = self->priv->_domain;
#line 439 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp13_ = gvir_domain_get_name (_tmp12_);
#line 439 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp14_ = _error_;
#line 439 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp15_ = _tmp14_->message;
#line 439 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_warning ("libvirt-machine.vala:439: Failed to shutdown '%s': %s", _tmp13_, _tmp15_);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (_error_);
#line 5342 "libvirt-machine.c"
	}
	__finally21:
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_clear_error (&_inner_error_);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 5353 "libvirt-machine.c"
	}
}


void boxes_libvirt_machine_try_shutdown (BoxesLibvirtMachine* self) {
	GError * _inner_error_ = NULL;
#line 443 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 5362 "libvirt-machine.c"
	{
		GVirDomain* _tmp0_;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp0_ = self->priv->_domain;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gvir_domain_shutdown (_tmp0_, (guint) 0, &_inner_error_);
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_inner_error_ != NULL) {
#line 5371 "libvirt-machine.c"
			goto __catch22_g_error;
		}
	}
	goto __finally22;
	__catch22_g_error:
	{
		GError* _error_ = NULL;
		GVirDomain* _tmp1_;
		const gchar* _tmp2_ = NULL;
		GError* _tmp3_;
		const gchar* _tmp4_;
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_error_ = _inner_error_;
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_inner_error_ = NULL;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp1_ = self->priv->_domain;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp2_ = gvir_domain_get_name (_tmp1_);
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp3_ = _error_;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_tmp4_ = _tmp3_->message;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_warning ("libvirt-machine.vala:447: Failed to reboot '%s': %s", _tmp2_, _tmp4_);
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_g_error_free0 (_error_);
#line 5399 "libvirt-machine.c"
	}
	__finally22:
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_inner_error_ != NULL) {
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_clear_error (&_inner_error_);
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 5410 "libvirt-machine.c"
	}
}


GVirDomainDisk* boxes_libvirt_machine_get_domain_disk (BoxesLibvirtMachine* self, GError** error) {
	GVirDomainDisk* result = NULL;
	GVirDomainDisk* _tmp0_;
	GVirDomainDisk* disk;
	GVirConfigDomain* _tmp1_;
	GList* _tmp2_ = NULL;
	GError * _inner_error_ = NULL;
#line 451 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (NULL, gvir_domain_disk_get_type ()) ? ((GVirDomainDisk*) NULL) : NULL);
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	disk = _tmp0_;
#line 454 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = self->priv->_domain_config;
#line 454 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = gvir_config_domain_get_devices (_tmp1_);
#line 5432 "libvirt-machine.c"
	{
		GList* device_config_collection = NULL;
		GList* device_config_it = NULL;
#line 454 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		device_config_collection = _tmp2_;
#line 454 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		for (device_config_it = device_config_collection; device_config_it != NULL; device_config_it = device_config_it->next) {
#line 5440 "libvirt-machine.c"
			GVirConfigDomainDevice* _tmp3_;
			GVirConfigDomainDevice* device_config = NULL;
#line 454 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_tmp3_ = _g_object_ref0 ((GVirConfigDomainDevice*) device_config_it->data);
#line 454 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			device_config = _tmp3_;
#line 5447 "libvirt-machine.c"
			{
				GVirConfigDomainDevice* _tmp4_;
#line 455 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp4_ = device_config;
#line 455 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp4_, gvir_config_domain_disk_get_type ())) {
#line 5454 "libvirt-machine.c"
					GVirConfigDomainDevice* _tmp5_;
					GVirConfigDomainDisk* _tmp6_;
					GVirConfigDomainDisk* disk_config;
					GVirConfigDomainDisk* _tmp7_;
					GVirConfigDomainDiskGuestDeviceType _tmp8_ = 0;
					GVirConfigDomainDiskGuestDeviceType disk_type;
					const gchar* _tmp9_ = NULL;
					GVirStorageVol* _tmp10_;
					const gchar* _tmp13_;
					gchar* _tmp14_;
					gchar* storage_volume_path;
					gboolean _tmp15_ = FALSE;
					GVirConfigDomainDiskGuestDeviceType _tmp16_;
					gboolean _tmp20_;
#line 456 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp5_ = device_config;
#line 456 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp6_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, gvir_config_domain_disk_get_type ()) ? ((GVirConfigDomainDisk*) _tmp5_) : NULL);
#line 456 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					disk_config = _tmp6_;
#line 457 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp7_ = disk_config;
#line 457 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp8_ = gvir_config_domain_disk_get_guest_device_type (_tmp7_);
#line 457 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					disk_type = _tmp8_;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp10_ = self->priv->_storage_volume;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					if (_tmp10_ != NULL) {
#line 5485 "libvirt-machine.c"
						GVirStorageVol* _tmp11_;
						const gchar* _tmp12_ = NULL;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp11_ = self->priv->_storage_volume;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp12_ = gvir_storage_vol_get_path (_tmp11_, &_inner_error_);
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp9_ = _tmp12_;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						if (_inner_error_ != NULL) {
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
							g_propagate_error (error, _inner_error_);
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
							_g_object_unref0 (disk_config);
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
							_g_object_unref0 (device_config);
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
							__g_list_free__g_object_unref0_0 (device_config_collection);
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
							_g_object_unref0 (disk);
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
							return NULL;
#line 5508 "libvirt-machine.c"
						}
					} else {
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp9_ = NULL;
#line 5513 "libvirt-machine.c"
					}
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp13_ = _tmp9_;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp14_ = g_strdup (_tmp13_);
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					storage_volume_path = _tmp14_;
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp16_ = disk_type;
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					if (_tmp16_ == GVIR_CONFIG_DOMAIN_DISK_GUEST_DEVICE_DISK) {
#line 5525 "libvirt-machine.c"
						GVirConfigDomainDisk* _tmp17_;
						const gchar* _tmp18_ = NULL;
						const gchar* _tmp19_;
#line 462 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp17_ = disk_config;
#line 462 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp18_ = gvir_config_domain_disk_get_source (_tmp17_);
#line 462 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp19_ = storage_volume_path;
#line 462 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp15_ = g_strcmp0 (_tmp18_, _tmp19_) == 0;
#line 5537 "libvirt-machine.c"
					} else {
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp15_ = FALSE;
#line 5541 "libvirt-machine.c"
					}
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp20_ = _tmp15_;
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					if (_tmp20_) {
#line 5547 "libvirt-machine.c"
						GVirDomain* _tmp21_;
						GVirConfigDomainDevice* _tmp22_;
						GObject* _tmp23_ = NULL;
						GObject* _tmp24_;
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp21_ = self->priv->_domain;
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp22_ = device_config;
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp23_ = g_object_new (gvir_domain_disk_get_type (), "domain", _tmp21_, "config", _tmp22_, NULL);
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_tmp24_ = G_IS_INITIALLY_UNOWNED (_tmp23_) ? g_object_ref_sink (_tmp23_) : _tmp23_;
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_g_object_unref0 (disk);
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						disk = G_TYPE_CHECK_INSTANCE_TYPE (_tmp24_, gvir_domain_disk_get_type ()) ? ((GVirDomainDisk*) _tmp24_) : NULL;
#line 467 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_g_free0 (storage_volume_path);
#line 467 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_g_object_unref0 (disk_config);
#line 467 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						_g_object_unref0 (device_config);
#line 467 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
						break;
#line 5572 "libvirt-machine.c"
					}
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_free0 (storage_volume_path);
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_object_unref0 (disk_config);
#line 5578 "libvirt-machine.c"
				}
#line 455 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_object_unref0 (device_config);
#line 5582 "libvirt-machine.c"
			}
		}
#line 455 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		__g_list_free__g_object_unref0_0 (device_config_collection);
#line 5587 "libvirt-machine.c"
	}
#line 472 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = disk;
#line 472 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 5593 "libvirt-machine.c"
}


static GVirDomainInterface* boxes_libvirt_machine_get_domain_network_interface (BoxesLibvirtMachine* self, GError** error) {
	GVirDomainInterface* result = NULL;
	GVirDomainInterface* _tmp0_;
	GVirDomainInterface* net;
	GVirConfigDomain* _tmp1_;
	GList* _tmp2_ = NULL;
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 476 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (NULL, gvir_domain_interface_get_type ()) ? ((GVirDomainInterface*) NULL) : NULL);
#line 476 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	net = _tmp0_;
#line 479 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = self->priv->_domain_config;
#line 479 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = gvir_config_domain_get_devices (_tmp1_);
#line 5613 "libvirt-machine.c"
	{
		GList* device_config_collection = NULL;
		GList* device_config_it = NULL;
#line 479 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		device_config_collection = _tmp2_;
#line 479 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		for (device_config_it = device_config_collection; device_config_it != NULL; device_config_it = device_config_it->next) {
#line 5621 "libvirt-machine.c"
			GVirConfigDomainDevice* _tmp3_;
			GVirConfigDomainDevice* device_config = NULL;
#line 479 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_tmp3_ = _g_object_ref0 ((GVirConfigDomainDevice*) device_config_it->data);
#line 479 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			device_config = _tmp3_;
#line 5628 "libvirt-machine.c"
			{
				GVirConfigDomainDevice* _tmp4_;
#line 480 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_tmp4_ = device_config;
#line 480 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp4_, gvir_config_domain_interface_get_type ())) {
#line 5635 "libvirt-machine.c"
					GVirDomain* _tmp5_;
					GVirConfigDomainDevice* _tmp6_;
					GObject* _tmp7_ = NULL;
					GObject* _tmp8_;
#line 481 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp5_ = self->priv->_domain;
#line 481 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp6_ = device_config;
#line 481 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp7_ = g_object_new (gvir_domain_interface_get_type (), "domain", _tmp5_, "config", _tmp6_, NULL);
#line 481 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_tmp8_ = G_IS_INITIALLY_UNOWNED (_tmp7_) ? g_object_ref_sink (_tmp7_) : _tmp7_;
#line 481 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_object_unref0 (net);
#line 481 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					net = G_TYPE_CHECK_INSTANCE_TYPE (_tmp8_, gvir_domain_interface_get_type ()) ? ((GVirDomainInterface*) _tmp8_) : NULL;
#line 484 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					_g_object_unref0 (device_config);
#line 484 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					break;
#line 5656 "libvirt-machine.c"
				}
#line 480 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_object_unref0 (device_config);
#line 5660 "libvirt-machine.c"
			}
		}
#line 480 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		__g_list_free__g_object_unref0_0 (device_config_collection);
#line 5665 "libvirt-machine.c"
	}
#line 488 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = net;
#line 488 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 5671 "libvirt-machine.c"
}


static void boxes_libvirt_machine_start_data_free (gpointer _data) {
	BoxesLibvirtMachineStartData* _data_;
	_data_ = _data;
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (_data_->self);
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_slice_free (BoxesLibvirtMachineStartData, _data_);
#line 5682 "libvirt-machine.c"
}


void boxes_libvirt_machine_start (BoxesLibvirtMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesLibvirtMachineStartData* _data_;
	BoxesLibvirtMachine* _tmp0_;
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_slice_new0 (BoxesLibvirtMachineStartData);
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_libvirt_machine_start);
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_libvirt_machine_start_data_free);
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->self = _tmp0_;
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_start_co (_data_);
#line 5701 "libvirt-machine.c"
}


void boxes_libvirt_machine_start_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_, GError** error) {
	BoxesLibvirtMachineStartData* _data_;
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (_res_), error)) {
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return;
#line 5711 "libvirt-machine.c"
	}
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 5715 "libvirt-machine.c"
}


static void boxes_libvirt_machine_start_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesLibvirtMachineStartData* _data_;
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_ = _user_data_;
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_source_object_ = source_object;
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_res_ = _res_;
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_start_co (_data_);
#line 5729 "libvirt-machine.c"
}


static gboolean boxes_libvirt_machine_start_co (BoxesLibvirtMachineStartData* _data_) {
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (_data_->_state_) {
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 0:
#line 5738 "libvirt-machine.c"
		goto _state_0;
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 1:
#line 5742 "libvirt-machine.c"
		goto _state_1;
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 2:
#line 5746 "libvirt-machine.c"
		goto _state_2;
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case 3:
#line 5750 "libvirt-machine.c"
		goto _state_3;
		default:
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_assert_not_reached ();
#line 5755 "libvirt-machine.c"
	}
	_state_0:
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp0_ = boxes_machine_get_state ((BoxesMachine*) _data_->self);
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp1_ = _data_->_tmp0_;
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_tmp1_ == BOXES_MACHINE_MACHINE_STATE_RUNNING) {
#line 493 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_state_ == 0) {
#line 493 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5768 "libvirt-machine.c"
		} else {
#line 493 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 5772 "libvirt-machine.c"
		}
#line 493 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_object_unref (_data_->_async_result);
#line 493 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 5778 "libvirt-machine.c"
	}
#line 495 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp2_ = boxes_machine_get_state ((BoxesMachine*) _data_->self);
#line 495 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_data_->_tmp3_ = _data_->_tmp2_;
#line 495 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_tmp3_ == BOXES_MACHINE_MACHINE_STATE_PAUSED) {
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp4_ = _data_->self->priv->_domain;
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_state_ = 1;
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gvir_domain_resume_async (_data_->_tmp4_, NULL, boxes_libvirt_machine_start_ready, _data_);
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		return FALSE;
#line 5794 "libvirt-machine.c"
		_state_1:
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		gvir_domain_resume_finish (_data_->_tmp4_, _data_->_res_, &_data_->_inner_error_);
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_inner_error_ != NULL) {
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_error_free (_data_->_inner_error_);
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_state_ == 0) {
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5808 "libvirt-machine.c"
			} else {
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_complete (_data_->_async_result);
#line 5812 "libvirt-machine.c"
			}
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			g_object_unref (_data_->_async_result);
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 5818 "libvirt-machine.c"
		}
	} else {
#line 497 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp5_ = boxes_machine_get_state ((BoxesMachine*) _data_->self);
#line 497 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		_data_->_tmp6_ = _data_->_tmp5_;
#line 497 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		if (_data_->_tmp6_ == BOXES_MACHINE_MACHINE_STATE_SLEEPING) {
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp7_ = _data_->self->priv->_domain;
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_state_ = 2;
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			gvir_domain_wakeup_async (_data_->_tmp7_, (guint) 0, NULL, boxes_libvirt_machine_start_ready, _data_);
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 5835 "libvirt-machine.c"
			_state_2:
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			gvir_domain_wakeup_finish (_data_->_tmp7_, _data_->_res_, &_data_->_inner_error_);
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_inner_error_ != NULL) {
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_error_free (_data_->_inner_error_);
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				if (_data_->_state_ == 0) {
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5849 "libvirt-machine.c"
				} else {
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_simple_async_result_complete (_data_->_async_result);
#line 5853 "libvirt-machine.c"
				}
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_object_unref (_data_->_async_result);
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				return FALSE;
#line 5859 "libvirt-machine.c"
			}
		} else {
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp8_ = _data_->self->priv->_domain;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp9_ = FALSE;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp9_ = gvir_domain_get_saved (_data_->_tmp8_);
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_tmp9_) {
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp10_ = NULL;
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp10_ = _ ("Restoring %s from disk");
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp11_ = boxes_collection_item_get_name ((BoxesCollectionItem*) _data_->self);
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp12_ = _data_->_tmp11_;
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp13_ = NULL;
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp13_ = g_strdup_printf (_data_->_tmp10_, _data_->_tmp12_);
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp14_ = _data_->_tmp13_;
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				boxes_machine_set_status ((BoxesMachine*) _data_->self, _data_->_tmp14_);
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (_data_->_tmp14_);
#line 5888 "libvirt-machine.c"
			} else {
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp15_ = NULL;
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp15_ = _ ("Starting %s");
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp16_ = boxes_collection_item_get_name ((BoxesCollectionItem*) _data_->self);
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp17_ = _data_->_tmp16_;
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp18_ = NULL;
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp18_ = g_strdup_printf (_data_->_tmp15_, _data_->_tmp17_);
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_data_->_tmp19_ = _data_->_tmp18_;
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				boxes_machine_set_status ((BoxesMachine*) _data_->self, _data_->_tmp19_);
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				_g_free0 (_data_->_tmp19_);
#line 5908 "libvirt-machine.c"
			}
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_tmp20_ = _data_->self->priv->_domain;
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			_data_->_state_ = 3;
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			gvir_domain_start_async (_data_->_tmp20_, (guint) 0, NULL, boxes_libvirt_machine_start_ready, _data_);
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			return FALSE;
#line 5918 "libvirt-machine.c"
			_state_3:
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			gvir_domain_start_finish (_data_->_tmp20_, _data_->_res_, &_data_->_inner_error_);
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
			if (_data_->_inner_error_ != NULL) {
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_simple_async_result_set_from_error (_data_->_async_result, _data_->_inner_error_);
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_error_free (_data_->_inner_error_);
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				if (_data_->_state_ == 0) {
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5932 "libvirt-machine.c"
				} else {
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
					g_simple_async_result_complete (_data_->_async_result);
#line 5936 "libvirt-machine.c"
				}
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				g_object_unref (_data_->_async_result);
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
				return FALSE;
#line 5942 "libvirt-machine.c"
			}
		}
	}
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	if (_data_->_state_ == 0) {
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 5950 "libvirt-machine.c"
	} else {
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 5954 "libvirt-machine.c"
	}
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_unref (_data_->_async_result);
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return FALSE;
#line 5960 "libvirt-machine.c"
}


GVirDomain* boxes_libvirt_machine_get_domain (BoxesLibvirtMachine* self) {
	GVirDomain* result;
	GVirDomain* _tmp0_;
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->_domain;
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = _tmp0_;
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 5975 "libvirt-machine.c"
}


void boxes_libvirt_machine_set_domain (BoxesLibvirtMachine* self, GVirDomain* value) {
	GVirDomain* _tmp0_;
	GVirDomain* _tmp1_;
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = value;
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->_domain);
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->_domain = _tmp1_;
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_notify ((GObject *) self, "domain");
#line 5994 "libvirt-machine.c"
}


GVirConfigDomain* boxes_libvirt_machine_get_domain_config (BoxesLibvirtMachine* self) {
	GVirConfigDomain* result;
	GVirConfigDomain* _tmp0_;
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->_domain_config;
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = _tmp0_;
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 6009 "libvirt-machine.c"
}


void boxes_libvirt_machine_set_domain_config (BoxesLibvirtMachine* self, GVirConfigDomain* value) {
	GVirConfigDomain* _tmp0_;
	GVirConfigDomain* _tmp1_;
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = value;
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->_domain_config);
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->_domain_config = _tmp1_;
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_notify ((GObject *) self, "domain-config");
#line 6028 "libvirt-machine.c"
}


GVirConnection* boxes_libvirt_machine_get_connection (BoxesLibvirtMachine* self) {
	GVirConnection* result;
	GVirConnection* _tmp0_;
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->_connection;
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = _tmp0_;
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 6043 "libvirt-machine.c"
}


void boxes_libvirt_machine_set_connection (BoxesLibvirtMachine* self, GVirConnection* value) {
	GVirConnection* _tmp0_;
	GVirConnection* _tmp1_;
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = value;
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->_connection);
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->_connection = _tmp1_;
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_notify ((GObject *) self, "connection");
#line 6062 "libvirt-machine.c"
}


GVirStorageVol* boxes_libvirt_machine_get_storage_volume (BoxesLibvirtMachine* self) {
	GVirStorageVol* result;
	GVirStorageVol* _tmp0_;
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->_storage_volume;
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = _tmp0_;
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 6077 "libvirt-machine.c"
}


void boxes_libvirt_machine_set_storage_volume (BoxesLibvirtMachine* self, GVirStorageVol* value) {
	GVirStorageVol* _tmp0_;
	GVirStorageVol* _tmp1_;
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = value;
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->_storage_volume);
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->_storage_volume = _tmp1_;
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_notify ((GObject *) self, "storage-volume");
#line 6096 "libvirt-machine.c"
}


BoxesVMCreator* boxes_libvirt_machine_get_vm_creator (BoxesLibvirtMachine* self) {
	BoxesVMCreator* result;
	BoxesVMCreator* _tmp0_;
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = self->priv->_vm_creator;
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = _tmp0_;
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 6111 "libvirt-machine.c"
}


static gpointer _boxes_vm_creator_ref0 (gpointer self) {
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return self ? boxes_vm_creator_ref (self) : NULL;
#line 6118 "libvirt-machine.c"
}


void boxes_libvirt_machine_set_vm_creator (BoxesLibvirtMachine* self, BoxesVMCreator* value) {
	BoxesVMCreator* _tmp0_;
	BoxesVMCreator* _tmp1_;
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = value;
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = _boxes_vm_creator_ref0 (_tmp0_);
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_boxes_vm_creator_unref0 (self->priv->_vm_creator);
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->_vm_creator = _tmp1_;
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_notify ((GObject *) self, "vm-creator");
#line 6137 "libvirt-machine.c"
}


gboolean boxes_libvirt_machine_get_save_on_quit (BoxesLibvirtMachine* self) {
	gboolean result;
	BoxesCollectionSource* _tmp0_;
	gboolean _tmp1_ = FALSE;
#line 15 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 15 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = ((BoxesMachine*) self)->source;
#line 15 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = boxes_iconfig_get_boolean ((BoxesIConfig*) _tmp0_, "source", "save-on-quit", FALSE);
#line 15 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	result = _tmp1_;
#line 15 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return result;
#line 6155 "libvirt-machine.c"
}


void boxes_libvirt_machine_set_save_on_quit (BoxesLibvirtMachine* self, gboolean value) {
	BoxesCollectionSource* _tmp0_;
	gboolean _tmp1_;
#line 16 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_return_if_fail (self != NULL);
#line 16 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = ((BoxesMachine*) self)->source;
#line 16 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = value;
#line 16 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_iconfig_set_boolean ((BoxesIConfig*) _tmp0_, "source", "save-on-quit", _tmp1_);
#line 16 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_notify ((GObject *) self, "save-on-quit");
#line 6172 "libvirt-machine.c"
}


static GObject * boxes_libvirt_machine_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	BoxesLibvirtMachine * self;
	BoxesLibvirtMachineMachineStat* _tmp0_ = NULL;
	GCancellable* _tmp1_;
#line 60 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	parent_class = G_OBJECT_CLASS (boxes_libvirt_machine_parent_class);
#line 60 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 60 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachine);
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = g_new0 (BoxesLibvirtMachineMachineStat, BOXES_LIBVIRT_MACHINE_STATS_SIZE);
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->stats = (_vala_BoxesLibvirtMachineMachineStat_array_free (self->priv->stats, self->priv->stats_length1), NULL);
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->stats = _tmp0_;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->stats_length1 = BOXES_LIBVIRT_MACHINE_STATS_SIZE;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->_stats_size_ = self->priv->stats_length1;
#line 62 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = g_cancellable_new ();
#line 62 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->stats_cancellable);
#line 62 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->stats_cancellable = _tmp1_;
#line 63 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_machine_set_can_save ((BoxesMachine*) self, TRUE);
#line 60 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return obj;
#line 6208 "libvirt-machine.c"
}


static GVirDomainDiskStats* _vala_GVirDomainDiskStats_copy (GVirDomainDiskStats* self) {
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return g_boxed_copy (gvir_domain_disk_stats_get_type (), self);
#line 6215 "libvirt-machine.c"
}


static gpointer __vala_GVirDomainDiskStats_copy0 (gpointer self) {
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return self ? _vala_GVirDomainDiskStats_copy (self) : NULL;
#line 6222 "libvirt-machine.c"
}


static GVirDomainInterfaceStats* _vala_GVirDomainInterfaceStats_copy (GVirDomainInterfaceStats* self) {
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return g_boxed_copy (gvir_domain_interface_stats_get_type (), self);
#line 6229 "libvirt-machine.c"
}


static gpointer __vala_GVirDomainInterfaceStats_copy0 (gpointer self) {
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return self ? _vala_GVirDomainInterfaceStats_copy (self) : NULL;
#line 6236 "libvirt-machine.c"
}


static void boxes_libvirt_machine_machine_stat_copy (const BoxesLibvirtMachineMachineStat* self, BoxesLibvirtMachineMachineStat* dest) {
	gint64 _tmp0_;
	gdouble _tmp1_;
	gdouble _tmp2_;
	gdouble _tmp3_;
	gdouble _tmp4_;
	GVirDomainDiskStats* _tmp5_;
	GVirDomainDiskStats* _tmp6_;
	gdouble _tmp7_;
	gdouble _tmp8_;
	GVirDomainInterfaceStats* _tmp9_;
	GVirDomainInterfaceStats* _tmp10_;
	gdouble _tmp11_;
	gdouble _tmp12_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp0_ = (*self).timestamp;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).timestamp = _tmp0_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp1_ = (*self).cpu_time;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).cpu_time = _tmp1_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp2_ = (*self).cpu_time_abs;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).cpu_time_abs = _tmp2_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp3_ = (*self).cpu_guest_percent;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).cpu_guest_percent = _tmp3_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp4_ = (*self).memory_percent;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).memory_percent = _tmp4_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp5_ = (*self).disk;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp6_ = __vala_GVirDomainDiskStats_copy0 (_tmp5_);
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainDiskStats_free0 ((*dest).disk);
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).disk = _tmp6_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp7_ = (*self).disk_read;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).disk_read = _tmp7_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp8_ = (*self).disk_write;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).disk_write = _tmp8_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp9_ = (*self).net;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp10_ = __vala_GVirDomainInterfaceStats_copy0 (_tmp9_);
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainInterfaceStats_free0 ((*dest).net);
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).net = _tmp10_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp11_ = (*self).net_read;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).net_read = _tmp11_;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_tmp12_ = (*self).net_write;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	(*dest).net_write = _tmp12_;
#line 6306 "libvirt-machine.c"
}


static void boxes_libvirt_machine_machine_stat_destroy (BoxesLibvirtMachineMachineStat* self) {
#line 42 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainDiskStats_free0 ((*self).disk);
#line 45 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	__vala_GVirDomainInterfaceStats_free0 ((*self).net);
#line 6315 "libvirt-machine.c"
}


static BoxesLibvirtMachineMachineStat* boxes_libvirt_machine_machine_stat_dup (const BoxesLibvirtMachineMachineStat* self) {
	BoxesLibvirtMachineMachineStat* dup;
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	dup = g_new0 (BoxesLibvirtMachineMachineStat, 1);
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_machine_stat_copy (self, dup);
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	return dup;
#line 6327 "libvirt-machine.c"
}


static void boxes_libvirt_machine_machine_stat_free (BoxesLibvirtMachineMachineStat* self) {
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_machine_stat_destroy (self);
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_free (self);
#line 6336 "libvirt-machine.c"
}


static GType boxes_libvirt_machine_machine_stat_get_type (void) {
	static volatile gsize boxes_libvirt_machine_machine_stat_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_libvirt_machine_machine_stat_type_id__volatile)) {
		GType boxes_libvirt_machine_machine_stat_type_id;
		boxes_libvirt_machine_machine_stat_type_id = g_boxed_type_register_static ("BoxesLibvirtMachineMachineStat", (GBoxedCopyFunc) boxes_libvirt_machine_machine_stat_dup, (GBoxedFreeFunc) boxes_libvirt_machine_machine_stat_free);
		g_once_init_leave (&boxes_libvirt_machine_machine_stat_type_id__volatile, boxes_libvirt_machine_machine_stat_type_id);
	}
	return boxes_libvirt_machine_machine_stat_type_id__volatile;
}


static void boxes_libvirt_machine_class_init (BoxesLibvirtMachineClass * klass) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	boxes_libvirt_machine_parent_class = g_type_class_peek_parent (klass);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_type_class_add_private (klass, sizeof (BoxesLibvirtMachinePrivate));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (klass)->disconnect_display = boxes_libvirt_machine_real_disconnect_display;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (klass)->connect_display = boxes_libvirt_machine_real_connect_display;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (klass)->connect_display_finish = boxes_libvirt_machine_real_connect_display_finish;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (klass)->get_properties = boxes_libvirt_machine_real_get_properties;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (klass)->take_screenshot = boxes_libvirt_machine_real_take_screenshot;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (klass)->take_screenshot_finish = boxes_libvirt_machine_real_take_screenshot_finish;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (klass)->delete = boxes_libvirt_machine_real_delete;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (klass)->save_real = boxes_libvirt_machine_real_save_real;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	BOXES_MACHINE_CLASS (klass)->save_real_finish = boxes_libvirt_machine_real_save_real_finish;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_boxes_libvirt_machine_get_property;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_boxes_libvirt_machine_set_property;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	G_OBJECT_CLASS (klass)->constructor = boxes_libvirt_machine_constructor;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	G_OBJECT_CLASS (klass)->finalize = boxes_libvirt_machine_finalize;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_LIBVIRT_MACHINE_DOMAIN, g_param_spec_object ("domain", "domain", "domain", gvir_domain_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_LIBVIRT_MACHINE_DOMAIN_CONFIG, g_param_spec_object ("domain-config", "domain-config", "domain-config", gvir_config_domain_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_LIBVIRT_MACHINE_CONNECTION, g_param_spec_object ("connection", "connection", "connection", gvir_connection_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_LIBVIRT_MACHINE_STORAGE_VOLUME, g_param_spec_object ("storage-volume", "storage-volume", "storage-volume", gvir_storage_vol_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_LIBVIRT_MACHINE_VM_CREATOR, boxes_param_spec_vm_creator ("vm-creator", "vm-creator", "vm-creator", BOXES_TYPE_VM_CREATOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_LIBVIRT_MACHINE_SAVE_ON_QUIT, g_param_spec_boolean ("save-on-quit", "save-on-quit", "save-on-quit", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	g_signal_new ("stats_updated", BOXES_TYPE_LIBVIRT_MACHINE, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 6396 "libvirt-machine.c"
}


static void boxes_libvirt_machine_instance_init (BoxesLibvirtMachine * self) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv = BOXES_LIBVIRT_MACHINE_GET_PRIVATE (self);
#line 6403 "libvirt-machine.c"
}


static void boxes_libvirt_machine_finalize (GObject* obj) {
	BoxesLibvirtMachine * self;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachine);
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->_domain);
#line 7 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->_domain_config);
#line 8 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->_connection);
#line 9 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->_storage_volume);
#line 10 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_boxes_vm_creator_unref0 (self->priv->_vm_creator);
#line 12 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->properties);
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	_g_object_unref0 (self->priv->stats_cancellable);
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->priv->stats = (_vala_BoxesLibvirtMachineMachineStat_array_free (self->priv->stats, self->priv->stats_length1), NULL);
#line 230 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->cpu_stats = (g_free (self->cpu_stats), NULL);
#line 231 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->io_stats = (g_free (self->io_stats), NULL);
#line 232 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	self->net_stats = (g_free (self->net_stats), NULL);
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	G_OBJECT_CLASS (boxes_libvirt_machine_parent_class)->finalize (obj);
#line 6435 "libvirt-machine.c"
}


GType boxes_libvirt_machine_get_type (void) {
	static volatile gsize boxes_libvirt_machine_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_libvirt_machine_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (BoxesLibvirtMachineClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) boxes_libvirt_machine_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BoxesLibvirtMachine), 0, (GInstanceInitFunc) boxes_libvirt_machine_instance_init, NULL };
		GType boxes_libvirt_machine_type_id;
		boxes_libvirt_machine_type_id = g_type_register_static (BOXES_TYPE_MACHINE, "BoxesLibvirtMachine", &g_define_type_info, 0);
		g_once_init_leave (&boxes_libvirt_machine_type_id__volatile, boxes_libvirt_machine_type_id);
	}
	return boxes_libvirt_machine_type_id__volatile;
}


static void _vala_boxes_libvirt_machine_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	BoxesLibvirtMachine * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachine);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (property_id) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_DOMAIN:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_value_set_object (value, boxes_libvirt_machine_get_domain (self));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_DOMAIN_CONFIG:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_value_set_object (value, boxes_libvirt_machine_get_domain_config (self));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_CONNECTION:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_value_set_object (value, boxes_libvirt_machine_get_connection (self));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_STORAGE_VOLUME:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_value_set_object (value, boxes_libvirt_machine_get_storage_volume (self));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_VM_CREATOR:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_value_set_vm_creator (value, boxes_libvirt_machine_get_vm_creator (self));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_SAVE_ON_QUIT:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		g_value_set_boolean (value, boxes_libvirt_machine_get_save_on_quit (self));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 6492 "libvirt-machine.c"
		default:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 6498 "libvirt-machine.c"
	}
}


static void _vala_boxes_libvirt_machine_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	BoxesLibvirtMachine * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachine);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
	switch (property_id) {
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_DOMAIN:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_domain (self, g_value_get_object (value));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_DOMAIN_CONFIG:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_domain_config (self, g_value_get_object (value));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_CONNECTION:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_connection (self, g_value_get_object (value));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_STORAGE_VOLUME:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_storage_volume (self, g_value_get_object (value));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_VM_CREATOR:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_vm_creator (self, boxes_value_get_vm_creator (value));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		case BOXES_LIBVIRT_MACHINE_SAVE_ON_QUIT:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		boxes_libvirt_machine_set_save_on_quit (self, g_value_get_boolean (value));
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 6544 "libvirt-machine.c"
		default:
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 5 "/extra-data/checkout/gnome/gnome-boxes/src/libvirt-machine.vala"
		break;
#line 6550 "libvirt-machine.c"
	}
}



