/* wizard.c generated by valac 0.18.0.79-04df6, the Vala compiler
 * generated from wizard.vala, do not modify */

/* This file is part of GNOME Boxes. License: LGPLv2+*/

#include <glib.h>
#include <glib-object.h>
#include <clutter/clutter.h>
#include <clutter-gtk/clutter-gtk.h>
#include <gtk/gtk.h>
#include <libcommon.h>
#include <gio/gio.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <libxml/uri.h>
#include <osinfo/osinfo.h>
#include <libgd/gd.h>
#include <libvirt-gconfig/libvirt-gconfig.h>
#include <libvirt-gobject/libvirt-gobject.h>


#define BOXES_TYPE_WIZARD_PAGE (boxes_wizard_page_get_type ())

#define BOXES_TYPE_UI (boxes_ui_get_type ())
#define BOXES_UI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_UI, BoxesUI))
#define BOXES_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_UI, BoxesUIClass))
#define BOXES_IS_UI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_UI))
#define BOXES_IS_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_UI))
#define BOXES_UI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_UI, BoxesUIClass))

typedef struct _BoxesUI BoxesUI;
typedef struct _BoxesUIClass BoxesUIClass;
typedef struct _BoxesUIPrivate BoxesUIPrivate;

#define BOXES_TYPE_UI_STATE (boxes_ui_state_get_type ())

#define BOXES_TYPE_WIZARD (boxes_wizard_get_type ())
#define BOXES_WIZARD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_WIZARD, BoxesWizard))
#define BOXES_WIZARD_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_WIZARD, BoxesWizardClass))
#define BOXES_IS_WIZARD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_WIZARD))
#define BOXES_IS_WIZARD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_WIZARD))
#define BOXES_WIZARD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_WIZARD, BoxesWizardClass))

typedef struct _BoxesWizard BoxesWizard;
typedef struct _BoxesWizardClass BoxesWizardClass;
typedef struct _BoxesWizardPrivate BoxesWizardPrivate;

#define BOXES_TYPE_WIZARD_SOURCE (boxes_wizard_source_get_type ())
#define BOXES_WIZARD_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_WIZARD_SOURCE, BoxesWizardSource))
#define BOXES_WIZARD_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_WIZARD_SOURCE, BoxesWizardSourceClass))
#define BOXES_IS_WIZARD_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_WIZARD_SOURCE))
#define BOXES_IS_WIZARD_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_WIZARD_SOURCE))
#define BOXES_WIZARD_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_WIZARD_SOURCE, BoxesWizardSourceClass))

typedef struct _BoxesWizardSource BoxesWizardSource;
typedef struct _BoxesWizardSourceClass BoxesWizardSourceClass;

#define BOXES_WIZARD_TYPE_WIZARD_SUMMARY (boxes_wizard_wizard_summary_get_type ())
#define BOXES_WIZARD_WIZARD_SUMMARY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_WIZARD_TYPE_WIZARD_SUMMARY, BoxesWizardWizardSummary))
#define BOXES_WIZARD_WIZARD_SUMMARY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_WIZARD_TYPE_WIZARD_SUMMARY, BoxesWizardWizardSummaryClass))
#define BOXES_WIZARD_IS_WIZARD_SUMMARY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_WIZARD_TYPE_WIZARD_SUMMARY))
#define BOXES_WIZARD_IS_WIZARD_SUMMARY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_WIZARD_TYPE_WIZARD_SUMMARY))
#define BOXES_WIZARD_WIZARD_SUMMARY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_WIZARD_TYPE_WIZARD_SUMMARY, BoxesWizardWizardSummaryClass))

typedef struct _BoxesWizardWizardSummary BoxesWizardWizardSummary;
typedef struct _BoxesWizardWizardSummaryClass BoxesWizardWizardSummaryClass;

#define BOXES_TYPE_MEDIA_MANAGER (boxes_media_manager_get_type ())
#define BOXES_MEDIA_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_MEDIA_MANAGER, BoxesMediaManager))
#define BOXES_MEDIA_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_MEDIA_MANAGER, BoxesMediaManagerClass))
#define BOXES_IS_MEDIA_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_MEDIA_MANAGER))
#define BOXES_IS_MEDIA_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_MEDIA_MANAGER))
#define BOXES_MEDIA_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_MEDIA_MANAGER, BoxesMediaManagerClass))

typedef struct _BoxesMediaManager BoxesMediaManager;
typedef struct _BoxesMediaManagerClass BoxesMediaManagerClass;

#define BOXES_TYPE_VM_CREATOR (boxes_vm_creator_get_type ())
#define BOXES_VM_CREATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_VM_CREATOR, BoxesVMCreator))
#define BOXES_VM_CREATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_VM_CREATOR, BoxesVMCreatorClass))
#define BOXES_IS_VM_CREATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_VM_CREATOR))
#define BOXES_IS_VM_CREATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_VM_CREATOR))
#define BOXES_VM_CREATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_VM_CREATOR, BoxesVMCreatorClass))

typedef struct _BoxesVMCreator BoxesVMCreator;
typedef struct _BoxesVMCreatorClass BoxesVMCreatorClass;

#define BOXES_TYPE_COLLECTION_ITEM (boxes_collection_item_get_type ())
#define BOXES_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItem))
#define BOXES_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))
#define BOXES_IS_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_IS_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_COLLECTION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))

typedef struct _BoxesCollectionItem BoxesCollectionItem;
typedef struct _BoxesCollectionItemClass BoxesCollectionItemClass;

#define BOXES_TYPE_MACHINE (boxes_machine_get_type ())
#define BOXES_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_MACHINE, BoxesMachine))
#define BOXES_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_MACHINE, BoxesMachineClass))
#define BOXES_IS_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_MACHINE))
#define BOXES_IS_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_MACHINE))
#define BOXES_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_MACHINE, BoxesMachineClass))

typedef struct _BoxesMachine BoxesMachine;
typedef struct _BoxesMachineClass BoxesMachineClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_ptr_array_unref0(var) ((var == NULL) ? NULL : (var = (g_ptr_array_unref (var), NULL)))
#define _boxes_vm_creator_unref0(var) ((var == NULL) ? NULL : (var = (boxes_vm_creator_unref (var), NULL)))

#define BOXES_TYPE_SOURCE_PAGE (boxes_source_page_get_type ())

#define BOXES_TYPE_APP (boxes_app_get_type ())
#define BOXES_APP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_APP, BoxesApp))
#define BOXES_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_APP, BoxesAppClass))
#define BOXES_IS_APP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_APP))
#define BOXES_IS_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_APP))
#define BOXES_APP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_APP, BoxesAppClass))

typedef struct _BoxesApp BoxesApp;
typedef struct _BoxesAppClass BoxesAppClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
typedef struct _BoxesWizardSourcePrivate BoxesWizardSourcePrivate;
typedef struct _Block69Data Block69Data;

#define BOXES_TYPE_LIBVIRT_MACHINE (boxes_libvirt_machine_get_type ())
#define BOXES_LIBVIRT_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachine))
#define BOXES_LIBVIRT_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachineClass))
#define BOXES_IS_LIBVIRT_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_LIBVIRT_MACHINE))
#define BOXES_IS_LIBVIRT_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_LIBVIRT_MACHINE))
#define BOXES_LIBVIRT_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachineClass))

typedef struct _BoxesLibvirtMachine BoxesLibvirtMachine;
typedef struct _BoxesLibvirtMachineClass BoxesLibvirtMachineClass;
typedef struct _BoxesWizardCreateData BoxesWizardCreateData;
#define _xmlFreeURI0(var) ((var == NULL) ? NULL : (var = (xmlFreeURI (var), NULL)))
typedef struct _Block70Data Block70Data;

#define BOXES_TYPE_INSTALLER_MEDIA (boxes_installer_media_get_type ())
#define BOXES_INSTALLER_MEDIA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_INSTALLER_MEDIA, BoxesInstallerMedia))
#define BOXES_INSTALLER_MEDIA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_INSTALLER_MEDIA, BoxesInstallerMediaClass))
#define BOXES_IS_INSTALLER_MEDIA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_INSTALLER_MEDIA))
#define BOXES_IS_INSTALLER_MEDIA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_INSTALLER_MEDIA))
#define BOXES_INSTALLER_MEDIA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_INSTALLER_MEDIA, BoxesInstallerMediaClass))

typedef struct _BoxesInstallerMedia BoxesInstallerMedia;
typedef struct _BoxesInstallerMediaClass BoxesInstallerMediaClass;
typedef struct _BoxesAppPrivate BoxesAppPrivate;

#define BOXES_TYPE_CLUTTER_WIDGET (boxes_clutter_widget_get_type ())
#define BOXES_CLUTTER_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidget))
#define BOXES_CLUTTER_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidgetClass))
#define BOXES_IS_CLUTTER_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_CLUTTER_WIDGET))
#define BOXES_IS_CLUTTER_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_CLUTTER_WIDGET))
#define BOXES_CLUTTER_WIDGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidgetClass))

typedef struct _BoxesClutterWidget BoxesClutterWidget;
typedef struct _BoxesClutterWidgetClass BoxesClutterWidgetClass;

#define BOXES_TYPE_SEARCHBAR (boxes_searchbar_get_type ())
#define BOXES_SEARCHBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SEARCHBAR, BoxesSearchbar))
#define BOXES_SEARCHBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SEARCHBAR, BoxesSearchbarClass))
#define BOXES_IS_SEARCHBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SEARCHBAR))
#define BOXES_IS_SEARCHBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SEARCHBAR))
#define BOXES_SEARCHBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SEARCHBAR, BoxesSearchbarClass))

typedef struct _BoxesSearchbar BoxesSearchbar;
typedef struct _BoxesSearchbarClass BoxesSearchbarClass;

#define BOXES_TYPE_TOPBAR (boxes_topbar_get_type ())
#define BOXES_TOPBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_TOPBAR, BoxesTopbar))
#define BOXES_TOPBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_TOPBAR, BoxesTopbarClass))
#define BOXES_IS_TOPBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_TOPBAR))
#define BOXES_IS_TOPBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_TOPBAR))
#define BOXES_TOPBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_TOPBAR, BoxesTopbarClass))

typedef struct _BoxesTopbar BoxesTopbar;
typedef struct _BoxesTopbarClass BoxesTopbarClass;

#define BOXES_TYPE_NOTIFICATIONBAR (boxes_notificationbar_get_type ())
#define BOXES_NOTIFICATIONBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbar))
#define BOXES_NOTIFICATIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbarClass))
#define BOXES_IS_NOTIFICATIONBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_NOTIFICATIONBAR))
#define BOXES_IS_NOTIFICATIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_NOTIFICATIONBAR))
#define BOXES_NOTIFICATIONBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbarClass))

typedef struct _BoxesNotificationbar BoxesNotificationbar;
typedef struct _BoxesNotificationbarClass BoxesNotificationbarClass;

#define BOXES_TYPE_REVEALER (boxes_revealer_get_type ())
#define BOXES_REVEALER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_REVEALER, BoxesRevealer))
#define BOXES_REVEALER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_REVEALER, BoxesRevealerClass))
#define BOXES_IS_REVEALER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_REVEALER))
#define BOXES_IS_REVEALER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_REVEALER))
#define BOXES_REVEALER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_REVEALER, BoxesRevealerClass))

typedef struct _BoxesRevealer BoxesRevealer;
typedef struct _BoxesRevealerClass BoxesRevealerClass;

#define BOXES_TYPE_SIDEBAR (boxes_sidebar_get_type ())
#define BOXES_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SIDEBAR, BoxesSidebar))
#define BOXES_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SIDEBAR, BoxesSidebarClass))
#define BOXES_IS_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SIDEBAR))
#define BOXES_IS_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SIDEBAR))
#define BOXES_SIDEBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SIDEBAR, BoxesSidebarClass))

typedef struct _BoxesSidebar BoxesSidebar;
typedef struct _BoxesSidebarClass BoxesSidebarClass;

#define BOXES_TYPE_SELECTIONBAR (boxes_selectionbar_get_type ())
#define BOXES_SELECTIONBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbar))
#define BOXES_SELECTIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbarClass))
#define BOXES_IS_SELECTIONBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SELECTIONBAR))
#define BOXES_IS_SELECTIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SELECTIONBAR))
#define BOXES_SELECTIONBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbarClass))

typedef struct _BoxesSelectionbar BoxesSelectionbar;
typedef struct _BoxesSelectionbarClass BoxesSelectionbarClass;

#define BOXES_TYPE_PROPERTIES (boxes_properties_get_type ())
#define BOXES_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_PROPERTIES, BoxesProperties))
#define BOXES_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_PROPERTIES, BoxesPropertiesClass))
#define BOXES_IS_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_PROPERTIES))
#define BOXES_IS_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_PROPERTIES))
#define BOXES_PROPERTIES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_PROPERTIES, BoxesPropertiesClass))

typedef struct _BoxesProperties BoxesProperties;
typedef struct _BoxesPropertiesClass BoxesPropertiesClass;

#define BOXES_TYPE_DISPLAY_PAGE (boxes_display_page_get_type ())
#define BOXES_DISPLAY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPage))
#define BOXES_DISPLAY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPageClass))
#define BOXES_IS_DISPLAY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_DISPLAY_PAGE))
#define BOXES_IS_DISPLAY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_DISPLAY_PAGE))
#define BOXES_DISPLAY_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPageClass))

typedef struct _BoxesDisplayPage BoxesDisplayPage;
typedef struct _BoxesDisplayPageClass BoxesDisplayPageClass;

#define BOXES_TYPE_EMPTY_BOXES (boxes_empty_boxes_get_type ())
#define BOXES_EMPTY_BOXES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxes))
#define BOXES_EMPTY_BOXES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxesClass))
#define BOXES_IS_EMPTY_BOXES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_EMPTY_BOXES))
#define BOXES_IS_EMPTY_BOXES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_EMPTY_BOXES))
#define BOXES_EMPTY_BOXES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxesClass))

typedef struct _BoxesEmptyBoxes BoxesEmptyBoxes;
typedef struct _BoxesEmptyBoxesClass BoxesEmptyBoxesClass;

#define BOXES_TYPE_COLLECTION (boxes_collection_get_type ())
#define BOXES_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION, BoxesCollection))
#define BOXES_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION, BoxesCollectionClass))
#define BOXES_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION))
#define BOXES_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION))
#define BOXES_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION, BoxesCollectionClass))

typedef struct _BoxesCollection BoxesCollection;
typedef struct _BoxesCollectionClass BoxesCollectionClass;

#define BOXES_TYPE_COLLECTION_FILTER (boxes_collection_filter_get_type ())
#define BOXES_COLLECTION_FILTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilter))
#define BOXES_COLLECTION_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilterClass))
#define BOXES_IS_COLLECTION_FILTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_FILTER))
#define BOXES_IS_COLLECTION_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_FILTER))
#define BOXES_COLLECTION_FILTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilterClass))

typedef struct _BoxesCollectionFilter BoxesCollectionFilter;
typedef struct _BoxesCollectionFilterClass BoxesCollectionFilterClass;

#define BOXES_TYPE_COLLECTION_VIEW (boxes_collection_view_get_type ())
#define BOXES_COLLECTION_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionView))
#define BOXES_COLLECTION_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionViewClass))
#define BOXES_IS_COLLECTION_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_VIEW))
#define BOXES_IS_COLLECTION_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_VIEW))
#define BOXES_COLLECTION_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionViewClass))

typedef struct _BoxesCollectionView BoxesCollectionView;
typedef struct _BoxesCollectionViewClass BoxesCollectionViewClass;
typedef struct _BoxesWizardReviewData BoxesWizardReviewData;

#define BOXES_TYPE_REMOTE_MACHINE (boxes_remote_machine_get_type ())
#define BOXES_REMOTE_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_REMOTE_MACHINE, BoxesRemoteMachine))
#define BOXES_REMOTE_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_REMOTE_MACHINE, BoxesRemoteMachineClass))
#define BOXES_IS_REMOTE_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_REMOTE_MACHINE))
#define BOXES_IS_REMOTE_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_REMOTE_MACHINE))
#define BOXES_REMOTE_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_REMOTE_MACHINE, BoxesRemoteMachineClass))

typedef struct _BoxesRemoteMachine BoxesRemoteMachine;
typedef struct _BoxesRemoteMachineClass BoxesRemoteMachineClass;
#define _boxes_pair_unref0(var) ((var == NULL) ? NULL : (var = (boxes_pair_unref (var), NULL)))
#define __g_list_free__boxes_pair_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__boxes_pair_unref0_ (var), NULL)))
typedef struct _Block71Data Block71Data;
#define __vala_GVirStorageVolInfo_free0(var) ((var == NULL) ? NULL : (var = (_vala_GVirStorageVolInfo_free (var), NULL)))
typedef struct _BoxesWizardDoReviewCancellableData BoxesWizardDoReviewCancellableData;
typedef struct _BoxesSidebarPrivate BoxesSidebarPrivate;

#define BOXES_TYPE_SIDEBAR_PAGE (boxes_sidebar_page_get_type ())
typedef struct _BoxesTopbarPrivate BoxesTopbarPrivate;

#define BOXES_TYPE_TOPBAR_PAGE (boxes_topbar_page_get_type ())
typedef struct _BoxesWizardWizardSummaryPrivate BoxesWizardWizardSummaryPrivate;
typedef struct _Block72Data Block72Data;
#define _g_list_free0(var) ((var == NULL) ? NULL : (var = (g_list_free (var), NULL)))

typedef enum  {
	BOXES_WIZARD_PAGE_INTRODUCTION,
	BOXES_WIZARD_PAGE_SOURCE,
	BOXES_WIZARD_PAGE_PREPARATION,
	BOXES_WIZARD_PAGE_SETUP,
	BOXES_WIZARD_PAGE_REVIEW,
	BOXES_WIZARD_PAGE_LAST
} BoxesWizardPage;

typedef enum  {
	BOXES_UI_STATE_NONE,
	BOXES_UI_STATE_COLLECTION,
	BOXES_UI_STATE_CREDS,
	BOXES_UI_STATE_DISPLAY,
	BOXES_UI_STATE_SETTINGS,
	BOXES_UI_STATE_WIZARD,
	BOXES_UI_STATE_PROPERTIES
} BoxesUIState;

struct _BoxesUI {
	GObject parent_instance;
	BoxesUIPrivate * priv;
	BoxesUIState previous_ui_state;
};

struct _BoxesUIClass {
	GObjectClass parent_class;
	void (*ui_state_changed) (BoxesUI* self);
	ClutterActor* (*get_actor) (BoxesUI* self);
};

struct _BoxesWizard {
	BoxesUI parent_instance;
	BoxesWizardPrivate * priv;
};

struct _BoxesWizardClass {
	BoxesUIClass parent_class;
};

struct _BoxesWizardPrivate {
	GtkClutterActor* gtk_actor;
	GPtrArray* steps;
	GtkNotebook* notebook;
	GtkButton* cancel_button;
	GtkButton* back_button;
	GtkButton* next_button;
	GtkButton* continue_button;
	GtkButton* create_button;
	GtkSizeGroup* toolbar_sizegroup;
	BoxesWizardSource* wizard_source;
	BoxesWizardWizardSummary* summary;
	BoxesCollectionSource* source;
	GtkProgressBar* prep_progress;
	GtkLabel* prep_media_label;
	GtkLabel* prep_status_label;
	GtkBox* setup_vbox;
	GtkLabel* review_label;
	GtkLabel* nokvm_label;
	GtkImage* installer_image;
	BoxesMediaManager* media_manager;
	BoxesVMCreator* vm_creator;
	BoxesMachine* _machine;
	BoxesWizardPage _page;
	GCancellable* review_cancellable;
	gboolean skip_review_for_live;
};

typedef enum  {
	BOXES_SOURCE_PAGE_MAIN,
	BOXES_SOURCE_PAGE_URL,
	BOXES_SOURCE_PAGE_LAST
} BoxesSourcePage;

struct _BoxesWizardSource {
	GObject parent_instance;
	BoxesWizardSourcePrivate * priv;
	GtkEntry* url_entry;
};

struct _BoxesWizardSourceClass {
	GObjectClass parent_class;
};

struct _Block69Data {
	int _ref_count_;
	BoxesWizard * self;
	gpointer _async_data_;
};

struct _BoxesWizardCreateData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesWizard* self;
	gboolean result;
	Block69Data* _data69_;
	BoxesCollectionSource* _tmp0_;
	BoxesVMCreator* _tmp1_;
	BoxesLibvirtMachine* _tmp2_;
	BoxesLibvirtMachine* _tmp3_;
	GCancellable* _tmp4_;
	gulong _tmp5_;
	gulong wait;
	gulong _tmp6_;
	BoxesLibvirtMachine* _tmp7_;
	BoxesLibvirtMachine* _tmp8_;
	GtkButton* _tmp9_;
	BoxesVMCreator* _tmp10_;
	BoxesLibvirtMachine* _tmp11_;
	BoxesLibvirtMachine* _tmp12_;
	GError* _error_;
	GError* _tmp13_;
	const gchar* _tmp14_;
	BoxesWizardSource* _tmp15_;
	BoxesCollectionSource* _tmp16_;
	BoxesApp* _tmp17_;
	BoxesCollectionSource* _tmp18_;
	GError * _inner_error_;
};

struct _Block70Data {
	int _ref_count_;
	BoxesWizard * self;
	BoxesActivityProgress* progress;
};

typedef void (*BoxesMediaManagerInstallerRecognized) (OsinfoMedia* os_media, OsinfoOs* os, void* user_data);
struct _BoxesApp {
	BoxesUI parent_instance;
	BoxesAppPrivate * priv;
	GtkApplicationWindow* window;
	GtkNotebook* notebook;
	BoxesClutterWidget* embed;
	ClutterStage* stage;
	ClutterBinLayout* stage_bin;
	ClutterActor* overlay_bin_actor;
	ClutterBinLayout* overlay_bin;
	BoxesCollectionItem* current_item;
	BoxesSearchbar* searchbar;
	BoxesTopbar* topbar;
	BoxesNotificationbar* notificationbar;
	BoxesRevealer* sidebar_revealer;
	BoxesRevealer* searchbar_revealer;
	BoxesSidebar* sidebar;
	BoxesSelectionbar* selectionbar;
	guint duration;
	GSettings* settings;
	BoxesWizard* wizard;
	BoxesProperties* properties;
	BoxesDisplayPage* display_page;
	BoxesEmptyBoxes* empty_boxes;
	BoxesCollection* collection;
	BoxesCollectionFilter* filter;
	BoxesCollectionView* view;
};

struct _BoxesAppClass {
	BoxesUIClass parent_class;
};

struct _BoxesWizardReviewData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesWizard* self;
	gboolean result;
	GCancellable* _tmp0_;
	GCancellable* _tmp1_;
	gboolean _tmp2_;
	gboolean _result_;
};

struct _Block71Data {
	int _ref_count_;
	BoxesWizard * self;
	GVirConfigDomain* config;
};

typedef void (*BoxesWizardWizardSummaryCustomizeFunc) (void* user_data);
struct _BoxesWizardDoReviewCancellableData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesWizard* self;
	gboolean result;
	GCancellable* _tmp0_;
	GtkLabel* _tmp1_;
	BoxesWizardWizardSummary* _tmp2_;
	BoxesCollectionSource* _tmp3_;
	BoxesCollectionSource* _tmp4_;
	BoxesRemoteMachine* _tmp5_;
	BoxesRemoteMachine* _tmp6_;
	BoxesRemoteMachine* _tmp7_;
	GError* _error_;
	GError* _tmp8_;
	const gchar* _tmp9_;
	gboolean _tmp10_;
	BoxesVMCreator* _tmp11_;
	BoxesLibvirtMachine* _tmp12_;
	BoxesLibvirtMachine* _tmp13_;
	gboolean _tmp14_;
	BoxesVMCreator* _tmp15_;
	GCancellable* _tmp16_;
	BoxesLibvirtMachine* _tmp17_;
	BoxesLibvirtMachine* _tmp18_;
	BoxesLibvirtMachine* _tmp19_;
	GError* cancel_error;
	GError* _vala1__error_;
	BoxesApp* _tmp20_;
	BoxesNotificationbar* _tmp21_;
	const gchar* _tmp22_;
	GdNotification* _tmp23_;
	GdNotification* _tmp24_;
	GError* _tmp25_;
	const gchar* _tmp26_;
	BoxesLibvirtMachine* _tmp27_;
	BoxesLibvirtMachine* _tmp28_;
	GCancellable* _tmp29_;
	gboolean _tmp30_;
	GtkLabel* _tmp31_;
	const gchar* _tmp32_;
	BoxesCollectionSource* _tmp33_;
	BoxesCollectionSource* _tmp34_;
	gchar* _tmp35_;
	gchar* _tmp36_;
	gchar* _tmp37_;
	xmlURI* _tmp38_;
	xmlURI* _tmp39_;
	xmlURI* uri;
	BoxesWizardWizardSummary* _tmp40_;
	const gchar* _tmp41_;
	BoxesCollectionSource* _tmp42_;
	gchar* _tmp43_;
	gchar* _tmp44_;
	gchar* _tmp45_;
	gboolean _tmp46_;
	xmlURI* _tmp47_;
	xmlURI* _tmp48_;
	const gchar* _tmp49_;
	gboolean _tmp50_;
	BoxesWizardWizardSummary* _tmp51_;
	const gchar* _tmp52_;
	xmlURI* _tmp53_;
	const gchar* _tmp54_;
	gchar* _tmp55_;
	gchar* _tmp56_;
	BoxesWizardWizardSummary* _tmp57_;
	const gchar* _tmp58_;
	BoxesCollectionSource* _tmp59_;
	gchar* _tmp60_;
	gchar* _tmp61_;
	gchar* _tmp62_;
	gchar* _tmp63_;
	gchar* _tmp64_;
	xmlURI* _tmp65_;
	const gchar* _tmp66_;
	const gchar* _tmp67_;
	GQuark _tmp69_;
	gint port;
	gint tls_port;
	BoxesCollectionSource* _tmp70_;
	gchar* _tmp71_;
	gchar* _tmp72_;
	gchar* _tmp73_;
	gint* _tmp74_;
	gint* _tmp75_;
	gint* _tmp76_;
	gint* _tmp77_;
	gint _tmp78_;
	BoxesWizardWizardSummary* _tmp79_;
	const gchar* _tmp80_;
	gint _tmp81_;
	gchar* _tmp82_;
	gchar* _tmp83_;
	gint _tmp84_;
	BoxesWizardWizardSummary* _tmp85_;
	const gchar* _tmp86_;
	gint _tmp87_;
	gchar* _tmp88_;
	gchar* _tmp89_;
	GError* _vala2__error_;
	GError* _tmp90_;
	const gchar* _tmp91_;
	xmlURI* _tmp92_;
	gint _tmp93_;
	BoxesWizardWizardSummary* _tmp94_;
	const gchar* _tmp95_;
	xmlURI* _tmp96_;
	gint _tmp97_;
	gchar* _tmp98_;
	gchar* _tmp99_;
	BoxesApp* _tmp100_;
	BoxesCollectionSource* _tmp101_;
	gchar* _tmp102_;
	gchar* _tmp103_;
	gchar* _tmp104_;
	gboolean _tmp105_;
	gboolean _tmp106_;
	GtkLabel* _tmp107_;
	const gchar* _tmp108_;
	BoxesLibvirtMachine* _tmp109_;
	BoxesLibvirtMachine* _tmp110_;
	BoxesVMCreator* _tmp111_;
	BoxesInstallerMedia* _tmp112_;
	BoxesInstallerMedia* _tmp113_;
	GList* _tmp114_;
	GList* property_collection;
	GList* property_it;
	BoxesPair* _tmp115_;
	BoxesPair* property;
	BoxesWizardWizardSummary* _tmp116_;
	BoxesPair* _tmp117_;
	gconstpointer _tmp118_;
	BoxesPair* _tmp119_;
	gconstpointer _tmp120_;
	Block71Data* _data71_;
	GVirConfigDomain* _tmp121_;
	GVirConfigDomain* _tmp122_;
	guint64 _tmp123_;
	guint64 _tmp124_;
	gchar* _tmp125_;
	gchar* memory;
	BoxesWizardWizardSummary* _tmp126_;
	const gchar* _tmp127_;
	const gchar* _tmp128_;
	GError* _vala3__error_;
	BoxesLibvirtMachine* _tmp129_;
	BoxesLibvirtMachine* _tmp130_;
	const gchar* _tmp131_;
	const gchar* _tmp132_;
	GError* _tmp133_;
	const gchar* _tmp134_;
	BoxesLibvirtMachine* _tmp135_;
	BoxesLibvirtMachine* _tmp136_;
	GVirStorageVol* _tmp137_;
	GVirStorageVol* _tmp138_;
	BoxesLibvirtMachine* _tmp139_;
	BoxesLibvirtMachine* _tmp140_;
	GVirStorageVol* _tmp141_;
	GVirStorageVol* _tmp142_;
	GVirStorageVolInfo* _tmp143_;
	GVirStorageVolInfo* volume_info;
	GVirStorageVolInfo* _tmp144_;
	guint64 _tmp145_;
	gchar* _tmp146_;
	gchar* capacity;
	BoxesWizardWizardSummary* _tmp147_;
	const gchar* _tmp148_;
	const gchar* _tmp149_;
	gchar* _tmp150_;
	gchar* _tmp151_;
	const gchar* _tmp152_;
	GError* _vala4__error_;
	BoxesLibvirtMachine* _tmp153_;
	BoxesLibvirtMachine* _tmp154_;
	GVirStorageVol* _tmp155_;
	GVirStorageVol* _tmp156_;
	const gchar* _tmp157_;
	GError* _tmp158_;
	const gchar* _tmp159_;
	GtkLabel* _tmp160_;
	BoxesLibvirtMachine* _tmp161_;
	BoxesLibvirtMachine* _tmp162_;
	GVirConfigDomain* _tmp163_;
	GVirConfigDomain* _tmp164_;
	GVirConfigDomainVirtType _tmp165_;
	BoxesWizardWizardSummary* _tmp166_;
	GError * _inner_error_;
};

struct _BoxesSidebar {
	BoxesUI parent_instance;
	BoxesSidebarPrivate * priv;
	GtkNotebook* notebook;
};

struct _BoxesSidebarClass {
	BoxesUIClass parent_class;
};

typedef enum  {
	BOXES_SIDEBAR_PAGE_COLLECTION,
	BOXES_SIDEBAR_PAGE_WIZARD,
	BOXES_SIDEBAR_PAGE_PROPERTIES
} BoxesSidebarPage;

struct _BoxesTopbar {
	BoxesUI parent_instance;
	BoxesTopbarPrivate * priv;
	GtkNotebook* notebook;
	gchar* _status;
};

struct _BoxesTopbarClass {
	BoxesUIClass parent_class;
};

typedef enum  {
	BOXES_TOPBAR_PAGE_COLLECTION,
	BOXES_TOPBAR_PAGE_SELECTION,
	BOXES_TOPBAR_PAGE_WIZARD,
	BOXES_TOPBAR_PAGE_PROPERTIES
} BoxesTopbarPage;

struct _BoxesWizardWizardSummary {
	GObject parent_instance;
	BoxesWizardWizardSummaryPrivate * priv;
};

struct _BoxesWizardWizardSummaryClass {
	GObjectClass parent_class;
};

struct _BoxesWizardWizardSummaryPrivate {
	GtkGrid* grid;
	gint current_row;
};

struct _Block72Data {
	int _ref_count_;
	BoxesWizardWizardSummary * self;
	BoxesWizardWizardSummaryCustomizeFunc customize_func;
	gpointer customize_func_target;
};


static gpointer boxes_wizard_parent_class = NULL;
extern BoxesApp* boxes_app_app;
static gpointer boxes_wizard_wizard_summary_parent_class = NULL;

GType boxes_wizard_page_get_type (void) G_GNUC_CONST;
GType boxes_ui_get_type (void) G_GNUC_CONST;
GType boxes_ui_state_get_type (void) G_GNUC_CONST;
GType boxes_wizard_get_type (void) G_GNUC_CONST;
GType boxes_wizard_source_get_type (void) G_GNUC_CONST;
static GType boxes_wizard_wizard_summary_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
GType boxes_media_manager_get_type (void) G_GNUC_CONST;
gpointer boxes_vm_creator_ref (gpointer instance);
void boxes_vm_creator_unref (gpointer instance);
GParamSpec* boxes_param_spec_vm_creator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void boxes_value_set_vm_creator (GValue* value, gpointer v_object);
void boxes_value_take_vm_creator (GValue* value, gpointer v_object);
gpointer boxes_value_get_vm_creator (const GValue* value);
GType boxes_vm_creator_get_type (void) G_GNUC_CONST;
GType boxes_collection_item_get_type (void) G_GNUC_CONST;
GType boxes_machine_get_type (void) G_GNUC_CONST;
#define BOXES_WIZARD_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), BOXES_TYPE_WIZARD, BoxesWizardPrivate))
enum  {
	BOXES_WIZARD_DUMMY_PROPERTY,
	BOXES_WIZARD_ACTOR,
	BOXES_WIZARD_MACHINE
};
static void boxes_wizard_wizard_source_update_next (BoxesWizard* self);
GType boxes_source_page_get_type (void) G_GNUC_CONST;
BoxesSourcePage boxes_wizard_source_get_page (BoxesWizardSource* self);
GtkWidget* boxes_wizard_source_get_selected (BoxesWizardSource* self);
const gchar* boxes_wizard_source_get_uri (BoxesWizardSource* self);
static void boxes_wizard_prepare_for_location (BoxesWizard* self, const gchar* location, gboolean probing, GError** error);
GType boxes_app_get_type (void) G_GNUC_CONST;
gboolean boxes_app_has_broker_for_source_type (BoxesApp* self, const gchar* type);
void boxes_wizard_source_update_url_page (BoxesWizardSource* self, const gchar* title, const gchar* text, const gchar* icon_name);
BoxesWizard* boxes_wizard_new (void);
BoxesWizard* boxes_wizard_construct (GType object_type);
BoxesUI* boxes_ui_construct (GType object_type);
BoxesWizardSource* boxes_wizard_source_new (BoxesMediaManager* media_manager);
BoxesWizardSource* boxes_wizard_source_construct (GType object_type, BoxesMediaManager* media_manager);
static void _boxes_wizard_wizard_source_update_next_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
static void _boxes_wizard_wizard_source_update_next_gtk_editable_changed (GtkEditable* _sender, gpointer self);
static void __lambda178_ (BoxesWizard* self);
static void boxes_wizard_set_page (BoxesWizard* self, BoxesWizardPage value);
static void ___lambda178__gtk_entry_activate (GtkEntry* _sender, gpointer self);
static void boxes_wizard_setup_ui (BoxesWizard* self);
void boxes_wizard_cleanup (BoxesWizard* self);
static void boxes_wizard_destroy_machine (BoxesWizard* self);
static void boxes_wizard_create_data_free (gpointer _data);
static void boxes_wizard_create (BoxesWizard* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
static gboolean boxes_wizard_create_finish (BoxesWizard* self, GAsyncResult* _res_);
static gboolean boxes_wizard_create_co (BoxesWizardCreateData* _data_);
static Block69Data* block69_data_ref (Block69Data* _data69_);
static void block69_data_unref (void * _userdata_);
GType boxes_libvirt_machine_get_type (void) G_GNUC_CONST;
static BoxesLibvirtMachine* boxes_wizard_get_libvirt_machine (BoxesWizard* self);
static void ____lambda176_ (Block69Data* _data69_);
static void _____lambda176__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
void boxes_vm_creator_launch_vm (BoxesVMCreator* self, BoxesLibvirtMachine* machine, GError** error);
void boxes_wizard_source_set_uri (BoxesWizardSource* self, const gchar* value);
void boxes_app_add_collection_source (BoxesApp* self, BoxesCollectionSource* source, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_app_add_collection_source_finish (BoxesApp* self, GAsyncResult* _res_);
void boxes_wizard_set_machine (BoxesWizard* self, BoxesMachine* value);
static void boxes_wizard_prepare_for_installer (BoxesWizard* self, const gchar* path, GError** error);
static void boxes_wizard_prepare_for_uri (BoxesWizard* self, const gchar* uri_as_text, GError** error);
void spice_validate_uri (const gchar* uri_as_text, gint** port, gint** tls_port, GError** error);
static Block70Data* block70_data_ref (Block70Data* _data70_);
static void block70_data_unref (void * _userdata_);
static void __lambda171_ (Block70Data* _data70_);
static void ___lambda171__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self);
void boxes_media_manager_create_installer_media_for_path (BoxesMediaManager* self, const gchar* path, BoxesMediaManagerInstallerRecognized on_installer_recognized, void* on_installer_recognized_target, BoxesActivityProgress* progress, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
GType boxes_installer_media_get_type (void) G_GNUC_CONST;
BoxesInstallerMedia* boxes_media_manager_create_installer_media_for_path_finish (BoxesMediaManager* self, GAsyncResult* _res_, GError** error);
static void boxes_wizard_on_installer_recognized (BoxesWizard* self, OsinfoMedia* os_media, OsinfoOs* os);
static void _boxes_wizard_on_installer_recognized_boxes_media_manager_installer_recognized (OsinfoMedia* os_media, OsinfoOs* os, gpointer self);
static void boxes_wizard_on_installer_media_instantiated (BoxesWizard* self, GObject* source_object, GAsyncResult* _result_);
static void _boxes_wizard_on_installer_media_instantiated_gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self);
void boxes_downloader_fetch_os_logo (GtkImage* image, OsinfoOs* os, gint size, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_downloader_fetch_os_logo_finish (GAsyncResult* _res_);
BoxesVMCreator* boxes_vm_creator_new (BoxesInstallerMedia* install_media);
BoxesVMCreator* boxes_vm_creator_construct (GType object_type, BoxesInstallerMedia* install_media);
GType boxes_clutter_widget_get_type (void) G_GNUC_CONST;
GType boxes_searchbar_get_type (void) G_GNUC_CONST;
GType boxes_topbar_get_type (void) G_GNUC_CONST;
GType boxes_notificationbar_get_type (void) G_GNUC_CONST;
GType boxes_revealer_get_type (void) G_GNUC_CONST;
GType boxes_sidebar_get_type (void) G_GNUC_CONST;
GType boxes_selectionbar_get_type (void) G_GNUC_CONST;
GType boxes_properties_get_type (void) G_GNUC_CONST;
GType boxes_display_page_get_type (void) G_GNUC_CONST;
GType boxes_empty_boxes_get_type (void) G_GNUC_CONST;
GType boxes_collection_get_type (void) G_GNUC_CONST;
GType boxes_collection_filter_get_type (void) G_GNUC_CONST;
GType boxes_collection_view_get_type (void) G_GNUC_CONST;
GdNotification* boxes_notificationbar_display_error (BoxesNotificationbar* self, const gchar* message, gint timeout);
#define BOXES_NOTIFICATIONBAR_DEFAULT_TIMEOUT 6
static gboolean boxes_wizard_prepare (BoxesWizard* self);
BoxesInstallerMedia* boxes_wizard_source_get_install_media (BoxesWizardSource* self);
static gboolean boxes_wizard_setup (BoxesWizard* self);
BoxesInstallerMedia* boxes_vm_creator_get_install_media (BoxesVMCreator* self);
void boxes_installer_media_populate_setup_vbox (BoxesInstallerMedia* self, GtkBox* setup_vbox);
static void boxes_wizard_review_data_free (gpointer _data);
static void boxes_wizard_review (BoxesWizard* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
static gboolean boxes_wizard_review_finish (BoxesWizard* self, GAsyncResult* _res_);
static gboolean boxes_wizard_review_co (BoxesWizardReviewData* _data_);
static void boxes_wizard_do_review_cancellable (BoxesWizard* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
static gboolean boxes_wizard_do_review_cancellable_finish (BoxesWizard* self, GAsyncResult* _res_);
static void boxes_wizard_review_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static void boxes_wizard_do_review_cancellable_data_free (gpointer _data);
static gboolean boxes_wizard_do_review_cancellable_co (BoxesWizardDoReviewCancellableData* _data_);
static void boxes_wizard_wizard_summary_clear (BoxesWizardWizardSummary* self);
BoxesRemoteMachine* boxes_remote_machine_new (BoxesCollectionSource* source, GError** error);
BoxesRemoteMachine* boxes_remote_machine_construct (GType object_type, BoxesCollectionSource* source, GError** error);
GType boxes_remote_machine_get_type (void) G_GNUC_CONST;
void boxes_vm_creator_create_vm (BoxesVMCreator* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
BoxesLibvirtMachine* boxes_vm_creator_create_vm_finish (BoxesVMCreator* self, GAsyncResult* _res_, GError** error);
static void boxes_wizard_do_review_cancellable_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static void boxes_wizard_wizard_summary_add_property (BoxesWizardWizardSummary* self, const gchar* name, const gchar* value);
GList* boxes_installer_media_get_vm_properties (BoxesInstallerMedia* self);
static void _boxes_pair_unref0_ (gpointer var);
static void _g_list_free__boxes_pair_unref0_ (GList* self);
static Block71Data* block71_data_ref (Block71Data* _data71_);
static void block71_data_unref (void * _userdata_);
static void _____lambda173_ (Block71Data* _data71_, GError** error);
GVirDomain* boxes_libvirt_machine_get_domain (BoxesLibvirtMachine* self);
static void ______lambda173__boxes_run_in_thread_func (gpointer self, GError** error);
const gchar* boxes_collection_item_get_name (BoxesCollectionItem* self);
GVirStorageVol* boxes_libvirt_machine_get_storage_volume (BoxesLibvirtMachine* self);
static void _vala_GVirStorageVolInfo_free (GVirStorageVolInfo* self);
GVirConfigDomain* boxes_libvirt_machine_get_domain_config (BoxesLibvirtMachine* self);
static void boxes_wizard_wizard_summary_append_customize_button (BoxesWizardWizardSummary* self, BoxesWizardWizardSummaryCustomizeFunc customize_func, void* customize_func_target);
static void __lambda174_ (BoxesWizard* self);
void boxes_app_select_item (BoxesApp* self, BoxesCollectionItem* item);
BoxesMachine* boxes_wizard_get_machine (BoxesWizard* self);
static void ___lambda174__boxes_wizard_wizard_summary_customize_func (gpointer self);
static void boxes_wizard_add_step (BoxesWizard* self, GtkWidget* widget, const gchar* title, BoxesWizardPage page);
GType boxes_sidebar_page_get_type (void) G_GNUC_CONST;
static gboolean boxes_wizard_skip_page (BoxesWizard* self, BoxesWizardPage page);
static BoxesWizardPage boxes_wizard_get_page (BoxesWizard* self);
gboolean boxes_installer_media_get_need_user_input_for_vm_creation (BoxesInstallerMedia* self);
gboolean boxes_installer_media_get_live (BoxesInstallerMedia* self);
GtkWidget* boxes_wizard_source_get_widget (BoxesWizardSource* self);
static BoxesWizardWizardSummary* boxes_wizard_wizard_summary_new (void);
static BoxesWizardWizardSummary* boxes_wizard_wizard_summary_construct (GType object_type);
static GtkWidget* boxes_wizard_wizard_summary_get_widget (BoxesWizardWizardSummary* self);
GType boxes_topbar_page_get_type (void) G_GNUC_CONST;
static void __lambda179_ (BoxesWizard* self);
void boxes_wizard_source_set_page (BoxesWizardSource* self, BoxesSourcePage value);
void boxes_ui_set_ui_state (BoxesUI* self, BoxesUIState value);
static void ___lambda179__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void __lambda180_ (BoxesWizard* self);
static void ___lambda180__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void __lambda181_ (BoxesWizard* self);
static void ___lambda181__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void __lambda182_ (BoxesWizard* self);
static void ___lambda182__gtk_button_clicked (GtkButton* _sender, gpointer self);
void boxes_wizard_open_with_uri (BoxesWizard* self, const gchar* uri, gboolean skip_review_for_live);
static void boxes_wizard_real_ui_state_changed (BoxesUI* base);
BoxesUIState boxes_ui_get_ui_state (BoxesUI* self);
void boxes_fade_actor (ClutterActor* actor, guint opacity);
ClutterActor* boxes_ui_get_actor (BoxesUI* self);
void boxes_app_delete_machine (BoxesApp* self, BoxesMachine* machine, gboolean by_user);
void boxes_wizard_source_set_selected (BoxesWizardSource* self, GtkWidget* value);
static void _boxes_wizard_page_____lambda175_ (BoxesWizard* self, GObject* obj, GAsyncResult* _result_);
static void __boxes_wizard_page_____lambda175__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self);
static void _boxes_wizard_page_____lambda177_ (BoxesWizard* self, GObject* obj, GAsyncResult* _result_);
static void __boxes_wizard_page_____lambda177__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self);
static GObject * boxes_wizard_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void _g_object_unref0_ (gpointer var);
BoxesMediaManager* boxes_media_manager_get_instance (void);
#define BOXES_WIZARD_WIZARD_SUMMARY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), BOXES_WIZARD_TYPE_WIZARD_SUMMARY, BoxesWizardWizardSummaryPrivate))
enum  {
	BOXES_WIZARD_WIZARD_SUMMARY_DUMMY_PROPERTY,
	BOXES_WIZARD_WIZARD_SUMMARY_WIDGET
};
static Block72Data* block72_data_ref (Block72Data* _data72_);
static void block72_data_unref (void * _userdata_);
static void __lambda170_ (Block72Data* _data72_);
static void ___lambda170__gtk_button_clicked (GtkButton* _sender, gpointer self);
static void boxes_wizard_wizard_summary_finalize (GObject* obj);
static void _vala_boxes_wizard_wizard_summary_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void boxes_wizard_finalize (GObject* obj);
static void _vala_boxes_wizard_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_boxes_wizard_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


GType boxes_wizard_page_get_type (void) {
	static volatile gsize boxes_wizard_page_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_wizard_page_type_id__volatile)) {
		static const GEnumValue values[] = {{BOXES_WIZARD_PAGE_INTRODUCTION, "BOXES_WIZARD_PAGE_INTRODUCTION", "introduction"}, {BOXES_WIZARD_PAGE_SOURCE, "BOXES_WIZARD_PAGE_SOURCE", "source"}, {BOXES_WIZARD_PAGE_PREPARATION, "BOXES_WIZARD_PAGE_PREPARATION", "preparation"}, {BOXES_WIZARD_PAGE_SETUP, "BOXES_WIZARD_PAGE_SETUP", "setup"}, {BOXES_WIZARD_PAGE_REVIEW, "BOXES_WIZARD_PAGE_REVIEW", "review"}, {BOXES_WIZARD_PAGE_LAST, "BOXES_WIZARD_PAGE_LAST", "last"}, {0, NULL, NULL}};
		GType boxes_wizard_page_type_id;
		boxes_wizard_page_type_id = g_enum_register_static ("BoxesWizardPage", values);
		g_once_init_leave (&boxes_wizard_page_type_id__volatile, boxes_wizard_page_type_id);
	}
	return boxes_wizard_page_type_id__volatile;
}


static void boxes_wizard_wizard_source_update_next (BoxesWizard* self) {
	GtkButton* _tmp0_;
	BoxesWizardSource* _tmp1_;
	BoxesSourcePage _tmp2_;
	BoxesSourcePage _tmp3_;
	GError * _inner_error_ = NULL;
#line 136 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 137 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->next_button;
#line 137 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_sensitive ((GtkWidget*) _tmp0_, FALSE);
#line 139 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = self->priv->wizard_source;
#line 139 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = boxes_wizard_source_get_page (_tmp1_);
#line 139 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = _tmp2_;
#line 139 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	switch (_tmp3_) {
#line 139 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case BOXES_SOURCE_PAGE_MAIN:
#line 958 "wizard.c"
		{
			GtkButton* _tmp4_;
			BoxesWizardSource* _tmp5_;
			GtkWidget* _tmp6_;
			GtkWidget* _tmp7_;
#line 141 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp4_ = self->priv->next_button;
#line 141 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp5_ = self->priv->wizard_source;
#line 141 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp6_ = boxes_wizard_source_get_selected (_tmp5_);
#line 141 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp7_ = _tmp6_;
#line 141 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			gtk_widget_set_sensitive ((GtkWidget*) _tmp4_, _tmp7_ != NULL);
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_object_unref0 (self->priv->source);
#line 142 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			self->priv->source = NULL;
#line 143 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			break;
#line 980 "wizard.c"
		}
#line 139 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case BOXES_SOURCE_PAGE_URL:
#line 984 "wizard.c"
		{
			GtkButton* _tmp8_;
			BoxesWizardSource* _tmp9_;
			const gchar* _tmp10_;
			const gchar* _tmp11_;
			gint _tmp12_;
			gint _tmp13_;
			const gchar* _tmp14_ = NULL;
			gchar* _tmp15_;
			gchar* text;
			gchar* _tmp16_;
			gchar* icon;
			BoxesWizardSource* _tmp34_;
			const gchar* _tmp35_ = NULL;
			const gchar* _tmp36_;
			const gchar* _tmp37_;
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp8_ = self->priv->next_button;
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp9_ = self->priv->wizard_source;
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp10_ = boxes_wizard_source_get_uri (_tmp9_);
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp11_ = _tmp10_;
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp12_ = strlen (_tmp11_);
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp13_ = _tmp12_;
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			gtk_widget_set_sensitive ((GtkWidget*) _tmp8_, _tmp13_ != 0);
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp14_ = _ ("Please enter desktop or collection URI");
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp15_ = g_strdup (_tmp14_);
#line 148 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			text = _tmp15_;
#line 149 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp16_ = g_strdup ("preferences-desktop-remote-desktop");
#line 149 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			icon = _tmp16_;
#line 1025 "wizard.c"
			{
				BoxesWizardSource* _tmp17_;
				const gchar* _tmp18_;
				const gchar* _tmp19_;
				gboolean _tmp20_ = FALSE;
				BoxesCollectionSource* _tmp21_;
				gboolean _tmp28_;
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp17_ = self->priv->wizard_source;
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp18_ = boxes_wizard_source_get_uri (_tmp17_);
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp19_ = _tmp18_;
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_wizard_prepare_for_location (self, _tmp19_, TRUE, &_inner_error_);
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_inner_error_ != NULL) {
#line 1043 "wizard.c"
					goto __catch94_g_error;
				}
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp21_ = self->priv->source;
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_tmp21_ != NULL) {
#line 1050 "wizard.c"
					BoxesApp* _tmp22_;
					BoxesCollectionSource* _tmp23_;
					gchar* _tmp24_;
					gchar* _tmp25_;
					gchar* _tmp26_;
					gboolean _tmp27_ = FALSE;
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp22_ = boxes_app_app;
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp23_ = self->priv->source;
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp24_ = boxes_collection_source_get_source_type (_tmp23_);
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp25_ = _tmp24_;
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp26_ = _tmp25_;
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp27_ = boxes_app_has_broker_for_source_type (_tmp22_, _tmp26_);
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp20_ = _tmp27_;
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_g_free0 (_tmp26_);
#line 1073 "wizard.c"
				} else {
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp20_ = FALSE;
#line 1077 "wizard.c"
				}
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp28_ = _tmp20_;
#line 153 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_tmp28_) {
#line 1083 "wizard.c"
					const gchar* _tmp29_ = NULL;
					gchar* _tmp30_;
					gchar* _tmp31_;
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp29_ = _ ("Will add boxes for all systems available from this account.");
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp30_ = g_strdup (_tmp29_);
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_g_free0 (text);
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					text = _tmp30_;
#line 155 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp31_ = g_strdup ("network-workgroup");
#line 155 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_g_free0 (icon);
#line 155 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					icon = _tmp31_;
#line 1101 "wizard.c"
				} else {
					const gchar* _tmp32_ = NULL;
					gchar* _tmp33_;
#line 157 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp32_ = _ ("Will add a single box.");
#line 157 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp33_ = g_strdup (_tmp32_);
#line 157 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_g_free0 (text);
#line 157 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					text = _tmp33_;
#line 1113 "wizard.c"
				}
			}
			goto __finally94;
			__catch94_g_error:
			{
				GError* _error_ = NULL;
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_error_ = _inner_error_;
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_inner_error_ = NULL;
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_error_free0 (_error_);
#line 1126 "wizard.c"
			}
			__finally94:
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_inner_error_ != NULL) {
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_free0 (icon);
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_free0 (text);
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_clear_error (&_inner_error_);
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return;
#line 1141 "wizard.c"
			}
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp34_ = self->priv->wizard_source;
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp35_ = _ ("Desktop Access");
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp36_ = text;
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp37_ = icon;
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_source_update_url_page (_tmp34_, _tmp35_, _tmp36_, _tmp37_);
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_free0 (icon);
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_free0 (text);
#line 164 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			break;
#line 1159 "wizard.c"
		}
		default:
		{
#line 167 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_warn_if_reached ();
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			break;
#line 1167 "wizard.c"
		}
	}
}


static void _boxes_wizard_wizard_source_update_next_g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 174 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_wizard_source_update_next (self);
#line 1176 "wizard.c"
}


static void _boxes_wizard_wizard_source_update_next_gtk_editable_changed (GtkEditable* _sender, gpointer self) {
#line 176 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_wizard_source_update_next (self);
#line 1183 "wizard.c"
}


static void __lambda178_ (BoxesWizard* self) {
#line 179 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_set_page (self, BOXES_WIZARD_PAGE_PREPARATION);
#line 1190 "wizard.c"
}


static void ___lambda178__gtk_entry_activate (GtkEntry* _sender, gpointer self) {
#line 178 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	__lambda178_ (self);
#line 1197 "wizard.c"
}


BoxesWizard* boxes_wizard_construct (GType object_type) {
	BoxesWizard * self = NULL;
	BoxesMediaManager* _tmp0_;
	BoxesWizardSource* _tmp1_;
	BoxesWizardSource* _tmp2_;
	BoxesWizardSource* _tmp3_;
	BoxesWizardSource* _tmp4_;
	GtkEntry* _tmp5_;
	BoxesWizardSource* _tmp6_;
	GtkEntry* _tmp7_;
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = (BoxesWizard*) boxes_ui_construct (object_type);
#line 173 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->media_manager;
#line 173 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = boxes_wizard_source_new (_tmp0_);
#line 173 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->wizard_source);
#line 173 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->wizard_source = _tmp1_;
#line 174 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = self->priv->wizard_source;
#line 174 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_object ((GObject*) _tmp2_, "notify::page", (GCallback) _boxes_wizard_wizard_source_update_next_g_object_notify, self, 0);
#line 175 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = self->priv->wizard_source;
#line 175 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_object ((GObject*) _tmp3_, "notify::selected", (GCallback) _boxes_wizard_wizard_source_update_next_g_object_notify, self, 0);
#line 176 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = self->priv->wizard_source;
#line 176 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = _tmp4_->url_entry;
#line 176 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_object ((GtkEditable*) _tmp5_, "changed", (GCallback) _boxes_wizard_wizard_source_update_next_gtk_editable_changed, self, 0);
#line 178 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp6_ = self->priv->wizard_source;
#line 178 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp7_ = _tmp6_->url_entry;
#line 178 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_object (_tmp7_, "activate", (GCallback) ___lambda178__gtk_entry_activate, self, 0);
#line 182 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_setup_ui (self);
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return self;
#line 1245 "wizard.c"
}


BoxesWizard* boxes_wizard_new (void) {
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return boxes_wizard_construct (BOXES_TYPE_WIZARD);
#line 1252 "wizard.c"
}


void boxes_wizard_cleanup (BoxesWizard* self) {
#line 185 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 186 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_destroy_machine (self);
#line 1261 "wizard.c"
}


static void boxes_wizard_create_data_free (gpointer _data) {
	BoxesWizardCreateData* _data_;
	_data_ = _data;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (_data_->self);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_slice_free (BoxesWizardCreateData, _data_);
#line 1272 "wizard.c"
}


static gpointer _g_object_ref0 (gpointer self) {
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return self ? g_object_ref (self) : NULL;
#line 1279 "wizard.c"
}


static void boxes_wizard_create (BoxesWizard* self, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesWizardCreateData* _data_;
	BoxesWizard* _tmp0_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_ = g_slice_new0 (BoxesWizardCreateData);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_wizard_create);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_wizard_create_data_free);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->self = _tmp0_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_create_co (_data_);
#line 1298 "wizard.c"
}


static gboolean boxes_wizard_create_finish (BoxesWizard* self, GAsyncResult* _res_) {
	gboolean result;
	BoxesWizardCreateData* _data_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = _data_->result;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 1311 "wizard.c"
}


static Block69Data* block69_data_ref (Block69Data* _data69_) {
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_atomic_int_inc (&_data69_->_ref_count_);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return _data69_;
#line 1320 "wizard.c"
}


static void block69_data_unref (void * _userdata_) {
	Block69Data* _data69_;
	_data69_ = (Block69Data*) _userdata_;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (g_atomic_int_dec_and_test (&_data69_->_ref_count_)) {
#line 1329 "wizard.c"
		BoxesWizard * self;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		self = _data69_->self;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_object_unref0 (self);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_slice_free (Block69Data, _data69_);
#line 1337 "wizard.c"
	}
}


static void ____lambda176_ (Block69Data* _data69_) {
	BoxesWizard * self;
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = _data69_->self;
#line 197 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_create_co (_data69_->_async_data_);
#line 1348 "wizard.c"
}


static void _____lambda176__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	____lambda176_ (self);
#line 1355 "wizard.c"
}


static gboolean boxes_wizard_create_co (BoxesWizardCreateData* _data_) {
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	switch (_data_->_state_) {
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case 0:
#line 1364 "wizard.c"
		goto _state_0;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case 1:
#line 1368 "wizard.c"
		goto _state_1;
		default:
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_assert_not_reached ();
#line 1373 "wizard.c"
	}
	_state_0:
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_data69_ = g_slice_new0 (Block69Data);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_data69_->_ref_count_ = 1;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_data69_->self = g_object_ref (_data_->self);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_data69_->_async_data_ = _data_;
#line 190 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp0_ = _data_->self->priv->source;
#line 190 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_tmp0_ == NULL) {
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp1_ = _data_->self->priv->vm_creator;
#line 191 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_return_val_if_fail (_data_->_tmp1_ != NULL, FALSE);
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp2_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp3_ = _data_->_tmp2_;
#line 193 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_tmp3_ == NULL) {
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp4_ = _data_->self->priv->review_cancellable;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_return_val_if_fail (_data_->_tmp4_ != NULL, FALSE);
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp5_ = 0UL;
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp5_ = g_signal_connect_data ((GObject*) _data_->self, "notify::machine", (GCallback) _____lambda176__g_object_notify, block69_data_ref (_data_->_data69_), (GClosureNotify) block69_data_unref, 0);
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->wait = _data_->_tmp5_;
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_state_ = 1;
#line 199 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			return FALSE;
#line 1412 "wizard.c"
			_state_1:
			;
#line 200 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp6_ = _data_->wait;
#line 200 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_signal_handler_disconnect ((GObject*) _data_->self, _data_->_tmp6_);
#line 201 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp7_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 201 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp8_ = _data_->_tmp7_;
#line 201 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_data_->_tmp8_ == NULL) {
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->result = FALSE;
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				block69_data_unref (_data_->_data69_);
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_data69_ = NULL;
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_data_->_state_ == 0) {
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 1435 "wizard.c"
				} else {
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					g_simple_async_result_complete (_data_->_async_result);
#line 1439 "wizard.c"
				}
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_object_unref (_data_->_async_result);
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return FALSE;
#line 1445 "wizard.c"
			}
		}
#line 204 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp9_ = _data_->self->priv->next_button;
#line 204 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		gtk_widget_set_sensitive ((GtkWidget*) _data_->_tmp9_, FALSE);
#line 1452 "wizard.c"
		{
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp10_ = _data_->self->priv->vm_creator;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp11_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp12_ = _data_->_tmp11_;
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_vm_creator_launch_vm (_data_->_tmp10_, _data_->_tmp12_, &_data_->_inner_error_);
#line 206 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_data_->_inner_error_ != NULL) {
#line 1464 "wizard.c"
				goto __catch95_g_error;
			}
		}
		goto __finally95;
		__catch95_g_error:
		{
#line 205 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_error_ = _data_->_inner_error_;
#line 205 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_inner_error_ = NULL;
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp13_ = _data_->_error_;
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp14_ = _data_->_tmp13_->message;
#line 208 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_warning ("wizard.vala:208: %s", _data_->_tmp14_);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->result = FALSE;
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_error_free0 (_data_->_error_);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			block69_data_unref (_data_->_data69_);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_data69_ = NULL;
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_data_->_state_ == 0) {
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 1493 "wizard.c"
			} else {
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_simple_async_result_complete (_data_->_async_result);
#line 1497 "wizard.c"
			}
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_object_unref (_data_->_async_result);
#line 210 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			return FALSE;
#line 1503 "wizard.c"
		}
		__finally95:
#line 205 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_inner_error_ != NULL) {
#line 205 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			block69_data_unref (_data_->_data69_);
#line 205 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_data69_ = NULL;
#line 205 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 205 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_clear_error (&_data_->_inner_error_);
#line 205 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			return FALSE;
#line 1518 "wizard.c"
		}
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_boxes_vm_creator_unref0 (_data_->self->priv->vm_creator);
#line 213 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->self->priv->vm_creator = NULL;
#line 214 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp15_ = _data_->self->priv->wizard_source;
#line 214 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_wizard_source_set_uri (_data_->_tmp15_, "");
#line 1528 "wizard.c"
	} else {
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp16_ = _data_->self->priv->source;
#line 216 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_iconfig_save ((BoxesIConfig*) _data_->_tmp16_);
#line 217 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp17_ = boxes_app_app;
#line 217 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp18_ = _data_->self->priv->source;
#line 217 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_app_add_collection_source (_data_->_tmp17_, _data_->_tmp18_, NULL, NULL);
#line 1540 "wizard.c"
	}
#line 220 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_set_machine (_data_->self, NULL);
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->result = TRUE;
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	block69_data_unref (_data_->_data69_);
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_data69_ = NULL;
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_state_ == 0) {
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 1554 "wizard.c"
	} else {
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 1558 "wizard.c"
	}
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_unref (_data_->_async_result);
#line 221 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return FALSE;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	block69_data_unref (_data_->_data69_);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_data69_ = NULL;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_state_ == 0) {
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 1572 "wizard.c"
	} else {
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 1576 "wizard.c"
	}
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_unref (_data_->_async_result);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return FALSE;
#line 1582 "wizard.c"
}


static void boxes_wizard_prepare_for_location (BoxesWizard* self, const gchar* location, gboolean probing, GError** error) {
	const gchar* _tmp0_;
	const gchar* _tmp2_;
	GFile* _tmp3_ = NULL;
	GFile* file;
	GFile* _tmp4_;
	gboolean _tmp5_ = FALSE;
	GError * _inner_error_ = NULL;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (location != NULL);
#line 225 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = location;
#line 225 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (g_strcmp0 (_tmp0_, "") == 0) {
#line 1602 "wizard.c"
		GError* _tmp1_;
#line 226 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp1_ = g_error_new_literal (BOXES_ERROR, BOXES_ERROR_INVALID, "empty location");
#line 226 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_inner_error_ = _tmp1_;
#line 226 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_propagate_error (error, _inner_error_);
#line 226 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return;
#line 1612 "wizard.c"
	}
#line 228 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = location;
#line 228 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = g_file_new_for_commandline_arg (_tmp2_);
#line 228 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	file = _tmp3_;
#line 230 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = file;
#line 230 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = g_file_is_native (_tmp4_);
#line 230 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp5_) {
#line 1626 "wizard.c"
		gboolean _tmp6_;
#line 232 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp6_ = probing;
#line 232 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (!_tmp6_) {
#line 1632 "wizard.c"
			GFile* _tmp7_;
			gchar* _tmp8_ = NULL;
			gchar* _tmp9_;
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp7_ = file;
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp8_ = g_file_get_path (_tmp7_);
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp9_ = _tmp8_;
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_prepare_for_installer (self, _tmp9_, &_inner_error_);
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_free0 (_tmp9_);
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_inner_error_ != NULL) {
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_propagate_error (error, _inner_error_);
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_object_unref0 (file);
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return;
#line 1654 "wizard.c"
			}
		}
	} else {
		gboolean uncertain = FALSE;
		GFile* _tmp10_;
		gchar* _tmp11_ = NULL;
		gchar* uri;
		const gchar* _tmp12_;
		gboolean _tmp13_ = FALSE;
		gchar* _tmp14_ = NULL;
		gchar* mimetype;
		gboolean _tmp15_;
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp10_ = file;
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp11_ = g_file_get_uri (_tmp10_);
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		uri = _tmp11_;
#line 238 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp12_ = uri;
#line 238 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp14_ = g_content_type_guess (_tmp12_, NULL, (gsize) 0, &_tmp13_);
#line 238 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		uncertain = _tmp13_;
#line 238 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		mimetype = _tmp14_;
#line 240 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp15_ = uncertain;
#line 240 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_tmp15_) {
#line 1685 "wizard.c"
			const gchar* _tmp16_;
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp16_ = uri;
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_prepare_for_uri (self, _tmp16_, &_inner_error_);
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_inner_error_ != NULL) {
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_propagate_error (error, _inner_error_);
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_free0 (mimetype);
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_free0 (uri);
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_object_unref0 (file);
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return;
#line 1703 "wizard.c"
			}
		} else {
			const gchar* _tmp17_;
			gchar* _tmp18_ = NULL;
			gchar* _tmp19_;
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp17_ = mimetype;
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp18_ = g_strdup_printf ("FIXME: %s", _tmp17_);
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp19_ = _tmp18_;
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_debug ("wizard.vala:243: %s", _tmp19_);
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_free0 (_tmp19_);
#line 1719 "wizard.c"
		}
#line 230 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_free0 (mimetype);
#line 230 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_free0 (uri);
#line 1725 "wizard.c"
	}
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (file);
#line 1729 "wizard.c"
}


static void boxes_wizard_prepare_for_uri (BoxesWizard* self, const gchar* uri_as_text, GError** error) {
	const gchar* _tmp0_;
	xmlURI* _tmp1_ = NULL;
	xmlURI* uri;
	xmlURI* _tmp2_;
	xmlURI* _tmp5_;
	const gchar* _tmp6_;
	gchar* _tmp7_;
	gchar* _tmp8_;
	const gchar* _tmp9_;
	const gchar* _tmp12_;
	xmlURI* _tmp13_;
	const gchar* _tmp14_;
	const gchar* _tmp15_;
	BoxesCollectionSource* _tmp16_;
	xmlURI* _tmp17_;
	const gchar* _tmp18_;
	GError * _inner_error_ = NULL;
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (uri_as_text != NULL);
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = uri_as_text;
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = xmlParseURI (_tmp0_);
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	uri = _tmp1_;
#line 250 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = uri;
#line 250 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp2_ == NULL) {
#line 1765 "wizard.c"
		const gchar* _tmp3_ = NULL;
		GError* _tmp4_;
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp3_ = _ ("Invalid URI");
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = g_error_new_literal (BOXES_ERROR, BOXES_ERROR_INVALID, _tmp3_);
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_inner_error_ = _tmp4_;
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_inner_error_->domain == BOXES_ERROR) {
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_propagate_error (error, _inner_error_);
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_xmlFreeURI0 (uri);
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			return;
#line 1782 "wizard.c"
		} else {
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_xmlFreeURI0 (uri);
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_clear_error (&_inner_error_);
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			return;
#line 1792 "wizard.c"
		}
	}
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = uri;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp6_ = _tmp5_->server;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp7_ = g_strdup (_tmp6_);
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp8_ = _tmp7_;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp9_ = _tmp8_;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp9_ == NULL) {
#line 1807 "wizard.c"
		const gchar* _tmp10_;
		gchar* _tmp11_;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp10_ = uri_as_text;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp11_ = g_strdup (_tmp10_);
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_free0 (_tmp8_);
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp8_ = _tmp11_;
#line 1818 "wizard.c"
	}
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp12_ = _tmp8_;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp13_ = uri;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp14_ = _tmp13_->scheme;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp15_ = uri_as_text;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp16_ = boxes_collection_source_new (_tmp12_, _tmp14_, _tmp15_);
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->source);
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->source = _tmp16_;
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp17_ = uri;
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp18_ = _tmp17_->scheme;
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (g_strcmp0 (_tmp18_, "spice") == 0) {
#line 1840 "wizard.c"
		const gchar* _tmp19_;
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp19_ = uri_as_text;
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		spice_validate_uri (_tmp19_, NULL, NULL, &_inner_error_);
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_inner_error_ != NULL) {
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_inner_error_->domain == BOXES_ERROR) {
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_propagate_error (error, _inner_error_);
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_free0 (_tmp8_);
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_xmlFreeURI0 (uri);
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return;
#line 1858 "wizard.c"
			} else {
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_free0 (_tmp8_);
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_xmlFreeURI0 (uri);
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_clear_error (&_inner_error_);
#line 256 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return;
#line 1870 "wizard.c"
			}
		}
	} else {
		xmlURI* _tmp20_;
		const gchar* _tmp21_;
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp20_ = uri;
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp21_ = _tmp20_->scheme;
#line 257 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (g_strcmp0 (_tmp21_, "vnc") == 0) {
#line 1882 "wizard.c"
		} else {
			xmlURI* _tmp22_;
			const gchar* _tmp23_;
			gboolean _tmp24_ = FALSE;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp22_ = uri;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp23_ = _tmp22_->scheme;
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp24_ = g_str_has_prefix (_tmp23_, "qemu");
#line 259 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_tmp24_) {
#line 1895 "wizard.c"
				BoxesCollectionSource* _tmp25_;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp25_ = self->priv->source;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_collection_source_set_source_type (_tmp25_, "libvirt");
#line 1901 "wizard.c"
			} else {
				BoxesApp* _tmp26_;
				xmlURI* _tmp27_;
				const gchar* _tmp28_;
				gboolean _tmp29_ = FALSE;
#line 262 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp26_ = boxes_app_app;
#line 262 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp27_ = uri;
#line 262 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp28_ = _tmp27_->scheme;
#line 262 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp29_ = boxes_app_has_broker_for_source_type (_tmp26_, _tmp28_);
#line 262 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_tmp29_) {
#line 1917 "wizard.c"
					BoxesCollectionSource* _tmp30_;
					xmlURI* _tmp31_;
					const gchar* _tmp32_;
#line 263 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp30_ = self->priv->source;
#line 263 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp31_ = uri;
#line 263 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp32_ = _tmp31_->scheme;
#line 263 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					boxes_collection_source_set_source_type (_tmp30_, _tmp32_);
#line 1929 "wizard.c"
				} else {
					const gchar* _tmp33_ = NULL;
					xmlURI* _tmp34_;
					const gchar* _tmp35_;
					gchar* _tmp36_ = NULL;
					gchar* _tmp37_;
					GError* _tmp38_;
					GError* _tmp39_;
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp33_ = _ ("Unsupported protocol '%s'");
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp34_ = uri;
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp35_ = _tmp34_->scheme;
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp36_ = g_strdup_printf (_tmp33_, _tmp35_);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp37_ = _tmp36_;
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp38_ = g_error_new_literal (BOXES_ERROR, BOXES_ERROR_INVALID, _tmp37_);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp39_ = _tmp38_;
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_g_free0 (_tmp37_);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_inner_error_ = _tmp39_;
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					if (_inner_error_->domain == BOXES_ERROR) {
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						g_propagate_error (error, _inner_error_);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_g_free0 (_tmp8_);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_xmlFreeURI0 (uri);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						return;
#line 1966 "wizard.c"
					} else {
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_g_free0 (_tmp8_);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_xmlFreeURI0 (uri);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						g_clear_error (&_inner_error_);
#line 265 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						return;
#line 1978 "wizard.c"
					}
				}
			}
		}
	}
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_free0 (_tmp8_);
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_xmlFreeURI0 (uri);
#line 1988 "wizard.c"
}


static Block70Data* block70_data_ref (Block70Data* _data70_) {
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_atomic_int_inc (&_data70_->_ref_count_);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return _data70_;
#line 1997 "wizard.c"
}


static void block70_data_unref (void * _userdata_) {
	Block70Data* _data70_;
	_data70_ = (Block70Data*) _userdata_;
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (g_atomic_int_dec_and_test (&_data70_->_ref_count_)) {
#line 2006 "wizard.c"
		BoxesWizard * self;
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		self = _data70_->self;
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_object_unref0 (_data70_->progress);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_object_unref0 (self);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_slice_free (Block70Data, _data70_);
#line 2016 "wizard.c"
	}
}


static void __lambda171_ (Block70Data* _data70_) {
	BoxesWizard * self;
	gdouble _tmp0_;
	gdouble _tmp1_;
	GtkProgressBar* _tmp2_;
	gdouble _tmp3_;
	gdouble _tmp4_;
#line 275 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = _data70_->self;
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = boxes_activity_progress_get_progress (_data70_->progress);
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = _tmp0_;
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = self->priv->prep_progress;
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = gtk_progress_bar_get_fraction (_tmp2_);
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = _tmp3_;
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if ((_tmp1_ - _tmp4_) >= 0.01) {
#line 2042 "wizard.c"
		GtkProgressBar* _tmp5_;
		gdouble _tmp6_;
		gdouble _tmp7_;
#line 277 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp5_ = self->priv->prep_progress;
#line 277 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp6_ = boxes_activity_progress_get_progress (_data70_->progress);
#line 277 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp7_ = _tmp6_;
#line 277 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		gtk_progress_bar_set_fraction (_tmp5_, _tmp7_);
#line 2054 "wizard.c"
	}
}


static void ___lambda171__g_object_notify (GObject* _sender, GParamSpec* pspec, gpointer self) {
#line 275 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	__lambda171_ (self);
#line 2062 "wizard.c"
}


static void _boxes_wizard_on_installer_recognized_boxes_media_manager_installer_recognized (OsinfoMedia* os_media, OsinfoOs* os, gpointer self) {
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_on_installer_recognized (self, os_media, os);
#line 2069 "wizard.c"
}


static void _boxes_wizard_on_installer_media_instantiated_gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self) {
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_on_installer_media_instantiated (self, source_object, res);
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_unref (self);
#line 2078 "wizard.c"
}


static void boxes_wizard_prepare_for_installer (BoxesWizard* self, const gchar* path, GError** error) {
	Block70Data* _data70_;
	GtkButton* _tmp0_;
	GtkLabel* _tmp1_;
	const gchar* _tmp2_ = NULL;
	GtkLabel* _tmp3_;
	const gchar* _tmp4_ = NULL;
	BoxesActivityProgress* _tmp5_;
	GtkLabel* _tmp6_;
	BoxesMediaManager* _tmp7_;
	const gchar* _tmp8_;
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (path != NULL);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data70_ = g_slice_new0 (Block70Data);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data70_->_ref_count_ = 1;
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data70_->self = g_object_ref (self);
#line 269 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->next_button;
#line 269 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_sensitive ((GtkWidget*) _tmp0_, FALSE);
#line 271 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = self->priv->prep_media_label;
#line 271 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = _ ("Unknown installer media");
#line 271 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_label (_tmp1_, _tmp2_);
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = self->priv->prep_status_label;
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = _ ("Analyzing...");
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_label (_tmp3_, _tmp4_);
#line 274 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = boxes_activity_progress_new ();
#line 274 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data70_->progress = _tmp5_;
#line 275 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_data ((GObject*) _data70_->progress, "notify::progress", (GCallback) ___lambda171__g_object_notify, block70_data_ref (_data70_), (GClosureNotify) block70_data_unref, 0);
#line 279 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp6_ = self->priv->prep_status_label;
#line 279 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_bind_property_with_closures ((GObject*) _data70_->progress, "info", (GObject*) _tmp6_, "label", G_BINDING_DEFAULT, (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)), (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)));
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp7_ = self->priv->media_manager;
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp8_ = path;
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_media_manager_create_installer_media_for_path (_tmp7_, _tmp8_, _boxes_wizard_on_installer_recognized_boxes_media_manager_installer_recognized, self, _data70_->progress, NULL, _boxes_wizard_on_installer_media_instantiated_gasync_ready_callback, g_object_ref (self));
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	block70_data_unref (_data70_);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data70_ = NULL;
#line 2139 "wizard.c"
}


static void boxes_wizard_on_installer_recognized (BoxesWizard* self, OsinfoMedia* os_media, OsinfoOs* os) {
	GtkLabel* _tmp0_;
	OsinfoOs* _tmp1_;
	const gchar* _tmp2_;
	const gchar* _tmp3_;
	GtkImage* _tmp4_;
	OsinfoOs* _tmp5_;
#line 288 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 288 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (os_media != NULL);
#line 288 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (os != NULL);
#line 289 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->prep_media_label;
#line 289 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = os;
#line 289 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = osinfo_product_get_name ((OsinfoProduct*) _tmp1_);
#line 289 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = _tmp2_;
#line 289 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_label (_tmp0_, _tmp3_);
#line 290 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = self->priv->installer_image;
#line 290 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = os;
#line 290 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_downloader_fetch_os_logo (_tmp4_, _tmp5_, 128, NULL, NULL);
#line 2172 "wizard.c"
}


static void boxes_wizard_on_installer_media_instantiated (BoxesWizard* self, GObject* source_object, GAsyncResult* _result_) {
	GtkButton* _tmp0_;
	GError * _inner_error_ = NULL;
#line 293 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 293 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (_result_ != NULL);
#line 294 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->next_button;
#line 294 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_sensitive ((GtkWidget*) _tmp0_, TRUE);
#line 2187 "wizard.c"
	{
		BoxesMediaManager* _tmp1_;
		GAsyncResult* _tmp2_;
		BoxesInstallerMedia* _tmp3_ = NULL;
		BoxesInstallerMedia* install_media;
		BoxesInstallerMedia* _tmp4_;
		BoxesVMCreator* _tmp5_;
		GtkProgressBar* _tmp6_;
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp1_ = self->priv->media_manager;
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp2_ = _result_;
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp3_ = boxes_media_manager_create_installer_media_for_path_finish (_tmp1_, _tmp2_, &_inner_error_);
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		install_media = _tmp3_;
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_inner_error_ != NULL) {
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (g_error_matches (_inner_error_, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
#line 2208 "wizard.c"
				goto __catch96_g_io_error_cancelled;
			}
			goto __catch96_g_error;
		}
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = install_media;
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp5_ = boxes_vm_creator_new (_tmp4_);
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_boxes_vm_creator_unref0 (self->priv->vm_creator);
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		self->priv->vm_creator = _tmp5_;
#line 299 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp6_ = self->priv->prep_progress;
#line 299 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		gtk_progress_bar_set_fraction (_tmp6_, 1.0);
#line 300 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_wizard_set_page (self, BOXES_WIZARD_PAGE_SETUP);
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_object_unref0 (install_media);
#line 2229 "wizard.c"
	}
	goto __finally96;
	__catch96_g_io_error_cancelled:
	{
		GError* cancel_error = NULL;
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		cancel_error = _inner_error_;
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_inner_error_ = NULL;
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_error_free0 (cancel_error);
#line 2241 "wizard.c"
	}
	goto __finally96;
	__catch96_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp7_;
		const gchar* _tmp8_;
		BoxesApp* _tmp9_;
		BoxesNotificationbar* _tmp10_;
		const gchar* _tmp11_ = NULL;
		GdNotification* _tmp12_ = NULL;
		GdNotification* _tmp13_;
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_error_ = _inner_error_;
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_inner_error_ = NULL;
#line 303 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp7_ = _error_;
#line 303 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp8_ = _tmp7_->message;
#line 303 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_debug ("wizard.vala:303: Failed to analyze installer image: %s", _tmp8_);
#line 304 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp9_ = boxes_app_app;
#line 304 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp10_ = _tmp9_->notificationbar;
#line 304 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp11_ = _ ("Failed to analyze installer media. Corrupted or incomplete media?");
#line 304 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp12_ = boxes_notificationbar_display_error (_tmp10_, _tmp11_, BOXES_NOTIFICATIONBAR_DEFAULT_TIMEOUT);
#line 304 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp13_ = _tmp12_;
#line 304 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_object_unref0 (_tmp13_);
#line 305 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_wizard_set_page (self, BOXES_WIZARD_PAGE_SOURCE);
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_error_free0 (_error_);
#line 2280 "wizard.c"
	}
	__finally96:
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_inner_error_ != NULL) {
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_clear_error (&_inner_error_);
#line 296 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return;
#line 2291 "wizard.c"
	}
}


static gboolean boxes_wizard_prepare (BoxesWizard* self) {
	gboolean result = FALSE;
	GtkImage* _tmp0_;
	BoxesWizardSource* _tmp1_;
	BoxesInstallerMedia* _tmp2_;
	BoxesInstallerMedia* _tmp3_;
	GError * _inner_error_ = NULL;
#line 309 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 310 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->installer_image;
#line 310 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_image_set_from_icon_name (_tmp0_, "media-optical", 0);
#line 312 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = self->priv->wizard_source;
#line 312 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = boxes_wizard_source_get_install_media (_tmp1_);
#line 312 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = _tmp2_;
#line 312 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp3_ != NULL) {
#line 2317 "wizard.c"
		BoxesWizardSource* _tmp4_;
		BoxesInstallerMedia* _tmp5_;
		BoxesInstallerMedia* _tmp6_;
		BoxesVMCreator* _tmp7_;
		GtkProgressBar* _tmp8_;
#line 313 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = self->priv->wizard_source;
#line 313 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp5_ = boxes_wizard_source_get_install_media (_tmp4_);
#line 313 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp6_ = _tmp5_;
#line 313 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp7_ = boxes_vm_creator_new (_tmp6_);
#line 313 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_boxes_vm_creator_unref0 (self->priv->vm_creator);
#line 313 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		self->priv->vm_creator = _tmp7_;
#line 314 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp8_ = self->priv->prep_progress;
#line 314 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		gtk_progress_bar_set_fraction (_tmp8_, 1.0);
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_wizard_set_page (self, BOXES_WIZARD_PAGE_SETUP);
#line 316 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		result = FALSE;
#line 316 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return result;
#line 2345 "wizard.c"
	} else {
		{
			BoxesWizardSource* _tmp9_;
			const gchar* _tmp10_;
			const gchar* _tmp11_;
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp9_ = self->priv->wizard_source;
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp10_ = boxes_wizard_source_get_uri (_tmp9_);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp11_ = _tmp10_;
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_prepare_for_location (self, _tmp11_, FALSE, &_inner_error_);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_inner_error_ != NULL) {
#line 2361 "wizard.c"
				goto __catch97_g_error;
			}
		}
		goto __finally97;
		__catch97_g_error:
		{
			GError* _error_ = NULL;
			BoxesApp* _tmp12_;
			BoxesNotificationbar* _tmp13_;
			GError* _tmp14_;
			const gchar* _tmp15_;
			GdNotification* _tmp16_ = NULL;
			GdNotification* _tmp17_;
#line 318 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_error_ = _inner_error_;
#line 318 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_inner_error_ = NULL;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp12_ = boxes_app_app;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp13_ = _tmp12_->notificationbar;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp14_ = _error_;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp15_ = _tmp14_->message;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp16_ = boxes_notificationbar_display_error (_tmp13_, _tmp15_, BOXES_NOTIFICATIONBAR_DEFAULT_TIMEOUT);
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp17_ = _tmp16_;
#line 321 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_object_unref0 (_tmp17_);
#line 323 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			result = FALSE;
#line 323 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_error_free0 (_error_);
#line 323 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			return result;
#line 2399 "wizard.c"
		}
		__finally97:
#line 318 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_inner_error_ != NULL) {
#line 318 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 318 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_clear_error (&_inner_error_);
#line 318 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			return FALSE;
#line 2410 "wizard.c"
		}
#line 326 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		result = TRUE;
#line 326 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return result;
#line 2416 "wizard.c"
	}
}


static gboolean boxes_wizard_setup (BoxesWizard* self) {
	gboolean result = FALSE;
	BoxesCollectionSource* _tmp0_;
	BoxesVMCreator* _tmp1_;
	BoxesVMCreator* _tmp2_;
	BoxesInstallerMedia* _tmp3_;
	BoxesInstallerMedia* _tmp4_;
	GtkButton* _tmp5_;
	BoxesVMCreator* _tmp6_;
	BoxesInstallerMedia* _tmp7_;
	BoxesInstallerMedia* _tmp8_;
	GtkBox* _tmp9_;
#line 330 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->source;
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp0_ != NULL) {
#line 333 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		result = TRUE;
#line 333 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return result;
#line 2443 "wizard.c"
	}
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = self->priv->vm_creator;
#line 335 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (_tmp1_ != NULL);
#line 337 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = self->priv->vm_creator;
#line 337 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = boxes_vm_creator_get_install_media (_tmp2_);
#line 337 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = _tmp3_;
#line 337 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = self->priv->continue_button;
#line 337 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_bind_property_with_closures ((GObject*) _tmp4_, "ready-to-create", (GObject*) _tmp5_, "sensitive", G_BINDING_SYNC_CREATE, (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)), (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)));
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp6_ = self->priv->vm_creator;
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp7_ = boxes_vm_creator_get_install_media (_tmp6_);
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp8_ = _tmp7_;
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp9_ = self->priv->setup_vbox;
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_installer_media_populate_setup_vbox (_tmp8_, _tmp9_);
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = TRUE;
#line 342 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 2473 "wizard.c"
}


static void boxes_wizard_review_data_free (gpointer _data) {
	BoxesWizardReviewData* _data_;
	_data_ = _data;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (_data_->self);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_slice_free (BoxesWizardReviewData, _data_);
#line 2484 "wizard.c"
}


static void boxes_wizard_review (BoxesWizard* self, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesWizardReviewData* _data_;
	BoxesWizard* _tmp0_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_ = g_slice_new0 (BoxesWizardReviewData);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_wizard_review);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_wizard_review_data_free);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->self = _tmp0_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_review_co (_data_);
#line 2503 "wizard.c"
}


static gboolean boxes_wizard_review_finish (BoxesWizard* self, GAsyncResult* _res_) {
	gboolean result;
	BoxesWizardReviewData* _data_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = _data_->result;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 2516 "wizard.c"
}


static void boxes_wizard_review_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesWizardReviewData* _data_;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_ = _user_data_;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_source_object_ = source_object;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_res_ = _res_;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_review_co (_data_);
#line 2530 "wizard.c"
}


static gboolean boxes_wizard_review_co (BoxesWizardReviewData* _data_) {
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	switch (_data_->_state_) {
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case 0:
#line 2539 "wizard.c"
		goto _state_0;
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case 1:
#line 2543 "wizard.c"
		goto _state_1;
		default:
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_assert_not_reached ();
#line 2548 "wizard.c"
	}
	_state_0:
#line 349 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp0_ = _data_->self->priv->review_cancellable;
#line 349 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (_data_->_tmp0_ == NULL);
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp1_ = g_cancellable_new ();
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (_data_->self->priv->review_cancellable);
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->self->priv->review_cancellable = _data_->_tmp1_;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_state_ = 1;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_do_review_cancellable (_data_->self, boxes_wizard_review_ready, _data_);
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return FALSE;
#line 2567 "wizard.c"
	_state_1:
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp2_ = FALSE;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp2_ = boxes_wizard_do_review_cancellable_finish (_data_->self, _data_->_res_);
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_result_ = _data_->_tmp2_;
#line 353 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (_data_->self->priv->review_cancellable);
#line 353 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->self->priv->review_cancellable = NULL;
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->self->priv->skip_review_for_live = FALSE;
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->result = _data_->_result_;
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_state_ == 0) {
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 2587 "wizard.c"
	} else {
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 2591 "wizard.c"
	}
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_unref (_data_->_async_result);
#line 356 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return FALSE;
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_state_ == 0) {
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 2601 "wizard.c"
	} else {
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 2605 "wizard.c"
	}
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_unref (_data_->_async_result);
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return FALSE;
#line 2611 "wizard.c"
}


static void boxes_wizard_do_review_cancellable_data_free (gpointer _data) {
	BoxesWizardDoReviewCancellableData* _data_;
	_data_ = _data;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (_data_->self);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_slice_free (BoxesWizardDoReviewCancellableData, _data_);
#line 2622 "wizard.c"
}


static void boxes_wizard_do_review_cancellable (BoxesWizard* self, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesWizardDoReviewCancellableData* _data_;
	BoxesWizard* _tmp0_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_ = g_slice_new0 (BoxesWizardDoReviewCancellableData);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_wizard_do_review_cancellable);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_wizard_do_review_cancellable_data_free);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->self = _tmp0_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_do_review_cancellable_co (_data_);
#line 2641 "wizard.c"
}


static gboolean boxes_wizard_do_review_cancellable_finish (BoxesWizard* self, GAsyncResult* _res_) {
	gboolean result;
	BoxesWizardDoReviewCancellableData* _data_;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = _data_->result;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 2654 "wizard.c"
}


static void boxes_wizard_do_review_cancellable_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesWizardDoReviewCancellableData* _data_;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_ = _user_data_;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_source_object_ = source_object;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_res_ = _res_;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_do_review_cancellable_co (_data_);
#line 2668 "wizard.c"
}


static gpointer _boxes_pair_ref0 (gpointer self) {
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return self ? boxes_pair_ref (self) : NULL;
#line 2675 "wizard.c"
}


static void _boxes_pair_unref0_ (gpointer var) {
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	(var == NULL) ? NULL : (var = (boxes_pair_unref (var), NULL));
#line 2682 "wizard.c"
}


static void _g_list_free__boxes_pair_unref0_ (GList* self) {
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_list_foreach (self, (GFunc) _boxes_pair_unref0_, NULL);
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_list_free (self);
#line 2691 "wizard.c"
}


static Block71Data* block71_data_ref (Block71Data* _data71_) {
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_atomic_int_inc (&_data71_->_ref_count_);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return _data71_;
#line 2700 "wizard.c"
}


static void block71_data_unref (void * _userdata_) {
	Block71Data* _data71_;
	_data71_ = (Block71Data*) _userdata_;
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (g_atomic_int_dec_and_test (&_data71_->_ref_count_)) {
#line 2709 "wizard.c"
		BoxesWizard * self;
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		self = _data71_->self;
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_object_unref0 (_data71_->config);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_object_unref0 (self);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_slice_free (Block71Data, _data71_);
#line 2719 "wizard.c"
	}
}


static void _____lambda173_ (Block71Data* _data71_, GError** error) {
	BoxesWizard * self;
	BoxesLibvirtMachine* _tmp0_;
	BoxesLibvirtMachine* _tmp1_;
	GVirDomain* _tmp2_;
	GVirDomain* _tmp3_;
	GVirConfigDomain* _tmp4_ = NULL;
	GVirConfigDomain* _tmp5_;
	GError * _inner_error_ = NULL;
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = _data71_->self;
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = boxes_wizard_get_libvirt_machine (self);
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = _tmp0_;
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = boxes_libvirt_machine_get_domain (_tmp1_);
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = _tmp2_;
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = gvir_domain_get_config (_tmp3_, (guint) GVIR_DOMAIN_XML_INACTIVE, &_inner_error_);
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = _tmp4_;
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_inner_error_ != NULL) {
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_propagate_error (error, _inner_error_);
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return;
#line 2753 "wizard.c"
	}
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (_data71_->config);
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data71_->config = _tmp5_;
#line 2759 "wizard.c"
}


static void ______lambda173__boxes_run_in_thread_func (gpointer self, GError** error) {
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_____lambda173_ (self, error);
#line 2766 "wizard.c"
}


static void _vala_GVirStorageVolInfo_free (GVirStorageVolInfo* self) {
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_boxed_free (gvir_storage_vol_info_get_type (), self);
#line 2773 "wizard.c"
}


static void __lambda174_ (BoxesWizard* self) {
	BoxesApp* _tmp0_;
	BoxesMachine* _tmp1_;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = boxes_app_app;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = self->priv->_machine;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_app_select_item (_tmp0_, (BoxesCollectionItem*) _tmp1_);
#line 2786 "wizard.c"
}


static void ___lambda174__boxes_wizard_wizard_summary_customize_func (gpointer self) {
#line 456 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	__lambda174_ (self);
#line 2793 "wizard.c"
}


static gboolean boxes_wizard_do_review_cancellable_co (BoxesWizardDoReviewCancellableData* _data_) {
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	switch (_data_->_state_) {
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case 0:
#line 2802 "wizard.c"
		goto _state_0;
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case 1:
#line 2806 "wizard.c"
		goto _state_1;
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case 2:
#line 2810 "wizard.c"
		goto _state_2;
		default:
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_assert_not_reached ();
#line 2815 "wizard.c"
	}
	_state_0:
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp0_ = _data_->self->priv->review_cancellable;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (_data_->_tmp0_ != NULL);
#line 362 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp1_ = _data_->self->priv->nokvm_label;
#line 362 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_hide ((GtkWidget*) _data_->_tmp1_);
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp2_ = _data_->self->priv->summary;
#line 363 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_wizard_summary_clear (_data_->_tmp2_);
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp3_ = _data_->self->priv->source;
#line 365 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_tmp3_ != NULL) {
#line 2834 "wizard.c"
		{
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp4_ = _data_->self->priv->source;
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp5_ = boxes_remote_machine_new (_data_->_tmp4_, &_data_->_inner_error_);
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp6_ = _data_->_tmp5_;
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_data_->_inner_error_ != NULL) {
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_data_->_inner_error_->domain == BOXES_ERROR) {
#line 2846 "wizard.c"
					goto __catch98_boxes_error;
				}
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_clear_error (&_data_->_inner_error_);
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return FALSE;
#line 2855 "wizard.c"
			}
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp7_ = _data_->_tmp6_;
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_set_machine (_data_->self, (BoxesMachine*) _data_->_tmp7_);
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_object_unref0 (_data_->_tmp7_);
#line 2863 "wizard.c"
		}
		goto __finally98;
		__catch98_boxes_error:
		{
#line 366 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_error_ = _data_->_inner_error_;
#line 366 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_inner_error_ = NULL;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp8_ = _data_->_error_;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp9_ = _data_->_tmp8_->message;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_warning ("wizard.vala:369: %s", _data_->_tmp9_);
#line 366 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_error_free0 (_data_->_error_);
#line 2880 "wizard.c"
		}
		__finally98:
#line 366 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_inner_error_ != NULL) {
#line 366 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 366 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_clear_error (&_data_->_inner_error_);
#line 366 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			return FALSE;
#line 2891 "wizard.c"
		}
	} else {
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp11_ = _data_->self->priv->vm_creator;
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_tmp11_ != NULL) {
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp12_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp13_ = _data_->_tmp12_;
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp10_ = _data_->_tmp13_ == NULL;
#line 2904 "wizard.c"
		} else {
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp10_ = FALSE;
#line 2908 "wizard.c"
		}
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp14_ = _data_->_tmp10_;
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_tmp14_) {
#line 2914 "wizard.c"
			{
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp15_ = _data_->self->priv->vm_creator;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp16_ = _data_->self->priv->review_cancellable;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_state_ = 1;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_vm_creator_create_vm (_data_->_tmp15_, _data_->_tmp16_, boxes_wizard_do_review_cancellable_ready, _data_);
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return FALSE;
#line 2926 "wizard.c"
				_state_1:
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp17_ = NULL;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp17_ = boxes_vm_creator_create_vm_finish (_data_->_tmp15_, _data_->_res_, &_data_->_inner_error_);
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp18_ = _data_->_tmp17_;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_data_->_inner_error_ != NULL) {
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					if (g_error_matches (_data_->_inner_error_, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
#line 2938 "wizard.c"
						goto __catch99_g_io_error_cancelled;
					}
					goto __catch99_g_error;
				}
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp19_ = _data_->_tmp18_;
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_wizard_set_machine (_data_->self, (BoxesMachine*) _data_->_tmp19_);
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_object_unref0 (_data_->_tmp19_);
#line 2949 "wizard.c"
			}
			goto __finally99;
			__catch99_g_io_error_cancelled:
			{
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->cancel_error = _data_->_inner_error_;
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_inner_error_ = NULL;
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_error_free0 (_data_->cancel_error);
#line 2960 "wizard.c"
			}
			goto __finally99;
			__catch99_g_error:
			{
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_vala1__error_ = _data_->_inner_error_;
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_inner_error_ = NULL;
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp20_ = boxes_app_app;
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp21_ = _data_->_tmp20_->notificationbar;
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp22_ = NULL;
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp22_ = _ ("Box setup failed");
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp23_ = NULL;
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp23_ = boxes_notificationbar_display_error (_data_->_tmp21_, _data_->_tmp22_, BOXES_NOTIFICATIONBAR_DEFAULT_TIMEOUT);
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp24_ = _data_->_tmp23_;
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_object_unref0 (_data_->_tmp24_);
#line 377 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp25_ = _data_->_vala1__error_;
#line 377 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp26_ = _data_->_tmp25_->message;
#line 377 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_warning ("wizard.vala:377: %s", _data_->_tmp26_);
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_error_free0 (_data_->_vala1__error_);
#line 2993 "wizard.c"
			}
			__finally99:
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_data_->_inner_error_ != NULL) {
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_clear_error (&_data_->_inner_error_);
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return FALSE;
#line 3004 "wizard.c"
			}
#line 380 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp27_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 380 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp28_ = _data_->_tmp27_;
#line 380 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_data_->_tmp28_ == NULL) {
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_object_notify ((GObject*) _data_->self, "machine");
#line 383 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->result = FALSE;
#line 383 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_data_->_state_ == 0) {
#line 383 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 3020 "wizard.c"
				} else {
#line 383 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					g_simple_async_result_complete (_data_->_async_result);
#line 3024 "wizard.c"
				}
#line 383 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_object_unref (_data_->_async_result);
#line 383 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return FALSE;
#line 3030 "wizard.c"
			}
		}
	}
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp29_ = _data_->self->priv->review_cancellable;
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp30_ = FALSE;
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp30_ = g_cancellable_is_cancelled (_data_->_tmp29_);
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_tmp30_) {
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->result = FALSE;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_state_ == 0) {
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 3048 "wizard.c"
		} else {
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 3052 "wizard.c"
		}
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_object_unref (_data_->_async_result);
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return FALSE;
#line 3058 "wizard.c"
	}
#line 390 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp31_ = _data_->self->priv->review_label;
#line 390 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp32_ = NULL;
#line 390 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp32_ = _ ("Will create a new box with the following properties:");
#line 390 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_text (_data_->_tmp31_, _data_->_tmp32_);
#line 392 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp33_ = _data_->self->priv->source;
#line 392 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_tmp33_ != NULL) {
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		static GQuark _tmp68_label0 = 0;
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		static GQuark _tmp68_label1 = 0;
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp34_ = _data_->self->priv->source;
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp35_ = boxes_collection_source_get_uri (_data_->_tmp34_);
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp36_ = _data_->_tmp35_;
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp37_ = _data_->_tmp36_;
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp38_ = NULL;
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp38_ = xmlParseURI (_data_->_tmp37_);
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp39_ = _data_->_tmp38_;
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_free0 (_data_->_tmp37_);
#line 393 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->uri = _data_->_tmp39_;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp40_ = _data_->self->priv->summary;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp41_ = NULL;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp41_ = _ ("Type");
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp42_ = _data_->self->priv->source;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp43_ = boxes_collection_source_get_source_type (_data_->_tmp42_);
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp44_ = _data_->_tmp43_;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp45_ = _data_->_tmp44_;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_wizard_wizard_summary_add_property (_data_->_tmp40_, _data_->_tmp41_, _data_->_tmp45_);
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_free0 (_data_->_tmp45_);
#line 397 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp47_ = _data_->uri;
#line 397 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_tmp47_ != NULL) {
#line 397 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp48_ = _data_->uri;
#line 397 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp49_ = _data_->_tmp48_->server;
#line 397 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp46_ = _data_->_tmp49_ != NULL;
#line 3122 "wizard.c"
		} else {
#line 397 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp46_ = FALSE;
#line 3126 "wizard.c"
		}
#line 397 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp50_ = _data_->_tmp46_;
#line 397 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_tmp50_) {
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp51_ = _data_->self->priv->summary;
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp52_ = NULL;
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp52_ = _ ("Host");
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp53_ = _data_->uri;
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp54_ = _data_->_tmp53_->server;
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp55_ = NULL;
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp55_ = g_utf8_strdown (_data_->_tmp54_, (gssize) (-1));
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp56_ = _data_->_tmp55_;
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_wizard_summary_add_property (_data_->_tmp51_, _data_->_tmp52_, _data_->_tmp56_);
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_free0 (_data_->_tmp56_);
#line 3152 "wizard.c"
		} else {
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp57_ = _data_->self->priv->summary;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp58_ = NULL;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp58_ = _ ("URI");
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp59_ = _data_->self->priv->source;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp60_ = boxes_collection_source_get_uri (_data_->_tmp59_);
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp61_ = _data_->_tmp60_;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp62_ = _data_->_tmp61_;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp63_ = NULL;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp63_ = g_utf8_strdown (_data_->_tmp62_, (gssize) (-1));
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp64_ = _data_->_tmp63_;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_wizard_summary_add_property (_data_->_tmp57_, _data_->_tmp58_, _data_->_tmp64_);
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_free0 (_data_->_tmp64_);
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_free0 (_data_->_tmp62_);
#line 3180 "wizard.c"
		}
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp65_ = _data_->uri;
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp66_ = _data_->_tmp65_->scheme;
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp69_ = 0U;
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp67_ = _data_->_tmp66_;
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp69_ = (NULL == _data_->_tmp67_) ? 0 : g_quark_from_string (_data_->_tmp67_);
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_tmp69_ == ((0 != _tmp68_label0) ? _tmp68_label0 : (_tmp68_label0 = g_quark_from_static_string ("spice")))) {
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			switch (0) {
#line 3196 "wizard.c"
				default:
				{
					{
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->port = 0;
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->tls_port = 0;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp70_ = _data_->self->priv->source;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp71_ = boxes_collection_source_get_uri (_data_->_tmp70_);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp72_ = _data_->_tmp71_;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp73_ = _data_->_tmp72_;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp74_ = NULL;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp75_ = NULL;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						spice_validate_uri (_data_->_tmp73_, &_data_->_tmp74_, &_data_->_tmp75_, &_data_->_inner_error_);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp76_ = _data_->_tmp74_;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->port = *_data_->_tmp76_;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp77_ = _data_->_tmp75_;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->tls_port = *_data_->_tmp77_;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_g_free0 (_data_->_tmp77_);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_g_free0 (_data_->_tmp76_);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_g_free0 (_data_->_tmp73_);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						if (_data_->_inner_error_ != NULL) {
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							if (_data_->_inner_error_->domain == BOXES_ERROR) {
#line 3236 "wizard.c"
								goto __catch100_boxes_error;
							}
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_xmlFreeURI0 (_data_->uri);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							g_clear_error (&_data_->_inner_error_);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							return FALSE;
#line 3247 "wizard.c"
						}
#line 408 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp78_ = _data_->port;
#line 408 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						if (_data_->_tmp78_ > 0) {
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp79_ = _data_->self->priv->summary;
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp80_ = NULL;
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp80_ = _ ("Port");
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp81_ = _data_->port;
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp82_ = NULL;
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp82_ = g_strdup_printf ("%i", _data_->_tmp81_);
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp83_ = _data_->_tmp82_;
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							boxes_wizard_wizard_summary_add_property (_data_->_tmp79_, _data_->_tmp80_, _data_->_tmp83_);
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_g_free0 (_data_->_tmp83_);
#line 3271 "wizard.c"
						}
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp84_ = _data_->tls_port;
#line 410 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						if (_data_->_tmp84_ > 0) {
#line 411 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp85_ = _data_->self->priv->summary;
#line 411 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp86_ = NULL;
#line 411 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp86_ = _ ("TLS Port");
#line 411 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp87_ = _data_->tls_port;
#line 411 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp88_ = NULL;
#line 411 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp88_ = g_strdup_printf ("%i", _data_->_tmp87_);
#line 411 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_data_->_tmp89_ = _data_->_tmp88_;
#line 411 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							boxes_wizard_wizard_summary_add_property (_data_->_tmp85_, _data_->_tmp86_, _data_->_tmp89_);
#line 411 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
							_g_free0 (_data_->_tmp89_);
#line 3295 "wizard.c"
						}
					}
					goto __finally100;
					__catch100_boxes_error:
					{
#line 404 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_vala2__error_ = _data_->_inner_error_;
#line 404 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_inner_error_ = NULL;
#line 414 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp90_ = _data_->_vala2__error_;
#line 414 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp91_ = _data_->_tmp90_->message;
#line 414 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						g_critical ("wizard.vala:414: %s", _data_->_tmp91_);
#line 404 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_g_error_free0 (_data_->_vala2__error_);
#line 3313 "wizard.c"
					}
					__finally100:
#line 404 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					if (_data_->_inner_error_ != NULL) {
#line 404 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_xmlFreeURI0 (_data_->uri);
#line 404 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 404 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						g_clear_error (&_data_->_inner_error_);
#line 404 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						return FALSE;
#line 3326 "wizard.c"
					}
#line 416 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					break;
#line 3330 "wizard.c"
				}
			}
		} else if (_data_->_tmp69_ == ((0 != _tmp68_label1) ? _tmp68_label1 : (_tmp68_label1 = g_quark_from_static_string ("vnc")))) {
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			switch (0) {
#line 3336 "wizard.c"
				default:
				{
#line 419 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp92_ = _data_->uri;
#line 419 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp93_ = _data_->_tmp92_->port;
#line 419 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					if (_data_->_tmp93_ > 0) {
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp94_ = _data_->self->priv->summary;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp95_ = NULL;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp95_ = _ ("Port");
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp96_ = _data_->uri;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp97_ = _data_->_tmp96_->port;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp98_ = NULL;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp98_ = g_strdup_printf ("%i", _data_->_tmp97_);
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp99_ = _data_->_tmp98_;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						boxes_wizard_wizard_summary_add_property (_data_->_tmp94_, _data_->_tmp95_, _data_->_tmp99_);
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_g_free0 (_data_->_tmp99_);
#line 3365 "wizard.c"
					}
#line 421 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					break;
#line 3369 "wizard.c"
				}
			}
		}
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp100_ = boxes_app_app;
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp101_ = _data_->self->priv->source;
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp102_ = boxes_collection_source_get_source_type (_data_->_tmp101_);
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp103_ = _data_->_tmp102_;
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp104_ = _data_->_tmp103_;
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp105_ = FALSE;
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp105_ = boxes_app_has_broker_for_source_type (_data_->_tmp100_, _data_->_tmp104_);
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp106_ = _data_->_tmp105_;
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_free0 (_data_->_tmp104_);
#line 424 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_tmp106_) {
#line 425 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp107_ = _data_->self->priv->review_label;
#line 425 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp108_ = NULL;
#line 425 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp108_ = _ ("Will add boxes for all systems available from this account:");
#line 425 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			gtk_label_set_text (_data_->_tmp107_, _data_->_tmp108_);
#line 3401 "wizard.c"
		}
#line 392 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_xmlFreeURI0 (_data_->uri);
#line 3405 "wizard.c"
	} else {
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp109_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data_->_tmp110_ = _data_->_tmp109_;
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_data_->_tmp110_ != NULL) {
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp111_ = _data_->self->priv->vm_creator;
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp112_ = boxes_vm_creator_get_install_media (_data_->_tmp111_);
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp113_ = _data_->_tmp112_;
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp114_ = NULL;
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp114_ = boxes_installer_media_get_vm_properties (_data_->_tmp113_);
#line 3423 "wizard.c"
			{
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->property_collection = _data_->_tmp114_;
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				for (_data_->property_it = _data_->property_collection; _data_->property_it != NULL; _data_->property_it = _data_->property_it->next) {
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp115_ = _boxes_pair_ref0 ((BoxesPair*) _data_->property_it->data);
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->property = _data_->_tmp115_;
#line 3433 "wizard.c"
					{
#line 429 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp116_ = _data_->self->priv->summary;
#line 429 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp117_ = _data_->property;
#line 429 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp118_ = _data_->_tmp117_->first;
#line 429 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp119_ = _data_->property;
#line 429 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_data_->_tmp120_ = _data_->_tmp119_->second;
#line 429 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						boxes_wizard_wizard_summary_add_property (_data_->_tmp116_, (const gchar*) _data_->_tmp118_, (const gchar*) _data_->_tmp120_);
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
						_boxes_pair_unref0 (_data_->property);
#line 3449 "wizard.c"
					}
				}
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				__g_list_free__boxes_pair_unref0_0 (_data_->property_collection);
#line 3454 "wizard.c"
			}
			{
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_data71_ = g_slice_new0 (Block71Data);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_data71_->_ref_count_ = 1;
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_data71_->self = g_object_ref (_data_->self);
#line 432 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp121_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (NULL, gvir_config_domain_get_type ()) ? ((GVirConfigDomain*) NULL) : NULL);
#line 432 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_data71_->config = _data_->_tmp121_;
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_state_ = 2;
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_run_in_thread (______lambda173__boxes_run_in_thread_func, block71_data_ref (_data_->_data71_), block71_data_unref, NULL, boxes_wizard_do_review_cancellable_ready, _data_);
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return FALSE;
#line 3473 "wizard.c"
				_state_2:
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_run_in_thread_finish (_data_->_res_, &_data_->_inner_error_);
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_data_->_inner_error_ != NULL) {
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					block71_data_unref (_data_->_data71_);
#line 433 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_data71_ = NULL;
#line 3483 "wizard.c"
					goto __catch101_g_error;
				}
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp122_ = _data_->_data71_->config;
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp123_ = gvir_config_domain_get_memory (_data_->_tmp122_);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp124_ = _data_->_tmp123_;
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp125_ = NULL;
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp125_ = g_format_size_full (_data_->_tmp124_ * OSINFO_KIBIBYTES, G_FORMAT_SIZE_IEC_UNITS);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->memory = _data_->_tmp125_;
#line 436 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp126_ = _data_->self->priv->summary;
#line 436 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp127_ = NULL;
#line 436 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp127_ = _ ("Memory");
#line 436 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp128_ = _data_->memory;
#line 436 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_wizard_wizard_summary_add_property (_data_->_tmp126_, _data_->_tmp127_, _data_->_tmp128_);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_free0 (_data_->memory);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				block71_data_unref (_data_->_data71_);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_data71_ = NULL;
#line 3514 "wizard.c"
			}
			goto __finally101;
			__catch101_g_error:
			{
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_vala3__error_ = _data_->_inner_error_;
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_inner_error_ = NULL;
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp129_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp130_ = _data_->_tmp129_;
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp131_ = boxes_collection_item_get_name ((BoxesCollectionItem*) _data_->_tmp130_);
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp132_ = _data_->_tmp131_;
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp133_ = _data_->_vala3__error_;
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_data_->_tmp134_ = _data_->_tmp133_->message;
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_warning ("wizard.vala:438: Failed to get configuration for machine '%s': %s", _data_->_tmp132_, _data_->_tmp134_);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_error_free0 (_data_->_vala3__error_);
#line 3539 "wizard.c"
			}
			__finally101:
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_data_->_inner_error_ != NULL) {
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				g_clear_error (&_data_->_inner_error_);
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return FALSE;
#line 3550 "wizard.c"
			}
#line 441 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp135_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 441 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp136_ = _data_->_tmp135_;
#line 441 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp137_ = boxes_libvirt_machine_get_storage_volume (_data_->_tmp136_);
#line 441 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp138_ = _data_->_tmp137_;
#line 441 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_data_->_tmp138_ != NULL) {
#line 3562 "wizard.c"
				{
#line 443 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp139_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 443 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp140_ = _data_->_tmp139_;
#line 443 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp141_ = boxes_libvirt_machine_get_storage_volume (_data_->_tmp140_);
#line 443 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp142_ = _data_->_tmp141_;
#line 443 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp143_ = NULL;
#line 443 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp143_ = gvir_storage_vol_get_info (_data_->_tmp142_, &_data_->_inner_error_);
#line 443 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->volume_info = _data_->_tmp143_;
#line 443 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					if (_data_->_inner_error_ != NULL) {
#line 3580 "wizard.c"
						goto __catch102_g_error;
					}
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp144_ = _data_->volume_info;
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp145_ = _data_->_tmp144_->capacity;
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp146_ = NULL;
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp146_ = g_format_size_full (_data_->_tmp145_, G_FORMAT_SIZE_DEFAULT);
#line 444 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->capacity = _data_->_tmp146_;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp147_ = _data_->self->priv->summary;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp148_ = NULL;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp148_ = _ ("Disk");
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp149_ = _data_->capacity;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp150_ = NULL;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp150_ = g_strdup_printf ("%s maximum", _data_->_tmp149_);
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp151_ = _data_->_tmp150_;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp152_ = NULL;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp152_ = _ (_data_->_tmp151_);
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					boxes_wizard_wizard_summary_add_property (_data_->_tmp147_, _data_->_tmp148_, _data_->_tmp152_);
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_g_free0 (_data_->_tmp151_);
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_g_free0 (_data_->capacity);
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					__vala_GVirStorageVolInfo_free0 (_data_->volume_info);
#line 3619 "wizard.c"
				}
				goto __finally102;
				__catch102_g_error:
				{
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_vala4__error_ = _data_->_inner_error_;
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_inner_error_ = NULL;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp153_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp154_ = _data_->_tmp153_;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp155_ = boxes_libvirt_machine_get_storage_volume (_data_->_tmp154_);
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp156_ = _data_->_tmp155_;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp157_ = NULL;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp157_ = gvir_storage_vol_get_name (_data_->_tmp156_);
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp158_ = _data_->_vala4__error_;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_data_->_tmp159_ = _data_->_tmp158_->message;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					g_warning ("wizard.vala:447: Failed to get information on volume '%s': %s", _data_->_tmp157_, _data_->_tmp159_);
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_g_error_free0 (_data_->_vala4__error_);
#line 3648 "wizard.c"
				}
				__finally102:
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (_data_->_inner_error_ != NULL) {
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					g_clear_error (&_data_->_inner_error_);
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					return FALSE;
#line 3659 "wizard.c"
				}
			}
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp160_ = _data_->self->priv->nokvm_label;
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp161_ = boxes_wizard_get_libvirt_machine (_data_->self);
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp162_ = _data_->_tmp161_;
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp163_ = boxes_libvirt_machine_get_domain_config (_data_->_tmp162_);
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp164_ = _data_->_tmp163_;
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp165_ = 0;
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_data_->_tmp165_ = gvir_config_domain_get_virt_type (_data_->_tmp164_);
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			gtk_widget_set_visible ((GtkWidget*) _data_->_tmp160_, _data_->_tmp165_ != GVIR_CONFIG_DOMAIN_VIRT_KVM);
#line 3678 "wizard.c"
		}
	}
#line 456 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->_tmp166_ = _data_->self->priv->summary;
#line 456 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_wizard_summary_append_customize_button (_data_->_tmp166_, ___lambda174__boxes_wizard_wizard_summary_customize_func, _data_->self);
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data_->result = TRUE;
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_state_ == 0) {
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 3691 "wizard.c"
	} else {
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 3695 "wizard.c"
	}
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_unref (_data_->_async_result);
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return FALSE;
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_data_->_state_ == 0) {
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 3705 "wizard.c"
	} else {
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 3709 "wizard.c"
	}
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_unref (_data_->_async_result);
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return FALSE;
#line 3715 "wizard.c"
}


static void g_ptr_array_set (GPtrArray* self, guint index, gpointer data) {
	gpointer _tmp0_;
	guint _tmp1_;
#line 4059 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_return_if_fail (self != NULL);
#line 4060 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	_tmp0_ = data;
#line 4060 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	data = NULL;
#line 4060 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_ptr_array_add (self, _tmp0_);
#line 4061 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	_tmp1_ = index;
#line 4061 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_ptr_array_remove_index_fast (self, _tmp1_);
#line 3734 "wizard.c"
}


static void boxes_wizard_add_step (BoxesWizard* self, GtkWidget* widget, const gchar* title, BoxesWizardPage page) {
	GtkNotebook* _tmp0_;
	GtkWidget* _tmp1_;
	BoxesApp* _tmp2_;
	BoxesSidebar* _tmp3_;
	GtkNotebook* _tmp4_;
	GtkWidget* _tmp5_ = NULL;
	GtkBox* _tmp6_;
	GtkBox* vbox;
	const gchar* _tmp7_;
	GtkLabel* _tmp8_;
	GtkLabel* label;
	GtkStyleContext* _tmp9_ = NULL;
	GPtrArray* _tmp10_;
	BoxesWizardPage _tmp11_;
	GtkLabel* _tmp12_;
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (widget != NULL);
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (title != NULL);
#line 465 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->notebook;
#line 465 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = widget;
#line 465 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_notebook_append_page (_tmp0_, _tmp1_, NULL);
#line 468 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = boxes_app_app;
#line 468 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = _tmp2_->sidebar;
#line 468 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = _tmp3_->notebook;
#line 468 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = gtk_notebook_get_nth_page (_tmp4_, (gint) BOXES_SIDEBAR_PAGE_WIZARD);
#line 468 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp6_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, GTK_TYPE_BOX) ? ((GtkBox*) _tmp5_) : NULL);
#line 468 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	vbox = _tmp6_;
#line 470 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp7_ = title;
#line 470 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp8_ = (GtkLabel*) gtk_label_new (_tmp7_);
#line 470 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp8_);
#line 470 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	label = _tmp8_;
#line 471 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_margin_left ((GtkWidget*) label, 25);
#line 472 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp9_ = gtk_widget_get_style_context ((GtkWidget*) label);
#line 472 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp9_, "boxes-step-label");
#line 473 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign ((GtkWidget*) label, GTK_ALIGN_START);
#line 474 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (vbox, (GtkWidget*) label, FALSE, FALSE, (guint) 10);
#line 476 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show_all ((GtkWidget*) vbox);
#line 477 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp10_ = self->priv->steps;
#line 477 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp11_ = page;
#line 477 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp12_ = _g_object_ref0 (label);
#line 477 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_ptr_array_set (_tmp10_, (guint) _tmp11_, _tmp12_);
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (label);
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (vbox);
#line 3810 "wizard.c"
}


static gboolean boxes_wizard_skip_page (BoxesWizard* self, BoxesWizardPage page) {
	gboolean result = FALSE;
	BoxesWizardPage _tmp0_;
	BoxesWizardPage _tmp1_;
	BoxesWizardPage _tmp2_;
	gboolean forwards;
	BoxesWizardPage _tmp3_;
	BoxesWizardPage skip_to;
	gboolean _tmp4_ = FALSE;
	BoxesCollectionSource* _tmp5_;
	gboolean _tmp8_;
	gboolean _tmp14_ = FALSE;
	gboolean _tmp15_;
	gboolean _tmp17_;
	BoxesVMCreator* _tmp19_;
	BoxesWizardPage _tmp48_;
	BoxesWizardPage _tmp49_;
#line 482 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 483 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = page;
#line 483 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = boxes_wizard_get_page (self);
#line 483 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = _tmp1_;
#line 483 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	forwards = _tmp0_ > _tmp2_;
#line 484 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = page;
#line 484 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	skip_to = _tmp3_;
#line 487 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = self->priv->source;
#line 487 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp5_ != NULL) {
#line 3849 "wizard.c"
		BoxesWizardPage _tmp6_;
		BoxesWizardPage _tmp7_;
#line 488 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp6_ = page;
#line 488 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp7_ = _tmp6_;
#line 488 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = (BOXES_WIZARD_PAGE_SOURCE < _tmp7_) && (_tmp7_ < BOXES_WIZARD_PAGE_REVIEW);
#line 3858 "wizard.c"
	} else {
#line 487 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = FALSE;
#line 3862 "wizard.c"
	}
#line 487 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp8_ = _tmp4_;
#line 487 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp8_) {
#line 3868 "wizard.c"
		BoxesWizardPage _tmp9_ = 0;
		gboolean _tmp10_;
		BoxesWizardPage _tmp13_;
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp10_ = forwards;
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_tmp10_) {
#line 3876 "wizard.c"
			BoxesWizardPage _tmp11_;
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp11_ = page;
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp9_ = _tmp11_ + 1;
#line 3882 "wizard.c"
		} else {
			BoxesWizardPage _tmp12_;
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp12_ = page;
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp9_ = _tmp12_ - 1;
#line 3889 "wizard.c"
		}
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp13_ = _tmp9_;
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		skip_to = _tmp13_;
#line 3895 "wizard.c"
	}
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp15_ = forwards;
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (!_tmp15_) {
#line 3901 "wizard.c"
		BoxesWizardPage _tmp16_;
#line 493 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp16_ = page;
#line 493 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp14_ = _tmp16_ == BOXES_WIZARD_PAGE_PREPARATION;
#line 3907 "wizard.c"
	} else {
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp14_ = FALSE;
#line 3911 "wizard.c"
	}
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp17_ = _tmp14_;
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp17_) {
#line 3917 "wizard.c"
		BoxesWizardPage _tmp18_;
#line 494 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp18_ = page;
#line 494 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		skip_to = _tmp18_ - 1;
#line 3923 "wizard.c"
	}
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp19_ = self->priv->vm_creator;
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp19_ != NULL) {
#line 3929 "wizard.c"
		gboolean _tmp20_ = FALSE;
		BoxesWizardPage _tmp21_;
		gboolean _tmp27_;
		gboolean _tmp33_ = FALSE;
		gboolean _tmp34_ = FALSE;
		gboolean _tmp35_ = FALSE;
		BoxesWizardPage _tmp36_;
		gboolean _tmp38_;
		gboolean _tmp44_;
		gboolean _tmp46_;
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp21_ = page;
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_tmp21_ == BOXES_WIZARD_PAGE_SETUP) {
#line 3944 "wizard.c"
			BoxesVMCreator* _tmp22_;
			BoxesInstallerMedia* _tmp23_;
			BoxesInstallerMedia* _tmp24_;
			gboolean _tmp25_;
			gboolean _tmp26_;
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp22_ = self->priv->vm_creator;
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp23_ = boxes_vm_creator_get_install_media (_tmp22_);
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp24_ = _tmp23_;
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp25_ = boxes_installer_media_get_need_user_input_for_vm_creation (_tmp24_);
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp26_ = _tmp25_;
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp20_ = !_tmp26_;
#line 3962 "wizard.c"
		} else {
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp20_ = FALSE;
#line 3966 "wizard.c"
		}
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp27_ = _tmp20_;
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_tmp27_) {
#line 3972 "wizard.c"
			BoxesWizardPage _tmp28_ = 0;
			gboolean _tmp29_;
			BoxesWizardPage _tmp32_;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp29_ = forwards;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			if (_tmp29_) {
#line 3980 "wizard.c"
				BoxesWizardPage _tmp30_;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp30_ = page;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp28_ = _tmp30_ + 1;
#line 3986 "wizard.c"
			} else {
				BoxesWizardPage _tmp31_;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp31_ = page;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp28_ = _tmp31_ - 1;
#line 3993 "wizard.c"
			}
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp32_ = _tmp28_;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			skip_to = _tmp32_;
#line 3999 "wizard.c"
		}
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp36_ = page;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_tmp36_ == BOXES_WIZARD_PAGE_REVIEW) {
#line 4005 "wizard.c"
			gboolean _tmp37_;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp37_ = forwards;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp35_ = _tmp37_;
#line 4011 "wizard.c"
		} else {
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp35_ = FALSE;
#line 4015 "wizard.c"
		}
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp38_ = _tmp35_;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_tmp38_) {
#line 4021 "wizard.c"
			BoxesVMCreator* _tmp39_;
			BoxesInstallerMedia* _tmp40_;
			BoxesInstallerMedia* _tmp41_;
			gboolean _tmp42_;
			gboolean _tmp43_;
#line 504 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp39_ = self->priv->vm_creator;
#line 504 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp40_ = boxes_vm_creator_get_install_media (_tmp39_);
#line 504 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp41_ = _tmp40_;
#line 504 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp42_ = boxes_installer_media_get_live (_tmp41_);
#line 504 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp43_ = _tmp42_;
#line 504 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp34_ = _tmp43_;
#line 4039 "wizard.c"
		} else {
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp34_ = FALSE;
#line 4043 "wizard.c"
		}
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp44_ = _tmp34_;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_tmp44_) {
#line 4049 "wizard.c"
			gboolean _tmp45_;
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp45_ = self->priv->skip_review_for_live;
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp33_ = _tmp45_;
#line 4055 "wizard.c"
		} else {
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp33_ = FALSE;
#line 4059 "wizard.c"
		}
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp46_ = _tmp33_;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_tmp46_) {
#line 4065 "wizard.c"
			BoxesWizardPage _tmp47_;
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp47_ = skip_to;
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			skip_to = _tmp47_ + 1;
#line 4071 "wizard.c"
		}
	}
#line 509 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp48_ = skip_to;
#line 509 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp49_ = page;
#line 509 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp48_ != _tmp49_) {
#line 4080 "wizard.c"
		BoxesWizardPage _tmp50_;
#line 510 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp50_ = skip_to;
#line 510 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_wizard_set_page (self, _tmp50_);
#line 511 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		result = TRUE;
#line 511 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return result;
#line 4090 "wizard.c"
	}
#line 514 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = FALSE;
#line 514 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 4096 "wizard.c"
}


static void __lambda179_ (BoxesWizard* self) {
	BoxesWizardSource* _tmp0_;
	BoxesApp* _tmp1_;
#line 651 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_destroy_machine (self);
#line 652 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->wizard_source;
#line 652 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_source_set_page (_tmp0_, BOXES_SOURCE_PAGE_MAIN);
#line 653 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = boxes_app_app;
#line 653 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_ui_set_ui_state ((BoxesUI*) _tmp1_, BOXES_UI_STATE_COLLECTION);
#line 4113 "wizard.c"
}


static void ___lambda179__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 650 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	__lambda179_ (self);
#line 4120 "wizard.c"
}


static void __lambda180_ (BoxesWizard* self) {
	BoxesWizardPage _tmp0_;
	BoxesWizardPage _tmp1_;
#line 660 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = boxes_wizard_get_page (self);
#line 660 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = _tmp0_;
#line 660 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_set_page (self, _tmp1_ - 1);
#line 4133 "wizard.c"
}


static void ___lambda180__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 659 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	__lambda180_ (self);
#line 4140 "wizard.c"
}


static void __lambda181_ (BoxesWizard* self) {
	BoxesWizardPage _tmp0_;
	BoxesWizardPage _tmp1_;
#line 668 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = boxes_wizard_get_page (self);
#line 668 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = _tmp0_;
#line 668 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_set_page (self, _tmp1_ + 1);
#line 4153 "wizard.c"
}


static void ___lambda181__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	__lambda181_ (self);
#line 4160 "wizard.c"
}


static void __lambda182_ (BoxesWizard* self) {
	BoxesWizardPage _tmp0_;
	BoxesWizardPage _tmp1_;
#line 676 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = boxes_wizard_get_page (self);
#line 676 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = _tmp0_;
#line 676 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_set_page (self, _tmp1_ + 1);
#line 4173 "wizard.c"
}


static void ___lambda182__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 675 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	__lambda182_ (self);
#line 4180 "wizard.c"
}


static void boxes_wizard_setup_ui (BoxesWizard* self) {
	GtkNotebook* _tmp0_;
	GtkNotebook* _tmp1_;
	GtkNotebook* _tmp2_;
	GtkNotebook* _tmp3_;
	GtkNotebook* _tmp4_;
	GtkStyleContext* _tmp5_ = NULL;
	GtkNotebook* _tmp6_;
	GtkClutterActor* _tmp7_;
	GtkClutterActor* _tmp8_;
	GtkWidget* _tmp9_ = NULL;
	GtkStyleContext* _tmp10_ = NULL;
	GtkClutterActor* _tmp11_;
	GtkClutterActor* _tmp12_;
	GtkClutterActor* _tmp13_;
	GtkClutterActor* _tmp14_;
	GtkClutterActor* _tmp15_;
	GtkClutterActor* _tmp16_;
	GtkBox* _tmp17_;
	GtkBox* hbox;
	GtkBox* _tmp18_;
	GtkBox* _tmp19_;
	const gchar* _tmp20_ = NULL;
	GtkBox* _tmp21_;
	gchar* _tmp22_ = NULL;
	gchar* _tmp23_;
	GtkImage* _tmp24_;
	GtkImage* _tmp25_;
	GtkLabel* _tmp26_;
	GtkLabel* label;
	GtkLabel* _tmp27_;
	GtkStyleContext* _tmp28_ = NULL;
	GtkLabel* _tmp29_;
	const gchar* _tmp30_ = NULL;
	GtkLabel* _tmp31_;
	GtkLabel* _tmp32_;
	GtkLabel* _tmp33_;
	GtkBox* _tmp34_;
	GtkLabel* _tmp35_;
	GtkBox* _tmp36_;
	GtkBox* _tmp37_;
	GtkBox* vbox;
	GtkBox* _tmp38_;
	GtkBox* _tmp39_;
	GtkBox* _tmp40_;
	const gchar* _tmp41_ = NULL;
	const gchar* _tmp42_ = NULL;
	GtkLabel* _tmp43_;
	GtkLabel* _tmp44_;
	GtkStyleContext* _tmp45_ = NULL;
	GtkLabel* _tmp46_;
	GtkLabel* _tmp47_;
	GtkLabel* _tmp48_;
	GtkBox* _tmp49_;
	GtkLabel* _tmp50_;
	GtkBox* _tmp51_;
	BoxesWizardSource* _tmp52_;
	GtkWidget* _tmp53_;
	GtkWidget* _tmp54_;
	BoxesWizardSource* _tmp55_;
	GtkWidget* _tmp56_;
	GtkWidget* _tmp57_;
	BoxesWizardSource* _tmp58_;
	GtkWidget* _tmp59_;
	GtkWidget* _tmp60_;
	const gchar* _tmp61_ = NULL;
	GtkLabel* _tmp62_;
	GtkLabel* _tmp63_;
	GtkStyleContext* _tmp64_ = NULL;
	GtkLabel* _tmp65_;
	GtkLabel* _tmp66_;
	GtkLabel* _tmp67_;
	GtkBox* _tmp68_;
	GtkLabel* _tmp69_;
	GtkBox* _tmp70_;
	GtkBox* _tmp71_;
	GtkBox* _tmp72_;
	GtkBox* _tmp73_;
	GtkBox* _tmp74_;
	const gchar* _tmp75_ = NULL;
	const gchar* _tmp76_ = NULL;
	GtkLabel* _tmp77_;
	GtkLabel* _tmp78_;
	GtkStyleContext* _tmp79_ = NULL;
	GtkLabel* _tmp80_;
	GtkLabel* _tmp81_;
	GtkLabel* _tmp82_;
	GtkBox* _tmp83_;
	GtkLabel* _tmp84_;
	GtkBox* _tmp85_;
	GtkBox* _tmp86_;
	GtkBox* _tmp87_;
	GtkBox* _tmp88_;
	GtkBox* _tmp89_;
	GtkImage* _tmp90_;
	GtkImage* _tmp91_;
	GtkBox* _tmp92_;
	GtkImage* _tmp93_;
	GtkBox* _tmp94_;
	GtkBox* prep_vbox;
	GtkBox* _tmp95_;
	GtkLabel* _tmp96_;
	GtkLabel* _tmp97_;
	GtkStyleContext* _tmp98_ = NULL;
	GtkLabel* _tmp99_;
	GtkLabel* _tmp100_;
	GtkLabel* _tmp101_;
	GtkLabel* _tmp102_;
	GtkStyleContext* _tmp103_ = NULL;
	GtkLabel* _tmp104_;
	GtkLabel* _tmp105_;
	GtkProgressBar* _tmp106_;
	GtkProgressBar* _tmp107_;
	GtkBox* _tmp108_;
	GtkBox* _tmp109_;
	GtkBox* _tmp110_;
	GtkBox* _tmp111_;
	GtkBox* _tmp112_;
	const gchar* _tmp113_ = NULL;
	GtkBox* _tmp114_;
	GtkBox* _tmp115_;
	GtkBox* _tmp116_;
	GtkBox* _tmp117_;
	GtkBox* _tmp118_;
	const gchar* _tmp119_ = NULL;
	GtkLabel* _tmp120_;
	GtkLabel* _tmp121_;
	GtkStyleContext* _tmp122_ = NULL;
	GtkLabel* _tmp123_;
	GtkLabel* _tmp124_;
	GtkLabel* _tmp125_;
	GtkBox* _tmp126_;
	GtkLabel* _tmp127_;
	BoxesWizardWizardSummary* _tmp128_;
	GtkBox* _tmp129_;
	BoxesWizardWizardSummary* _tmp130_;
	GtkWidget* _tmp131_;
	GtkWidget* _tmp132_;
	const gchar* _tmp133_ = NULL;
	GtkLabel* _tmp134_;
	GtkLabel* _tmp135_;
	GtkStyleContext* _tmp136_ = NULL;
	GtkLabel* _tmp137_;
	GtkLabel* _tmp138_;
	GtkBox* _tmp139_;
	GtkLabel* _tmp140_;
	GtkBox* _tmp141_;
	BoxesApp* _tmp142_;
	BoxesTopbar* _tmp143_;
	GtkNotebook* _tmp144_;
	GtkWidget* _tmp145_ = NULL;
	GtkBox* _tmp146_;
	GdMainToolbar* _tmp147_;
	GdMainToolbar* toolbar;
	GtkStyleContext* _tmp148_ = NULL;
	GtkBox* _tmp149_;
	const gchar* _tmp150_ = NULL;
	GtkSizeGroup* _tmp151_;
	const gchar* _tmp152_ = NULL;
	GtkWidget* _tmp153_ = NULL;
	GtkButton* _tmp154_;
	GtkButton* _tmp155_;
	GtkButton* _tmp156_;
	GtkSizeGroup* _tmp157_;
	GtkButton* _tmp158_;
	const gchar* _tmp159_ = NULL;
	GtkWidget* _tmp160_ = NULL;
	GtkButton* _tmp161_;
	GtkButton* _tmp162_;
	GtkButton* _tmp163_;
	GtkSizeGroup* _tmp164_;
	GtkButton* _tmp165_;
	const gchar* _tmp166_ = NULL;
	GtkWidget* _tmp167_ = NULL;
	GtkButton* _tmp168_;
	GtkButton* _tmp169_;
	GtkButton* _tmp170_;
	GtkStyleContext* _tmp171_ = NULL;
	GtkButton* _tmp172_;
	GtkSizeGroup* _tmp173_;
	GtkButton* _tmp174_;
	const gchar* _tmp175_ = NULL;
	GtkWidget* _tmp176_ = NULL;
	GtkButton* _tmp177_;
	GtkButton* _tmp178_;
	GtkButton* _tmp179_;
	GtkStyleContext* _tmp180_ = NULL;
	GtkButton* _tmp181_;
	GtkSizeGroup* _tmp182_;
	GtkButton* _tmp183_;
	GtkBox* _tmp184_;
	GtkNotebook* _tmp185_;
#line 517 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 518 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = (GtkNotebook*) gtk_notebook_new ();
#line 518 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp0_);
#line 518 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->notebook);
#line 518 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->notebook = _tmp0_;
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = self->priv->notebook;
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set ((GtkWidget*) _tmp1_, "margin", 15, NULL);
#line 520 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = self->priv->notebook;
#line 520 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set ((GtkWidget*) _tmp2_, "width-request", 500, NULL);
#line 521 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = self->priv->notebook;
#line 521 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_notebook_set_show_tabs (_tmp3_, FALSE);
#line 522 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = self->priv->notebook;
#line 522 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = gtk_widget_get_style_context ((GtkWidget*) _tmp4_);
#line 522 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp5_, "boxes-bg");
#line 523 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp6_ = self->priv->notebook;
#line 523 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp7_ = (GtkClutterActor*) gtk_clutter_actor_new_with_contents ((GtkWidget*) _tmp6_);
#line 523 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp7_);
#line 523 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->gtk_actor);
#line 523 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->gtk_actor = _tmp7_;
#line 524 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp8_ = self->priv->gtk_actor;
#line 524 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp9_ = gtk_clutter_actor_get_widget (_tmp8_);
#line 524 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp10_ = gtk_widget_get_style_context (_tmp9_);
#line 524 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp10_, "boxes-bg");
#line 525 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp11_ = self->priv->gtk_actor;
#line 525 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	clutter_actor_set_name ((ClutterActor*) _tmp11_, "wizard");
#line 526 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp12_ = self->priv->gtk_actor;
#line 526 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	clutter_actor_set_opacity ((ClutterActor*) _tmp12_, (guint) 0);
#line 527 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp13_ = self->priv->gtk_actor;
#line 527 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	clutter_actor_set_x_align ((ClutterActor*) _tmp13_, CLUTTER_ACTOR_ALIGN_FILL);
#line 528 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp14_ = self->priv->gtk_actor;
#line 528 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	clutter_actor_set_y_align ((ClutterActor*) _tmp14_, CLUTTER_ACTOR_ALIGN_FILL);
#line 529 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp15_ = self->priv->gtk_actor;
#line 529 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	clutter_actor_set_x_expand ((ClutterActor*) _tmp15_, TRUE);
#line 530 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp16_ = self->priv->gtk_actor;
#line 530 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	clutter_actor_set_y_expand ((ClutterActor*) _tmp16_, TRUE);
#line 533 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp17_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
#line 533 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp17_);
#line 533 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	hbox = _tmp17_;
#line 534 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp18_ = hbox;
#line 534 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign ((GtkWidget*) _tmp18_, GTK_ALIGN_CENTER);
#line 535 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp19_ = hbox;
#line 535 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp20_ = _ ("Introduction");
#line 535 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_add_step (self, (GtkWidget*) _tmp19_, _tmp20_, BOXES_WIZARD_PAGE_INTRODUCTION);
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp21_ = hbox;
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp22_ = boxes_get_pixmap ("boxes-create.png");
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp23_ = _tmp22_;
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp24_ = (GtkImage*) gtk_image_new_from_file (_tmp23_);
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp24_);
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp25_ = _tmp24_;
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_container_add ((GtkContainer*) _tmp21_, (GtkWidget*) _tmp25_);
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (_tmp25_);
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_free0 (_tmp23_);
#line 537 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp26_ = (GtkLabel*) gtk_label_new (NULL);
#line 537 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp26_);
#line 537 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	label = _tmp26_;
#line 538 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp27_ = label;
#line 538 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp28_ = gtk_widget_get_style_context ((GtkWidget*) _tmp27_);
#line 538 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp28_, "boxes-wizard-label");
#line 539 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp29_ = label;
#line 539 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp30_ = _ ("Creating a Box will allow you to use another operating system directly" \
" from your existing login.\n" \
"\n" \
"You may connect to an existing machine <b><i>over the network</i></b> " \
"or create a <b><i>virtual machine</i></b> that runs locally on your ow" \
"n.");
#line 539 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_markup (_tmp29_, _tmp30_);
#line 540 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp31_ = label;
#line 540 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set (_tmp31_, "wrap", TRUE, NULL);
#line 542 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp32_ = label;
#line 542 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_width_chars (_tmp32_, 30);
#line 543 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp33_ = label;
#line 543 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_max_width_chars (_tmp33_, 40);
#line 544 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp34_ = hbox;
#line 544 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp35_ = label;
#line 544 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_container_add ((GtkContainer*) _tmp34_, (GtkWidget*) _tmp35_);
#line 545 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp36_ = hbox;
#line 545 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp36_);
#line 548 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp37_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 30);
#line 548 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp37_);
#line 548 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	vbox = _tmp37_;
#line 549 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp38_ = vbox;
#line 549 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_valign ((GtkWidget*) _tmp38_, GTK_ALIGN_CENTER);
#line 550 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp39_ = vbox;
#line 550 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign ((GtkWidget*) _tmp39_, GTK_ALIGN_CENTER);
#line 551 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp40_ = vbox;
#line 551 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp41_ = _ ("Source Selection");
#line 551 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_add_step (self, (GtkWidget*) _tmp40_, _tmp41_, BOXES_WIZARD_PAGE_SOURCE);
#line 552 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp42_ = _ ("Insert operating system installation media or select a source below");
#line 552 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp43_ = (GtkLabel*) gtk_label_new (_tmp42_);
#line 552 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp43_);
#line 552 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (label);
#line 552 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	label = _tmp43_;
#line 553 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp44_ = label;
#line 553 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp45_ = gtk_widget_get_style_context ((GtkWidget*) _tmp44_);
#line 553 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp45_, "boxes-wizard-label");
#line 554 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp46_ = label;
#line 554 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set (_tmp46_, "wrap", TRUE, NULL);
#line 556 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp47_ = label;
#line 556 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_width_chars (_tmp47_, 30);
#line 557 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp48_ = label;
#line 557 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set ((GtkMisc*) _tmp48_, "xalign", 0.0f, NULL);
#line 558 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp49_ = vbox;
#line 558 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp50_ = label;
#line 558 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp49_, (GtkWidget*) _tmp50_, FALSE, FALSE, (guint) 0);
#line 559 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp51_ = vbox;
#line 559 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp52_ = self->priv->wizard_source;
#line 559 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp53_ = boxes_wizard_source_get_widget (_tmp52_);
#line 559 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp54_ = _tmp53_;
#line 559 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp51_, _tmp54_, FALSE, FALSE, (guint) 0);
#line 560 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp55_ = self->priv->wizard_source;
#line 560 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp56_ = boxes_wizard_source_get_widget (_tmp55_);
#line 560 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp57_ = _tmp56_;
#line 560 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_hexpand (_tmp57_, TRUE);
#line 561 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp58_ = self->priv->wizard_source;
#line 561 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp59_ = boxes_wizard_source_get_widget (_tmp58_);
#line 561 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp60_ = _tmp59_;
#line 561 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign (_tmp60_, GTK_ALIGN_CENTER);
#line 562 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp61_ = _ ("Any trademarks shown above are used merely for identification of softw" \
"are products you have already obtained and are the property of their r" \
"espective owners.");
#line 562 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp62_ = (GtkLabel*) gtk_label_new (_tmp61_);
#line 562 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp62_);
#line 562 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (label);
#line 562 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	label = _tmp62_;
#line 563 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp63_ = label;
#line 563 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp64_ = gtk_widget_get_style_context ((GtkWidget*) _tmp63_);
#line 563 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp64_, "boxes-logo-notice-label");
#line 564 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp65_ = label;
#line 564 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set (_tmp65_, "wrap", TRUE, NULL);
#line 566 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp66_ = label;
#line 566 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_width_chars (_tmp66_, 30);
#line 567 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp67_ = label;
#line 567 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_max_width_chars (_tmp67_, 50);
#line 568 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp68_ = vbox;
#line 568 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp69_ = label;
#line 568 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp68_, (GtkWidget*) _tmp69_, FALSE, FALSE, (guint) 0);
#line 569 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp70_ = vbox;
#line 569 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp70_);
#line 572 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp71_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 30);
#line 572 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp71_);
#line 572 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (vbox);
#line 572 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	vbox = _tmp71_;
#line 573 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp72_ = vbox;
#line 573 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_valign ((GtkWidget*) _tmp72_, GTK_ALIGN_CENTER);
#line 574 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp73_ = vbox;
#line 574 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign ((GtkWidget*) _tmp73_, GTK_ALIGN_CENTER);
#line 575 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp74_ = vbox;
#line 575 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp75_ = _ ("Preparation");
#line 575 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_add_step (self, (GtkWidget*) _tmp74_, _tmp75_, BOXES_WIZARD_PAGE_PREPARATION);
#line 576 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp76_ = _ ("Preparing to create new box");
#line 576 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp77_ = (GtkLabel*) gtk_label_new (_tmp76_);
#line 576 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp77_);
#line 576 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (label);
#line 576 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	label = _tmp77_;
#line 577 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp78_ = label;
#line 577 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp79_ = gtk_widget_get_style_context ((GtkWidget*) _tmp78_);
#line 577 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp79_, "boxes-wizard-label");
#line 578 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp80_ = label;
#line 578 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set (_tmp80_, "wrap", TRUE, NULL);
#line 580 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp81_ = label;
#line 580 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_width_chars (_tmp81_, 30);
#line 581 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp82_ = label;
#line 581 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set ((GtkMisc*) _tmp82_, "xalign", 0.0f, NULL);
#line 582 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp83_ = vbox;
#line 582 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp84_ = label;
#line 582 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp83_, (GtkWidget*) _tmp84_, FALSE, FALSE, (guint) 0);
#line 584 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp85_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
#line 584 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp85_);
#line 584 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (hbox);
#line 584 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	hbox = _tmp85_;
#line 585 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp86_ = hbox;
#line 585 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_valign ((GtkWidget*) _tmp86_, GTK_ALIGN_CENTER);
#line 586 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp87_ = hbox;
#line 586 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign ((GtkWidget*) _tmp87_, GTK_ALIGN_CENTER);
#line 587 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp88_ = vbox;
#line 587 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp89_ = hbox;
#line 587 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp88_, (GtkWidget*) _tmp89_, TRUE, TRUE, (guint) 0);
#line 589 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp90_ = (GtkImage*) gtk_image_new_from_icon_name ("media-optical", 0);
#line 589 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp90_);
#line 589 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->installer_image);
#line 589 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->installer_image = _tmp90_;
#line 590 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp91_ = self->priv->installer_image;
#line 590 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_image_set_pixel_size (_tmp91_, 128);
#line 591 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp92_ = hbox;
#line 591 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp93_ = self->priv->installer_image;
#line 591 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp92_, (GtkWidget*) _tmp93_, FALSE, FALSE, (guint) 0);
#line 592 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp94_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 10);
#line 592 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp94_);
#line 592 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	prep_vbox = _tmp94_;
#line 593 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_set_homogeneous (prep_vbox, TRUE);
#line 594 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_valign ((GtkWidget*) prep_vbox, GTK_ALIGN_CENTER);
#line 595 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp95_ = hbox;
#line 595 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp95_, (GtkWidget*) prep_vbox, TRUE, TRUE, (guint) 0);
#line 596 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp96_ = (GtkLabel*) gtk_label_new (NULL);
#line 596 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp96_);
#line 596 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->prep_media_label);
#line 596 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->prep_media_label = _tmp96_;
#line 597 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp97_ = self->priv->prep_media_label;
#line 597 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp98_ = gtk_widget_get_style_context ((GtkWidget*) _tmp97_);
#line 597 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp98_, "boxes-wizard-media-os-label");
#line 598 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp99_ = self->priv->prep_media_label;
#line 598 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign ((GtkWidget*) _tmp99_, GTK_ALIGN_START);
#line 599 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp100_ = self->priv->prep_media_label;
#line 599 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (prep_vbox, (GtkWidget*) _tmp100_, FALSE, FALSE, (guint) 0);
#line 600 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp101_ = (GtkLabel*) gtk_label_new (NULL);
#line 600 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp101_);
#line 600 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->prep_status_label);
#line 600 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->prep_status_label = _tmp101_;
#line 601 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp102_ = self->priv->prep_status_label;
#line 601 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp103_ = gtk_widget_get_style_context ((GtkWidget*) _tmp102_);
#line 601 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp103_, "boxes-wizard-label");
#line 602 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp104_ = self->priv->prep_status_label;
#line 602 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign ((GtkWidget*) _tmp104_, GTK_ALIGN_START);
#line 603 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp105_ = self->priv->prep_status_label;
#line 603 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (prep_vbox, (GtkWidget*) _tmp105_, FALSE, FALSE, (guint) 0);
#line 604 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp106_ = (GtkProgressBar*) gtk_progress_bar_new ();
#line 604 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp106_);
#line 604 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->prep_progress);
#line 604 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->prep_progress = _tmp106_;
#line 605 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp107_ = self->priv->prep_progress;
#line 605 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (prep_vbox, (GtkWidget*) _tmp107_, FALSE, FALSE, (guint) 0);
#line 606 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp108_ = vbox;
#line 606 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp108_);
#line 609 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp109_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 30);
#line 609 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp109_);
#line 609 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->setup_vbox);
#line 609 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->setup_vbox = _tmp109_;
#line 610 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp110_ = self->priv->setup_vbox;
#line 610 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_valign ((GtkWidget*) _tmp110_, GTK_ALIGN_CENTER);
#line 611 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp111_ = self->priv->setup_vbox;
#line 611 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign ((GtkWidget*) _tmp111_, GTK_ALIGN_CENTER);
#line 612 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp112_ = self->priv->setup_vbox;
#line 612 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp113_ = _ ("Setup");
#line 612 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_add_step (self, (GtkWidget*) _tmp112_, _tmp113_, BOXES_WIZARD_PAGE_SETUP);
#line 613 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp114_ = self->priv->setup_vbox;
#line 613 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp114_);
#line 616 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp115_ = (GtkBox*) gtk_box_new (GTK_ORIENTATION_VERTICAL, 30);
#line 616 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp115_);
#line 616 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (vbox);
#line 616 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	vbox = _tmp115_;
#line 617 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp116_ = vbox;
#line 617 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_valign ((GtkWidget*) _tmp116_, GTK_ALIGN_CENTER);
#line 618 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp117_ = vbox;
#line 618 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_halign ((GtkWidget*) _tmp117_, GTK_ALIGN_CENTER);
#line 619 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp118_ = vbox;
#line 619 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp119_ = _ ("Review");
#line 619 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_add_step (self, (GtkWidget*) _tmp118_, _tmp119_, BOXES_WIZARD_PAGE_REVIEW);
#line 621 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp120_ = (GtkLabel*) gtk_label_new (NULL);
#line 621 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp120_);
#line 621 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->review_label);
#line 621 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->review_label = _tmp120_;
#line 622 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp121_ = self->priv->review_label;
#line 622 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp122_ = gtk_widget_get_style_context ((GtkWidget*) _tmp121_);
#line 622 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp122_, "boxes-wizard-label");
#line 623 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp123_ = self->priv->review_label;
#line 623 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set ((GtkMisc*) _tmp123_, "xalign", 0.0f, NULL);
#line 624 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp124_ = self->priv->review_label;
#line 624 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set (_tmp124_, "wrap", TRUE, NULL);
#line 625 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp125_ = self->priv->review_label;
#line 625 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_width_chars (_tmp125_, 30);
#line 626 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp126_ = vbox;
#line 626 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp127_ = self->priv->review_label;
#line 626 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp126_, (GtkWidget*) _tmp127_, FALSE, FALSE, (guint) 0);
#line 628 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp128_ = boxes_wizard_wizard_summary_new ();
#line 628 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->summary);
#line 628 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->summary = _tmp128_;
#line 629 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp129_ = vbox;
#line 629 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp130_ = self->priv->summary;
#line 629 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp131_ = boxes_wizard_wizard_summary_get_widget (_tmp130_);
#line 629 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp132_ = _tmp131_;
#line 629 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp129_, _tmp132_, TRUE, TRUE, (guint) 0);
#line 630 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp133_ = _ ("Virtualization extensions are unavailable on your system. Expect this " \
"box to be extremely slow. If your system is recent enough (made in or " \
"after 2008), these extensions are probably available on your system an" \
"d you may need to enable them in your system's BIOS setup.");
#line 630 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp134_ = (GtkLabel*) gtk_label_new (_tmp133_);
#line 630 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp134_);
#line 630 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->nokvm_label);
#line 630 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->nokvm_label = _tmp134_;
#line 631 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp135_ = self->priv->nokvm_label;
#line 631 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp136_ = gtk_widget_get_style_context ((GtkWidget*) _tmp135_);
#line 631 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp136_, "boxes-logo-notice-label");
#line 632 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp137_ = self->priv->nokvm_label;
#line 632 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set (_tmp137_, "wrap", TRUE, NULL);
#line 633 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp138_ = self->priv->nokvm_label;
#line 633 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_label_set_max_width_chars (_tmp138_, 50);
#line 634 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp139_ = vbox;
#line 634 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp140_ = self->priv->nokvm_label;
#line 634 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp139_, (GtkWidget*) _tmp140_, FALSE, FALSE, (guint) 0);
#line 635 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp141_ = vbox;
#line 635 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp141_);
#line 638 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp142_ = boxes_app_app;
#line 638 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp143_ = _tmp142_->topbar;
#line 638 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp144_ = _tmp143_->notebook;
#line 638 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp145_ = gtk_notebook_get_nth_page (_tmp144_, (gint) BOXES_TOPBAR_PAGE_WIZARD);
#line 638 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp146_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp145_, GTK_TYPE_BOX) ? ((GtkBox*) _tmp145_) : NULL);
#line 638 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (hbox);
#line 638 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	hbox = _tmp146_;
#line 640 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp147_ = (GdMainToolbar*) gd_main_toolbar_new ();
#line 640 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp147_);
#line 640 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	toolbar = _tmp147_;
#line 641 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp148_ = gtk_widget_get_style_context ((GtkWidget*) toolbar);
#line 641 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp148_, GTK_STYLE_CLASS_MENUBAR);
#line 642 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set ((GtkToolbar*) toolbar, "toolbar-style", GTK_TOOLBAR_TEXT, NULL);
#line 643 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp149_ = hbox;
#line 643 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_box_pack_start (_tmp149_, (GtkWidget*) toolbar, TRUE, TRUE, (guint) 0);
#line 645 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp150_ = _ ("Create a Box");
#line 645 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gd_main_toolbar_set_labels (toolbar, _tmp150_, NULL);
#line 647 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp151_ = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
#line 647 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->toolbar_sizegroup);
#line 647 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->toolbar_sizegroup = _tmp151_;
#line 648 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp152_ = _ ("_Cancel");
#line 648 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp153_ = gd_main_toolbar_add_button (toolbar, NULL, _tmp152_, TRUE);
#line 648 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp154_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp153_, GTK_TYPE_BUTTON) ? ((GtkButton*) _tmp153_) : NULL);
#line 648 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->cancel_button);
#line 648 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->cancel_button = _tmp154_;
#line 649 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp155_ = self->priv->cancel_button;
#line 649 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_button_set_use_underline (_tmp155_, TRUE);
#line 650 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp156_ = self->priv->cancel_button;
#line 650 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_object (_tmp156_, "clicked", (GCallback) ___lambda179__gtk_button_clicked, self, 0);
#line 655 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp157_ = self->priv->toolbar_sizegroup;
#line 655 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp158_ = self->priv->cancel_button;
#line 655 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_size_group_add_widget (_tmp157_, (GtkWidget*) _tmp158_);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp159_ = _ ("_Back");
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp160_ = gd_main_toolbar_add_button (toolbar, NULL, _tmp159_, FALSE);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp161_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp160_, GTK_TYPE_BUTTON) ? ((GtkButton*) _tmp160_) : NULL);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->back_button);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->back_button = _tmp161_;
#line 658 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp162_ = self->priv->back_button;
#line 658 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_button_set_use_underline (_tmp162_, TRUE);
#line 659 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp163_ = self->priv->back_button;
#line 659 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_object (_tmp163_, "clicked", (GCallback) ___lambda180__gtk_button_clicked, self, 0);
#line 662 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp164_ = self->priv->toolbar_sizegroup;
#line 662 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp165_ = self->priv->back_button;
#line 662 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_size_group_add_widget (_tmp164_, (GtkWidget*) _tmp165_);
#line 664 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp166_ = _ ("C_ontinue");
#line 664 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp167_ = gd_main_toolbar_add_button (toolbar, NULL, _tmp166_, FALSE);
#line 664 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp168_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp167_, GTK_TYPE_BUTTON) ? ((GtkButton*) _tmp167_) : NULL);
#line 664 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->continue_button);
#line 664 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->continue_button = _tmp168_;
#line 665 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp169_ = self->priv->continue_button;
#line 665 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_button_set_use_underline (_tmp169_, TRUE);
#line 666 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp170_ = self->priv->continue_button;
#line 666 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp171_ = gtk_widget_get_style_context ((GtkWidget*) _tmp170_);
#line 666 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp171_, "boxes-continue");
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp172_ = self->priv->continue_button;
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_object (_tmp172_, "clicked", (GCallback) ___lambda181__gtk_button_clicked, self, 0);
#line 670 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp173_ = self->priv->toolbar_sizegroup;
#line 670 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp174_ = self->priv->continue_button;
#line 670 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_size_group_add_widget (_tmp173_, (GtkWidget*) _tmp174_);
#line 672 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp175_ = _ ("C_reate");
#line 672 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp176_ = gd_main_toolbar_add_button (toolbar, NULL, _tmp175_, FALSE);
#line 672 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp177_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp176_, GTK_TYPE_BUTTON) ? ((GtkButton*) _tmp176_) : NULL);
#line 672 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->create_button);
#line 672 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->create_button = _tmp177_;
#line 673 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp178_ = self->priv->create_button;
#line 673 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_button_set_use_underline (_tmp178_, TRUE);
#line 674 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp179_ = self->priv->create_button;
#line 674 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp180_ = gtk_widget_get_style_context ((GtkWidget*) _tmp179_);
#line 674 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp180_, "boxes-continue");
#line 675 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp181_ = self->priv->create_button;
#line 675 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_object (_tmp181_, "clicked", (GCallback) ___lambda182__gtk_button_clicked, self, 0);
#line 678 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp182_ = self->priv->toolbar_sizegroup;
#line 678 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp183_ = self->priv->create_button;
#line 678 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_size_group_add_widget (_tmp182_, (GtkWidget*) _tmp183_);
#line 680 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp184_ = hbox;
#line 680 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp184_);
#line 681 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp185_ = self->priv->notebook;
#line 681 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp185_);
#line 517 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (toolbar);
#line 517 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (prep_vbox);
#line 517 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (vbox);
#line 517 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (label);
#line 517 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (hbox);
#line 5104 "wizard.c"
}


void boxes_wizard_open_with_uri (BoxesWizard* self, const gchar* uri, gboolean skip_review_for_live) {
	BoxesApp* _tmp0_;
	gboolean _tmp1_;
	BoxesWizardSource* _tmp2_;
	BoxesWizardSource* _tmp3_;
	const gchar* _tmp4_;
#line 684 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 684 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (uri != NULL);
#line 685 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = boxes_app_app;
#line 685 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_ui_set_ui_state ((BoxesUI*) _tmp0_, BOXES_UI_STATE_WIZARD);
#line 686 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = skip_review_for_live;
#line 686 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->skip_review_for_live = _tmp1_;
#line 688 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_set_page (self, BOXES_WIZARD_PAGE_SOURCE);
#line 689 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = self->priv->wizard_source;
#line 689 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_source_set_page (_tmp2_, BOXES_SOURCE_PAGE_URL);
#line 690 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = self->priv->wizard_source;
#line 690 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = uri;
#line 690 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_source_set_uri (_tmp3_, _tmp4_);
#line 691 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_set_page (self, BOXES_WIZARD_PAGE_PREPARATION);
#line 5140 "wizard.c"
}


static void boxes_wizard_real_ui_state_changed (BoxesUI* base) {
	BoxesWizard * self;
	BoxesUIState _tmp0_;
	BoxesUIState _tmp1_;
	gint _tmp4_ = 0;
	BoxesUIState _tmp5_;
	BoxesUIState _tmp6_;
	ClutterActor* _tmp7_;
	ClutterActor* _tmp8_;
	gint _tmp9_;
#line 694 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = (BoxesWizard*) base;
#line 695 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 695 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = _tmp0_;
#line 695 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp1_ == BOXES_UI_STATE_WIZARD) {
#line 5162 "wizard.c"
		BoxesUIState _tmp2_;
#line 696 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp2_ = ((BoxesUI*) self)->previous_ui_state;
#line 696 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		if (_tmp2_ == BOXES_UI_STATE_PROPERTIES) {
#line 697 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_review (self, NULL, NULL);
#line 5170 "wizard.c"
		} else {
			BoxesWizardSource* _tmp3_;
#line 699 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp3_ = self->priv->wizard_source;
#line 699 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_source_set_uri (_tmp3_, "");
#line 700 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_set_page (self, BOXES_WIZARD_PAGE_INTRODUCTION);
#line 5179 "wizard.c"
		}
	}
#line 704 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 704 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp6_ = _tmp5_;
#line 704 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp6_ == BOXES_UI_STATE_WIZARD) {
#line 704 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = 255;
#line 5190 "wizard.c"
	} else {
#line 704 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = 0;
#line 5194 "wizard.c"
	}
#line 704 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp7_ = boxes_ui_get_actor ((BoxesUI*) self);
#line 704 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp8_ = _tmp7_;
#line 704 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp9_ = _tmp4_;
#line 704 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_fade_actor (_tmp8_, (guint) _tmp9_);
#line 5204 "wizard.c"
}


static void boxes_wizard_destroy_machine (BoxesWizard* self) {
	GCancellable* _tmp0_;
	BoxesMachine* _tmp2_;
#line 707 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 708 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->review_cancellable;
#line 708 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp0_ != NULL) {
#line 5217 "wizard.c"
		GCancellable* _tmp1_;
#line 709 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp1_ = self->priv->review_cancellable;
#line 709 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_cancellable_cancel (_tmp1_);
#line 5223 "wizard.c"
	}
#line 711 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = self->priv->_machine;
#line 711 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp2_ != NULL) {
#line 5229 "wizard.c"
		BoxesApp* _tmp3_;
		BoxesMachine* _tmp4_;
#line 712 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp3_ = boxes_app_app;
#line 712 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = self->priv->_machine;
#line 712 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_app_delete_machine (_tmp3_, _tmp4_, TRUE);
#line 713 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_wizard_set_machine (self, NULL);
#line 5240 "wizard.c"
	}
}


static ClutterActor* boxes_wizard_real_get_actor (BoxesUI* base) {
	ClutterActor* result;
	BoxesWizard* self;
	GtkClutterActor* _tmp0_;
#line 14 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = (BoxesWizard*) base;
#line 14 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->gtk_actor;
#line 14 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = (ClutterActor*) _tmp0_;
#line 14 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 5257 "wizard.c"
}


BoxesMachine* boxes_wizard_get_machine (BoxesWizard* self) {
	BoxesMachine* result;
	BoxesMachine* _tmp0_;
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->_machine;
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = _tmp0_;
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 5272 "wizard.c"
}


void boxes_wizard_set_machine (BoxesWizard* self, BoxesMachine* value) {
	BoxesMachine* _tmp0_;
	BoxesMachine* _tmp1_;
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = value;
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->_machine);
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->_machine = _tmp1_;
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_notify ((GObject *) self, "machine");
#line 5291 "wizard.c"
}


static BoxesLibvirtMachine* boxes_wizard_get_libvirt_machine (BoxesWizard* self) {
	BoxesLibvirtMachine* result;
	BoxesMachine* _tmp0_;
#line 40 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 40 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->_machine;
#line 40 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = G_TYPE_CHECK_INSTANCE_TYPE (_tmp0_, BOXES_TYPE_LIBVIRT_MACHINE) ? ((BoxesLibvirtMachine*) _tmp0_) : NULL;
#line 40 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 5306 "wizard.c"
}


static BoxesWizardPage boxes_wizard_get_page (BoxesWizard* self) {
	BoxesWizardPage result;
	BoxesWizardPage _tmp0_;
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->_page;
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = _tmp0_;
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 5321 "wizard.c"
}


static void _boxes_wizard_page_____lambda175_ (BoxesWizard* self, GObject* obj, GAsyncResult* _result_) {
	GtkButton* _tmp0_;
	GAsyncResult* _tmp1_;
	gboolean _tmp2_ = FALSE;
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (_result_ != NULL);
#line 89 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->next_button;
#line 89 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_sensitive ((GtkWidget*) _tmp0_, TRUE);
#line 90 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = _result_;
#line 90 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = boxes_wizard_review_finish (self, _tmp1_);
#line 90 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (!_tmp2_) {
#line 5341 "wizard.c"
		BoxesWizardPage _tmp3_;
		BoxesWizardPage _tmp4_;
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp3_ = boxes_wizard_get_page (self);
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = _tmp3_;
#line 91 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_wizard_set_page (self, _tmp4_ - 1);
#line 5350 "wizard.c"
	}
}


static void __boxes_wizard_page_____lambda175__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self) {
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_boxes_wizard_page_____lambda175_ (self, source_object, res);
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_unref (self);
#line 5360 "wizard.c"
}


static void _boxes_wizard_page_____lambda177_ (BoxesWizard* self, GObject* obj, GAsyncResult* _result_) {
	GAsyncResult* _tmp0_;
	gboolean _tmp1_ = FALSE;
#line 96 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (_result_ != NULL);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = _result_;
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = boxes_wizard_create_finish (self, _tmp0_);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp1_) {
#line 5375 "wizard.c"
		BoxesApp* _tmp2_;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp2_ = boxes_app_app;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_ui_set_ui_state ((BoxesUI*) _tmp2_, BOXES_UI_STATE_COLLECTION);
#line 5381 "wizard.c"
	} else {
		BoxesApp* _tmp3_;
		BoxesNotificationbar* _tmp4_;
		const gchar* _tmp5_ = NULL;
		GdNotification* _tmp6_ = NULL;
		GdNotification* _tmp7_;
#line 100 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp3_ = boxes_app_app;
#line 100 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp4_ = _tmp3_->notificationbar;
#line 100 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp5_ = _ ("Box creation failed");
#line 100 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp6_ = boxes_notificationbar_display_error (_tmp4_, _tmp5_, BOXES_NOTIFICATIONBAR_DEFAULT_TIMEOUT);
#line 100 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp7_ = _tmp6_;
#line 100 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_object_unref0 (_tmp7_);
#line 5400 "wizard.c"
	}
}


static void __boxes_wizard_page_____lambda177__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self) {
#line 96 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_boxes_wizard_page_____lambda177_ (self, source_object, res);
#line 96 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_unref (self);
#line 5410 "wizard.c"
}


static gint g_ptr_array_get_length (GPtrArray* self) {
	gint result;
	guint _tmp0_;
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, 0);
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	_tmp0_ = self->len;
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	result = (gint) _tmp0_;
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	return result;
#line 5425 "wizard.c"
}


static void g_ptr_array_set_length (GPtrArray* self, gint value) {
	gint _tmp0_;
#line 4081 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_return_if_fail (self != NULL);
#line 4081 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	_tmp0_ = value;
#line 4081 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_ptr_array_set_size (self, _tmp0_);
#line 5437 "wizard.c"
}


static void boxes_wizard_set_page (BoxesWizard* self, BoxesWizardPage value) {
	GtkButton* _tmp0_;
	BoxesWizardPage _tmp1_;
	BoxesWizardPage _tmp2_;
	BoxesWizardPage _tmp3_;
	BoxesWizardPage _tmp4_;
	gboolean forwards;
	BoxesWizardPage _tmp5_;
	gboolean _tmp16_;
	BoxesWizardPage _tmp33_;
	gboolean _tmp34_ = FALSE;
	BoxesWizardPage _tmp35_;
	GtkNotebook* _tmp36_;
	BoxesWizardPage _tmp37_;
	GPtrArray* _tmp51_;
	BoxesWizardPage _tmp52_;
	BoxesWizardPage _tmp53_;
	gconstpointer _tmp54_ = NULL;
	GtkStyleContext* _tmp55_ = NULL;
#line 45 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 46 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->back_button;
#line 46 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = value;
#line 46 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_set_sensitive ((GtkWidget*) _tmp0_, _tmp1_ != BOXES_WIZARD_PAGE_INTRODUCTION);
#line 48 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = value;
#line 48 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = boxes_wizard_get_page (self);
#line 48 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = _tmp3_;
#line 48 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	forwards = _tmp2_ > _tmp4_;
#line 50 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = value;
#line 50 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	switch (_tmp5_) {
#line 50 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case BOXES_WIZARD_PAGE_INTRODUCTION:
#line 5482 "wizard.c"
		{
			GtkButton* _tmp6_;
			GtkButton* _tmp7_;
			GtkButton* _tmp8_;
			GtkButton* _tmp9_;
			GtkButton* _tmp10_;
			GtkButton* _tmp11_;
#line 52 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp6_ = self->priv->create_button;
#line 52 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			gtk_widget_set_visible ((GtkWidget*) _tmp6_, FALSE);
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp7_ = self->priv->continue_button;
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			gtk_widget_set_visible ((GtkWidget*) _tmp7_, TRUE);
#line 54 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp8_ = self->priv->continue_button;
#line 54 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp9_ = _g_object_ref0 (_tmp8_);
#line 54 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_g_object_unref0 (self->priv->next_button);
#line 54 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			self->priv->next_button = _tmp9_;
#line 55 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp10_ = self->priv->next_button;
#line 55 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			gtk_widget_set_sensitive ((GtkWidget*) _tmp10_, TRUE);
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp11_ = self->priv->next_button;
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			gtk_widget_grab_focus ((GtkWidget*) _tmp11_);
#line 57 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			break;
#line 5516 "wizard.c"
		}
#line 50 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case BOXES_WIZARD_PAGE_SOURCE:
#line 5520 "wizard.c"
		{
			BoxesWizardSource* _tmp12_;
			BoxesWizardSource* _tmp13_;
			BoxesSourcePage _tmp14_;
			BoxesSourcePage _tmp15_;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp12_ = self->priv->wizard_source;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp13_ = self->priv->wizard_source;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp14_ = boxes_wizard_source_get_page (_tmp13_);
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp15_ = _tmp14_;
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			boxes_wizard_source_set_page (_tmp12_, _tmp15_);
#line 62 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			break;
#line 5538 "wizard.c"
		}
		default:
#line 50 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		break;
#line 5543 "wizard.c"
	}
#line 65 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp16_ = forwards;
#line 65 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp16_) {
#line 5549 "wizard.c"
		BoxesWizardPage _tmp17_;
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp17_ = value;
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		switch (_tmp17_) {
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			case BOXES_WIZARD_PAGE_SOURCE:
#line 5557 "wizard.c"
			{
				BoxesWizardSource* _tmp18_;
				BoxesWizardSource* _tmp19_;
#line 68 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp18_ = self->priv->wizard_source;
#line 68 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_wizard_source_set_selected (_tmp18_, NULL);
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp19_ = self->priv->wizard_source;
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_wizard_source_set_page (_tmp19_, BOXES_SOURCE_PAGE_MAIN);
#line 70 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				break;
#line 5571 "wizard.c"
			}
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			case BOXES_WIZARD_PAGE_PREPARATION:
#line 5575 "wizard.c"
			{
				gboolean _tmp20_ = FALSE;
#line 73 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp20_ = boxes_wizard_prepare (self);
#line 73 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (!_tmp20_) {
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					return;
#line 5584 "wizard.c"
				}
#line 75 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				break;
#line 5588 "wizard.c"
			}
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			case BOXES_WIZARD_PAGE_SETUP:
#line 5592 "wizard.c"
			{
				gboolean _tmp21_ = FALSE;
#line 78 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp21_ = boxes_wizard_setup (self);
#line 78 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (!_tmp21_) {
#line 79 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					return;
#line 5601 "wizard.c"
				}
#line 80 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				break;
#line 5605 "wizard.c"
			}
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			case BOXES_WIZARD_PAGE_REVIEW:
#line 5609 "wizard.c"
			{
				GtkButton* _tmp22_;
				GtkButton* _tmp23_;
				GtkButton* _tmp24_;
				GtkButton* _tmp25_;
				GtkButton* _tmp26_;
#line 83 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp22_ = self->priv->continue_button;
#line 83 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				gtk_widget_set_visible ((GtkWidget*) _tmp22_, FALSE);
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp23_ = self->priv->create_button;
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				gtk_widget_set_visible ((GtkWidget*) _tmp23_, TRUE);
#line 85 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp24_ = self->priv->create_button;
#line 85 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp25_ = _g_object_ref0 (_tmp24_);
#line 85 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_object_unref0 (self->priv->next_button);
#line 85 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				self->priv->next_button = _tmp25_;
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp26_ = self->priv->next_button;
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				gtk_widget_set_sensitive ((GtkWidget*) _tmp26_, FALSE);
#line 88 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_wizard_review (self, __boxes_wizard_page_____lambda175__gasync_ready_callback, g_object_ref (self));
#line 93 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				break;
#line 5640 "wizard.c"
			}
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			case BOXES_WIZARD_PAGE_LAST:
#line 5644 "wizard.c"
			{
#line 96 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_wizard_create (self, __boxes_wizard_page_____lambda177__gasync_ready_callback, g_object_ref (self));
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				return;
#line 5650 "wizard.c"
			}
			default:
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			break;
#line 5655 "wizard.c"
		}
	} else {
		BoxesWizardPage _tmp27_;
		BoxesWizardPage _tmp28_;
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp27_ = boxes_wizard_get_page (self);
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_tmp28_ = _tmp27_;
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		switch (_tmp28_) {
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			case BOXES_WIZARD_PAGE_REVIEW:
#line 5668 "wizard.c"
			{
				GtkButton* _tmp29_;
				GtkButton* _tmp30_;
				GtkButton* _tmp31_;
				GtkButton* _tmp32_;
#line 107 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp29_ = self->priv->create_button;
#line 107 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				gtk_widget_set_visible ((GtkWidget*) _tmp29_, FALSE);
#line 108 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp30_ = self->priv->continue_button;
#line 108 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				gtk_widget_set_visible ((GtkWidget*) _tmp30_, TRUE);
#line 109 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp31_ = self->priv->continue_button;
#line 109 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp32_ = _g_object_ref0 (_tmp31_);
#line 109 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_object_unref0 (self->priv->next_button);
#line 109 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				self->priv->next_button = _tmp32_;
#line 110 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				boxes_wizard_destroy_machine (self);
#line 111 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				break;
#line 5694 "wizard.c"
			}
			default:
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			break;
#line 5699 "wizard.c"
		}
	}
#line 115 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp33_ = value;
#line 115 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp34_ = boxes_wizard_skip_page (self, _tmp33_);
#line 115 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp34_) {
#line 116 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return;
#line 5710 "wizard.c"
	}
#line 118 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp35_ = value;
#line 118 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->_page = _tmp35_;
#line 119 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp36_ = self->priv->notebook;
#line 119 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp37_ = value;
#line 119 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_notebook_set_current_page (_tmp36_, (gint) _tmp37_);
#line 5722 "wizard.c"
	{
		gint i;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		i = 0;
#line 5727 "wizard.c"
		{
			gboolean _tmp38_;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			_tmp38_ = TRUE;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			while (TRUE) {
#line 5734 "wizard.c"
				gboolean _tmp39_;
				gint _tmp41_;
				GPtrArray* _tmp42_;
				gint _tmp43_;
				gint _tmp44_;
				GPtrArray* _tmp45_;
				gint _tmp46_;
				gconstpointer _tmp47_ = NULL;
				GtkLabel* _tmp48_;
				GtkLabel* label;
				GtkLabel* _tmp49_;
				GtkStyleContext* _tmp50_ = NULL;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp39_ = _tmp38_;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (!_tmp39_) {
#line 5751 "wizard.c"
					gint _tmp40_;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					_tmp40_ = i;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					i = _tmp40_ + 1;
#line 5757 "wizard.c"
				}
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp38_ = FALSE;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp41_ = i;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp42_ = self->priv->steps;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp43_ = g_ptr_array_get_length (_tmp42_);
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp44_ = _tmp43_;
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				if (!(_tmp41_ < _tmp44_)) {
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
					break;
#line 5773 "wizard.c"
				}
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp45_ = self->priv->steps;
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp46_ = i;
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp47_ = g_ptr_array_index (_tmp45_, (guint) _tmp46_);
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp48_ = _g_object_ref0 ((GtkLabel*) _tmp47_);
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				label = _tmp48_;
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp49_ = label;
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp50_ = gtk_widget_get_style_context ((GtkWidget*) _tmp49_);
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				gtk_style_context_remove_class (_tmp50_, "boxes-wizard-current-page-label");
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_g_object_unref0 (label);
#line 5793 "wizard.c"
			}
		}
	}
#line 126 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp51_ = self->priv->steps;
#line 126 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp52_ = boxes_wizard_get_page (self);
#line 126 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp53_ = _tmp52_;
#line 126 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp54_ = g_ptr_array_index (_tmp51_, (guint) _tmp53_);
#line 126 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp55_ = gtk_widget_get_style_context ((GtkWidget*) ((GtkLabel*) _tmp54_));
#line 126 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp55_, "boxes-wizard-current-page-label");
#line 5809 "wizard.c"
}


static void _g_object_unref0_ (gpointer var) {
#line 131 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
#line 5816 "wizard.c"
}


static GObject * boxes_wizard_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	BoxesWizard * self;
	GPtrArray* _tmp0_;
	GPtrArray* _tmp1_;
	BoxesMediaManager* _tmp2_ = NULL;
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	parent_class = G_OBJECT_CLASS (boxes_wizard_parent_class);
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BOXES_TYPE_WIZARD, BoxesWizard);
#line 131 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = g_ptr_array_new_with_free_func (_g_object_unref0_);
#line 131 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_ptr_array_unref0 (self->priv->steps);
#line 131 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->steps = _tmp0_;
#line 132 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = self->priv->steps;
#line 132 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_ptr_array_set_length (_tmp1_, (gint) BOXES_WIZARD_PAGE_LAST);
#line 133 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = boxes_media_manager_get_instance ();
#line 133 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->media_manager);
#line 133 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->media_manager = _tmp2_;
#line 130 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return obj;
#line 5851 "wizard.c"
}


static BoxesWizardWizardSummary* boxes_wizard_wizard_summary_construct (GType object_type) {
	BoxesWizardWizardSummary * self = NULL;
	GtkGrid* _tmp0_;
	GtkGrid* _tmp1_;
	GtkGrid* _tmp2_;
#line 724 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = (BoxesWizardWizardSummary*) g_object_new (object_type, NULL);
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = (GtkGrid*) gtk_grid_new ();
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp0_);
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->grid);
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->grid = _tmp0_;
#line 726 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = self->priv->grid;
#line 726 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_grid_set_row_spacing (_tmp1_, 10);
#line 727 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = self->priv->grid;
#line 727 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_grid_set_column_spacing (_tmp2_, 20);
#line 728 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->current_row = 0;
#line 730 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_wizard_summary_clear (self);
#line 724 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return self;
#line 5884 "wizard.c"
}


static BoxesWizardWizardSummary* boxes_wizard_wizard_summary_new (void) {
#line 724 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return boxes_wizard_wizard_summary_construct (BOXES_WIZARD_TYPE_WIZARD_SUMMARY);
#line 5891 "wizard.c"
}


static void boxes_wizard_wizard_summary_add_property (BoxesWizardWizardSummary* self, const gchar* name, const gchar* value) {
	const gchar* _tmp0_;
	const gchar* _tmp1_;
	GtkLabel* _tmp2_;
	GtkLabel* label_name;
	GtkLabel* _tmp3_;
	GtkStyleContext* _tmp4_ = NULL;
	GtkLabel* _tmp5_;
	GtkGrid* _tmp6_;
	GtkLabel* _tmp7_;
	gint _tmp8_;
	const gchar* _tmp9_;
	GtkLabel* _tmp10_;
	GtkLabel* label_value;
	GtkLabel* _tmp11_;
	GtkStyleContext* _tmp12_ = NULL;
	GtkLabel* _tmp13_;
	GtkGrid* _tmp14_;
	GtkLabel* _tmp15_;
	gint _tmp16_;
	gint _tmp17_;
	GtkGrid* _tmp18_;
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (name != NULL);
#line 734 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = value;
#line 734 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp0_ == NULL) {
#line 735 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return;
#line 5927 "wizard.c"
	}
#line 737 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = name;
#line 737 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = (GtkLabel*) gtk_label_new (_tmp1_);
#line 737 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp2_);
#line 737 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	label_name = _tmp2_;
#line 738 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = label_name;
#line 738 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = gtk_widget_get_style_context ((GtkWidget*) _tmp3_);
#line 738 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp4_, "boxes-wizard-summary-prop-name-label");
#line 739 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = label_name;
#line 739 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set ((GtkMisc*) _tmp5_, "xalign", 1.0f, NULL);
#line 740 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp6_ = self->priv->grid;
#line 740 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp7_ = label_name;
#line 740 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp8_ = self->priv->current_row;
#line 740 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_grid_attach (_tmp6_, (GtkWidget*) _tmp7_, 0, _tmp8_, 1, 1);
#line 742 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp9_ = value;
#line 742 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp10_ = (GtkLabel*) gtk_label_new (_tmp9_);
#line 742 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp10_);
#line 742 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	label_value = _tmp10_;
#line 743 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp11_ = label_value;
#line 743 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp12_ = gtk_widget_get_style_context ((GtkWidget*) _tmp11_);
#line 743 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp12_, "boxes-wizard-summary-prop-value-label");
#line 744 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp13_ = label_value;
#line 744 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_set ((GtkMisc*) _tmp13_, "xalign", 0.0f, NULL);
#line 745 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp14_ = self->priv->grid;
#line 745 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp15_ = label_value;
#line 745 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp16_ = self->priv->current_row;
#line 745 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_grid_attach (_tmp14_, (GtkWidget*) _tmp15_, 1, _tmp16_, 1, 1);
#line 747 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp17_ = self->priv->current_row;
#line 747 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->current_row = _tmp17_ + 1;
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp18_ = self->priv->grid;
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp18_);
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (label_value);
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (label_name);
#line 5993 "wizard.c"
}


static Block72Data* block72_data_ref (Block72Data* _data72_) {
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_atomic_int_inc (&_data72_->_ref_count_);
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return _data72_;
#line 6002 "wizard.c"
}


static void block72_data_unref (void * _userdata_) {
	Block72Data* _data72_;
	_data72_ = (Block72Data*) _userdata_;
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (g_atomic_int_dec_and_test (&_data72_->_ref_count_)) {
#line 6011 "wizard.c"
		BoxesWizardWizardSummary * self;
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		self = _data72_->self;
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_object_unref0 (self);
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_slice_free (Block72Data, _data72_);
#line 6019 "wizard.c"
	}
}


static void __lambda170_ (Block72Data* _data72_) {
	BoxesWizardWizardSummary * self;
	BoxesWizardWizardSummaryCustomizeFunc _tmp0_;
	void* _tmp0__target;
#line 761 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = _data72_->self;
#line 761 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = _data72_->customize_func;
#line 761 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0__target = _data72_->customize_func_target;
#line 761 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ (_tmp0__target);
#line 6036 "wizard.c"
}


static void ___lambda170__gtk_button_clicked (GtkButton* _sender, gpointer self) {
#line 761 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	__lambda170_ (self);
#line 6043 "wizard.c"
}


static void boxes_wizard_wizard_summary_append_customize_button (BoxesWizardWizardSummary* self, BoxesWizardWizardSummaryCustomizeFunc customize_func, void* customize_func_target) {
	Block72Data* _data72_;
	BoxesWizardWizardSummaryCustomizeFunc _tmp0_;
	void* _tmp0__target;
	gint _tmp1_;
	const gchar* _tmp2_ = NULL;
	GtkButton* _tmp3_;
	GtkButton* button;
	GtkButton* _tmp4_;
	GtkStyleContext* _tmp5_ = NULL;
	GtkGrid* _tmp6_;
	GtkButton* _tmp7_;
	gint _tmp8_;
	GtkButton* _tmp9_;
	GtkButton* _tmp10_;
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data72_ = g_slice_new0 (Block72Data);
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data72_->_ref_count_ = 1;
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data72_->self = g_object_ref (self);
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = customize_func;
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0__target = customize_func_target;
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data72_->customize_func = _tmp0_;
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data72_->customize_func_target = _tmp0__target;
#line 753 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = self->priv->current_row;
#line 753 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	if (_tmp1_ == 0) {
#line 754 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		block72_data_unref (_data72_);
#line 754 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_data72_ = NULL;
#line 754 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		return;
#line 6088 "wizard.c"
	}
#line 756 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp2_ = _ ("C_ustomize...");
#line 756 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp3_ = (GtkButton*) gtk_button_new_with_mnemonic (_tmp2_);
#line 756 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_ref_sink (_tmp3_);
#line 756 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	button = _tmp3_;
#line 757 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp4_ = button;
#line 757 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp5_ = gtk_widget_get_style_context ((GtkWidget*) _tmp4_);
#line 757 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_style_context_add_class (_tmp5_, "boxes-wizard-summary-customize-button");
#line 758 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp6_ = self->priv->grid;
#line 758 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp7_ = button;
#line 758 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp8_ = self->priv->current_row;
#line 758 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_grid_attach (_tmp6_, (GtkWidget*) _tmp7_, 2, _tmp8_ - 1, 1, 1);
#line 759 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp9_ = button;
#line 759 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	gtk_widget_show ((GtkWidget*) _tmp9_);
#line 761 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp10_ = button;
#line 761 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_signal_connect_data (_tmp10_, "clicked", (GCallback) ___lambda170__gtk_button_clicked, block72_data_ref (_data72_), (GClosureNotify) block72_data_unref, 0);
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (button);
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	block72_data_unref (_data72_);
#line 751 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_data72_ = NULL;
#line 6126 "wizard.c"
}


static void boxes_wizard_wizard_summary_clear (BoxesWizardWizardSummary* self) {
	GtkGrid* _tmp0_;
	GList* _tmp1_ = NULL;
#line 764 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_if_fail (self != NULL);
#line 765 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->grid;
#line 765 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp1_ = gtk_container_get_children ((GtkContainer*) _tmp0_);
#line 6139 "wizard.c"
	{
		GList* child_collection = NULL;
		GList* child_it = NULL;
#line 765 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		child_collection = _tmp1_;
#line 765 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		for (child_it = child_collection; child_it != NULL; child_it = child_it->next) {
#line 6147 "wizard.c"
			GtkWidget* child = NULL;
#line 765 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
			child = (GtkWidget*) child_it->data;
#line 6151 "wizard.c"
			{
				GtkGrid* _tmp2_;
				GtkWidget* _tmp3_;
#line 766 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp2_ = self->priv->grid;
#line 766 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				_tmp3_ = child;
#line 766 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
				gtk_container_remove ((GtkContainer*) _tmp2_, _tmp3_);
#line 6161 "wizard.c"
			}
		}
#line 765 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		_g_list_free0 (child_collection);
#line 6166 "wizard.c"
	}
#line 769 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv->current_row = 0;
#line 6170 "wizard.c"
}


static GtkWidget* boxes_wizard_wizard_summary_get_widget (BoxesWizardWizardSummary* self) {
	GtkWidget* result;
	GtkGrid* _tmp0_;
#line 720 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 720 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_tmp0_ = self->priv->grid;
#line 720 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	result = (GtkWidget*) _tmp0_;
#line 720 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	return result;
#line 6185 "wizard.c"
}


static void boxes_wizard_wizard_summary_class_init (BoxesWizardWizardSummaryClass * klass) {
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_wizard_summary_parent_class = g_type_class_peek_parent (klass);
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_type_class_add_private (klass, sizeof (BoxesWizardWizardSummaryPrivate));
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_boxes_wizard_wizard_summary_get_property;
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	G_OBJECT_CLASS (klass)->finalize = boxes_wizard_wizard_summary_finalize;
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_WIZARD_WIZARD_SUMMARY_WIDGET, g_param_spec_object ("widget", "widget", "widget", GTK_TYPE_WIDGET, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 6200 "wizard.c"
}


static void boxes_wizard_wizard_summary_instance_init (BoxesWizardWizardSummary * self) {
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv = BOXES_WIZARD_WIZARD_SUMMARY_GET_PRIVATE (self);
#line 6207 "wizard.c"
}


static void boxes_wizard_wizard_summary_finalize (GObject* obj) {
	BoxesWizardWizardSummary * self;
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BOXES_WIZARD_TYPE_WIZARD_SUMMARY, BoxesWizardWizardSummary);
#line 721 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->grid);
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	G_OBJECT_CLASS (boxes_wizard_wizard_summary_parent_class)->finalize (obj);
#line 6219 "wizard.c"
}


static GType boxes_wizard_wizard_summary_get_type (void) {
	static volatile gsize boxes_wizard_wizard_summary_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_wizard_wizard_summary_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (BoxesWizardWizardSummaryClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) boxes_wizard_wizard_summary_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BoxesWizardWizardSummary), 0, (GInstanceInitFunc) boxes_wizard_wizard_summary_instance_init, NULL };
		GType boxes_wizard_wizard_summary_type_id;
		boxes_wizard_wizard_summary_type_id = g_type_register_static (G_TYPE_OBJECT, "BoxesWizardWizardSummary", &g_define_type_info, 0);
		g_once_init_leave (&boxes_wizard_wizard_summary_type_id__volatile, boxes_wizard_wizard_summary_type_id);
	}
	return boxes_wizard_wizard_summary_type_id__volatile;
}


static void _vala_boxes_wizard_wizard_summary_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	BoxesWizardWizardSummary * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_WIZARD_TYPE_WIZARD_SUMMARY, BoxesWizardWizardSummary);
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	switch (property_id) {
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case BOXES_WIZARD_WIZARD_SUMMARY_WIDGET:
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_value_set_object (value, boxes_wizard_wizard_summary_get_widget (self));
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		break;
#line 6246 "wizard.c"
		default:
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 717 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		break;
#line 6252 "wizard.c"
	}
}


static void boxes_wizard_class_init (BoxesWizardClass * klass) {
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	boxes_wizard_parent_class = g_type_class_peek_parent (klass);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_type_class_add_private (klass, sizeof (BoxesWizardPrivate));
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	BOXES_UI_CLASS (klass)->ui_state_changed = boxes_wizard_real_ui_state_changed;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	BOXES_UI_CLASS (klass)->get_actor = boxes_wizard_real_get_actor;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_boxes_wizard_get_property;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_boxes_wizard_set_property;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	G_OBJECT_CLASS (klass)->constructor = boxes_wizard_constructor;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	G_OBJECT_CLASS (klass)->finalize = boxes_wizard_finalize;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_WIZARD_ACTOR, g_param_spec_object ("actor", "actor", "actor", clutter_actor_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_WIZARD_MACHINE, g_param_spec_object ("machine", "machine", "machine", BOXES_TYPE_MACHINE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 6278 "wizard.c"
}


static void boxes_wizard_instance_init (BoxesWizard * self) {
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self->priv = BOXES_WIZARD_GET_PRIVATE (self);
#line 6285 "wizard.c"
}


static void boxes_wizard_finalize (GObject* obj) {
	BoxesWizard * self;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BOXES_TYPE_WIZARD, BoxesWizard);
#line 16 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->gtk_actor);
#line 17 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_ptr_array_unref0 (self->priv->steps);
#line 18 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->notebook);
#line 19 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->cancel_button);
#line 20 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->back_button);
#line 21 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->next_button);
#line 22 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->continue_button);
#line 23 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->create_button);
#line 24 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->toolbar_sizegroup);
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->wizard_source);
#line 26 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->summary);
#line 27 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->source);
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->prep_progress);
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->prep_media_label);
#line 30 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->prep_status_label);
#line 31 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->setup_vbox);
#line 32 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->review_label);
#line 33 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->nokvm_label);
#line 34 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->installer_image);
#line 36 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->media_manager);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_boxes_vm_creator_unref0 (self->priv->vm_creator);
#line 39 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->_machine);
#line 345 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	_g_object_unref0 (self->priv->review_cancellable);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	G_OBJECT_CLASS (boxes_wizard_parent_class)->finalize (obj);
#line 6341 "wizard.c"
}


GType boxes_wizard_get_type (void) {
	static volatile gsize boxes_wizard_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_wizard_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (BoxesWizardClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) boxes_wizard_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BoxesWizard), 0, (GInstanceInitFunc) boxes_wizard_instance_init, NULL };
		GType boxes_wizard_type_id;
		boxes_wizard_type_id = g_type_register_static (BOXES_TYPE_UI, "BoxesWizard", &g_define_type_info, 0);
		g_once_init_leave (&boxes_wizard_type_id__volatile, boxes_wizard_type_id);
	}
	return boxes_wizard_type_id__volatile;
}


static void _vala_boxes_wizard_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	BoxesWizard * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_WIZARD, BoxesWizard);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	switch (property_id) {
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case BOXES_WIZARD_ACTOR:
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_value_set_object (value, boxes_ui_get_actor ((BoxesUI*) self));
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		break;
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case BOXES_WIZARD_MACHINE:
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		g_value_set_object (value, boxes_wizard_get_machine (self));
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		break;
#line 6374 "wizard.c"
		default:
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		break;
#line 6380 "wizard.c"
	}
}


static void _vala_boxes_wizard_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	BoxesWizard * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_WIZARD, BoxesWizard);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
	switch (property_id) {
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		case BOXES_WIZARD_MACHINE:
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		boxes_wizard_set_machine (self, g_value_get_object (value));
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		break;
#line 6396 "wizard.c"
		default:
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 13 "/extra-data/checkout/gnome/gnome-boxes/src/wizard.vala"
		break;
#line 6402 "wizard.c"
	}
}



