/* app.c generated by valac 0.18.0.79-04df6, the Vala compiler
 * generated from app.vala, do not modify */

/* This file is part of GNOME Boxes. License: LGPLv2+*/

#include <glib.h>
#include <glib-object.h>
#include <gio/gio.h>
#include <libcommon.h>
#include <gtk/gtk.h>
#include <clutter/clutter.h>
#include <clutter-gtk/clutter-gtk.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <version.h>
#include <config.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <glib/gstdio.h>
#include <libvirt-gobject/libvirt-gobject.h>
#include <gdk/gdk.h>
#include <float.h>
#include <math.h>
#include <libgd/gd.h>


#define BOXES_TYPE_BROKER (boxes_broker_get_type ())
#define BOXES_BROKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_BROKER, BoxesBroker))
#define BOXES_BROKER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_BROKER, BoxesBrokerClass))
#define BOXES_IS_BROKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_BROKER))
#define BOXES_IS_BROKER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_BROKER))
#define BOXES_BROKER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_BROKER, BoxesBrokerClass))

typedef struct _BoxesBroker BoxesBroker;
typedef struct _BoxesBrokerClass BoxesBrokerClass;
typedef struct _BoxesBrokerPrivate BoxesBrokerPrivate;

#define BOXES_TYPE_APP_PAGE (boxes_app_page_get_type ())

#define BOXES_TYPE_APPLICATION (boxes_application_get_type ())
#define BOXES_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_APPLICATION, BoxesApplication))
#define BOXES_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_APPLICATION, BoxesApplicationClass))
#define BOXES_IS_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_APPLICATION))
#define BOXES_IS_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_APPLICATION))
#define BOXES_APPLICATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_APPLICATION, BoxesApplicationClass))

typedef struct _BoxesApplication BoxesApplication;
typedef struct _BoxesApplicationClass BoxesApplicationClass;
typedef struct _BoxesApplicationPrivate BoxesApplicationPrivate;

#define BOXES_TYPE_UI (boxes_ui_get_type ())
#define BOXES_UI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_UI, BoxesUI))
#define BOXES_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_UI, BoxesUIClass))
#define BOXES_IS_UI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_UI))
#define BOXES_IS_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_UI))
#define BOXES_UI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_UI, BoxesUIClass))

typedef struct _BoxesUI BoxesUI;
typedef struct _BoxesUIClass BoxesUIClass;

#define BOXES_TYPE_APP (boxes_app_get_type ())
#define BOXES_APP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_APP, BoxesApp))
#define BOXES_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_APP, BoxesAppClass))
#define BOXES_IS_APP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_APP))
#define BOXES_IS_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_APP))
#define BOXES_APP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_APP, BoxesAppClass))

typedef struct _BoxesApp BoxesApp;
typedef struct _BoxesAppClass BoxesAppClass;
typedef struct _BoxesUIPrivate BoxesUIPrivate;

#define BOXES_TYPE_UI_STATE (boxes_ui_state_get_type ())
typedef struct _BoxesAppPrivate BoxesAppPrivate;

#define BOXES_TYPE_CLUTTER_WIDGET (boxes_clutter_widget_get_type ())
#define BOXES_CLUTTER_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidget))
#define BOXES_CLUTTER_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidgetClass))
#define BOXES_IS_CLUTTER_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_CLUTTER_WIDGET))
#define BOXES_IS_CLUTTER_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_CLUTTER_WIDGET))
#define BOXES_CLUTTER_WIDGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_CLUTTER_WIDGET, BoxesClutterWidgetClass))

typedef struct _BoxesClutterWidget BoxesClutterWidget;
typedef struct _BoxesClutterWidgetClass BoxesClutterWidgetClass;

#define BOXES_TYPE_COLLECTION_ITEM (boxes_collection_item_get_type ())
#define BOXES_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItem))
#define BOXES_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))
#define BOXES_IS_COLLECTION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_IS_COLLECTION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_ITEM))
#define BOXES_COLLECTION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_ITEM, BoxesCollectionItemClass))

typedef struct _BoxesCollectionItem BoxesCollectionItem;
typedef struct _BoxesCollectionItemClass BoxesCollectionItemClass;

#define BOXES_TYPE_SEARCHBAR (boxes_searchbar_get_type ())
#define BOXES_SEARCHBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SEARCHBAR, BoxesSearchbar))
#define BOXES_SEARCHBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SEARCHBAR, BoxesSearchbarClass))
#define BOXES_IS_SEARCHBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SEARCHBAR))
#define BOXES_IS_SEARCHBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SEARCHBAR))
#define BOXES_SEARCHBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SEARCHBAR, BoxesSearchbarClass))

typedef struct _BoxesSearchbar BoxesSearchbar;
typedef struct _BoxesSearchbarClass BoxesSearchbarClass;

#define BOXES_TYPE_TOPBAR (boxes_topbar_get_type ())
#define BOXES_TOPBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_TOPBAR, BoxesTopbar))
#define BOXES_TOPBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_TOPBAR, BoxesTopbarClass))
#define BOXES_IS_TOPBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_TOPBAR))
#define BOXES_IS_TOPBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_TOPBAR))
#define BOXES_TOPBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_TOPBAR, BoxesTopbarClass))

typedef struct _BoxesTopbar BoxesTopbar;
typedef struct _BoxesTopbarClass BoxesTopbarClass;

#define BOXES_TYPE_NOTIFICATIONBAR (boxes_notificationbar_get_type ())
#define BOXES_NOTIFICATIONBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbar))
#define BOXES_NOTIFICATIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbarClass))
#define BOXES_IS_NOTIFICATIONBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_NOTIFICATIONBAR))
#define BOXES_IS_NOTIFICATIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_NOTIFICATIONBAR))
#define BOXES_NOTIFICATIONBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_NOTIFICATIONBAR, BoxesNotificationbarClass))

typedef struct _BoxesNotificationbar BoxesNotificationbar;
typedef struct _BoxesNotificationbarClass BoxesNotificationbarClass;

#define BOXES_TYPE_REVEALER (boxes_revealer_get_type ())
#define BOXES_REVEALER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_REVEALER, BoxesRevealer))
#define BOXES_REVEALER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_REVEALER, BoxesRevealerClass))
#define BOXES_IS_REVEALER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_REVEALER))
#define BOXES_IS_REVEALER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_REVEALER))
#define BOXES_REVEALER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_REVEALER, BoxesRevealerClass))

typedef struct _BoxesRevealer BoxesRevealer;
typedef struct _BoxesRevealerClass BoxesRevealerClass;

#define BOXES_TYPE_SIDEBAR (boxes_sidebar_get_type ())
#define BOXES_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SIDEBAR, BoxesSidebar))
#define BOXES_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SIDEBAR, BoxesSidebarClass))
#define BOXES_IS_SIDEBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SIDEBAR))
#define BOXES_IS_SIDEBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SIDEBAR))
#define BOXES_SIDEBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SIDEBAR, BoxesSidebarClass))

typedef struct _BoxesSidebar BoxesSidebar;
typedef struct _BoxesSidebarClass BoxesSidebarClass;

#define BOXES_TYPE_SELECTIONBAR (boxes_selectionbar_get_type ())
#define BOXES_SELECTIONBAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbar))
#define BOXES_SELECTIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbarClass))
#define BOXES_IS_SELECTIONBAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_SELECTIONBAR))
#define BOXES_IS_SELECTIONBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_SELECTIONBAR))
#define BOXES_SELECTIONBAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_SELECTIONBAR, BoxesSelectionbarClass))

typedef struct _BoxesSelectionbar BoxesSelectionbar;
typedef struct _BoxesSelectionbarClass BoxesSelectionbarClass;

#define BOXES_TYPE_WIZARD (boxes_wizard_get_type ())
#define BOXES_WIZARD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_WIZARD, BoxesWizard))
#define BOXES_WIZARD_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_WIZARD, BoxesWizardClass))
#define BOXES_IS_WIZARD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_WIZARD))
#define BOXES_IS_WIZARD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_WIZARD))
#define BOXES_WIZARD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_WIZARD, BoxesWizardClass))

typedef struct _BoxesWizard BoxesWizard;
typedef struct _BoxesWizardClass BoxesWizardClass;

#define BOXES_TYPE_PROPERTIES (boxes_properties_get_type ())
#define BOXES_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_PROPERTIES, BoxesProperties))
#define BOXES_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_PROPERTIES, BoxesPropertiesClass))
#define BOXES_IS_PROPERTIES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_PROPERTIES))
#define BOXES_IS_PROPERTIES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_PROPERTIES))
#define BOXES_PROPERTIES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_PROPERTIES, BoxesPropertiesClass))

typedef struct _BoxesProperties BoxesProperties;
typedef struct _BoxesPropertiesClass BoxesPropertiesClass;

#define BOXES_TYPE_DISPLAY_PAGE (boxes_display_page_get_type ())
#define BOXES_DISPLAY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPage))
#define BOXES_DISPLAY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPageClass))
#define BOXES_IS_DISPLAY_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_DISPLAY_PAGE))
#define BOXES_IS_DISPLAY_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_DISPLAY_PAGE))
#define BOXES_DISPLAY_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_DISPLAY_PAGE, BoxesDisplayPageClass))

typedef struct _BoxesDisplayPage BoxesDisplayPage;
typedef struct _BoxesDisplayPageClass BoxesDisplayPageClass;

#define BOXES_TYPE_EMPTY_BOXES (boxes_empty_boxes_get_type ())
#define BOXES_EMPTY_BOXES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxes))
#define BOXES_EMPTY_BOXES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxesClass))
#define BOXES_IS_EMPTY_BOXES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_EMPTY_BOXES))
#define BOXES_IS_EMPTY_BOXES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_EMPTY_BOXES))
#define BOXES_EMPTY_BOXES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_EMPTY_BOXES, BoxesEmptyBoxesClass))

typedef struct _BoxesEmptyBoxes BoxesEmptyBoxes;
typedef struct _BoxesEmptyBoxesClass BoxesEmptyBoxesClass;

#define BOXES_TYPE_COLLECTION (boxes_collection_get_type ())
#define BOXES_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION, BoxesCollection))
#define BOXES_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION, BoxesCollectionClass))
#define BOXES_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION))
#define BOXES_IS_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION))
#define BOXES_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION, BoxesCollectionClass))

typedef struct _BoxesCollection BoxesCollection;
typedef struct _BoxesCollectionClass BoxesCollectionClass;

#define BOXES_TYPE_COLLECTION_FILTER (boxes_collection_filter_get_type ())
#define BOXES_COLLECTION_FILTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilter))
#define BOXES_COLLECTION_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilterClass))
#define BOXES_IS_COLLECTION_FILTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_FILTER))
#define BOXES_IS_COLLECTION_FILTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_FILTER))
#define BOXES_COLLECTION_FILTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_FILTER, BoxesCollectionFilterClass))

typedef struct _BoxesCollectionFilter BoxesCollectionFilter;
typedef struct _BoxesCollectionFilterClass BoxesCollectionFilterClass;

#define BOXES_TYPE_COLLECTION_VIEW (boxes_collection_view_get_type ())
#define BOXES_COLLECTION_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionView))
#define BOXES_COLLECTION_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionViewClass))
#define BOXES_IS_COLLECTION_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_COLLECTION_VIEW))
#define BOXES_IS_COLLECTION_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_COLLECTION_VIEW))
#define BOXES_COLLECTION_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_COLLECTION_VIEW, BoxesCollectionViewClass))

typedef struct _BoxesCollectionView BoxesCollectionView;
typedef struct _BoxesCollectionViewClass BoxesCollectionViewClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_hash_table_unref0(var) ((var == NULL) ? NULL : (var = (g_hash_table_unref (var), NULL)))

#define BOXES_TYPE_SELECTION_CRITERIA (boxes_selection_criteria_get_type ())

#define BOXES_TYPE_LIBVIRT_BROKER (boxes_libvirt_broker_get_type ())
#define BOXES_LIBVIRT_BROKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_LIBVIRT_BROKER, BoxesLibvirtBroker))
#define BOXES_LIBVIRT_BROKER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_LIBVIRT_BROKER, BoxesLibvirtBrokerClass))
#define BOXES_IS_LIBVIRT_BROKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_LIBVIRT_BROKER))
#define BOXES_IS_LIBVIRT_BROKER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_LIBVIRT_BROKER))
#define BOXES_LIBVIRT_BROKER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_LIBVIRT_BROKER, BoxesLibvirtBrokerClass))

typedef struct _BoxesLibvirtBroker BoxesLibvirtBroker;
typedef struct _BoxesLibvirtBrokerClass BoxesLibvirtBrokerClass;

#define BOXES_TYPE_OVIRT_BROKER (boxes_ovirt_broker_get_type ())
#define BOXES_OVIRT_BROKER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_OVIRT_BROKER, BoxesOvirtBroker))
#define BOXES_OVIRT_BROKER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_OVIRT_BROKER, BoxesOvirtBrokerClass))
#define BOXES_IS_OVIRT_BROKER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_OVIRT_BROKER))
#define BOXES_IS_OVIRT_BROKER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_OVIRT_BROKER))
#define BOXES_OVIRT_BROKER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_OVIRT_BROKER, BoxesOvirtBrokerClass))

typedef struct _BoxesOvirtBroker BoxesOvirtBroker;
typedef struct _BoxesOvirtBrokerClass BoxesOvirtBrokerClass;
typedef struct _Block1Data Block1Data;
typedef struct _Block2Data Block2Data;
#define _g_option_context_free0(var) ((var == NULL) ? NULL : (var = (g_option_context_free (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
typedef struct _Block3Data Block3Data;
typedef struct _Block4Data Block4Data;
typedef struct _BoxesCollectionPrivate BoxesCollectionPrivate;

#define BOXES_TYPE_MACHINE (boxes_machine_get_type ())
#define BOXES_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_MACHINE, BoxesMachine))
#define BOXES_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_MACHINE, BoxesMachineClass))
#define BOXES_IS_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_MACHINE))
#define BOXES_IS_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_MACHINE))
#define BOXES_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_MACHINE, BoxesMachineClass))

typedef struct _BoxesMachine BoxesMachine;
typedef struct _BoxesMachineClass BoxesMachineClass;
typedef struct _BoxesCollectionItemPrivate BoxesCollectionItemPrivate;

#define BOXES_TYPE_IPROPERTIES_PROVIDER (boxes_iproperties_provider_get_type ())
#define BOXES_IPROPERTIES_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER, BoxesIPropertiesProvider))
#define BOXES_IS_IPROPERTIES_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER))
#define BOXES_IPROPERTIES_PROVIDER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), BOXES_TYPE_IPROPERTIES_PROVIDER, BoxesIPropertiesProviderIface))

typedef struct _BoxesIPropertiesProvider BoxesIPropertiesProvider;
typedef struct _BoxesIPropertiesProviderIface BoxesIPropertiesProviderIface;

#define BOXES_TYPE_PROPERTIES_PAGE (boxes_properties_page_get_type ())

#define TYPE_PROPERTY_CREATION_FLAG (property_creation_flag_get_type ())

#define BOXES_TYPE_PROPERTY (boxes_property_get_type ())
#define BOXES_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_PROPERTY, BoxesProperty))
#define BOXES_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_PROPERTY, BoxesPropertyClass))
#define BOXES_IS_PROPERTY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_PROPERTY))
#define BOXES_IS_PROPERTY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_PROPERTY))
#define BOXES_PROPERTY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_PROPERTY, BoxesPropertyClass))

typedef struct _BoxesProperty BoxesProperty;
typedef struct _BoxesPropertyClass BoxesPropertyClass;
typedef struct _BoxesMachinePrivate BoxesMachinePrivate;

#define BOXES_TYPE_REMOTE_MACHINE (boxes_remote_machine_get_type ())
#define BOXES_REMOTE_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_REMOTE_MACHINE, BoxesRemoteMachine))
#define BOXES_REMOTE_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_REMOTE_MACHINE, BoxesRemoteMachineClass))
#define BOXES_IS_REMOTE_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_REMOTE_MACHINE))
#define BOXES_IS_REMOTE_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_REMOTE_MACHINE))
#define BOXES_REMOTE_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_REMOTE_MACHINE, BoxesRemoteMachineClass))

typedef struct _BoxesRemoteMachine BoxesRemoteMachine;
typedef struct _BoxesRemoteMachineClass BoxesRemoteMachineClass;
typedef struct _BoxesAppAddCollectionSourceData BoxesAppAddCollectionSourceData;
typedef struct _BoxesAppSetupSourcesData BoxesAppSetupSourcesData;
#define _g_variant_unref0(var) ((var == NULL) ? NULL : (var = (g_variant_unref (var), NULL)))
typedef struct _BoxesRevealerPrivate BoxesRevealerPrivate;
typedef struct _Block5Data Block5Data;
typedef struct _BoxesCollectionViewPrivate BoxesCollectionViewPrivate;
typedef struct _Block6Data Block6Data;

#define BOXES_TYPE_LIBVIRT_MACHINE (boxes_libvirt_machine_get_type ())
#define BOXES_LIBVIRT_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachine))
#define BOXES_LIBVIRT_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachineClass))
#define BOXES_IS_LIBVIRT_MACHINE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BOXES_TYPE_LIBVIRT_MACHINE))
#define BOXES_IS_LIBVIRT_MACHINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BOXES_TYPE_LIBVIRT_MACHINE))
#define BOXES_LIBVIRT_MACHINE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), BOXES_TYPE_LIBVIRT_MACHINE, BoxesLibvirtMachineClass))

typedef struct _BoxesLibvirtMachine BoxesLibvirtMachine;
typedef struct _BoxesLibvirtMachineClass BoxesLibvirtMachineClass;
typedef struct _Block7Data Block7Data;
#define _g_main_context_unref0(var) ((var == NULL) ? NULL : (var = (g_main_context_unref (var), NULL)))
#define __g_list_free__g_object_unref0_0(var) ((var == NULL) ? NULL : (var = (_g_list_free__g_object_unref0_ (var), NULL)))
typedef struct _Block8Data Block8Data;
typedef struct _Block9Data Block9Data;

struct _BoxesBroker {
	GObject parent_instance;
	BoxesBrokerPrivate * priv;
};

struct _BoxesBrokerClass {
	GObjectClass parent_class;
	void (*add_source) (BoxesBroker* self, BoxesCollectionSource* source, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*add_source_finish) (BoxesBroker* self, GAsyncResult* _res_);
};

typedef enum  {
	BOXES_APP_PAGE_MAIN,
	BOXES_APP_PAGE_DISPLAY
} BoxesAppPage;

struct _BoxesApplication {
	GtkApplication parent_instance;
	BoxesApplicationPrivate * priv;
};

struct _BoxesApplicationClass {
	GtkApplicationClass parent_class;
};

typedef enum  {
	BOXES_UI_STATE_NONE,
	BOXES_UI_STATE_COLLECTION,
	BOXES_UI_STATE_CREDS,
	BOXES_UI_STATE_DISPLAY,
	BOXES_UI_STATE_SETTINGS,
	BOXES_UI_STATE_WIZARD,
	BOXES_UI_STATE_PROPERTIES
} BoxesUIState;

struct _BoxesUI {
	GObject parent_instance;
	BoxesUIPrivate * priv;
	BoxesUIState previous_ui_state;
};

struct _BoxesUIClass {
	GObjectClass parent_class;
	void (*ui_state_changed) (BoxesUI* self);
	ClutterActor* (*get_actor) (BoxesUI* self);
};

struct _BoxesApp {
	BoxesUI parent_instance;
	BoxesAppPrivate * priv;
	GtkApplicationWindow* window;
	GtkNotebook* notebook;
	BoxesClutterWidget* embed;
	ClutterStage* stage;
	ClutterBinLayout* stage_bin;
	ClutterActor* overlay_bin_actor;
	ClutterBinLayout* overlay_bin;
	BoxesCollectionItem* current_item;
	BoxesSearchbar* searchbar;
	BoxesTopbar* topbar;
	BoxesNotificationbar* notificationbar;
	BoxesRevealer* sidebar_revealer;
	BoxesRevealer* searchbar_revealer;
	BoxesSidebar* sidebar;
	BoxesSelectionbar* selectionbar;
	guint duration;
	GSettings* settings;
	BoxesWizard* wizard;
	BoxesProperties* properties;
	BoxesDisplayPage* display_page;
	BoxesEmptyBoxes* empty_boxes;
	BoxesCollection* collection;
	BoxesCollectionFilter* filter;
	BoxesCollectionView* view;
};

struct _BoxesAppClass {
	BoxesUIClass parent_class;
};

struct _BoxesAppPrivate {
	gchar* _uri;
	gboolean is_ready;
	BoxesApplication* application;
	GHashTable* brokers;
	GHashTable* sources;
	guint configure_id;
	GBinding* status_bind;
	gulong got_error_id;
	gboolean ui_is_setup;
	gboolean _selection_mode;
};

typedef enum  {
	BOXES_SELECTION_CRITERIA_ALL,
	BOXES_SELECTION_CRITERIA_NONE,
	BOXES_SELECTION_CRITERIA_RUNNING
} BoxesSelectionCriteria;

typedef void (*BoxesAppCallReadyFunc) (void* user_data);
struct _Block1Data {
	int _ref_count_;
	BoxesApp * self;
	BoxesAppCallReadyFunc func;
	gpointer func_target;
	GDestroyNotify func_target_destroy_notify;
};

struct _Block2Data {
	int _ref_count_;
	BoxesApp * self;
	BoxesApp* app;
};

struct _Block3Data {
	int _ref_count_;
	Block2Data * _data2_;
	gchar* uuid;
};

struct _Block4Data {
	int _ref_count_;
	Block2Data * _data2_;
	gchar* arg;
	GFile* file;
};

struct _BoxesCollection {
	GObject parent_instance;
	BoxesCollectionPrivate * priv;
	GPtrArray* items;
};

struct _BoxesCollectionClass {
	GObjectClass parent_class;
};

struct _BoxesCollectionItem {
	BoxesUI parent_instance;
	BoxesCollectionItemPrivate * priv;
};

struct _BoxesCollectionItemClass {
	BoxesUIClass parent_class;
};

typedef enum  {
	BOXES_PROPERTIES_PAGE_LOGIN,
	BOXES_PROPERTIES_PAGE_SYSTEM,
	BOXES_PROPERTIES_PAGE_DISPLAY,
	BOXES_PROPERTIES_PAGE_DEVICES,
	BOXES_PROPERTIES_PAGE_LAST
} BoxesPropertiesPage;

typedef enum  {
	PROPERTY_CREATION_FLAG_NONE = 0,
	PROPERTY_CREATION_FLAG_NO_USB = 1 << 0
} PropertyCreationFlag;

struct _BoxesIPropertiesProviderIface {
	GTypeInterface parent_iface;
	GList* (*get_properties) (BoxesIPropertiesProvider* self, BoxesPropertiesPage page, PropertyCreationFlag flags);
};

struct _BoxesMachine {
	BoxesCollectionItem parent_instance;
	BoxesMachinePrivate * priv;
	BoxesCollectionSource* source;
	BoxesBoxConfig* config;
	gboolean stay_on_display;
	gboolean suspend_at_exit;
	gboolean deleted;
};

struct _BoxesMachineClass {
	BoxesCollectionItemClass parent_class;
	void (*save_real) (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*save_real_finish) (BoxesMachine* self, GAsyncResult* _res_, GError** error);
	void (*take_screenshot) (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GdkPixbuf* (*take_screenshot_finish) (BoxesMachine* self, GAsyncResult* _res_, GError** error);
	GList* (*get_properties) (BoxesMachine* self, BoxesPropertiesPage page, PropertyCreationFlag flags);
	void (*connect_display) (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*connect_display_finish) (BoxesMachine* self, GAsyncResult* _res_, GError** error);
	void (*disconnect_display) (BoxesMachine* self);
	void (*delete) (BoxesMachine* self, gboolean by_user);
};

struct _BoxesAppAddCollectionSourceData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesApp* self;
	BoxesCollectionSource* source;
	BoxesCollectionSource* _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	BoxesCollectionSource* _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	GQuark _tmp8_;
	BoxesCollectionSource* _tmp9_;
	BoxesRemoteMachine* _tmp10_;
	BoxesRemoteMachine* machine;
	BoxesCollection* _tmp11_;
	BoxesRemoteMachine* _tmp12_;
	GError* _error_;
	GError* _tmp13_;
	const gchar* _tmp14_;
	GHashTable* _tmp15_;
	BoxesCollectionSource* _tmp16_;
	gchar* _tmp17_;
	gchar* _tmp18_;
	gchar* _tmp19_;
	gconstpointer _tmp20_;
	BoxesBroker* _tmp21_;
	BoxesBroker* _tmp22_;
	BoxesBroker* broker;
	BoxesBroker* _tmp23_;
	BoxesBroker* _tmp24_;
	BoxesCollectionSource* _tmp25_;
	GHashTable* _tmp26_;
	BoxesCollectionSource* _tmp27_;
	gchar* _tmp28_;
	gchar* _tmp29_;
	BoxesCollectionSource* _tmp30_;
	BoxesCollectionSource* _tmp31_;
	BoxesCollectionSource* _tmp32_;
	gchar* _tmp33_;
	gchar* _tmp34_;
	gchar* _tmp35_;
	gboolean _tmp36_;
	BoxesCollectionSource* _tmp37_;
	gchar* _tmp38_;
	gchar* _tmp39_;
	gchar* _tmp40_;
	GError * _inner_error_;
};

struct _BoxesAppSetupSourcesData {
	int _state_;
	GObject* _source_object_;
	GAsyncResult* _res_;
	GSimpleAsyncResult* _async_result;
	BoxesApp* self;
	gchar* _tmp0_;
	gchar* _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	gchar* _tmp4_;
	gchar* _tmp5_;
	GFile* _tmp6_;
	GFile* _tmp7_;
	GFile* src;
	gchar* _tmp8_;
	gchar* _tmp9_;
	GFile* _tmp10_;
	GFile* _tmp11_;
	GFile* dst;
	GFile* _tmp12_;
	GFile* _tmp13_;
	GError* _error_;
	GError* _tmp14_;
	const gchar* _tmp15_;
	BoxesCollectionSource* _tmp16_;
	BoxesCollectionSource* source;
	GError* _vala1__error_;
	GError* _tmp17_;
	const gchar* _tmp18_;
	GVirConnection* _tmp19_;
	GVirConnection* _tmp20_;
	GVirConnection* _tmp21_;
	gboolean _tmp22_;
	BoxesApplication* _tmp23_;
	gchar* _tmp24_;
	gchar* _tmp25_;
	GFile* _tmp26_;
	GFile* _tmp27_;
	GFile* dir;
	GError * _inner_error_;
};

struct _BoxesRevealer {
	ClutterActor parent_instance;
	BoxesRevealerPrivate * priv;
	guint duration;
	gboolean resize;
};

struct _BoxesRevealerClass {
	ClutterActorClass parent_class;
};

struct _Block5Data {
	int _ref_count_;
	BoxesApp * self;
	ClutterActor* actor;
	gulong id;
	gulong completed_id;
};

struct _BoxesCollectionView {
	BoxesUI parent_instance;
	BoxesCollectionViewPrivate * priv;
	GdMainIconView* icon_view;
};

struct _BoxesCollectionViewClass {
	BoxesUIClass parent_class;
};

struct _Block6Data {
	int _ref_count_;
	BoxesApp * self;
	gint waiting_counter;
};

struct _Block7Data {
	int _ref_count_;
	Block6Data * _data6_;
	BoxesLibvirtMachine* machine;
};

struct _Block8Data {
	int _ref_count_;
	BoxesApp * self;
	GList* selected_items;
};

typedef void (*BoxesNotificationbarOKFunc) (void* user_data);
typedef void (*BoxesNotificationbarCancelFunc) (void* user_data);
struct _Block9Data {
	int _ref_count_;
	BoxesApp * self;
	BoxesMachine* machine;
};


static gpointer boxes_broker_parent_class = NULL;
static gpointer boxes_application_parent_class = NULL;
extern BoxesApp* boxes_app_app;
static gpointer boxes_app_parent_class = NULL;
BoxesApp* boxes_app_app = NULL;
static gboolean boxes_app_opt_fullscreen;
static gboolean boxes_app_opt_fullscreen = FALSE;
static gboolean boxes_app_opt_help;
static gboolean boxes_app_opt_help = FALSE;
static gchar* boxes_app_opt_open_uuid;
static gchar* boxes_app_opt_open_uuid = NULL;
static gchar** boxes_app_opt_uris;
static gint boxes_app_opt_uris_length1;
static gchar** boxes_app_opt_uris = NULL;
static gint boxes_app_opt_uris_length1 = 0;
static gint _boxes_app_opt_uris_size_ = 0;
static gchar** boxes_app_opt_search;
static gint boxes_app_opt_search_length1;
static gchar** boxes_app_opt_search = NULL;
static gint boxes_app_opt_search_length1 = 0;
static gint _boxes_app_opt_search_size_ = 0;

GType boxes_broker_get_type (void) G_GNUC_CONST;
enum  {
	BOXES_BROKER_DUMMY_PROPERTY
};
void boxes_broker_add_source (BoxesBroker* self, BoxesCollectionSource* source, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_broker_add_source_finish (BoxesBroker* self, GAsyncResult* _res_);
BoxesBroker* boxes_broker_construct (GType object_type);
GType boxes_app_page_get_type (void) G_GNUC_CONST;
GType boxes_application_get_type (void) G_GNUC_CONST;
enum  {
	BOXES_APPLICATION_DUMMY_PROPERTY
};
BoxesApplication* boxes_application_new (void);
BoxesApplication* boxes_application_construct (GType object_type);
static void boxes_application_real_startup (GApplication* base);
GType boxes_ui_get_type (void) G_GNUC_CONST;
GType boxes_app_get_type (void) G_GNUC_CONST;
void boxes_app_startup (BoxesApp* self);
static void boxes_application_real_activate (GApplication* base);
void boxes_app_activate (BoxesApp* self);
static gint boxes_application_real_command_line (GApplication* base, GApplicationCommandLine* cmdline);
gint boxes_app_command_line (BoxesApp* self, GApplicationCommandLine* cmdline);
GType boxes_ui_state_get_type (void) G_GNUC_CONST;
GType boxes_clutter_widget_get_type (void) G_GNUC_CONST;
GType boxes_collection_item_get_type (void) G_GNUC_CONST;
GType boxes_searchbar_get_type (void) G_GNUC_CONST;
GType boxes_topbar_get_type (void) G_GNUC_CONST;
GType boxes_notificationbar_get_type (void) G_GNUC_CONST;
GType boxes_revealer_get_type (void) G_GNUC_CONST;
GType boxes_sidebar_get_type (void) G_GNUC_CONST;
GType boxes_selectionbar_get_type (void) G_GNUC_CONST;
GType boxes_wizard_get_type (void) G_GNUC_CONST;
GType boxes_properties_get_type (void) G_GNUC_CONST;
GType boxes_display_page_get_type (void) G_GNUC_CONST;
GType boxes_empty_boxes_get_type (void) G_GNUC_CONST;
GType boxes_collection_get_type (void) G_GNUC_CONST;
GType boxes_collection_filter_get_type (void) G_GNUC_CONST;
GType boxes_collection_view_get_type (void) G_GNUC_CONST;
#define BOXES_APP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), BOXES_TYPE_APP, BoxesAppPrivate))
enum  {
	BOXES_APP_DUMMY_PROPERTY,
	BOXES_APP_ACTOR,
	BOXES_APP_FULLSCREEN,
	BOXES_APP_URI,
	BOXES_APP_DEFAULT_CONNECTION,
	BOXES_APP_DEFAULT_SOURCE,
	BOXES_APP_SELECTION_MODE,
	BOXES_APP_SELECTED_ITEMS
};
#define BOXES_APP_configure_id_timeout ((guint) 100)
BoxesApp* boxes_app_new (void);
BoxesApp* boxes_app_construct (GType object_type);
BoxesUI* boxes_ui_construct (GType object_type);
static void _g_free0_ (gpointer var);
static void _g_object_unref0_ (gpointer var);
BoxesCollectionFilter* boxes_collection_filter_new (void);
BoxesCollectionFilter* boxes_collection_filter_construct (GType object_type);
static void __lambda185_ (BoxesApp* self);
gboolean boxes_app_quit (BoxesApp* self);
static void ___lambda185__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self);
static void __lambda186_ (BoxesApp* self);
void boxes_ui_set_ui_state (BoxesUI* self, BoxesUIState value);
static void ___lambda186__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self);
static void __lambda187_ (BoxesApp* self);
GType boxes_selection_criteria_get_type (void) G_GNUC_CONST;
void boxes_collection_view_select (BoxesCollectionView* self, BoxesSelectionCriteria selection);
static void ___lambda187__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self);
static void __lambda188_ (BoxesApp* self);
static void ___lambda188__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self);
static void __lambda189_ (BoxesApp* self);
static void ___lambda189__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self);
static void __lambda190_ (BoxesApp* self);
static void ___lambda190__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self);
BoxesCollection* boxes_collection_new (void);
BoxesCollection* boxes_collection_construct (GType object_type);
static void __lambda191_ (BoxesApp* self, BoxesCollectionItem* item);
void boxes_collection_view_add_item (BoxesCollectionView* self, BoxesCollectionItem* item);
static void ___lambda191__boxes_collection_item_added (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self);
static void __lambda192_ (BoxesApp* self, BoxesCollectionItem* item);
void boxes_collection_view_remove_item (BoxesCollectionView* self, BoxesCollectionItem* item);
static void ___lambda192__boxes_collection_item_removed (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self);
GType boxes_libvirt_broker_get_type (void) G_GNUC_CONST;
BoxesLibvirtBroker* boxes_libvirt_broker_get_default (void);
GType boxes_ovirt_broker_get_type (void) G_GNUC_CONST;
BoxesOvirtBroker* boxes_ovirt_broker_get_default (void);
static void boxes_app_setup_sources (BoxesApp* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
static void boxes_app_setup_sources_finish (BoxesApp* self, GAsyncResult* _res_);
static void __lambda196_ (BoxesApp* self, GObject* obj, GAsyncResult* _result_);
static void ___lambda196__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self);
void boxes_check_cpu_vt_capability (GAsyncReadyCallback _callback_, gpointer _user_data_);
gboolean boxes_check_cpu_vt_capability_finish (GAsyncResult* _res_);
void boxes_check_module_kvm_loaded (GAsyncReadyCallback _callback_, gpointer _user_data_);
gboolean boxes_check_module_kvm_loaded_finish (GAsyncResult* _res_);
gboolean boxes_app_has_broker_for_source_type (BoxesApp* self, const gchar* type);
void boxes_app_call_when_ready (BoxesApp* self, BoxesAppCallReadyFunc func, void* func_target, GDestroyNotify func_target_destroy_notify);
static Block1Data* block1_data_ref (Block1Data* _data1_);
static void block1_data_unref (void * _userdata_);
static void __lambda183_ (Block1Data* _data1_);
static void ___lambda183__boxes_app_ready (BoxesApp* _sender, gpointer self);
static void boxes_app_setup_ui (BoxesApp* self);
static Block2Data* block2_data_ref (Block2Data* _data2_);
static void block2_data_unref (void * _userdata_);
static Block3Data* block3_data_ref (Block3Data* _data3_);
static void block3_data_unref (void * _userdata_);
static void ___lambda201_ (Block3Data* _data3_);
gboolean boxes_app_open_uuid (BoxesApp* self, const gchar* uuid);
static void ____lambda201__boxes_app_call_ready_func (gpointer self);
static Block4Data* block4_data_ref (Block4Data* _data4_);
static void block4_data_unref (void * _userdata_);
static void _____lambda202_ (Block4Data* _data4_);
void boxes_wizard_open_with_uri (BoxesWizard* self, const gchar* uri, gboolean skip_review_for_live);
static void ______lambda202__boxes_app_call_ready_func (gpointer self);
static void _____lambda203_ (Block4Data* _data4_);
void boxes_app_open (BoxesApp* self, const gchar* name);
static void ______lambda203__boxes_app_call_ready_func (gpointer self);
static void ___lambda204_ (BoxesApp* self);
void boxes_searchbar_set_text (BoxesSearchbar* self, const gchar* value);
void boxes_searchbar_set_visible (BoxesSearchbar* self, gboolean value);
BoxesUIState boxes_ui_get_ui_state (BoxesUI* self);
void boxes_revealer_set_revealed (BoxesRevealer* self, gboolean value);
static void ____lambda204__boxes_app_call_ready_func (gpointer self);
void boxes_app_set_fullscreen (BoxesApp* self, gboolean value);
gint boxes_app_run (BoxesApp* self, gchar** args, int args_length1);
void boxes_app_shutdown (BoxesApp* self);
void boxes_collection_view_set_visible (BoxesCollectionView* self, gboolean value);
const gchar* boxes_collection_item_get_name (BoxesCollectionItem* self);
void boxes_app_select_item (BoxesApp* self, BoxesCollectionItem* item);
GType boxes_machine_get_type (void) G_GNUC_CONST;
GType boxes_properties_page_get_type (void) G_GNUC_CONST;
GType property_creation_flag_get_type (void) G_GNUC_CONST;
GType boxes_property_get_type (void) G_GNUC_CONST;
GType boxes_iproperties_provider_get_type (void) G_GNUC_CONST;
void boxes_app_delete_machine (BoxesApp* self, BoxesMachine* machine, gboolean by_user);
void boxes_machine_delete (BoxesMachine* self, gboolean by_user);
void boxes_collection_remove_item (BoxesCollection* self, BoxesCollectionItem* item);
static void boxes_app_add_collection_source_data_free (gpointer _data);
void boxes_app_add_collection_source (BoxesApp* self, BoxesCollectionSource* source, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_app_add_collection_source_finish (BoxesApp* self, GAsyncResult* _res_);
static gboolean boxes_app_add_collection_source_co (BoxesAppAddCollectionSourceData* _data_);
BoxesRemoteMachine* boxes_remote_machine_new (BoxesCollectionSource* source, GError** error);
BoxesRemoteMachine* boxes_remote_machine_construct (GType object_type, BoxesCollectionSource* source, GError** error);
GType boxes_remote_machine_get_type (void) G_GNUC_CONST;
void boxes_collection_add_item (BoxesCollection* self, BoxesCollectionItem* item);
static void boxes_app_add_collection_source_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
static void boxes_app_setup_sources_data_free (gpointer _data);
static gboolean boxes_app_setup_sources_co (BoxesAppSetupSourcesData* _data_);
static void boxes_app_setup_sources_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_);
GVirConnection* boxes_app_get_default_connection (BoxesApp* self);
static gboolean __lambda195_ (BoxesApp* self, const gchar* filename, GError** error);
static gboolean ___lambda195__boxes_foreach_filename_from_dir_func (const gchar* filename, gpointer self, GError** error);
static void boxes_app_save_window_geometry (BoxesApp* self);
static gboolean boxes_app_get_maximized (BoxesApp* self);
static GVariant* _variant_new1 (gint* value, gint value_length1);
static GVariant* _variant_new2 (gint* value, gint value_length1);
static gint _variant_get3 (GVariant* value);
static gint _variant_get4 (GVariant* value);
static gint _variant_get5 (GVariant* value);
static gint _variant_get6 (GVariant* value);
static gboolean __lambda197_ (BoxesApp* self);
gboolean boxes_app_get_fullscreen (BoxesApp* self);
static gboolean __lambda198_ (BoxesApp* self);
static gboolean ___lambda198__gsource_func (gpointer self);
static gboolean ___lambda197__gtk_widget_configure_event (GtkWidget* _sender, GdkEventConfigure* event, gpointer self);
static gboolean __lambda199_ (BoxesApp* self, GdkEventWindowState* event);
static gboolean ___lambda199__gtk_widget_window_state_event (GtkWidget* _sender, GdkEventWindowState* event, gpointer self);
BoxesClutterWidget* boxes_clutter_widget_new (void);
BoxesClutterWidget* boxes_clutter_widget_construct (GType object_type);
BoxesDisplayPage* boxes_display_page_new (void);
BoxesDisplayPage* boxes_display_page_construct (GType object_type);
GtkWidget* boxes_display_page_get_widget (BoxesDisplayPage* self);
void boxes_gdk_rgba_to_clutter_color (GdkRGBA* gdk_rgba, ClutterColor* result);
void boxes_get_boxes_bg_color (GdkRGBA* result);
static gboolean __lambda200_ (BoxesApp* self);
static gboolean ___lambda200__gtk_widget_delete_event (GtkWidget* _sender, GdkEventAny* event, gpointer self);
static gboolean boxes_app_on_key_pressed (BoxesApp* self, GtkWidget* widget, GdkEventKey* event);
static gboolean _boxes_app_on_key_pressed_gtk_widget_key_press_event (GtkWidget* _sender, GdkEventKey* event, gpointer self);
BoxesSidebar* boxes_sidebar_new (void);
BoxesSidebar* boxes_sidebar_construct (GType object_type);
BoxesCollectionView* boxes_collection_view_new (void);
BoxesCollectionView* boxes_collection_view_construct (GType object_type);
BoxesSearchbar* boxes_searchbar_new (void);
BoxesSearchbar* boxes_searchbar_construct (GType object_type);
BoxesTopbar* boxes_topbar_new (void);
BoxesTopbar* boxes_topbar_construct (GType object_type);
BoxesNotificationbar* boxes_notificationbar_new (void);
BoxesNotificationbar* boxes_notificationbar_construct (GType object_type);
BoxesSelectionbar* boxes_selectionbar_new (void);
BoxesSelectionbar* boxes_selectionbar_construct (GType object_type);
BoxesWizard* boxes_wizard_new (void);
BoxesWizard* boxes_wizard_construct (GType object_type);
BoxesProperties* boxes_properties_new (void);
BoxesProperties* boxes_properties_construct (GType object_type);
BoxesEmptyBoxes* boxes_empty_boxes_new (void);
BoxesEmptyBoxes* boxes_empty_boxes_construct (GType object_type);
BoxesRevealer* boxes_revealer_new (gboolean horizontal);
BoxesRevealer* boxes_revealer_construct (GType object_type, gboolean horizontal);
ClutterActor* boxes_ui_get_actor (BoxesUI* self);
void boxes_revealer_unreveal (BoxesRevealer* self);
ClutterActor* boxes_notificationbar_get_actor (BoxesNotificationbar* self);
ClutterActor* boxes_selectionbar_get_actor (BoxesSelectionbar* self);
static void boxes_app_set_main_ui_state (BoxesApp* self);
static void boxes_app_position_item_actor_at_icon (BoxesApp* self, BoxesCollectionItem* item);
void boxes_collection_view_get_item_pos (BoxesCollectionView* self, BoxesCollectionItem* item, gfloat* x, gfloat* y);
#define BOXES_MACHINE_SCREENSHOT_WIDTH 180
static void boxes_app_real_ui_state_changed (BoxesUI* base);
void boxes_topbar_set_status (BoxesTopbar* self, const gchar* value);
gboolean boxes_searchbar_get_visible (BoxesSearchbar* self);
static Block5Data* block5_data_ref (Block5Data* _data5_);
static void block5_data_unref (void * _userdata_);
static void ____lambda205_ (BoxesApp* self, GtkAllocation* allocation);
static gboolean __lambda206_ (BoxesApp* self);
static gboolean ___lambda206__gsource_func (gpointer self);
static void _____lambda205__gtk_widget_size_allocate (GtkWidget* _sender, GtkAllocation* allocation, gpointer self);
static void ____lambda207_ (Block5Data* _data5_);
void boxes_actor_remove (ClutterActor* actor);
static void _____lambda207__clutter_actor_transitions_completed (ClutterActor* _sender, gpointer self);
const gchar* boxes_ui_state_to_string (BoxesUIState self);
void boxes_app_suspend_machines (BoxesApp* self);
static Block6Data* block6_data_ref (Block6Data* _data6_);
static void block6_data_unref (void * _userdata_);
GType boxes_libvirt_machine_get_type (void) G_GNUC_CONST;
static Block7Data* block7_data_ref (Block7Data* _data7_);
static void block7_data_unref (void * _userdata_);
void boxes_libvirt_machine_suspend (BoxesLibvirtMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_libvirt_machine_suspend_finish (BoxesLibvirtMachine* self, GAsyncResult* _res_, GError** error);
static void ______lambda208_ (Block7Data* _data7_);
static void _______lambda208__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self);
void boxes_notificationbar_cancel (BoxesNotificationbar* self);
void boxes_wizard_cleanup (BoxesWizard* self);
void boxes_app_show_properties (BoxesApp* self);
GList* boxes_collection_view_get_selected_items (BoxesCollectionView* self);
void boxes_app_set_selection_mode (BoxesApp* self, gboolean value);
static void _g_list_free__g_object_unref0_ (GList* self);
void boxes_app_remove_selected_items (BoxesApp* self);
static Block8Data* block8_data_ref (Block8Data* _data8_);
static void block8_data_unref (void * _userdata_);
static void __lambda158_ (Block8Data* _data8_);
static void ___lambda158__boxes_notificationbar_ok_func (gpointer self);
static void __lambda159_ (Block8Data* _data8_);
static void ___lambda159__boxes_notificationbar_cancel_func (gpointer self);
GdNotification* boxes_notificationbar_display_for_action (BoxesNotificationbar* self, const gchar* message, const gchar* action_label, BoxesNotificationbarOKFunc action_func, void* action_func_target, GDestroyNotify action_func_target_destroy_notify, BoxesNotificationbarCancelFunc ignore_func, void* ignore_func_target, GDestroyNotify ignore_func_target_destroy_notify, gint timeout);
#define BOXES_NOTIFICATIONBAR_DEFAULT_TIMEOUT 6
gboolean boxes_app_get_selection_mode (BoxesApp* self);
void boxes_app_connect_to (BoxesApp* self, BoxesMachine* machine, gfloat x, gfloat y);
static Block9Data* block9_data_ref (Block9Data* _data9_);
static void block9_data_unref (void * _userdata_);
void boxes_allocate_actor_no_animation (ClutterActor* actor, gfloat x, gfloat y, gfloat width, gfloat height);
#define BOXES_MACHINE_SCREENSHOT_HEIGHT 134
static void __lambda133_ (BoxesApp* self, const gchar* message);
GdNotification* boxes_notificationbar_display_error (BoxesNotificationbar* self, const gchar* message, gint timeout);
static void ___lambda133__boxes_machine_got_error (BoxesMachine* _sender, const gchar* message, gpointer self);
void boxes_machine_connect_display (BoxesMachine* self, GAsyncReadyCallback _callback_, gpointer _user_data_);
void boxes_machine_connect_display_finish (BoxesMachine* self, GAsyncResult* _res_, GError** error);
static void __lambda134_ (Block9Data* _data9_, GObject* obj, GAsyncResult* res);
static void ___lambda134__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self);
const gchar* boxes_app_get_uri (BoxesApp* self);
void boxes_app_set_uri (BoxesApp* self, const gchar* value);
GVirConnection* boxes_libvirt_broker_get_connection (BoxesLibvirtBroker* self, const gchar* name);
BoxesCollectionSource* boxes_app_get_default_source (BoxesApp* self);
GList* boxes_app_get_selected_items (BoxesApp* self);
static void boxes_app_finalize (GObject* obj);
static void _vala_boxes_app_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_boxes_app_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);

static const GOptionEntry BOXES_APP_options[8] = {{"version", (gchar) 0, 0, G_OPTION_ARG_NONE, NULL, "Display version number", NULL}, {"help", 'h', (gint) G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &boxes_app_opt_help, NULL, NULL}, {"full-screen", 'f', 0, G_OPTION_ARG_NONE, &boxes_app_opt_fullscreen, "Open in full screen", NULL}, {"checks", (gchar) 0, 0, G_OPTION_ARG_NONE, NULL, "Check virtualization capabilities", NULL}, {"open-uuid", (gchar) 0, 0, G_OPTION_ARG_STRING, &boxes_app_opt_open_uuid, "Open box with UUID", NULL}, {"search", (gchar) 0, 0, G_OPTION_ARG_STRING_ARRAY, &boxes_app_opt_search, "Search term", NULL}, {"", (gchar) 0, 0, G_OPTION_ARG_STRING_ARRAY, &boxes_app_opt_uris, "URI to display, broker or installer media", NULL}, {NULL}};

void boxes_broker_add_source (BoxesBroker* self, BoxesCollectionSource* source, GAsyncReadyCallback _callback_, gpointer _user_data_) {
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	BOXES_BROKER_GET_CLASS (self)->add_source (self, source, _callback_, _user_data_);
#line 956 "app.c"
}


void boxes_broker_add_source_finish (BoxesBroker* self, GAsyncResult* _res_) {
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	BOXES_BROKER_GET_CLASS (self)->add_source_finish (self, _res_);
#line 963 "app.c"
}


BoxesBroker* boxes_broker_construct (GType object_type) {
	BoxesBroker * self = NULL;
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = (BoxesBroker*) g_object_new (object_type, NULL);
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return self;
#line 973 "app.c"
}


static void boxes_broker_class_init (BoxesBrokerClass * klass) {
#line 6 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_broker_parent_class = g_type_class_peek_parent (klass);
#line 980 "app.c"
}


static void boxes_broker_instance_init (BoxesBroker * self) {
}


GType boxes_broker_get_type (void) {
	static volatile gsize boxes_broker_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_broker_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (BoxesBrokerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) boxes_broker_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BoxesBroker), 0, (GInstanceInitFunc) boxes_broker_instance_init, NULL };
		GType boxes_broker_type_id;
		boxes_broker_type_id = g_type_register_static (G_TYPE_OBJECT, "BoxesBroker", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_once_init_leave (&boxes_broker_type_id__volatile, boxes_broker_type_id);
	}
	return boxes_broker_type_id__volatile;
}


GType boxes_app_page_get_type (void) {
	static volatile gsize boxes_app_page_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_app_page_type_id__volatile)) {
		static const GEnumValue values[] = {{BOXES_APP_PAGE_MAIN, "BOXES_APP_PAGE_MAIN", "main"}, {BOXES_APP_PAGE_DISPLAY, "BOXES_APP_PAGE_DISPLAY", "display"}, {0, NULL, NULL}};
		GType boxes_app_page_type_id;
		boxes_app_page_type_id = g_enum_register_static ("BoxesAppPage", values);
		g_once_init_leave (&boxes_app_page_type_id__volatile, boxes_app_page_type_id);
	}
	return boxes_app_page_type_id__volatile;
}


BoxesApplication* boxes_application_construct (GType object_type) {
	BoxesApplication * self = NULL;
	GApplicationFlags _tmp0_;
	GApplicationFlags _tmp1_;
#line 18 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = (BoxesApplication*) g_object_new (object_type, NULL);
#line 19 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_application_set_application_id ((GApplication*) self, "org.gnome.Boxes");
#line 20 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = g_application_get_flags ((GApplication*) self);
#line 20 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _tmp0_;
#line 20 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_application_set_flags ((GApplication*) self, _tmp1_ | G_APPLICATION_HANDLES_COMMAND_LINE);
#line 18 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return self;
#line 1028 "app.c"
}


BoxesApplication* boxes_application_new (void) {
#line 18 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return boxes_application_construct (BOXES_TYPE_APPLICATION);
#line 1035 "app.c"
}


static void boxes_application_real_startup (GApplication* base) {
	BoxesApplication * self;
	BoxesApp* _tmp0_;
#line 23 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = (BoxesApplication*) base;
#line 24 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	G_APPLICATION_CLASS (boxes_application_parent_class)->startup ((GApplication*) G_TYPE_CHECK_INSTANCE_CAST (self, GTK_TYPE_APPLICATION, GtkApplication));
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = boxes_app_app;
#line 25 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_startup (_tmp0_);
#line 1050 "app.c"
}


static void boxes_application_real_activate (GApplication* base) {
	BoxesApplication * self;
	BoxesApp* _tmp0_;
#line 28 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = (BoxesApplication*) base;
#line 29 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	G_APPLICATION_CLASS (boxes_application_parent_class)->activate ((GApplication*) G_TYPE_CHECK_INSTANCE_CAST (self, GTK_TYPE_APPLICATION, GtkApplication));
#line 30 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = boxes_app_app;
#line 30 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_activate (_tmp0_);
#line 1065 "app.c"
}


static gint boxes_application_real_command_line (GApplication* base, GApplicationCommandLine* cmdline) {
	BoxesApplication * self;
	gint result = 0;
	BoxesApp* _tmp0_;
	GApplicationCommandLine* _tmp1_;
	gint _tmp2_ = 0;
#line 33 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = (BoxesApplication*) base;
#line 33 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (cmdline != NULL, 0);
#line 34 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = boxes_app_app;
#line 34 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = cmdline;
#line 34 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = boxes_app_command_line (_tmp0_, _tmp1_);
#line 34 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = _tmp2_;
#line 34 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 1089 "app.c"
}


static void boxes_application_class_init (BoxesApplicationClass * klass) {
#line 17 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_application_parent_class = g_type_class_peek_parent (klass);
#line 17 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	G_APPLICATION_CLASS (klass)->startup = boxes_application_real_startup;
#line 17 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	G_APPLICATION_CLASS (klass)->activate = boxes_application_real_activate;
#line 17 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	G_APPLICATION_CLASS (klass)->command_line = boxes_application_real_command_line;
#line 1102 "app.c"
}


static void boxes_application_instance_init (BoxesApplication * self) {
}


GType boxes_application_get_type (void) {
	static volatile gsize boxes_application_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_application_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (BoxesApplicationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) boxes_application_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BoxesApplication), 0, (GInstanceInitFunc) boxes_application_instance_init, NULL };
		GType boxes_application_type_id;
		boxes_application_type_id = g_type_register_static (GTK_TYPE_APPLICATION, "BoxesApplication", &g_define_type_info, 0);
		g_once_init_leave (&boxes_application_type_id__volatile, boxes_application_type_id);
	}
	return boxes_application_type_id__volatile;
}


static gpointer _g_object_ref0 (gpointer self) {
#line 95 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return self ? g_object_ref (self) : NULL;
#line 1125 "app.c"
}


static void _g_free0_ (gpointer var) {
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	var = (g_free (var), NULL);
#line 1132 "app.c"
}


static void _g_object_unref0_ (gpointer var) {
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	(var == NULL) ? NULL : (var = (g_object_unref (var), NULL));
#line 1139 "app.c"
}


static void __lambda185_ (BoxesApp* self) {
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_quit (self);
#line 1146 "app.c"
}


static void ___lambda185__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self) {
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda185_ (self);
#line 1153 "app.c"
}


static void __lambda186_ (BoxesApp* self) {
#line 106 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_ui_set_ui_state ((BoxesUI*) self, BOXES_UI_STATE_WIZARD);
#line 1160 "app.c"
}


static void ___lambda186__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self) {
#line 106 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda186_ (self);
#line 1167 "app.c"
}


static void __lambda187_ (BoxesApp* self) {
	BoxesCollectionView* _tmp0_;
#line 110 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 110 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_collection_view_select (_tmp0_, BOXES_SELECTION_CRITERIA_ALL);
#line 1177 "app.c"
}


static void ___lambda187__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self) {
#line 110 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda187_ (self);
#line 1184 "app.c"
}


static void __lambda188_ (BoxesApp* self) {
	BoxesCollectionView* _tmp0_;
#line 114 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 114 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_collection_view_select (_tmp0_, BOXES_SELECTION_CRITERIA_RUNNING);
#line 1194 "app.c"
}


static void ___lambda188__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self) {
#line 114 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda188_ (self);
#line 1201 "app.c"
}


static void __lambda189_ (BoxesApp* self) {
	BoxesCollectionView* _tmp0_;
#line 118 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 118 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_collection_view_select (_tmp0_, BOXES_SELECTION_CRITERIA_NONE);
#line 1211 "app.c"
}


static void ___lambda189__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self) {
#line 118 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda189_ (self);
#line 1218 "app.c"
}


static void __lambda190_ (BoxesApp* self) {
	gchar* _tmp0_;
	gchar* _tmp1_;
	gchar* _tmp2_;
	gchar* _tmp3_;
	gchar** _tmp4_ = NULL;
	gchar** authors;
	gint authors_length1;
	gint _authors_size_;
	gchar* _tmp5_;
	gchar* _tmp6_;
	gchar** _tmp7_ = NULL;
	gchar** artists;
	gint artists_length1;
	gint _artists_size_;
	GtkApplicationWindow* _tmp8_;
	const gchar* _tmp9_ = NULL;
	const gchar* _tmp10_ = NULL;
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = g_strdup ("Alexander Larsson <alexl@redhat.com>");
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = g_strdup ("Christophe Fergeau <cfergeau@redhat.com>");
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = g_strdup ("Marc-André Lureau <marcandre.lureau@gmail.com>");
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = g_strdup ("Zeeshan Ali (Khattak) <zeeshanak@gnome.org>");
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = g_new0 (gchar*, 4 + 1);
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_[0] = _tmp0_;
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_[1] = _tmp1_;
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_[2] = _tmp2_;
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_[3] = _tmp3_;
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	authors = _tmp4_;
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	authors_length1 = 4;
#line 123 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_authors_size_ = authors_length1;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = g_strdup ("Jon McCann <jmccann@redhat.com>");
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = g_strdup ("Jakub Steiner <jsteiner@redhat.com>");
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_ = g_new0 (gchar*, 2 + 1);
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_[0] = _tmp5_;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_[1] = _tmp6_;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	artists = _tmp7_;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	artists_length1 = 2;
#line 129 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_artists_size_ = artists_length1;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_ = self->window;
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp9_ = _ ("translator-credits");
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp10_ = _ ("A simple GNOME 3 application to access remote or virtual systems");
#line 134 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_show_about_dialog ((GtkWindow*) _tmp8_, "artists", artists, "authors", authors, "translator-credits", _tmp9_, "comments", _tmp10_, "copyright", "Copyright 2011 Red Hat, Inc.", "license-type", GTK_LICENSE_LGPL_2_1, "logo-icon-name", "gnome-boxes", "version", BUILD_VERSION, "website", "http://live.gnome.org/Boxes", "wrap-license", TRUE, NULL);
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	artists = (_vala_array_free (artists, artists_length1, (GDestroyNotify) g_free), NULL);
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	authors = (_vala_array_free (authors, authors_length1, (GDestroyNotify) g_free), NULL);
#line 1292 "app.c"
}


static void ___lambda190__g_simple_action_activate (GSimpleAction* _sender, GVariant* parameter, gpointer self) {
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda190_ (self);
#line 1299 "app.c"
}


BoxesApp* boxes_app_construct (GType object_type) {
	BoxesApp * self = NULL;
	BoxesApp* _tmp0_;
	BoxesApplication* _tmp1_;
	GSettings* _tmp2_;
	GHashFunc _tmp3_;
	GEqualFunc _tmp4_;
	GHashTable* _tmp5_;
	GHashFunc _tmp6_;
	GEqualFunc _tmp7_;
	GHashTable* _tmp8_;
	BoxesCollectionFilter* _tmp9_;
	GSimpleAction* _tmp10_;
	GSimpleAction* action;
	GSimpleAction* _tmp11_;
	BoxesApplication* _tmp12_;
	GSimpleAction* _tmp13_;
	GSimpleAction* _tmp14_;
	GSimpleAction* _tmp15_;
	BoxesApplication* _tmp16_;
	GSimpleAction* _tmp17_;
	GSimpleAction* _tmp18_;
	GSimpleAction* _tmp19_;
	BoxesApplication* _tmp20_;
	GSimpleAction* _tmp21_;
	GSimpleAction* _tmp22_;
	GSimpleAction* _tmp23_;
	BoxesApplication* _tmp24_;
	GSimpleAction* _tmp25_;
	GSimpleAction* _tmp26_;
	GSimpleAction* _tmp27_;
	BoxesApplication* _tmp28_;
	GSimpleAction* _tmp29_;
	GSimpleAction* _tmp30_;
	GSimpleAction* _tmp31_;
	BoxesApplication* _tmp32_;
	GSimpleAction* _tmp33_;
#line 94 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = (BoxesApp*) boxes_ui_construct (object_type);
#line 95 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 95 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (boxes_app_app);
#line 95 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_app = _tmp0_;
#line 96 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = boxes_application_new ();
#line 96 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->priv->application);
#line 96 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->application = _tmp1_;
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = g_settings_new ("org.gnome.boxes");
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->settings);
#line 97 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->settings = _tmp2_;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = g_str_hash;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = g_str_equal;
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = g_hash_table_new_full (_tmp3_, _tmp4_, _g_free0_, _g_object_unref0_);
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_hash_table_unref0 (self->priv->sources);
#line 98 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->sources = _tmp5_;
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = g_str_hash;
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_ = g_str_equal;
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_ = g_hash_table_new_full (_tmp6_, _tmp7_, _g_free0_, _g_object_unref0_);
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_hash_table_unref0 (self->priv->brokers);
#line 99 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->brokers = _tmp8_;
#line 100 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp9_ = boxes_collection_filter_new ();
#line 100 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->filter);
#line 100 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->filter = _tmp9_;
#line 101 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp10_ = g_simple_action_new ("quit", NULL);
#line 101 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	action = _tmp10_;
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp11_ = action;
#line 102 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object (_tmp11_, "activate", (GCallback) ___lambda185__g_simple_action_activate, self, 0);
#line 103 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp12_ = self->priv->application;
#line 103 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp13_ = action;
#line 103 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_action_map_add_action ((GActionMap*) _tmp12_, (GAction*) _tmp13_);
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_ = g_simple_action_new ("new", NULL);
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (action);
#line 105 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	action = _tmp14_;
#line 106 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp15_ = action;
#line 106 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object (_tmp15_, "activate", (GCallback) ___lambda186__g_simple_action_activate, self, 0);
#line 107 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp16_ = self->priv->application;
#line 107 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp17_ = action;
#line 107 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_action_map_add_action ((GActionMap*) _tmp16_, (GAction*) _tmp17_);
#line 109 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp18_ = g_simple_action_new ("select-all", NULL);
#line 109 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (action);
#line 109 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	action = _tmp18_;
#line 110 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp19_ = action;
#line 110 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object (_tmp19_, "activate", (GCallback) ___lambda187__g_simple_action_activate, self, 0);
#line 111 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp20_ = self->priv->application;
#line 111 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp21_ = action;
#line 111 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_action_map_add_action ((GActionMap*) _tmp20_, (GAction*) _tmp21_);
#line 113 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp22_ = g_simple_action_new ("select-running", NULL);
#line 113 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (action);
#line 113 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	action = _tmp22_;
#line 114 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp23_ = action;
#line 114 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object (_tmp23_, "activate", (GCallback) ___lambda188__g_simple_action_activate, self, 0);
#line 115 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp24_ = self->priv->application;
#line 115 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp25_ = action;
#line 115 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_action_map_add_action ((GActionMap*) _tmp24_, (GAction*) _tmp25_);
#line 117 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp26_ = g_simple_action_new ("select-none", NULL);
#line 117 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (action);
#line 117 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	action = _tmp26_;
#line 118 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp27_ = action;
#line 118 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object (_tmp27_, "activate", (GCallback) ___lambda189__g_simple_action_activate, self, 0);
#line 119 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp28_ = self->priv->application;
#line 119 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp29_ = action;
#line 119 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_action_map_add_action ((GActionMap*) _tmp28_, (GAction*) _tmp29_);
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp30_ = g_simple_action_new ("about", NULL);
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (action);
#line 121 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	action = _tmp30_;
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp31_ = action;
#line 122 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object (_tmp31_, "activate", (GCallback) ___lambda190__g_simple_action_activate, self, 0);
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp32_ = self->priv->application;
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp33_ = action;
#line 146 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_action_map_add_action ((GActionMap*) _tmp32_, (GAction*) _tmp33_);
#line 94 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (action);
#line 94 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return self;
#line 1484 "app.c"
}


BoxesApp* boxes_app_new (void) {
#line 94 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return boxes_app_construct (BOXES_TYPE_APP);
#line 1491 "app.c"
}


static void __lambda191_ (BoxesApp* self, BoxesCollectionItem* item) {
	BoxesCollectionView* _tmp0_;
	BoxesCollectionItem* _tmp1_;
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (item != NULL);
#line 169 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 169 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = item;
#line 169 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_collection_view_add_item (_tmp0_, _tmp1_);
#line 1506 "app.c"
}


static void ___lambda191__boxes_collection_item_added (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self) {
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda191_ (self, item);
#line 1513 "app.c"
}


static void __lambda192_ (BoxesApp* self, BoxesCollectionItem* item) {
	BoxesCollectionView* _tmp0_;
	BoxesCollectionItem* _tmp1_;
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (item != NULL);
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = item;
#line 172 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_collection_view_remove_item (_tmp0_, _tmp1_);
#line 1528 "app.c"
}


static void ___lambda192__boxes_collection_item_removed (BoxesCollection* _sender, BoxesCollectionItem* item, gpointer self) {
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda192_ (self, item);
#line 1535 "app.c"
}


static void __lambda196_ (BoxesApp* self, GObject* obj, GAsyncResult* _result_) {
	GAsyncResult* _tmp0_;
#line 179 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (_result_ != NULL);
#line 180 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _result_;
#line 180 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_setup_sources_finish (self, _tmp0_);
#line 181 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->is_ready = TRUE;
#line 182 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_emit_by_name (self, "ready");
#line 1551 "app.c"
}


static void ___lambda196__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self) {
#line 179 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda196_ (self, source_object, res);
#line 179 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_unref (self);
#line 1560 "app.c"
}


void boxes_app_startup (BoxesApp* self) {
	gchar** _tmp0_ = NULL;
	gchar** args;
	gint args_length1;
	gint _args_size_;
	gchar** _tmp1_;
	gint _tmp1__length1;
	gchar** args2;
	gint args2_length1;
	gint _args2_size_;
	GMenu* _tmp2_;
	GMenu* menu;
	GMenu* _tmp3_;
	const gchar* _tmp4_ = NULL;
	GMenu* _tmp5_;
	GMenu* display_section;
	GMenu* _tmp6_;
	GMenu* _tmp7_;
	GMenu* _tmp8_;
	const gchar* _tmp9_ = NULL;
	GMenu* _tmp10_;
	const gchar* _tmp11_ = NULL;
	BoxesApplication* _tmp12_;
	GMenu* _tmp13_;
	BoxesCollection* _tmp14_;
	GSettings* _tmp15_;
	gint _tmp16_ = 0;
	BoxesCollection* _tmp17_;
	BoxesCollection* _tmp18_;
	GHashTable* _tmp19_;
	gchar* _tmp20_;
	BoxesLibvirtBroker* _tmp21_ = NULL;
#line 149 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = g_new0 (gchar*, 0 + 1);
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	args = _tmp0_;
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	args_length1 = 0;
#line 150 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_args_size_ = args_length1;
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = args;
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1__length1 = args_length1;
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	args2 = _tmp1_;
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	args2_length1 = _tmp1__length1;
#line 151 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_args2_size_ = args2_length1;
#line 152 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_clutter_init (&args2_length1, &args2);
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = g_menu_new ();
#line 154 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	menu = _tmp2_;
#line 155 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = menu;
#line 155 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = _ ("New");
#line 155 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_menu_append (_tmp3_, _tmp4_, "app.new");
#line 157 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = g_menu_new ();
#line 157 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	display_section = _tmp5_;
#line 158 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = menu;
#line 158 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_ = display_section;
#line 158 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_menu_append_section (_tmp6_, NULL, (GMenuModel*) _tmp7_);
#line 160 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_ = menu;
#line 160 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp9_ = _ ("About Boxes");
#line 160 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_menu_append (_tmp8_, _tmp9_, "app.about");
#line 161 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp10_ = menu;
#line 161 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp11_ = _ ("Quit");
#line 161 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_menu_append (_tmp10_, _tmp11_, "app.quit");
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp12_ = self->priv->application;
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp13_ = menu;
#line 163 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_application_set_app_menu ((GtkApplication*) _tmp12_, (GMenuModel*) _tmp13_);
#line 165 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_ = boxes_collection_new ();
#line 165 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->collection);
#line 165 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->collection = _tmp14_;
#line 166 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp15_ = self->settings;
#line 166 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp16_ = g_settings_get_int (_tmp15_, "animation-duration");
#line 166 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->duration = (guint) _tmp16_;
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp17_ = self->collection;
#line 168 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object (_tmp17_, "item-added", (GCallback) ___lambda191__boxes_collection_item_added, self, 0);
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp18_ = self->collection;
#line 171 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object (_tmp18_, "item-removed", (GCallback) ___lambda192__boxes_collection_item_removed, self, 0);
#line 175 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp19_ = self->priv->brokers;
#line 175 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp20_ = g_strdup ("libvirt");
#line 175 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp21_ = boxes_libvirt_broker_get_default ();
#line 175 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_hash_table_insert (_tmp19_, _tmp20_, (BoxesBroker*) _tmp21_);
#line 176 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (HAVE_OVIRT) {
#line 1686 "app.c"
		GHashTable* _tmp22_;
		gchar* _tmp23_;
		BoxesOvirtBroker* _tmp24_ = NULL;
#line 177 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp22_ = self->priv->brokers;
#line 177 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp23_ = g_strdup ("ovirt");
#line 177 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp24_ = boxes_ovirt_broker_get_default ();
#line 177 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_hash_table_insert (_tmp22_, _tmp23_, (BoxesBroker*) _tmp24_);
#line 1698 "app.c"
	}
#line 179 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_setup_sources (self, ___lambda196__gasync_ready_callback, g_object_ref (self));
#line 185 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_check_cpu_vt_capability (NULL, NULL);
#line 186 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_check_module_kvm_loaded (NULL, NULL);
#line 149 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (display_section);
#line 149 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (menu);
#line 149 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
#line 1712 "app.c"
}


gboolean boxes_app_has_broker_for_source_type (BoxesApp* self, const gchar* type) {
	gboolean result = FALSE;
	GHashTable* _tmp0_;
	const gchar* _tmp1_;
	gboolean _tmp2_ = FALSE;
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 189 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (type != NULL, FALSE);
#line 190 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->priv->brokers;
#line 190 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = type;
#line 190 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = g_hash_table_contains (_tmp0_, _tmp1_);
#line 190 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = _tmp2_;
#line 190 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 1735 "app.c"
}


static Block1Data* block1_data_ref (Block1Data* _data1_) {
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_atomic_int_inc (&_data1_->_ref_count_);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return _data1_;
#line 1744 "app.c"
}


static void block1_data_unref (void * _userdata_) {
	Block1Data* _data1_;
	_data1_ = (Block1Data*) _userdata_;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (g_atomic_int_dec_and_test (&_data1_->_ref_count_)) {
#line 1753 "app.c"
		BoxesApp * self;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self = _data1_->self;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		(_data1_->func_target_destroy_notify == NULL) ? NULL : (_data1_->func_target_destroy_notify (_data1_->func_target), NULL);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data1_->func = NULL;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data1_->func_target = NULL;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data1_->func_target_destroy_notify = NULL;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (self);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_slice_free (Block1Data, _data1_);
#line 1769 "app.c"
	}
}


static void __lambda183_ (Block1Data* _data1_) {
	BoxesApp * self;
	BoxesAppCallReadyFunc _tmp0_;
	void* _tmp0__target;
#line 197 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = _data1_->self;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _data1_->func;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0__target = _data1_->func_target;
#line 198 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ (_tmp0__target);
#line 1786 "app.c"
}


static void ___lambda183__boxes_app_ready (BoxesApp* _sender, gpointer self) {
#line 197 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda183_ (self);
#line 1793 "app.c"
}


void boxes_app_call_when_ready (BoxesApp* self, BoxesAppCallReadyFunc func, void* func_target, GDestroyNotify func_target_destroy_notify) {
	Block1Data* _data1_;
	BoxesAppCallReadyFunc _tmp0_;
	void* _tmp0__target;
	gboolean _tmp1_;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_ = g_slice_new0 (Block1Data);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_->_ref_count_ = 1;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_->self = g_object_ref (self);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = func;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0__target = func_target;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	(_data1_->func_target_destroy_notify == NULL) ? NULL : (_data1_->func_target_destroy_notify (_data1_->func_target), NULL);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_->func = NULL;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_->func_target = NULL;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_->func_target_destroy_notify = NULL;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_->func = _tmp0_;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_->func_target = _tmp0__target;
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_->func_target_destroy_notify = func_target_destroy_notify;
#line 195 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = self->priv->is_ready;
#line 195 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp1_) {
#line 1832 "app.c"
		BoxesAppCallReadyFunc _tmp2_;
		void* _tmp2__target;
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp2_ = _data1_->func;
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp2__target = _data1_->func_target;
#line 196 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp2_ (_tmp2__target);
#line 1841 "app.c"
	}
#line 197 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_data (self, "ready", (GCallback) ___lambda183__boxes_app_ready, block1_data_ref (_data1_), (GClosureNotify) block1_data_unref, 0);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	block1_data_unref (_data1_);
#line 194 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data1_ = NULL;
#line 1849 "app.c"
}


void boxes_app_activate (BoxesApp* self) {
	GtkApplicationWindow* _tmp0_;
#line 202 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 203 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_setup_ui (self);
#line 204 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->window;
#line 204 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_window_present ((GtkWindow*) _tmp0_);
#line 1863 "app.c"
}


static Block2Data* block2_data_ref (Block2Data* _data2_) {
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_atomic_int_inc (&_data2_->_ref_count_);
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return _data2_;
#line 1872 "app.c"
}


static void block2_data_unref (void * _userdata_) {
	Block2Data* _data2_;
	_data2_ = (Block2Data*) _userdata_;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (g_atomic_int_dec_and_test (&_data2_->_ref_count_)) {
#line 1881 "app.c"
		BoxesApp * self;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self = _data2_->self;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (_data2_->app);
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (self);
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_slice_free (Block2Data, _data2_);
#line 1891 "app.c"
	}
}


static Block3Data* block3_data_ref (Block3Data* _data3_) {
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_atomic_int_inc (&_data3_->_ref_count_);
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return _data3_;
#line 1901 "app.c"
}


static void block3_data_unref (void * _userdata_) {
	Block3Data* _data3_;
	_data3_ = (Block3Data*) _userdata_;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (g_atomic_int_dec_and_test (&_data3_->_ref_count_)) {
#line 1910 "app.c"
		BoxesApp * self;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self = _data3_->_data2_->self;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_data3_->uuid);
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block2_data_unref (_data3_->_data2_);
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data3_->_data2_ = NULL;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_slice_free (Block3Data, _data3_);
#line 1922 "app.c"
	}
}


static void ___lambda201_ (Block3Data* _data3_) {
	Block2Data* _data2_;
	BoxesApp * self;
	BoxesApp* _tmp0_;
	const gchar* _tmp1_;
#line 263 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data2_ = _data3_->_data2_;
#line 263 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = _data2_->self;
#line 264 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _data2_->app;
#line 264 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _data3_->uuid;
#line 264 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_open_uuid (_tmp0_, _tmp1_);
#line 1942 "app.c"
}


static void ____lambda201__boxes_app_call_ready_func (gpointer self) {
#line 263 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	___lambda201_ (self);
#line 1949 "app.c"
}


static Block4Data* block4_data_ref (Block4Data* _data4_) {
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_atomic_int_inc (&_data4_->_ref_count_);
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return _data4_;
#line 1958 "app.c"
}


static void block4_data_unref (void * _userdata_) {
	Block4Data* _data4_;
	_data4_ = (Block4Data*) _userdata_;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (g_atomic_int_dec_and_test (&_data4_->_ref_count_)) {
#line 1967 "app.c"
		BoxesApp * self;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self = _data4_->_data2_->self;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (_data4_->file);
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_data4_->arg);
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block2_data_unref (_data4_->_data2_);
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data4_->_data2_ = NULL;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_slice_free (Block4Data, _data4_);
#line 1981 "app.c"
	}
}


static void _____lambda202_ (Block4Data* _data4_) {
	Block2Data* _data2_;
	BoxesApp * self;
	BoxesWizard* _tmp0_;
	GFile* _tmp1_;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_;
#line 271 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data2_ = _data4_->_data2_;
#line 271 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = _data2_->self;
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->wizard;
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _data4_->file;
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = g_file_get_uri (_tmp1_);
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _tmp2_;
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_wizard_open_with_uri (_tmp0_, _tmp3_, TRUE);
#line 272 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (_tmp3_);
#line 2009 "app.c"
}


static void ______lambda202__boxes_app_call_ready_func (gpointer self) {
#line 271 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_____lambda202_ (self);
#line 2016 "app.c"
}


static void _____lambda203_ (Block4Data* _data4_) {
	Block2Data* _data2_;
	BoxesApp * self;
	const gchar* _tmp0_;
#line 275 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data2_ = _data4_->_data2_;
#line 275 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = _data2_->self;
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _data4_->arg;
#line 276 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_open (self, _tmp0_);
#line 2032 "app.c"
}


static void ______lambda203__boxes_app_call_ready_func (gpointer self) {
#line 275 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_____lambda203_ (self);
#line 2039 "app.c"
}


static void ___lambda204_ (BoxesApp* self) {
	BoxesSearchbar* _tmp0_;
	gchar** _tmp1_;
	gint _tmp1__length1;
	gchar* _tmp2_ = NULL;
	gchar* _tmp3_;
	BoxesSearchbar* _tmp4_;
	BoxesUIState _tmp5_;
	BoxesUIState _tmp6_;
#line 283 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->searchbar;
#line 283 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = boxes_app_opt_search;
#line 283 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1__length1 = boxes_app_opt_search_length1;
#line 283 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = g_strjoinv (" ", _tmp1_);
#line 283 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _tmp2_;
#line 283 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_searchbar_set_text (_tmp0_, _tmp3_);
#line 283 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (_tmp3_);
#line 284 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = self->searchbar;
#line 284 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_searchbar_set_visible (_tmp4_, TRUE);
#line 285 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 285 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = _tmp5_;
#line 285 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp6_ == BOXES_UI_STATE_COLLECTION) {
#line 2076 "app.c"
		BoxesRevealer* _tmp7_;
#line 286 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp7_ = self->searchbar_revealer;
#line 286 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_revealer_set_revealed (_tmp7_, TRUE);
#line 2082 "app.c"
	}
}


static void ____lambda204__boxes_app_call_ready_func (gpointer self) {
#line 282 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	___lambda204_ (self);
#line 2090 "app.c"
}


gint boxes_app_command_line (BoxesApp* self, GApplicationCommandLine* cmdline) {
	gint result = 0;
	Block2Data* _data2_;
	const gchar* _tmp0_ = NULL;
	gchar* _tmp1_;
	gchar* parameter_string;
	const gchar* _tmp2_;
	GOptionContext* _tmp3_;
	GOptionContext* opt_context;
	GOptionContext* _tmp4_;
	GOptionContext* _tmp5_;
	gboolean _tmp18_;
	gboolean _tmp23_ = FALSE;
	gchar** _tmp24_;
	gint _tmp24__length1;
	gboolean _tmp29_;
	BoxesApplication* _tmp36_;
	BoxesApp* _tmp37_;
	const gchar* _tmp38_;
	gchar** _tmp54_;
	gint _tmp54__length1;
	gboolean _tmp55_;
	GError * _inner_error_ = NULL;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (cmdline != NULL, 0);
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data2_ = g_slice_new0 (Block2Data);
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data2_->_ref_count_ = 1;
#line 224 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data2_->self = g_object_ref (self);
#line 225 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_opt_fullscreen = FALSE;
#line 226 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_opt_help = FALSE;
#line 227 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (boxes_app_opt_open_uuid);
#line 227 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_opt_open_uuid = NULL;
#line 228 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_opt_uris = (_vala_array_free (boxes_app_opt_uris, boxes_app_opt_uris_length1, (GDestroyNotify) g_free), NULL);
#line 228 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_opt_uris = NULL;
#line 228 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_opt_uris_length1 = 0;
#line 228 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_boxes_app_opt_uris_size_ = boxes_app_opt_uris_length1;
#line 229 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_opt_search = (_vala_array_free (boxes_app_opt_search, boxes_app_opt_search_length1, (GDestroyNotify) g_free), NULL);
#line 229 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_opt_search = NULL;
#line 229 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_opt_search_length1 = 0;
#line 229 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_boxes_app_opt_search_size_ = boxes_app_opt_search_length1;
#line 231 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _ ("- A simple application to access remote or virtual machines");
#line 231 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 231 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	parameter_string = _tmp1_;
#line 232 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = parameter_string;
#line 232 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = g_option_context_new (_tmp2_);
#line 232 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	opt_context = _tmp3_;
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = opt_context;
#line 233 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_option_context_add_main_entries (_tmp4_, BOXES_APP_options, NULL);
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = opt_context;
#line 234 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_option_context_set_help_enabled (_tmp5_, FALSE);
#line 2171 "app.c"
	{
		GApplicationCommandLine* _tmp6_;
		gint _tmp7_ = 0;
		gchar** _tmp8_ = NULL;
		gchar** args1;
		gint args1_length1;
		gint _args1_size_;
		gchar** _tmp9_;
		gint _tmp9__length1;
		gchar** args2;
		gint args2_length1;
		gint _args2_size_;
		GOptionContext* _tmp10_;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp6_ = cmdline;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp8_ = g_application_command_line_get_arguments (_tmp6_, &_tmp7_);
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		args1 = _tmp8_;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		args1_length1 = _tmp7_;
#line 237 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_args1_size_ = args1_length1;
#line 238 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp9_ = args1;
#line 238 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp9__length1 = args1_length1;
#line 238 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		args2 = _tmp9_;
#line 238 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		args2_length1 = _tmp9__length1;
#line 238 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_args2_size_ = args2_length1;
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp10_ = opt_context;
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_option_context_parse (_tmp10_, &args2_length1, &args2, &_inner_error_);
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_inner_error_ != NULL) {
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			args1 = (_vala_array_free (args1, args1_length1, (GDestroyNotify) g_free), NULL);
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_inner_error_->domain == G_OPTION_ERROR) {
#line 2215 "app.c"
				goto __catch0_g_option_error;
			}
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			args1 = (_vala_array_free (args1, args1_length1, (GDestroyNotify) g_free), NULL);
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_option_context_free0 (opt_context);
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_free0 (parameter_string);
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			block2_data_unref (_data2_);
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data2_ = NULL;
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_clear_error (&_inner_error_);
#line 239 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			return 0;
#line 2234 "app.c"
		}
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		args1 = (_vala_array_free (args1, args1_length1, (GDestroyNotify) g_free), NULL);
#line 2238 "app.c"
	}
	goto __finally0;
	__catch0_g_option_error:
	{
		GError* _error_ = NULL;
		GApplicationCommandLine* _tmp11_;
		GError* _tmp12_;
		const gchar* _tmp13_;
		GApplicationCommandLine* _tmp14_;
		GOptionContext* _tmp15_;
		gchar* _tmp16_ = NULL;
		gchar* _tmp17_;
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_error_ = _inner_error_;
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_inner_error_ = NULL;
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp11_ = cmdline;
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp12_ = _error_;
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp13_ = _tmp12_->message;
#line 241 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_application_command_line_printerr (_tmp11_, "%s\n", _tmp13_);
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp14_ = cmdline;
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp15_ = opt_context;
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp16_ = g_option_context_get_help (_tmp15_, TRUE, NULL);
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp17_ = _tmp16_;
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_application_command_line_printerr (_tmp14_, "%s", _tmp17_);
#line 242 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_tmp17_);
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		result = 1;
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_error_free0 (_error_);
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_option_context_free0 (opt_context);
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (parameter_string);
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block2_data_unref (_data2_);
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data2_ = NULL;
#line 243 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return result;
#line 2289 "app.c"
	}
	__finally0:
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_inner_error_ != NULL) {
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_option_context_free0 (opt_context);
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (parameter_string);
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block2_data_unref (_data2_);
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data2_ = NULL;
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_clear_error (&_inner_error_);
#line 236 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return 0;
#line 2308 "app.c"
	}
#line 246 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp18_ = boxes_app_opt_help;
#line 246 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp18_) {
#line 2314 "app.c"
		GApplicationCommandLine* _tmp19_;
		GOptionContext* _tmp20_;
		gchar* _tmp21_ = NULL;
		gchar* _tmp22_;
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp19_ = cmdline;
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp20_ = opt_context;
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp21_ = g_option_context_get_help (_tmp20_, TRUE, NULL);
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp22_ = _tmp21_;
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_application_command_line_printerr (_tmp19_, "%s", _tmp22_);
#line 247 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_tmp22_);
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		result = 1;
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_option_context_free0 (opt_context);
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (parameter_string);
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block2_data_unref (_data2_);
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data2_ = NULL;
#line 248 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return result;
#line 2343 "app.c"
	}
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp24_ = boxes_app_opt_uris;
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp24__length1 = boxes_app_opt_uris_length1;
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp24__length1 > 1) {
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp23_ = TRUE;
#line 2353 "app.c"
	} else {
		gboolean _tmp25_ = FALSE;
		const gchar* _tmp26_;
		gboolean _tmp28_;
#line 252 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp26_ = boxes_app_opt_open_uuid;
#line 252 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_tmp26_ != NULL) {
#line 2362 "app.c"
			gchar** _tmp27_;
			gint _tmp27__length1;
#line 252 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp27_ = boxes_app_opt_uris;
#line 252 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp27__length1 = boxes_app_opt_uris_length1;
#line 252 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp25_ = _tmp27_ != NULL;
#line 2371 "app.c"
		} else {
#line 252 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp25_ = FALSE;
#line 2375 "app.c"
		}
#line 252 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp28_ = _tmp25_;
#line 252 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp23_ = _tmp28_;
#line 2381 "app.c"
	}
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp29_ = _tmp23_;
#line 251 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp29_) {
#line 2387 "app.c"
		GApplicationCommandLine* _tmp30_;
		const gchar* _tmp31_ = NULL;
		GApplicationCommandLine* _tmp32_;
		GOptionContext* _tmp33_;
		gchar* _tmp34_ = NULL;
		gchar* _tmp35_;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp30_ = cmdline;
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp31_ = _ ("Too many command line arguments specified.\n");
#line 253 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_application_command_line_printerr (_tmp30_, "%s", _tmp31_);
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp32_ = cmdline;
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp33_ = opt_context;
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp34_ = g_option_context_get_help (_tmp33_, TRUE, NULL);
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp35_ = _tmp34_;
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_application_command_line_printerr (_tmp32_, "%s", _tmp35_);
#line 254 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_tmp35_);
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		result = 1;
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_option_context_free0 (opt_context);
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (parameter_string);
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block2_data_unref (_data2_);
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data2_ = NULL;
#line 255 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return result;
#line 2424 "app.c"
	}
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp36_ = self->priv->application;
#line 258 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_application_activate ((GApplication*) _tmp36_);
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp37_ = _g_object_ref0 (self);
#line 260 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data2_->app = _tmp37_;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp38_ = boxes_app_opt_open_uuid;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp38_ != NULL) {
#line 2438 "app.c"
		Block3Data* _data3_;
		const gchar* _tmp39_;
		gchar* _tmp40_;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data3_ = g_slice_new0 (Block3Data);
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data3_->_ref_count_ = 1;
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data3_->_data2_ = block2_data_ref (_data2_);
#line 262 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp39_ = boxes_app_opt_open_uuid;
#line 262 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp40_ = g_strdup (_tmp39_);
#line 262 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data3_->uuid = _tmp40_;
#line 263 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_call_when_ready (self, ____lambda201__boxes_app_call_ready_func, block3_data_ref (_data3_), block3_data_unref);
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block3_data_unref (_data3_);
#line 261 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data3_ = NULL;
#line 2460 "app.c"
	} else {
		gchar** _tmp41_;
		gint _tmp41__length1;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp41_ = boxes_app_opt_uris;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp41__length1 = boxes_app_opt_uris_length1;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_tmp41_ != NULL) {
#line 2470 "app.c"
			Block4Data* _data4_;
			gchar** _tmp42_;
			gint _tmp42__length1;
			const gchar* _tmp43_;
			gchar* _tmp44_;
			const gchar* _tmp45_;
			GFile* _tmp46_ = NULL;
			gboolean _tmp47_ = FALSE;
			GFile* _tmp48_;
			gboolean _tmp49_ = FALSE;
			gboolean _tmp53_;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data4_ = g_slice_new0 (Block4Data);
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data4_->_ref_count_ = 1;
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data4_->_data2_ = block2_data_ref (_data2_);
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp42_ = boxes_app_opt_uris;
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp42__length1 = boxes_app_opt_uris_length1;
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp43_ = _tmp42_[0];
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp44_ = g_strdup (_tmp43_);
#line 267 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data4_->arg = _tmp44_;
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp45_ = _data4_->arg;
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp46_ = g_file_new_for_commandline_arg (_tmp45_);
#line 268 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data4_->file = _tmp46_;
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp48_ = _data4_->file;
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp49_ = g_file_query_exists (_tmp48_, NULL);
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_tmp49_) {
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp47_ = TRUE;
#line 2512 "app.c"
			} else {
				const gchar* _tmp50_;
				gchar* _tmp51_ = NULL;
				gchar* _tmp52_;
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp50_ = _data4_->arg;
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp51_ = g_uri_parse_scheme (_tmp50_);
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp52_ = _tmp51_;
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp47_ = _tmp52_ != NULL;
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_free0 (_tmp52_);
#line 2527 "app.c"
			}
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp53_ = _tmp47_;
#line 270 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_tmp53_) {
#line 271 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_app_call_when_ready (self, ______lambda202__boxes_app_call_ready_func, block4_data_ref (_data4_), block4_data_unref);
#line 2535 "app.c"
			} else {
#line 275 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_app_call_when_ready (self, ______lambda203__boxes_app_call_ready_func, block4_data_ref (_data4_), block4_data_unref);
#line 2539 "app.c"
			}
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			block4_data_unref (_data4_);
#line 266 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data4_ = NULL;
#line 2545 "app.c"
		}
	}
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp54_ = boxes_app_opt_search;
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp54__length1 = boxes_app_opt_search_length1;
#line 281 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp54_ != NULL) {
#line 282 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_call_when_ready (self, ____lambda204__boxes_app_call_ready_func, g_object_ref (self), g_object_unref);
#line 2556 "app.c"
	}
#line 291 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp55_ = boxes_app_opt_fullscreen;
#line 291 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp55_) {
#line 2562 "app.c"
		BoxesApp* _tmp56_;
#line 292 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp56_ = _data2_->app;
#line 292 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_set_fullscreen (_tmp56_, TRUE);
#line 2568 "app.c"
	}
#line 294 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = 0;
#line 294 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_option_context_free0 (opt_context);
#line 294 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (parameter_string);
#line 294 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	block2_data_unref (_data2_);
#line 294 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data2_ = NULL;
#line 294 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 2582 "app.c"
}


gint boxes_app_run (BoxesApp* self, gchar** args, int args_length1) {
	gint result = 0;
	BoxesApplication* _tmp0_;
	gchar** _tmp1_;
	gint _tmp1__length1;
	gint _tmp2_ = 0;
#line 297 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, 0);
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->priv->application;
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = args;
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1__length1 = args_length1;
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = g_application_run ((GApplication*) _tmp0_, _tmp1__length1, _tmp1_);
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = _tmp2_;
#line 298 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 2606 "app.c"
}


void boxes_app_shutdown (BoxesApp* self) {
#line 304 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 305 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->priv->application);
#line 305 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->application = NULL;
#line 2617 "app.c"
}


void boxes_app_open (BoxesApp* self, const gchar* name) {
	BoxesCollectionView* _tmp0_;
	BoxesCollection* _tmp1_;
	GPtrArray* _tmp2_;
	gpointer* _tmp3_;
	gint _tmp3__length1;
#line 308 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 308 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (name != NULL);
#line 309 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_ui_set_ui_state ((BoxesUI*) self, BOXES_UI_STATE_COLLECTION);
#line 312 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 312 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_collection_view_set_visible (_tmp0_, FALSE);
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = self->collection;
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = _tmp1_->items;
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _tmp2_->pdata;
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3__length1 = (gint) _tmp2_->len;
#line 2645 "app.c"
	{
		BoxesCollectionItem** item_collection = NULL;
		gint item_collection_length1 = 0;
		gint _item_collection_size_ = 0;
		gint item_it = 0;
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		item_collection = _tmp3_;
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		item_collection_length1 = _tmp3__length1;
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		for (item_it = 0; item_it < _tmp3__length1; item_it = item_it + 1) {
#line 2657 "app.c"
			BoxesCollectionItem* _tmp4_;
			BoxesCollectionItem* item = NULL;
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp4_ = _g_object_ref0 (item_collection[item_it]);
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			item = _tmp4_;
#line 2664 "app.c"
			{
				BoxesCollectionItem* _tmp5_;
				const gchar* _tmp6_;
				const gchar* _tmp7_;
				const gchar* _tmp8_;
#line 316 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp5_ = item;
#line 316 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp6_ = boxes_collection_item_get_name (_tmp5_);
#line 316 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp7_ = _tmp6_;
#line 316 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp8_ = name;
#line 316 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (g_strcmp0 (_tmp7_, _tmp8_) == 0) {
#line 2680 "app.c"
					BoxesCollectionItem* _tmp9_;
#line 317 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp9_ = item;
#line 317 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					boxes_app_select_item (self, _tmp9_);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_g_object_unref0 (item);
#line 319 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					break;
#line 2690 "app.c"
				}
#line 315 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (item);
#line 2694 "app.c"
			}
		}
	}
}


gboolean boxes_app_open_uuid (BoxesApp* self, const gchar* uuid) {
	gboolean result = FALSE;
	BoxesCollectionView* _tmp0_;
	BoxesCollection* _tmp1_;
	GPtrArray* _tmp2_;
	gpointer* _tmp3_;
	gint _tmp3__length1;
#line 324 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 324 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (uuid != NULL, FALSE);
#line 325 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_ui_set_ui_state ((BoxesUI*) self, BOXES_UI_STATE_COLLECTION);
#line 328 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 328 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_collection_view_set_visible (_tmp0_, FALSE);
#line 331 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = self->collection;
#line 331 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = _tmp1_->items;
#line 331 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _tmp2_->pdata;
#line 331 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3__length1 = (gint) _tmp2_->len;
#line 2726 "app.c"
	{
		BoxesCollectionItem** item_collection = NULL;
		gint item_collection_length1 = 0;
		gint _item_collection_size_ = 0;
		gint item_it = 0;
#line 331 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		item_collection = _tmp3_;
#line 331 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		item_collection_length1 = _tmp3__length1;
#line 331 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		for (item_it = 0; item_it < _tmp3__length1; item_it = item_it + 1) {
#line 2738 "app.c"
			BoxesCollectionItem* _tmp4_;
			BoxesCollectionItem* item = NULL;
#line 331 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp4_ = _g_object_ref0 (item_collection[item_it]);
#line 331 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			item = _tmp4_;
#line 2745 "app.c"
			{
				BoxesCollectionItem* _tmp5_;
				BoxesCollectionItem* _tmp6_;
				BoxesMachine* _tmp7_;
				BoxesMachine* machine;
				BoxesMachine* _tmp8_;
				BoxesBoxConfig* _tmp9_;
				gchar* _tmp10_;
				gchar* _tmp11_;
				gchar* _tmp12_;
				const gchar* _tmp13_;
				gboolean _tmp14_;
				BoxesCollectionItem* _tmp15_;
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp5_ = item;
#line 332 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (!G_TYPE_CHECK_INSTANCE_TYPE (_tmp5_, BOXES_TYPE_MACHINE)) {
#line 333 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_g_object_unref0 (item);
#line 333 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					continue;
#line 2767 "app.c"
				}
#line 334 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp6_ = item;
#line 334 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp7_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp6_, BOXES_TYPE_MACHINE) ? ((BoxesMachine*) _tmp6_) : NULL);
#line 334 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				machine = _tmp7_;
#line 336 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp8_ = machine;
#line 336 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp9_ = _tmp8_->config;
#line 336 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp10_ = boxes_box_config_get_uuid (_tmp9_);
#line 336 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp11_ = _tmp10_;
#line 336 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp12_ = _tmp11_;
#line 336 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp13_ = uuid;
#line 336 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp14_ = g_strcmp0 (_tmp12_, _tmp13_) != 0;
#line 336 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_free0 (_tmp12_);
#line 336 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (_tmp14_) {
#line 337 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_g_object_unref0 (machine);
#line 337 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_g_object_unref0 (item);
#line 337 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					continue;
#line 2799 "app.c"
				}
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp15_ = item;
#line 339 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_app_select_item (self, _tmp15_);
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				result = TRUE;
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (machine);
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (item);
#line 340 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				return result;
#line 2813 "app.c"
			}
		}
	}
#line 343 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = FALSE;
#line 343 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 2821 "app.c"
}


void boxes_app_delete_machine (BoxesApp* self, BoxesMachine* machine, gboolean by_user) {
	BoxesMachine* _tmp0_;
	gboolean _tmp1_;
	BoxesCollection* _tmp2_;
	BoxesMachine* _tmp3_;
#line 346 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 346 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (machine != NULL);
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = machine;
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = by_user;
#line 347 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_machine_delete (_tmp0_, _tmp1_);
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = self->collection;
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = machine;
#line 348 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_collection_remove_item (_tmp2_, (BoxesCollectionItem*) _tmp3_);
#line 2846 "app.c"
}


static void boxes_app_add_collection_source_data_free (gpointer _data) {
	BoxesAppAddCollectionSourceData* _data_;
	_data_ = _data;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_data_->source);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_data_->self);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_slice_free (BoxesAppAddCollectionSourceData, _data_);
#line 2859 "app.c"
}


void boxes_app_add_collection_source (BoxesApp* self, BoxesCollectionSource* source, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesAppAddCollectionSourceData* _data_;
	BoxesApp* _tmp0_;
	BoxesCollectionSource* _tmp1_;
	BoxesCollectionSource* _tmp2_;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_ = g_slice_new0 (BoxesAppAddCollectionSourceData);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_app_add_collection_source);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_app_add_collection_source_data_free);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->self = _tmp0_;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = source;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = _g_object_ref0 (_tmp1_);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_data_->source);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->source = _tmp2_;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_add_collection_source_co (_data_);
#line 2888 "app.c"
}


void boxes_app_add_collection_source_finish (BoxesApp* self, GAsyncResult* _res_) {
	BoxesAppAddCollectionSourceData* _data_;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 2896 "app.c"
}


static void boxes_app_add_collection_source_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesAppAddCollectionSourceData* _data_;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_ = _user_data_;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_source_object_ = source_object;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_res_ = _res_;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_add_collection_source_co (_data_);
#line 2910 "app.c"
}


static gboolean boxes_app_add_collection_source_co (BoxesAppAddCollectionSourceData* _data_) {
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	static GQuark _tmp7_label0 = 0;
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	static GQuark _tmp7_label1 = 0;
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	switch (_data_->_state_) {
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case 0:
#line 2923 "app.c"
		goto _state_0;
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case 1:
#line 2927 "app.c"
		goto _state_1;
		default:
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_assert_not_reached ();
#line 2932 "app.c"
	}
	_state_0:
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp0_ = _data_->source;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp1_ = boxes_collection_source_get_enabled (_data_->_tmp0_);
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp2_ = _data_->_tmp1_;
#line 352 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (!_data_->_tmp2_) {
#line 353 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_data_->_state_ == 0) {
#line 353 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 2947 "app.c"
		} else {
#line 353 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_simple_async_result_complete (_data_->_async_result);
#line 2951 "app.c"
		}
#line 353 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_object_unref (_data_->_async_result);
#line 353 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return FALSE;
#line 2957 "app.c"
	}
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp3_ = _data_->source;
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp4_ = boxes_collection_source_get_source_type (_data_->_tmp3_);
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp5_ = _data_->_tmp4_;
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp8_ = 0U;
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp6_ = _data_->_tmp5_;
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp8_ = (NULL == _data_->_tmp6_) ? 0 : g_quark_from_string (_data_->_tmp6_);
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_free (_data_->_tmp6_);
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if ((_data_->_tmp8_ == ((0 != _tmp7_label0) ? _tmp7_label0 : (_tmp7_label0 = g_quark_from_static_string ("vnc")))) || (_data_->_tmp8_ == ((0 != _tmp7_label1) ? _tmp7_label1 : (_tmp7_label1 = g_quark_from_static_string ("spice"))))) {
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		switch (0) {
#line 2977 "app.c"
			default:
			{
				{
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp9_ = _data_->source;
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp10_ = boxes_remote_machine_new (_data_->_tmp9_, &_data_->_inner_error_);
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->machine = _data_->_tmp10_;
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					if (_data_->_inner_error_ != NULL) {
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						if (_data_->_inner_error_->domain == BOXES_ERROR) {
#line 2991 "app.c"
							goto __catch1_boxes_error;
						}
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						g_critical ("file %s: line %d: unexpected error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						g_clear_error (&_data_->_inner_error_);
#line 359 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						return FALSE;
#line 3000 "app.c"
					}
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp11_ = _data_->self->collection;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp12_ = _data_->machine;
#line 360 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					boxes_collection_add_item (_data_->_tmp11_, (BoxesCollectionItem*) _data_->_tmp12_);
#line 358 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_g_object_unref0 (_data_->machine);
#line 3010 "app.c"
				}
				goto __finally1;
				__catch1_boxes_error:
				{
#line 358 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_error_ = _data_->_inner_error_;
#line 358 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_inner_error_ = NULL;
#line 362 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp13_ = _data_->_error_;
#line 362 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp14_ = _data_->_tmp13_->message;
#line 362 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					g_warning ("app.vala:362: %s", _data_->_tmp14_);
#line 358 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_g_error_free0 (_data_->_error_);
#line 3027 "app.c"
				}
				__finally1:
#line 358 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (_data_->_inner_error_ != NULL) {
#line 358 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 358 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					g_clear_error (&_data_->_inner_error_);
#line 358 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					return FALSE;
#line 3038 "app.c"
				}
#line 364 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				break;
#line 3042 "app.c"
			}
		}
	} else {
#line 355 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		switch (0) {
#line 3048 "app.c"
			default:
			{
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp15_ = _data_->self->priv->brokers;
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp16_ = _data_->source;
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp17_ = boxes_collection_source_get_source_type (_data_->_tmp16_);
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp18_ = _data_->_tmp17_;
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp19_ = _data_->_tmp18_;
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp20_ = NULL;
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp20_ = g_hash_table_lookup (_data_->_tmp15_, _data_->_tmp19_);
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp21_ = _g_object_ref0 ((BoxesBroker*) _data_->_tmp20_);
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp22_ = _data_->_tmp21_;
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_free0 (_data_->_tmp19_);
#line 367 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->broker = _data_->_tmp22_;
#line 368 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data_->_tmp23_ = _data_->broker;
#line 368 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (_data_->_tmp23_ != NULL) {
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp24_ = _data_->broker;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp25_ = _data_->source;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_state_ = 1;
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					boxes_broker_add_source (_data_->_tmp24_, _data_->_tmp25_, boxes_app_add_collection_source_ready, _data_);
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					return FALSE;
#line 3087 "app.c"
					_state_1:
#line 369 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					boxes_broker_add_source_finish (_data_->_tmp24_, _data_->_res_);
#line 370 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp26_ = _data_->self->priv->sources;
#line 370 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp27_ = _data_->source;
#line 370 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp28_ = boxes_collection_source_get_name (_data_->_tmp27_);
#line 370 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp29_ = _data_->_tmp28_;
#line 370 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp30_ = _data_->source;
#line 370 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp31_ = _g_object_ref0 (_data_->_tmp30_);
#line 370 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					g_hash_table_insert (_data_->_tmp26_, _data_->_tmp29_, _data_->_tmp31_);
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp32_ = _data_->source;
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp33_ = boxes_collection_source_get_name (_data_->_tmp32_);
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp34_ = _data_->_tmp33_;
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp35_ = _data_->_tmp34_;
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp36_ = g_strcmp0 (_data_->_tmp35_, "QEMU Session") == 0;
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_g_free0 (_data_->_tmp35_);
#line 371 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					if (_data_->_tmp36_) {
#line 372 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						g_object_notify ((GObject*) _data_->self, "default-connection");
#line 373 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						g_object_notify ((GObject*) _data_->self, "default-source");
#line 3123 "app.c"
					}
				} else {
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp37_ = _data_->source;
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp38_ = boxes_collection_source_get_source_type (_data_->_tmp37_);
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp39_ = _data_->_tmp38_;
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data_->_tmp40_ = _data_->_tmp39_;
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					g_warning ("app.vala:376: Unsupported source type %s", _data_->_tmp40_);
#line 376 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_g_free0 (_data_->_tmp40_);
#line 3138 "app.c"
				}
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (_data_->broker);
#line 378 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				break;
#line 3144 "app.c"
			}
		}
	}
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_data_->_state_ == 0) {
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 3152 "app.c"
	} else {
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 3156 "app.c"
	}
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_unref (_data_->_async_result);
#line 351 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return FALSE;
#line 3162 "app.c"
}


static void boxes_app_setup_sources_data_free (gpointer _data) {
	BoxesAppSetupSourcesData* _data_;
	_data_ = _data;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_data_->self);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_slice_free (BoxesAppSetupSourcesData, _data_);
#line 3173 "app.c"
}


static void boxes_app_setup_sources (BoxesApp* self, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	BoxesAppSetupSourcesData* _data_;
	BoxesApp* _tmp0_;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_ = g_slice_new0 (BoxesAppSetupSourcesData);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_async_result = g_simple_async_result_new (G_OBJECT (self), _callback_, _user_data_, boxes_app_setup_sources);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_simple_async_result_set_op_res_gpointer (_data_->_async_result, _data_, boxes_app_setup_sources_data_free);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _g_object_ref0 (self);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->self = _tmp0_;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_setup_sources_co (_data_);
#line 3192 "app.c"
}


static void boxes_app_setup_sources_finish (BoxesApp* self, GAsyncResult* _res_) {
	BoxesAppSetupSourcesData* _data_;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_ = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (_res_));
#line 3200 "app.c"
}


static void boxes_app_setup_sources_ready (GObject* source_object, GAsyncResult* _res_, gpointer _user_data_) {
	BoxesAppSetupSourcesData* _data_;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_ = _user_data_;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_source_object_ = source_object;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_res_ = _res_;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_setup_sources_co (_data_);
#line 3214 "app.c"
}


static gboolean __lambda195_ (BoxesApp* self, const gchar* filename, GError** error) {
	gboolean result = FALSE;
	const gchar* _tmp0_;
	BoxesCollectionSource* _tmp1_;
	BoxesCollectionSource* source;
	GError * _inner_error_ = NULL;
#line 406 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (filename != NULL, FALSE);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = filename;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = boxes_collection_source_new_with_file (_tmp0_, &_inner_error_);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	source = _tmp1_;
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_inner_error_ != NULL) {
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_propagate_error (error, _inner_error_);
#line 407 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return FALSE;
#line 3238 "app.c"
	}
#line 408 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_add_collection_source (self, source, NULL, NULL);
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = FALSE;
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (source);
#line 409 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3248 "app.c"
}


static gboolean ___lambda195__boxes_foreach_filename_from_dir_func (const gchar* filename, gpointer self, GError** error) {
	gboolean result;
	result = __lambda195_ (self, filename, error);
#line 406 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3257 "app.c"
}


static gboolean boxes_app_setup_sources_co (BoxesAppSetupSourcesData* _data_) {
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	switch (_data_->_state_) {
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case 0:
#line 3266 "app.c"
		goto _state_0;
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case 1:
#line 3270 "app.c"
		goto _state_1;
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case 2:
#line 3274 "app.c"
		goto _state_2;
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case 3:
#line 3278 "app.c"
		goto _state_3;
		default:
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_assert_not_reached ();
#line 3283 "app.c"
	}
	_state_0:
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp0_ = NULL;
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp0_ = boxes_get_user_pkgconfig_source ("QEMU Session");
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp1_ = _data_->_tmp0_;
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp2_ = FALSE;
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp2_ = g_file_test (_data_->_tmp1_, G_FILE_TEST_IS_REGULAR);
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp3_ = !_data_->_tmp2_;
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (_data_->_tmp1_);
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_data_->_tmp3_) {
#line 385 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp4_ = NULL;
#line 385 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp4_ = boxes_get_pkgdata_source ("QEMU_Session");
#line 385 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp5_ = _data_->_tmp4_;
#line 385 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp6_ = NULL;
#line 385 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp6_ = g_file_new_for_path (_data_->_tmp5_);
#line 385 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp7_ = _data_->_tmp6_;
#line 385 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_data_->_tmp5_);
#line 385 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->src = _data_->_tmp7_;
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp8_ = NULL;
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp8_ = boxes_get_user_pkgconfig_source ("QEMU Session");
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp9_ = _data_->_tmp8_;
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp10_ = NULL;
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp10_ = g_file_new_for_path (_data_->_tmp9_);
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp11_ = _data_->_tmp10_;
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_data_->_tmp9_);
#line 386 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->dst = _data_->_tmp11_;
#line 3334 "app.c"
		{
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data_->_tmp12_ = _data_->src;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data_->_tmp13_ = _data_->dst;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data_->_state_ = 1;
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_file_copy_async (_data_->_tmp12_, _data_->_tmp13_, G_FILE_COPY_NONE, G_PRIORITY_DEFAULT, NULL, NULL, NULL, boxes_app_setup_sources_ready, _data_);
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			return FALSE;
#line 3346 "app.c"
			_state_1:
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_file_copy_finish (_data_->_tmp12_, _data_->_res_, &_data_->_inner_error_);
#line 388 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_data_->_inner_error_ != NULL) {
#line 3352 "app.c"
				goto __catch2_g_error;
			}
		}
		goto __finally2;
		__catch2_g_error:
		{
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data_->_error_ = _data_->_inner_error_;
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data_->_inner_error_ = NULL;
#line 390 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data_->_tmp14_ = _data_->_error_;
#line 390 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_data_->_tmp15_ = _data_->_tmp14_->message;
#line 390 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_critical ("app.vala:390: Can't setup default sources: %s", _data_->_tmp15_);
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_error_free0 (_data_->_error_);
#line 3371 "app.c"
		}
		__finally2:
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_data_->_inner_error_ != NULL) {
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_object_unref0 (_data_->dst);
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_object_unref0 (_data_->src);
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_clear_error (&_data_->_inner_error_);
#line 387 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			return FALSE;
#line 3386 "app.c"
		}
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (_data_->dst);
#line 384 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (_data_->src);
#line 3392 "app.c"
	}
	{
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp16_ = boxes_collection_source_new_with_file ("QEMU Session", &_data_->_inner_error_);
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->source = _data_->_tmp16_;
#line 395 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_data_->_inner_error_ != NULL) {
#line 3401 "app.c"
			goto __catch3_g_error;
		}
#line 396 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_state_ = 2;
#line 396 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_add_collection_source (_data_->self, _data_->source, boxes_app_setup_sources_ready, _data_);
#line 396 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return FALSE;
#line 3410 "app.c"
		_state_2:
#line 396 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_add_collection_source_finish (_data_->self, _data_->_res_);
#line 394 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (_data_->source);
#line 3416 "app.c"
	}
	goto __finally3;
	__catch3_g_error:
	{
#line 394 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_vala1__error_ = _data_->_inner_error_;
#line 394 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_inner_error_ = NULL;
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp17_ = _data_->_vala1__error_;
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp18_ = _data_->_tmp17_->message;
#line 398 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_warning ("app.vala:398: %s", _data_->_tmp18_);
#line 394 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_error_free0 (_data_->_vala1__error_);
#line 3433 "app.c"
	}
	__finally3:
#line 394 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_data_->_inner_error_ != NULL) {
#line 394 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _data_->_inner_error_->message, g_quark_to_string (_data_->_inner_error_->domain), _data_->_inner_error_->code);
#line 394 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_clear_error (&_data_->_inner_error_);
#line 394 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return FALSE;
#line 3444 "app.c"
	}
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp19_ = boxes_app_get_default_connection (_data_->self);
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp20_ = _data_->_tmp19_;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp21_ = _data_->_tmp20_;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp22_ = _data_->_tmp21_ == NULL;
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_data_->_tmp21_);
#line 400 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_data_->_tmp22_) {
#line 401 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_printerr ("Missing or failing default libvirt connection\n");
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data_->_tmp23_ = _data_->self->priv->application;
#line 402 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_application_release ((GApplication*) _data_->_tmp23_);
#line 3464 "app.c"
	}
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp24_ = NULL;
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp24_ = boxes_get_user_pkgconfig_source (NULL);
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp25_ = _data_->_tmp24_;
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp26_ = NULL;
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp26_ = g_file_new_for_path (_data_->_tmp25_);
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_tmp27_ = _data_->_tmp26_;
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (_data_->_tmp25_);
#line 405 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->dir = _data_->_tmp27_;
#line 406 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data_->_state_ = 3;
#line 406 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_foreach_filename_from_dir (_data_->dir, ___lambda195__boxes_foreach_filename_from_dir_func, _data_->self, boxes_app_setup_sources_ready, _data_);
#line 406 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return FALSE;
#line 3488 "app.c"
	_state_3:
#line 406 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_foreach_filename_from_dir_finish (_data_->_res_);
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_data_->dir);
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_data_->_state_ == 0) {
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_simple_async_result_complete_in_idle (_data_->_async_result);
#line 3498 "app.c"
	} else {
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_simple_async_result_complete (_data_->_async_result);
#line 3502 "app.c"
	}
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_unref (_data_->_async_result);
#line 382 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return FALSE;
#line 3508 "app.c"
}


static GVariant* _variant_new1 (gint* value, gint value_length1) {
	gint* _tmp10_;
	GVariantBuilder _tmp11_;
	int _tmp12_;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp10_ = value;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_variant_builder_init (&_tmp11_, G_VARIANT_TYPE ("ai"));
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	for (_tmp12_ = 0; _tmp12_ < value_length1; _tmp12_++) {
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_variant_builder_add_value (&_tmp11_, g_variant_new_int32 (*_tmp10_));
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp10_++;
#line 3526 "app.c"
	}
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return g_variant_ref_sink (g_variant_builder_end (&_tmp11_));
#line 3530 "app.c"
}


static GVariant* _variant_new2 (gint* value, gint value_length1) {
	gint* _tmp22_;
	GVariantBuilder _tmp23_;
	int _tmp24_;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp22_ = value;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_variant_builder_init (&_tmp23_, G_VARIANT_TYPE ("ai"));
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	for (_tmp24_ = 0; _tmp24_ < value_length1; _tmp24_++) {
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_variant_builder_add_value (&_tmp23_, g_variant_new_int32 (*_tmp22_));
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp22_++;
#line 3548 "app.c"
	}
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return g_variant_ref_sink (g_variant_builder_end (&_tmp23_));
#line 3552 "app.c"
}


static void boxes_app_save_window_geometry (BoxesApp* self) {
	gint width = 0;
	gint height = 0;
	gint x = 0;
	gint y = 0;
	gboolean _tmp0_;
	gboolean _tmp1_;
	GtkApplicationWindow* _tmp2_;
	gint _tmp3_ = 0;
	gint _tmp4_ = 0;
	GSettings* _tmp5_;
	gint _tmp6_;
	gint _tmp7_;
	gint* _tmp8_ = NULL;
	gint* _tmp9_;
	gint _tmp9__length1;
	GVariant* _tmp13_;
	GtkApplicationWindow* _tmp14_;
	gint _tmp15_ = 0;
	gint _tmp16_ = 0;
	GSettings* _tmp17_;
	gint _tmp18_;
	gint _tmp19_;
	gint* _tmp20_ = NULL;
	gint* _tmp21_;
	gint _tmp21__length1;
	GVariant* _tmp25_;
#line 413 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 416 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = boxes_app_get_maximized (self);
#line 416 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _tmp0_;
#line 416 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp1_) {
#line 417 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return;
#line 3593 "app.c"
	}
#line 419 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = self->window;
#line 419 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_window_get_size ((GtkWindow*) _tmp2_, &_tmp3_, &_tmp4_);
#line 419 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	width = _tmp3_;
#line 419 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	height = _tmp4_;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = self->settings;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = width;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_ = height;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_ = g_new0 (gint, 2);
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_[0] = _tmp6_;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_[1] = _tmp7_;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp9_ = _tmp8_;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp9__length1 = 2;
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp13_ = _variant_new1 (_tmp9_, 2);
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_settings_set_value (_tmp5_, "window-size", _tmp13_);
#line 420 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp9_ = (g_free (_tmp9_), NULL);
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_ = self->window;
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_window_get_position ((GtkWindow*) _tmp14_, &_tmp15_, &_tmp16_);
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	x = _tmp15_;
#line 422 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	y = _tmp16_;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp17_ = self->settings;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp18_ = x;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp19_ = y;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp20_ = g_new0 (gint, 2);
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp20_[0] = _tmp18_;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp20_[1] = _tmp19_;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp21_ = _tmp20_;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp21__length1 = 2;
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp25_ = _variant_new2 (_tmp21_, 2);
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_settings_set_value (_tmp17_, "window-position", _tmp25_);
#line 423 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp21_ = (g_free (_tmp21_), NULL);
#line 3655 "app.c"
}


static gint _variant_get3 (GVariant* value) {
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return g_variant_get_int32 (value);
#line 3662 "app.c"
}


static gint _variant_get4 (GVariant* value) {
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return g_variant_get_int32 (value);
#line 3669 "app.c"
}


static gint _variant_get5 (GVariant* value) {
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return g_variant_get_int32 (value);
#line 3676 "app.c"
}


static gint _variant_get6 (GVariant* value) {
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return g_variant_get_int32 (value);
#line 3683 "app.c"
}


static gboolean __lambda198_ (BoxesApp* self) {
	gboolean result = FALSE;
#line 476 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->configure_id = (guint) 0;
#line 477 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_save_window_geometry (self);
#line 479 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = FALSE;
#line 479 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3697 "app.c"
}


static gboolean ___lambda198__gsource_func (gpointer self) {
	gboolean result;
	result = __lambda198_ (self);
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3706 "app.c"
}


static gboolean __lambda197_ (BoxesApp* self) {
	gboolean result = FALSE;
	gboolean _tmp0_;
	gboolean _tmp1_;
	guint _tmp2_;
	guint _tmp4_ = 0U;
#line 470 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = boxes_app_get_fullscreen (self);
#line 470 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _tmp0_;
#line 470 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp1_) {
#line 471 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		result = FALSE;
#line 471 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return result;
#line 3726 "app.c"
	}
#line 473 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = self->priv->configure_id;
#line 473 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp2_ != ((guint) 0)) {
#line 3732 "app.c"
		guint _tmp3_;
#line 474 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp3_ = self->priv->configure_id;
#line 474 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_source_remove (_tmp3_);
#line 3738 "app.c"
	}
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = g_timeout_add_full (G_PRIORITY_DEFAULT, BOXES_APP_configure_id_timeout, ___lambda198__gsource_func, g_object_ref (self), g_object_unref);
#line 475 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->configure_id = _tmp4_;
#line 482 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = FALSE;
#line 482 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3748 "app.c"
}


static gboolean ___lambda197__gtk_widget_configure_event (GtkWidget* _sender, GdkEventConfigure* event, gpointer self) {
	gboolean result;
	result = __lambda197_ (self);
#line 469 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3757 "app.c"
}


static gboolean __lambda199_ (BoxesApp* self, GdkEventWindowState* event) {
	gboolean result = FALSE;
	GdkEventWindowState _tmp0_;
	GdkWindowState _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	GSettings* _tmp4_;
	gboolean _tmp5_;
	gboolean _tmp6_;
#line 484 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 485 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = *event;
#line 485 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _tmp0_.changed_mask;
#line 485 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if ((_tmp1_ & GDK_WINDOW_STATE_FULLSCREEN) == GDK_WINDOW_STATE_FULLSCREEN) {
#line 486 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_object_notify ((GObject*) self, "fullscreen");
#line 3780 "app.c"
	}
#line 488 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = boxes_app_get_fullscreen (self);
#line 488 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _tmp2_;
#line 488 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp3_) {
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		result = FALSE;
#line 489 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return result;
#line 3792 "app.c"
	}
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = self->settings;
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = boxes_app_get_maximized (self);
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = _tmp5_;
#line 491 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_settings_set_boolean (_tmp4_, "window-maximized", _tmp6_);
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = FALSE;
#line 492 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3806 "app.c"
}


static gboolean ___lambda199__gtk_widget_window_state_event (GtkWidget* _sender, GdkEventWindowState* event, gpointer self) {
	gboolean result;
	result = __lambda199_ (self, event);
#line 484 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3815 "app.c"
}


static gboolean __lambda200_ (BoxesApp* self) {
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
#line 508 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = boxes_app_quit (self);
#line 508 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = _tmp0_;
#line 508 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3828 "app.c"
}


static gboolean ___lambda200__gtk_widget_delete_event (GtkWidget* _sender, GdkEventAny* event, gpointer self) {
	gboolean result;
	result = __lambda200_ (self);
#line 508 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3837 "app.c"
}


static gboolean _boxes_app_on_key_pressed_gtk_widget_key_press_event (GtkWidget* _sender, GdkEventKey* event, gpointer self) {
	gboolean result;
	result = boxes_app_on_key_pressed (self, _sender, event);
#line 510 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 3846 "app.c"
}


static void boxes_app_setup_ui (BoxesApp* self) {
	gboolean _tmp0_;
	GtkSettings* _tmp1_ = NULL;
	GtkCssProvider* _tmp2_;
	GtkCssProvider* provider;
	BoxesApplication* _tmp10_;
	GtkApplicationWindow* _tmp11_;
	GtkApplicationWindow* _tmp12_;
	GtkApplicationWindow* _tmp13_;
	GSettings* _tmp14_;
	GVariant* _tmp15_ = NULL;
	GVariant* size;
	GVariant* _tmp16_;
	gsize _tmp17_ = 0UL;
	GSettings* _tmp31_;
	gboolean _tmp32_ = FALSE;
	GSettings* _tmp34_;
	GVariant* _tmp35_ = NULL;
	GVariant* position;
	GVariant* _tmp36_;
	gsize _tmp37_ = 0UL;
	GtkApplicationWindow* _tmp51_;
	GtkApplicationWindow* _tmp52_;
	GtkNotebook* _tmp53_;
	GtkNotebook* _tmp54_;
	GtkNotebook* _tmp55_;
	GtkApplicationWindow* _tmp56_;
	GtkNotebook* _tmp57_;
	BoxesClutterWidget* _tmp58_;
	GtkNotebook* _tmp59_;
	BoxesClutterWidget* _tmp60_;
	BoxesDisplayPage* _tmp61_;
	GtkNotebook* _tmp62_;
	BoxesDisplayPage* _tmp63_;
	GtkWidget* _tmp64_;
	GtkWidget* _tmp65_;
	BoxesClutterWidget* _tmp66_;
	ClutterActor* _tmp67_ = NULL;
	ClutterStage* _tmp68_;
	ClutterStage* _tmp69_;
	GdkRGBA _tmp70_ = {0};
	ClutterColor _tmp71_ = {0};
	GtkApplicationWindow* _tmp72_;
	GtkApplicationWindow* _tmp73_;
	ClutterBinLayout* _tmp74_;
	ClutterStage* _tmp75_;
	ClutterBinLayout* _tmp76_;
	ClutterStage* _tmp77_;
	GtkClutterTexture* _tmp78_;
	GtkClutterTexture* background;
	GtkClutterTexture* _tmp79_;
	GtkClutterTexture* _tmp86_;
	GtkClutterTexture* _tmp87_;
	GtkClutterTexture* _tmp88_;
	GtkClutterTexture* _tmp89_;
	GtkClutterTexture* _tmp90_;
	ClutterStage* _tmp91_;
	GtkClutterTexture* _tmp92_;
	BoxesSidebar* _tmp93_;
	BoxesCollectionView* _tmp94_;
	BoxesSearchbar* _tmp95_;
	BoxesTopbar* _tmp96_;
	BoxesNotificationbar* _tmp97_;
	BoxesSelectionbar* _tmp98_;
	BoxesWizard* _tmp99_;
	BoxesProperties* _tmp100_;
	BoxesEmptyBoxes* _tmp101_;
	ClutterActor* _tmp102_;
	ClutterActor* vbox_actor;
	ClutterActor* _tmp103_;
	ClutterBoxLayout* _tmp104_;
	ClutterBoxLayout* vbox;
	ClutterActor* _tmp105_;
	ClutterBoxLayout* _tmp106_;
	ClutterBoxLayout* _tmp107_;
	ClutterActor* _tmp108_;
	ClutterActor* _tmp109_;
	ClutterActor* _tmp110_;
	ClutterActor* _tmp111_;
	ClutterStage* _tmp112_;
	ClutterActor* _tmp113_;
	BoxesRevealer* _tmp114_;
	BoxesRevealer* topbar_revealer;
	BoxesRevealer* _tmp115_;
	BoxesRevealer* _tmp116_;
	ClutterActor* _tmp117_;
	BoxesRevealer* _tmp118_;
	BoxesRevealer* _tmp119_;
	BoxesTopbar* _tmp120_;
	ClutterActor* _tmp121_;
	ClutterActor* _tmp122_;
	BoxesRevealer* _tmp123_;
	BoxesRevealer* _tmp124_;
	BoxesRevealer* _tmp125_;
	BoxesRevealer* _tmp126_;
	BoxesRevealer* _tmp127_;
	ClutterActor* _tmp128_;
	BoxesRevealer* _tmp129_;
	BoxesRevealer* _tmp130_;
	BoxesSearchbar* _tmp131_;
	ClutterActor* _tmp132_;
	ClutterActor* _tmp133_;
	ClutterActor* _tmp134_;
	ClutterActor* below_bin_actor;
	ClutterActor* _tmp135_;
	ClutterBinLayout* _tmp136_;
	ClutterBinLayout* below_bin;
	ClutterActor* _tmp137_;
	ClutterBinLayout* _tmp138_;
	ClutterActor* _tmp139_;
	ClutterActor* _tmp140_;
	ClutterActor* _tmp141_;
	ClutterActor* _tmp142_;
	ClutterActor* _tmp143_;
	BoxesCollectionView* _tmp144_;
	ClutterActor* _tmp145_;
	ClutterActor* _tmp146_;
	ClutterActor* _tmp147_;
	ClutterActor* hbox_actor;
	ClutterActor* _tmp148_;
	ClutterBoxLayout* _tmp149_;
	ClutterBoxLayout* hbox;
	ClutterActor* _tmp150_;
	ClutterBoxLayout* _tmp151_;
	ClutterActor* _tmp152_;
	ClutterActor* _tmp153_;
	ClutterActor* _tmp154_;
	ClutterActor* _tmp155_;
	ClutterActor* _tmp156_;
	ClutterActor* _tmp157_;
	ClutterActor* _tmp158_;
	ClutterActor* _tmp159_;
	ClutterBinLayout* _tmp160_;
	ClutterActor* _tmp161_;
	ClutterBinLayout* _tmp162_;
	ClutterActor* _tmp163_;
	ClutterActor* _tmp164_;
	ClutterActor* _tmp165_;
	ClutterActor* _tmp166_;
	ClutterActor* _tmp167_;
	ClutterActor* _tmp168_;
	BoxesRevealer* _tmp169_;
	BoxesRevealer* _tmp170_;
	BoxesRevealer* _tmp171_;
	ClutterActor* _tmp172_;
	BoxesRevealer* _tmp173_;
	BoxesRevealer* _tmp174_;
	BoxesRevealer* _tmp175_;
	BoxesSidebar* _tmp176_;
	ClutterActor* _tmp177_;
	ClutterActor* _tmp178_;
	ClutterActor* _tmp179_;
	ClutterActor* content_bin_actor;
	ClutterActor* _tmp180_;
	ClutterActor* _tmp181_;
	ClutterActor* _tmp182_;
	ClutterActor* _tmp183_;
	ClutterActor* _tmp184_;
	ClutterBinLayout* _tmp185_;
	ClutterBinLayout* content_bin;
	ClutterActor* _tmp186_;
	ClutterBinLayout* _tmp187_;
	ClutterActor* _tmp188_;
	ClutterActor* _tmp189_;
	ClutterActor* _tmp190_;
	BoxesNotificationbar* _tmp191_;
	ClutterActor* _tmp192_;
	ClutterActor* _tmp193_;
	ClutterActor* _tmp194_;
	BoxesSelectionbar* _tmp195_;
	ClutterActor* _tmp196_;
	ClutterActor* _tmp197_;
	ClutterActor* _tmp198_;
	BoxesWizard* _tmp199_;
	ClutterActor* _tmp200_;
	ClutterActor* _tmp201_;
	ClutterActor* _tmp202_;
	BoxesProperties* _tmp203_;
	ClutterActor* _tmp204_;
	ClutterActor* _tmp205_;
	ClutterActor* _tmp206_;
	BoxesEmptyBoxes* _tmp207_;
	ClutterActor* _tmp208_;
	ClutterActor* _tmp209_;
	BoxesProperties* _tmp210_;
	ClutterActor* _tmp211_;
	ClutterActor* _tmp212_;
	BoxesSelectionbar* _tmp213_;
	ClutterActor* _tmp214_;
	ClutterActor* _tmp215_;
	BoxesEmptyBoxes* _tmp216_;
	ClutterActor* _tmp217_;
	ClutterActor* _tmp218_;
	GtkNotebook* _tmp219_;
	GError * _inner_error_ = NULL;
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->priv->ui_is_setup;
#line 428 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp0_) {
#line 429 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return;
#line 4053 "app.c"
	}
#line 430 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->ui_is_setup = TRUE;
#line 431 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_window_set_default_icon_name ("gnome-boxes");
#line 432 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = gtk_settings_get_default ();
#line 432 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_set (_tmp1_, "gtk-application-prefer-dark-theme", TRUE, NULL);
#line 434 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = gtk_css_provider_new ();
#line 434 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	provider = _tmp2_;
#line 4067 "app.c"
	{
		gchar* _tmp3_ = NULL;
		gchar* sheet;
		GtkCssProvider* _tmp4_;
		const gchar* _tmp5_;
		GdkScreen* _tmp6_ = NULL;
		GtkCssProvider* _tmp7_;
#line 436 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp3_ = boxes_get_style ("gtk-style.css");
#line 436 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		sheet = _tmp3_;
#line 437 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp4_ = provider;
#line 437 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp5_ = sheet;
#line 437 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		gtk_css_provider_load_from_path (_tmp4_, _tmp5_, &_inner_error_);
#line 437 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_inner_error_ != NULL) {
#line 437 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_free0 (sheet);
#line 4089 "app.c"
			goto __catch4_g_error;
		}
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp6_ = gdk_screen_get_default ();
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp7_ = provider;
#line 438 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		gtk_style_context_add_provider_for_screen (_tmp6_, (GtkStyleProvider*) _tmp7_, (guint) 600);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (sheet);
#line 4100 "app.c"
	}
	goto __finally4;
	__catch4_g_error:
	{
		GError* _error_ = NULL;
		GError* _tmp8_;
		const gchar* _tmp9_;
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_error_ = _inner_error_;
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_inner_error_ = NULL;
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp8_ = _error_;
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp9_ = _tmp8_->message;
#line 442 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_warning ("app.vala:442: %s", _tmp9_);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_error_free0 (_error_);
#line 4120 "app.c"
	}
	__finally4:
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_inner_error_ != NULL) {
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (provider);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_clear_error (&_inner_error_);
#line 435 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return;
#line 4133 "app.c"
	}
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp10_ = self->priv->application;
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp11_ = (GtkApplicationWindow*) gtk_application_window_new ((GtkApplication*) _tmp10_);
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp11_);
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->window);
#line 445 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->window = _tmp11_;
#line 446 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp12_ = self->window;
#line 446 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_application_window_set_show_menubar (_tmp12_, FALSE);
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp13_ = self->window;
#line 447 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_window_set_hide_titlebar_when_maximized ((GtkWindow*) _tmp13_, TRUE);
#line 450 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_ = self->settings;
#line 450 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp15_ = g_settings_get_value (_tmp14_, "window-size");
#line 450 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	size = _tmp15_;
#line 451 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp16_ = size;
#line 451 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp17_ = g_variant_n_children (_tmp16_);
#line 451 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp17_ == ((gsize) 2)) {
#line 4165 "app.c"
		GVariant* _tmp18_;
		GVariant* _tmp19_ = NULL;
		GVariant* _tmp20_;
		gint _tmp21_;
		gint _tmp22_;
		gint width;
		GVariant* _tmp23_;
		GVariant* _tmp24_ = NULL;
		GVariant* _tmp25_;
		gint _tmp26_;
		gint _tmp27_;
		gint height;
		GtkApplicationWindow* _tmp28_;
		gint _tmp29_;
		gint _tmp30_;
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp18_ = size;
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp19_ = g_variant_get_child_value (_tmp18_, (gsize) 0);
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp20_ = _tmp19_;
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp21_ = _variant_get3 (_tmp20_);
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp22_ = _tmp21_;
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_variant_unref0 (_tmp20_);
#line 452 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		width = _tmp22_;
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp23_ = size;
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp24_ = g_variant_get_child_value (_tmp23_, (gsize) 1);
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp25_ = _tmp24_;
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp26_ = _variant_get4 (_tmp25_);
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp27_ = _tmp26_;
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_variant_unref0 (_tmp25_);
#line 453 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		height = _tmp27_;
#line 455 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp28_ = self->window;
#line 455 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp29_ = width;
#line 455 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp30_ = height;
#line 455 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		gtk_window_set_default_size ((GtkWindow*) _tmp28_, _tmp29_, _tmp30_);
#line 4217 "app.c"
	}
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp31_ = self->settings;
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp32_ = g_settings_get_boolean (_tmp31_, "window-maximized");
#line 458 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp32_) {
#line 4225 "app.c"
		GtkApplicationWindow* _tmp33_;
#line 459 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp33_ = self->window;
#line 459 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		gtk_window_maximize ((GtkWindow*) _tmp33_);
#line 4231 "app.c"
	}
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp34_ = self->settings;
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp35_ = g_settings_get_value (_tmp34_, "window-position");
#line 461 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	position = _tmp35_;
#line 462 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp36_ = position;
#line 462 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp37_ = g_variant_n_children (_tmp36_);
#line 462 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp37_ == ((gsize) 2)) {
#line 4245 "app.c"
		GVariant* _tmp38_;
		GVariant* _tmp39_ = NULL;
		GVariant* _tmp40_;
		gint _tmp41_;
		gint _tmp42_;
		gint x;
		GVariant* _tmp43_;
		GVariant* _tmp44_ = NULL;
		GVariant* _tmp45_;
		gint _tmp46_;
		gint _tmp47_;
		gint y;
		GtkApplicationWindow* _tmp48_;
		gint _tmp49_;
		gint _tmp50_;
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp38_ = position;
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp39_ = g_variant_get_child_value (_tmp38_, (gsize) 0);
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp40_ = _tmp39_;
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp41_ = _variant_get5 (_tmp40_);
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp42_ = _tmp41_;
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_variant_unref0 (_tmp40_);
#line 463 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		x = _tmp42_;
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp43_ = position;
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp44_ = g_variant_get_child_value (_tmp43_, (gsize) 1);
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp45_ = _tmp44_;
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp46_ = _variant_get6 (_tmp45_);
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp47_ = _tmp46_;
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_variant_unref0 (_tmp45_);
#line 464 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		y = _tmp47_;
#line 466 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp48_ = self->window;
#line 466 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp49_ = x;
#line 466 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp50_ = y;
#line 466 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		gtk_window_move ((GtkWindow*) _tmp48_, _tmp49_, _tmp50_);
#line 4297 "app.c"
	}
#line 469 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp51_ = self->window;
#line 469 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object ((GtkWidget*) _tmp51_, "configure-event", (GCallback) ___lambda197__gtk_widget_configure_event, self, 0);
#line 484 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp52_ = self->window;
#line 484 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object ((GtkWidget*) _tmp52_, "window-state-event", (GCallback) ___lambda199__gtk_widget_window_state_event, self, 0);
#line 495 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp53_ = (GtkNotebook*) gtk_notebook_new ();
#line 495 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp53_);
#line 495 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->notebook);
#line 495 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->notebook = _tmp53_;
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp54_ = self->notebook;
#line 496 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_notebook_set_show_border (_tmp54_, FALSE);
#line 497 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp55_ = self->notebook;
#line 497 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_notebook_set_show_tabs (_tmp55_, FALSE);
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp56_ = self->window;
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp57_ = self->notebook;
#line 498 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_container_add ((GtkContainer*) _tmp56_, (GtkWidget*) _tmp57_);
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp58_ = boxes_clutter_widget_new ();
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp58_);
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->embed);
#line 499 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->embed = _tmp58_;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp59_ = self->notebook;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp60_ = self->embed;
#line 500 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_notebook_append_page (_tmp59_, (GtkWidget*) _tmp60_, NULL);
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp61_ = boxes_display_page_new ();
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->display_page);
#line 502 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->display_page = _tmp61_;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp62_ = self->notebook;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp63_ = self->display_page;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp64_ = boxes_display_page_get_widget (_tmp63_);
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp65_ = _tmp64_;
#line 503 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_notebook_append_page (_tmp62_, _tmp65_, NULL);
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp66_ = self->embed;
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp67_ = gtk_clutter_embed_get_stage ((GtkClutterEmbed*) _tmp66_);
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp68_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp67_, clutter_stage_get_type ()) ? ((ClutterStage*) _tmp67_) : NULL);
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->stage);
#line 505 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->stage = _tmp68_;
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp69_ = self->stage;
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_get_boxes_bg_color (&_tmp70_);
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_gdk_rgba_to_clutter_color (&_tmp70_, &_tmp71_);
#line 506 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_background_color ((ClutterActor*) _tmp69_, &_tmp71_);
#line 508 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp72_ = self->window;
#line 508 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object ((GtkWidget*) _tmp72_, "delete-event", (GCallback) ___lambda200__gtk_widget_delete_event, self, 0);
#line 510 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp73_ = self->window;
#line 510 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_connect_object ((GtkWidget*) _tmp73_, "key-press-event", (GCallback) _boxes_app_on_key_pressed_gtk_widget_key_press_event, self, G_CONNECT_AFTER);
#line 512 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp74_ = (ClutterBinLayout*) clutter_bin_layout_new (CLUTTER_BIN_ALIGNMENT_START, CLUTTER_BIN_ALIGNMENT_START);
#line 512 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp74_);
#line 512 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->stage_bin);
#line 512 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->stage_bin = _tmp74_;
#line 514 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp75_ = self->stage;
#line 514 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp76_ = self->stage_bin;
#line 514 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_layout_manager ((ClutterActor*) _tmp75_, (ClutterLayoutManager*) _tmp76_);
#line 515 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp77_ = self->stage;
#line 515 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name ((ClutterActor*) _tmp77_, "boxes-stage");
#line 517 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp78_ = (GtkClutterTexture*) gtk_clutter_texture_new ();
#line 517 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp78_);
#line 517 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	background = _tmp78_;
#line 518 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp79_ = background;
#line 518 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name ((ClutterActor*) _tmp79_, "background");
#line 4413 "app.c"
	{
		gchar* _tmp80_ = NULL;
		gchar* _tmp81_;
		GdkPixbuf* _tmp82_;
		GdkPixbuf* _tmp83_;
		GdkPixbuf* pixbuf;
		GtkClutterTexture* _tmp84_;
		GdkPixbuf* _tmp85_;
#line 520 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp80_ = boxes_get_style ("assets/boxes-dark.png");
#line 520 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp81_ = _tmp80_;
#line 520 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp82_ = gdk_pixbuf_new_from_file (_tmp81_, &_inner_error_);
#line 520 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp83_ = _tmp82_;
#line 520 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_tmp81_);
#line 520 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		pixbuf = _tmp83_;
#line 520 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_inner_error_ != NULL) {
#line 4436 "app.c"
			goto __catch5_g_error;
		}
#line 521 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp84_ = background;
#line 521 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp85_ = pixbuf;
#line 521 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		gtk_clutter_texture_set_from_pixbuf (_tmp84_, _tmp85_, &_inner_error_);
#line 521 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_inner_error_ != NULL) {
#line 521 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_object_unref0 (pixbuf);
#line 4449 "app.c"
			goto __catch5_g_error;
		}
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (pixbuf);
#line 4454 "app.c"
	}
	goto __finally5;
	__catch5_g_error:
	{
		GError* e = NULL;
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		e = _inner_error_;
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_inner_error_ = NULL;
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_error_free0 (e);
#line 4466 "app.c"
	}
	__finally5:
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_inner_error_ != NULL) {
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (background);
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_variant_unref0 (position);
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_variant_unref0 (size);
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (provider);
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_clear_error (&_inner_error_);
#line 519 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return;
#line 4485 "app.c"
	}
#line 524 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp86_ = background;
#line 524 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_texture_set_repeat ((ClutterTexture*) _tmp86_, TRUE, TRUE);
#line 525 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp87_ = background;
#line 525 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_align ((ClutterActor*) _tmp87_, CLUTTER_ACTOR_ALIGN_FILL);
#line 526 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp88_ = background;
#line 526 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_align ((ClutterActor*) _tmp88_, CLUTTER_ACTOR_ALIGN_FILL);
#line 527 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp89_ = background;
#line 527 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_expand ((ClutterActor*) _tmp89_, TRUE);
#line 528 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp90_ = background;
#line 528 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_expand ((ClutterActor*) _tmp90_, TRUE);
#line 529 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp91_ = self->stage;
#line 529 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp92_ = background;
#line 529 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child ((ClutterActor*) _tmp91_, (ClutterActor*) _tmp92_);
#line 531 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp93_ = boxes_sidebar_new ();
#line 531 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->sidebar);
#line 531 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->sidebar = _tmp93_;
#line 532 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp94_ = boxes_collection_view_new ();
#line 532 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->view);
#line 532 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->view = _tmp94_;
#line 533 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp95_ = boxes_searchbar_new ();
#line 533 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->searchbar);
#line 533 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->searchbar = _tmp95_;
#line 534 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp96_ = boxes_topbar_new ();
#line 534 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->topbar);
#line 534 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->topbar = _tmp96_;
#line 535 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp97_ = boxes_notificationbar_new ();
#line 535 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->notificationbar);
#line 535 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->notificationbar = _tmp97_;
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp98_ = boxes_selectionbar_new ();
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->selectionbar);
#line 536 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->selectionbar = _tmp98_;
#line 537 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp99_ = boxes_wizard_new ();
#line 537 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->wizard);
#line 537 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->wizard = _tmp99_;
#line 538 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp100_ = boxes_properties_new ();
#line 538 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->properties);
#line 538 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->properties = _tmp100_;
#line 539 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp101_ = boxes_empty_boxes_new ();
#line 539 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->empty_boxes);
#line 539 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->empty_boxes = _tmp101_;
#line 541 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp102_ = clutter_actor_new ();
#line 541 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp102_);
#line 541 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	vbox_actor = _tmp102_;
#line 542 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp103_ = vbox_actor;
#line 542 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name (_tmp103_, "top-vbox");
#line 543 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp104_ = (ClutterBoxLayout*) clutter_box_layout_new ();
#line 543 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp104_);
#line 543 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	vbox = _tmp104_;
#line 544 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp105_ = vbox_actor;
#line 544 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp106_ = vbox;
#line 544 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_layout_manager (_tmp105_, (ClutterLayoutManager*) _tmp106_);
#line 545 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp107_ = vbox;
#line 545 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_box_layout_set_orientation (_tmp107_, CLUTTER_ORIENTATION_VERTICAL);
#line 546 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp108_ = vbox_actor;
#line 546 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_align (_tmp108_, CLUTTER_ACTOR_ALIGN_FILL);
#line 547 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp109_ = vbox_actor;
#line 547 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_align (_tmp109_, CLUTTER_ACTOR_ALIGN_FILL);
#line 548 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp110_ = vbox_actor;
#line 548 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_expand (_tmp110_, TRUE);
#line 549 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp111_ = vbox_actor;
#line 549 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_expand (_tmp111_, TRUE);
#line 551 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp112_ = self->stage;
#line 551 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp113_ = vbox_actor;
#line 551 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child ((ClutterActor*) _tmp112_, _tmp113_);
#line 553 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp114_ = boxes_revealer_new (TRUE);
#line 553 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp114_);
#line 553 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	topbar_revealer = _tmp114_;
#line 554 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp115_ = topbar_revealer;
#line 554 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name ((ClutterActor*) _tmp115_, "topbar-revealer");
#line 555 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp116_ = topbar_revealer;
#line 555 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_expand ((ClutterActor*) _tmp116_, TRUE);
#line 556 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp117_ = vbox_actor;
#line 556 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp118_ = topbar_revealer;
#line 556 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp117_, (ClutterActor*) _tmp118_);
#line 557 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp119_ = topbar_revealer;
#line 557 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp120_ = self->topbar;
#line 557 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp121_ = boxes_ui_get_actor ((BoxesUI*) _tmp120_);
#line 557 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp122_ = _tmp121_;
#line 557 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_container_add ((ClutterContainer*) _tmp119_, _tmp122_, NULL);
#line 559 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp123_ = boxes_revealer_new (TRUE);
#line 559 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp123_);
#line 559 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->searchbar_revealer);
#line 559 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->searchbar_revealer = _tmp123_;
#line 560 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp124_ = self->searchbar_revealer;
#line 560 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp124_->resize = TRUE;
#line 561 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp125_ = self->searchbar_revealer;
#line 561 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_revealer_unreveal (_tmp125_);
#line 562 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp126_ = self->searchbar_revealer;
#line 562 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name ((ClutterActor*) _tmp126_, "searchbar-revealer");
#line 563 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp127_ = self->searchbar_revealer;
#line 563 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_expand ((ClutterActor*) _tmp127_, TRUE);
#line 564 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp128_ = vbox_actor;
#line 564 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp129_ = self->searchbar_revealer;
#line 564 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp128_, (ClutterActor*) _tmp129_);
#line 565 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp130_ = self->searchbar_revealer;
#line 565 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp131_ = self->searchbar;
#line 565 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp132_ = boxes_ui_get_actor ((BoxesUI*) _tmp131_);
#line 565 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp133_ = _tmp132_;
#line 565 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_container_add ((ClutterContainer*) _tmp130_, _tmp133_, NULL);
#line 567 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp134_ = clutter_actor_new ();
#line 567 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp134_);
#line 567 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	below_bin_actor = _tmp134_;
#line 568 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp135_ = below_bin_actor;
#line 568 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name (_tmp135_, "below-bin");
#line 569 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp136_ = (ClutterBinLayout*) clutter_bin_layout_new (CLUTTER_BIN_ALIGNMENT_START, CLUTTER_BIN_ALIGNMENT_START);
#line 569 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp136_);
#line 569 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	below_bin = _tmp136_;
#line 571 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp137_ = below_bin_actor;
#line 571 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp138_ = below_bin;
#line 571 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_layout_manager (_tmp137_, (ClutterLayoutManager*) _tmp138_);
#line 573 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp139_ = below_bin_actor;
#line 573 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_expand (_tmp139_, TRUE);
#line 574 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp140_ = below_bin_actor;
#line 574 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_expand (_tmp140_, TRUE);
#line 575 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp141_ = vbox_actor;
#line 575 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp142_ = below_bin_actor;
#line 575 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp141_, _tmp142_);
#line 577 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp143_ = below_bin_actor;
#line 577 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp144_ = self->view;
#line 577 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp145_ = boxes_ui_get_actor ((BoxesUI*) _tmp144_);
#line 577 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp146_ = _tmp145_;
#line 577 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp143_, _tmp146_);
#line 579 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp147_ = clutter_actor_new ();
#line 579 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp147_);
#line 579 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	hbox_actor = _tmp147_;
#line 580 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp148_ = hbox_actor;
#line 580 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name (_tmp148_, "top-hbox");
#line 581 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp149_ = (ClutterBoxLayout*) clutter_box_layout_new ();
#line 581 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp149_);
#line 581 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	hbox = _tmp149_;
#line 582 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp150_ = hbox_actor;
#line 582 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp151_ = hbox;
#line 582 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_layout_manager (_tmp150_, (ClutterLayoutManager*) _tmp151_);
#line 583 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp152_ = hbox_actor;
#line 583 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_align (_tmp152_, CLUTTER_ACTOR_ALIGN_FILL);
#line 584 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp153_ = hbox_actor;
#line 584 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_align (_tmp153_, CLUTTER_ACTOR_ALIGN_FILL);
#line 585 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp154_ = hbox_actor;
#line 585 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_expand (_tmp154_, TRUE);
#line 586 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp155_ = hbox_actor;
#line 586 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_expand (_tmp155_, TRUE);
#line 588 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp156_ = below_bin_actor;
#line 588 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp157_ = hbox_actor;
#line 588 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp156_, _tmp157_);
#line 590 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp158_ = clutter_actor_new ();
#line 590 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp158_);
#line 590 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->overlay_bin_actor);
#line 590 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->overlay_bin_actor = _tmp158_;
#line 591 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp159_ = self->overlay_bin_actor;
#line 591 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name (_tmp159_, "overlay-bin");
#line 592 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp160_ = (ClutterBinLayout*) clutter_bin_layout_new (CLUTTER_BIN_ALIGNMENT_START, CLUTTER_BIN_ALIGNMENT_START);
#line 592 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp160_);
#line 592 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->overlay_bin);
#line 592 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->overlay_bin = _tmp160_;
#line 594 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp161_ = self->overlay_bin_actor;
#line 594 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp162_ = self->overlay_bin;
#line 594 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_layout_manager (_tmp161_, (ClutterLayoutManager*) _tmp162_);
#line 595 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp163_ = self->overlay_bin_actor;
#line 595 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_align (_tmp163_, CLUTTER_ACTOR_ALIGN_FILL);
#line 596 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp164_ = self->overlay_bin_actor;
#line 596 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_align (_tmp164_, CLUTTER_ACTOR_ALIGN_FILL);
#line 597 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp165_ = self->overlay_bin_actor;
#line 597 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_expand (_tmp165_, TRUE);
#line 598 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp166_ = self->overlay_bin_actor;
#line 598 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_expand (_tmp166_, TRUE);
#line 599 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp167_ = below_bin_actor;
#line 599 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp168_ = self->overlay_bin_actor;
#line 599 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp167_, _tmp168_);
#line 601 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp169_ = boxes_revealer_new (FALSE);
#line 601 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp169_);
#line 601 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->sidebar_revealer);
#line 601 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->sidebar_revealer = _tmp169_;
#line 602 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp170_ = self->sidebar_revealer;
#line 602 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name ((ClutterActor*) _tmp170_, "sidebar-revealer");
#line 603 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp171_ = self->sidebar_revealer;
#line 603 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_expand ((ClutterActor*) _tmp171_, TRUE);
#line 604 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp172_ = hbox_actor;
#line 604 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp173_ = self->sidebar_revealer;
#line 604 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp172_, (ClutterActor*) _tmp173_);
#line 605 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp174_ = self->sidebar_revealer;
#line 605 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_revealer_unreveal (_tmp174_);
#line 606 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp175_ = self->sidebar_revealer;
#line 606 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp176_ = self->sidebar;
#line 606 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp177_ = boxes_ui_get_actor ((BoxesUI*) _tmp176_);
#line 606 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp178_ = _tmp177_;
#line 606 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_container_add ((ClutterContainer*) _tmp175_, _tmp178_, NULL);
#line 608 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp179_ = clutter_actor_new ();
#line 608 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp179_);
#line 608 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	content_bin_actor = _tmp179_;
#line 609 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp180_ = content_bin_actor;
#line 609 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_name (_tmp180_, "content-bin");
#line 610 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp181_ = content_bin_actor;
#line 610 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_align (_tmp181_, CLUTTER_ACTOR_ALIGN_FILL);
#line 611 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp182_ = content_bin_actor;
#line 611 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_align (_tmp182_, CLUTTER_ACTOR_ALIGN_FILL);
#line 612 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp183_ = content_bin_actor;
#line 612 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_x_expand (_tmp183_, TRUE);
#line 613 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp184_ = content_bin_actor;
#line 613 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_y_expand (_tmp184_, TRUE);
#line 614 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp185_ = (ClutterBinLayout*) clutter_bin_layout_new (CLUTTER_BIN_ALIGNMENT_START, CLUTTER_BIN_ALIGNMENT_START);
#line 614 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_ref_sink (_tmp185_);
#line 614 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	content_bin = _tmp185_;
#line 616 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp186_ = content_bin_actor;
#line 616 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp187_ = content_bin;
#line 616 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_layout_manager (_tmp186_, (ClutterLayoutManager*) _tmp187_);
#line 617 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp188_ = hbox_actor;
#line 617 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp189_ = content_bin_actor;
#line 617 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp188_, _tmp189_);
#line 619 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp190_ = below_bin_actor;
#line 619 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp191_ = self->notificationbar;
#line 619 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp192_ = boxes_notificationbar_get_actor (_tmp191_);
#line 619 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp193_ = _tmp192_;
#line 619 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp190_, _tmp193_);
#line 620 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp194_ = below_bin_actor;
#line 620 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp195_ = self->selectionbar;
#line 620 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp196_ = boxes_selectionbar_get_actor (_tmp195_);
#line 620 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp197_ = _tmp196_;
#line 620 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_add_child (_tmp194_, _tmp197_);
#line 622 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp198_ = content_bin_actor;
#line 622 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp199_ = self->wizard;
#line 622 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp200_ = boxes_ui_get_actor ((BoxesUI*) _tmp199_);
#line 622 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp201_ = _tmp200_;
#line 622 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_container_add ((ClutterContainer*) _tmp198_, _tmp201_, NULL);
#line 623 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp202_ = content_bin_actor;
#line 623 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp203_ = self->properties;
#line 623 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp204_ = boxes_ui_get_actor ((BoxesUI*) _tmp203_);
#line 623 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp205_ = _tmp204_;
#line 623 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_container_add ((ClutterContainer*) _tmp202_, _tmp205_, NULL);
#line 624 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp206_ = below_bin_actor;
#line 624 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp207_ = self->empty_boxes;
#line 624 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp208_ = boxes_ui_get_actor ((BoxesUI*) _tmp207_);
#line 624 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp209_ = _tmp208_;
#line 624 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_insert_child_below (_tmp206_, _tmp209_, NULL);
#line 626 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp210_ = self->properties;
#line 626 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp211_ = boxes_ui_get_actor ((BoxesUI*) _tmp210_);
#line 626 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp212_ = _tmp211_;
#line 626 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_hide (_tmp212_);
#line 627 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp213_ = self->selectionbar;
#line 627 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp214_ = boxes_selectionbar_get_actor (_tmp213_);
#line 627 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp215_ = _tmp214_;
#line 627 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_hide (_tmp215_);
#line 628 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp216_ = self->empty_boxes;
#line 628 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp217_ = boxes_ui_get_actor ((BoxesUI*) _tmp216_);
#line 628 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp218_ = _tmp217_;
#line 628 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_hide (_tmp218_);
#line 630 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp219_ = self->notebook;
#line 630 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_widget_show_all ((GtkWidget*) _tmp219_);
#line 632 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_ui_set_ui_state ((BoxesUI*) self, BOXES_UI_STATE_COLLECTION);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (content_bin);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (content_bin_actor);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (hbox);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (hbox_actor);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (below_bin);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (below_bin_actor);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (topbar_revealer);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (vbox);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (vbox_actor);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (background);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_variant_unref0 (position);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_variant_unref0 (size);
#line 427 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (provider);
#line 5009 "app.c"
}


static void boxes_app_set_main_ui_state (BoxesApp* self) {
	GtkNotebook* _tmp0_;
#line 635 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 636 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->notebook;
#line 636 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_set (_tmp0_, "page", (gint) BOXES_APP_PAGE_MAIN, NULL);
#line 5021 "app.c"
}


static void boxes_app_position_item_actor_at_icon (BoxesApp* self, BoxesCollectionItem* item) {
	gfloat item_x = 0.0F;
	gfloat item_y = 0.0F;
	BoxesCollectionView* _tmp0_;
	BoxesCollectionItem* _tmp1_;
	gfloat _tmp2_ = 0.0F;
	gfloat _tmp3_ = 0.0F;
	BoxesCollectionItem* _tmp4_;
	ClutterActor* _tmp5_;
	ClutterActor* _tmp6_;
	ClutterActor* _tmp7_;
	ClutterActor* actor;
	guint _tmp8_ = 0U;
	guint old_duration;
#line 639 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 639 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (item != NULL);
#line 641 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 641 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = item;
#line 641 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_collection_view_get_item_pos (_tmp0_, _tmp1_, &_tmp2_, &_tmp3_);
#line 641 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	item_x = _tmp2_;
#line 641 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	item_y = _tmp3_;
#line 642 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = item;
#line 642 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = boxes_ui_get_actor ((BoxesUI*) _tmp4_);
#line 642 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = _tmp5_;
#line 642 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_ = _g_object_ref0 (_tmp6_);
#line 642 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	actor = _tmp7_;
#line 643 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_ = clutter_actor_get_easing_duration (actor);
#line 643 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	old_duration = _tmp8_;
#line 647 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_easing_duration (actor, (guint) 0);
#line 648 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_set (actor, "fixed-x", item_x, NULL);
#line 649 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_set (actor, "fixed-y", item_y, NULL);
#line 650 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_set (actor, "natural-width", (gfloat) BOXES_MACHINE_SCREENSHOT_WIDTH, NULL);
#line 650 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_set (actor, "min-width", (gfloat) BOXES_MACHINE_SCREENSHOT_WIDTH, NULL);
#line 651 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_easing_duration (actor, old_duration);
#line 639 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (actor);
#line 5081 "app.c"
}


static Block5Data* block5_data_ref (Block5Data* _data5_) {
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_atomic_int_inc (&_data5_->_ref_count_);
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return _data5_;
#line 5090 "app.c"
}


static void block5_data_unref (void * _userdata_) {
	Block5Data* _data5_;
	_data5_ = (Block5Data*) _userdata_;
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (g_atomic_int_dec_and_test (&_data5_->_ref_count_)) {
#line 5099 "app.c"
		BoxesApp * self;
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self = _data5_->self;
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (_data5_->actor);
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (self);
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_slice_free (Block5Data, _data5_);
#line 5109 "app.c"
	}
}


static gboolean __lambda206_ (BoxesApp* self) {
	gboolean result = FALSE;
	BoxesCollectionItem* _tmp0_;
#line 693 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->current_item;
#line 693 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_position_item_actor_at_icon (self, _tmp0_);
#line 694 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = FALSE;
#line 694 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 5125 "app.c"
}


static gboolean ___lambda206__gsource_func (gpointer self) {
	gboolean result;
	result = __lambda206_ (self);
#line 692 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 5134 "app.c"
}


static void ____lambda205_ (BoxesApp* self, GtkAllocation* allocation) {
#line 690 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (allocation != NULL);
#line 692 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_idle_add_full (G_PRIORITY_HIGH, ___lambda206__gsource_func, g_object_ref (self), g_object_unref);
#line 5143 "app.c"
}


static void _____lambda205__gtk_widget_size_allocate (GtkWidget* _sender, GtkAllocation* allocation, gpointer self) {
#line 690 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	____lambda205_ (self, allocation);
#line 5150 "app.c"
}


static void ____lambda207_ (Block5Data* _data5_) {
	BoxesApp * self;
	ClutterActor* _tmp0_;
	gulong _tmp1_;
	BoxesCollectionView* _tmp2_;
	GdMainIconView* _tmp3_;
	gulong _tmp4_;
	gboolean _tmp5_ = FALSE;
	BoxesApp* _tmp6_;
	BoxesUIState _tmp7_;
	BoxesUIState _tmp8_;
	gboolean _tmp14_;
#line 698 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = _data5_->self;
#line 699 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _data5_->actor;
#line 699 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _data5_->completed_id;
#line 699 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_handler_disconnect ((GObject*) _tmp0_, _tmp1_);
#line 700 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = self->view;
#line 700 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _tmp2_->icon_view;
#line 700 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = _data5_->id;
#line 700 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_handler_disconnect ((GObject*) _tmp3_, _tmp4_);
#line 701 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = boxes_app_app;
#line 701 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_ = boxes_ui_get_ui_state ((BoxesUI*) _tmp6_);
#line 701 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_ = _tmp7_;
#line 701 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp8_ == BOXES_UI_STATE_COLLECTION) {
#line 701 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp5_ = TRUE;
#line 5192 "app.c"
	} else {
		BoxesApp* _tmp9_;
		BoxesCollectionItem* _tmp10_;
		ClutterActor* _tmp11_;
		ClutterActor* _tmp12_;
		ClutterActor* _tmp13_;
#line 702 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp9_ = boxes_app_app;
#line 702 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp10_ = _tmp9_->current_item;
#line 702 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp11_ = boxes_ui_get_actor ((BoxesUI*) _tmp10_);
#line 702 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp12_ = _tmp11_;
#line 702 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp13_ = _data5_->actor;
#line 702 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp5_ = _tmp12_ != _tmp13_;
#line 5211 "app.c"
	}
#line 701 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_ = _tmp5_;
#line 701 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp14_) {
#line 5217 "app.c"
		ClutterActor* _tmp15_;
#line 703 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp15_ = _data5_->actor;
#line 703 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_actor_remove (_tmp15_);
#line 5223 "app.c"
	}
}


static void _____lambda207__clutter_actor_transitions_completed (ClutterActor* _sender, gpointer self) {
#line 698 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	____lambda207_ (self);
#line 5231 "app.c"
}


static void boxes_app_real_ui_state_changed (BoxesUI* base) {
	BoxesApp * self;
	BoxesSidebar* _tmp0_;
	BoxesUI* _tmp1_;
	BoxesSearchbar* _tmp2_;
	BoxesUI* _tmp3_;
	BoxesTopbar* _tmp4_;
	BoxesUI* _tmp5_;
	BoxesCollectionView* _tmp6_;
	BoxesUI* _tmp7_;
	BoxesProperties* _tmp8_;
	BoxesUI* _tmp9_;
	BoxesWizard* _tmp10_;
	BoxesUI* _tmp11_;
	BoxesEmptyBoxes* _tmp12_;
	BoxesUI* _tmp13_;
	BoxesUI** _tmp14_ = NULL;
	BoxesUIState _tmp19_;
	BoxesUIState _tmp20_;
	BoxesUIState _tmp21_;
	BoxesUIState _tmp22_;
	BoxesUIState _tmp24_;
	BoxesUIState _tmp25_;
	BoxesCollectionItem* _tmp62_;
#line 654 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = (BoxesApp*) base;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->sidebar;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _g_object_ref0 ((BoxesUI*) _tmp0_);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = self->searchbar;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _g_object_ref0 ((BoxesUI*) _tmp2_);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = self->topbar;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = _g_object_ref0 ((BoxesUI*) _tmp4_);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = self->view;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_ = _g_object_ref0 ((BoxesUI*) _tmp6_);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_ = self->properties;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp9_ = _g_object_ref0 ((BoxesUI*) _tmp8_);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp10_ = self->wizard;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp11_ = _g_object_ref0 ((BoxesUI*) _tmp10_);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp12_ = self->empty_boxes;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp13_ = _g_object_ref0 ((BoxesUI*) _tmp12_);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_ = g_new0 (BoxesUI*, 7 + 1);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_[0] = _tmp1_;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_[1] = _tmp3_;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_[2] = _tmp5_;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_[3] = _tmp7_;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_[4] = _tmp9_;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_[5] = _tmp11_;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_[6] = _tmp13_;
#line 5305 "app.c"
	{
		BoxesUI** ui_collection = NULL;
		gint ui_collection_length1 = 0;
		gint _ui_collection_size_ = 0;
		gint ui_it = 0;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		ui_collection = _tmp14_;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		ui_collection_length1 = 7;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		for (ui_it = 0; ui_it < 7; ui_it = ui_it + 1) {
#line 5317 "app.c"
			BoxesUI* _tmp15_;
			BoxesUI* ui = NULL;
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp15_ = _g_object_ref0 (ui_collection[ui_it]);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			ui = _tmp15_;
#line 5324 "app.c"
			{
				BoxesUI* _tmp16_;
				BoxesUIState _tmp17_;
				BoxesUIState _tmp18_;
#line 658 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp16_ = ui;
#line 658 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp17_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 658 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp18_ = _tmp17_;
#line 658 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_ui_set_ui_state (_tmp16_, _tmp18_);
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (ui);
#line 5339 "app.c"
			}
		}
#line 657 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		ui_collection = (_vala_array_free (ui_collection, ui_collection_length1, (GDestroyNotify) g_object_unref), NULL);
#line 5344 "app.c"
	}
#line 661 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp19_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 661 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp20_ = _tmp19_;
#line 661 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp20_ != BOXES_UI_STATE_DISPLAY) {
#line 662 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_set_main_ui_state (self);
#line 5354 "app.c"
	}
#line 664 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp21_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 664 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp22_ = _tmp21_;
#line 664 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp22_ != BOXES_UI_STATE_COLLECTION) {
#line 5362 "app.c"
		BoxesRevealer* _tmp23_;
#line 665 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp23_ = self->searchbar_revealer;
#line 665 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_revealer_set_revealed (_tmp23_, FALSE);
#line 5368 "app.c"
	}
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp24_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp25_ = _tmp24_;
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	switch (_tmp25_) {
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_UI_STATE_COLLECTION:
#line 5378 "app.c"
		{
			BoxesTopbar* _tmp26_;
			BoxesCollectionItem* _tmp27_;
			BoxesCollectionView* _tmp33_;
			BoxesRevealer* _tmp34_;
			BoxesSearchbar* _tmp35_;
			gboolean _tmp36_;
			gboolean _tmp37_;
			BoxesCollectionItem* _tmp38_;
#line 669 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp26_ = self->topbar;
#line 669 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			boxes_topbar_set_status (_tmp26_, NULL);
#line 670 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_object_unref0 (self->priv->status_bind);
#line 670 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			self->priv->status_bind = NULL;
#line 671 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp27_ = self->current_item;
#line 671 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp27_, BOXES_TYPE_MACHINE)) {
#line 5400 "app.c"
				BoxesCollectionItem* _tmp28_;
				BoxesMachine* _tmp29_;
				BoxesMachine* machine;
				gulong _tmp30_;
#line 672 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp28_ = self->current_item;
#line 672 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp29_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp28_, BOXES_TYPE_MACHINE) ? ((BoxesMachine*) _tmp28_) : NULL);
#line 672 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				machine = _tmp29_;
#line 673 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp30_ = self->priv->got_error_id;
#line 673 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (_tmp30_ != ((gulong) 0)) {
#line 5415 "app.c"
					BoxesMachine* _tmp31_;
					gulong _tmp32_;
#line 674 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp31_ = machine;
#line 674 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp32_ = self->priv->got_error_id;
#line 674 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					g_signal_handler_disconnect ((GObject*) _tmp31_, _tmp32_);
#line 675 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					self->priv->got_error_id = (gulong) 0;
#line 5426 "app.c"
				}
#line 671 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (machine);
#line 5430 "app.c"
			}
#line 678 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			boxes_app_set_fullscreen (self, FALSE);
#line 679 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp33_ = self->view;
#line 679 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			boxes_collection_view_set_visible (_tmp33_, TRUE);
#line 680 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp34_ = self->searchbar_revealer;
#line 680 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp35_ = self->searchbar;
#line 680 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp36_ = boxes_searchbar_get_visible (_tmp35_);
#line 680 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp37_ = _tmp36_;
#line 680 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			boxes_revealer_set_revealed (_tmp34_, _tmp37_);
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp38_ = self->current_item;
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_tmp38_ != NULL) {
#line 5452 "app.c"
				Block5Data* _data5_;
				BoxesCollectionItem* _tmp39_;
				ClutterActor* _tmp40_;
				ClutterActor* _tmp41_;
				ClutterActor* _tmp42_;
				ClutterActor* _tmp43_;
				BoxesCollectionItem* _tmp44_;
				BoxesCollectionView* _tmp45_;
				GdMainIconView* _tmp46_;
				gulong _tmp47_ = 0UL;
				ClutterActor* _tmp48_;
				gulong _tmp49_ = 0UL;
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data5_ = g_slice_new0 (Block5Data);
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data5_->_ref_count_ = 1;
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data5_->self = g_object_ref (self);
#line 684 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp39_ = self->current_item;
#line 684 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp40_ = boxes_ui_get_actor ((BoxesUI*) _tmp39_);
#line 684 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp41_ = _tmp40_;
#line 684 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp42_ = _g_object_ref0 (_tmp41_);
#line 684 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data5_->actor = _tmp42_;
#line 686 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp43_ = _data5_->actor;
#line 686 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				clutter_actor_show (_tmp43_);
#line 687 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp44_ = self->current_item;
#line 687 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_app_position_item_actor_at_icon (self, _tmp44_);
#line 690 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp45_ = self->view;
#line 690 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp46_ = _tmp45_->icon_view;
#line 690 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp47_ = g_signal_connect_object ((GtkWidget*) _tmp46_, "size-allocate", (GCallback) _____lambda205__gtk_widget_size_allocate, self, 0);
#line 690 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data5_->id = _tmp47_;
#line 697 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data5_->completed_id = (gulong) 0;
#line 698 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp48_ = _data5_->actor;
#line 698 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp49_ = g_signal_connect_data (_tmp48_, "transitions-completed", (GCallback) _____lambda207__clutter_actor_transitions_completed, block5_data_ref (_data5_), (GClosureNotify) block5_data_unref, 0);
#line 698 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data5_->completed_id = _tmp49_;
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				block5_data_unref (_data5_);
#line 683 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_data5_ = NULL;
#line 5509 "app.c"
			}
#line 707 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			break;
#line 5513 "app.c"
		}
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_UI_STATE_CREDS:
#line 5517 "app.c"
		{
#line 710 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			break;
#line 5521 "app.c"
		}
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_UI_STATE_WIZARD:
#line 5525 "app.c"
		{
			BoxesCollectionItem* _tmp50_;
#line 713 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp50_ = self->current_item;
#line 713 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_tmp50_ != NULL) {
#line 5532 "app.c"
				BoxesCollectionItem* _tmp51_;
				ClutterActor* _tmp52_;
				ClutterActor* _tmp53_;
#line 714 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp51_ = self->current_item;
#line 714 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp52_ = boxes_ui_get_actor ((BoxesUI*) _tmp51_);
#line 714 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp53_ = _tmp52_;
#line 714 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_actor_remove (_tmp53_);
#line 5544 "app.c"
			}
#line 715 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			break;
#line 5548 "app.c"
		}
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_UI_STATE_PROPERTIES:
#line 5552 "app.c"
		{
			BoxesCollectionItem* _tmp54_;
			ClutterActor* _tmp55_;
			ClutterActor* _tmp56_;
#line 718 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp54_ = self->current_item;
#line 718 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp55_ = boxes_ui_get_actor ((BoxesUI*) _tmp54_);
#line 718 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp56_ = _tmp55_;
#line 718 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			clutter_actor_hide (_tmp56_);
#line 719 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			break;
#line 5567 "app.c"
		}
#line 667 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_UI_STATE_DISPLAY:
#line 5571 "app.c"
		{
#line 722 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			break;
#line 5575 "app.c"
		}
		default:
		{
			BoxesUIState _tmp57_;
			BoxesUIState _tmp58_;
			GEnumValue* _tmp59_;
			gchar* _tmp60_ = NULL;
			gchar* _tmp61_;
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp57_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp58_ = _tmp57_;
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp59_ = g_enum_get_value (g_type_class_ref (BOXES_TYPE_UI_STATE), _tmp58_);
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp60_ = g_strdup_printf ("Unhandled UI state %s", (_tmp59_ != NULL) ? _tmp59_->value_name : NULL);
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp61_ = _tmp60_;
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_warning ("app.vala:725: %s", _tmp61_);
#line 725 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_free0 (_tmp61_);
#line 726 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			break;
#line 5600 "app.c"
		}
	}
#line 729 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp62_ = self->current_item;
#line 729 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp62_ != NULL) {
#line 5607 "app.c"
		BoxesCollectionItem* _tmp63_;
		BoxesUIState _tmp64_;
		BoxesUIState _tmp65_;
#line 730 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp63_ = self->current_item;
#line 730 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp64_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 730 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp65_ = _tmp64_;
#line 730 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_ui_set_ui_state ((BoxesUI*) _tmp63_, _tmp65_);
#line 5619 "app.c"
	}
}


static Block6Data* block6_data_ref (Block6Data* _data6_) {
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_atomic_int_inc (&_data6_->_ref_count_);
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return _data6_;
#line 5629 "app.c"
}


static void block6_data_unref (void * _userdata_) {
	Block6Data* _data6_;
	_data6_ = (Block6Data*) _userdata_;
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (g_atomic_int_dec_and_test (&_data6_->_ref_count_)) {
#line 5638 "app.c"
		BoxesApp * self;
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self = _data6_->self;
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (self);
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_slice_free (Block6Data, _data6_);
#line 5646 "app.c"
	}
}


static Block7Data* block7_data_ref (Block7Data* _data7_) {
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_atomic_int_inc (&_data7_->_ref_count_);
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return _data7_;
#line 5656 "app.c"
}


static void block7_data_unref (void * _userdata_) {
	Block7Data* _data7_;
	_data7_ = (Block7Data*) _userdata_;
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (g_atomic_int_dec_and_test (&_data7_->_ref_count_)) {
#line 5665 "app.c"
		BoxesApp * self;
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self = _data7_->_data6_->self;
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (_data7_->machine);
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block6_data_unref (_data7_->_data6_);
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data7_->_data6_ = NULL;
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_slice_free (Block7Data, _data7_);
#line 5677 "app.c"
	}
}


static void ______lambda208_ (Block7Data* _data7_) {
	Block6Data* _data6_;
	BoxesApp * self;
	BoxesLibvirtMachine* _tmp0_;
	const gchar* _tmp1_;
	const gchar* _tmp2_;
	gint _tmp3_;
#line 754 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data6_ = _data7_->_data6_;
#line 754 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = _data6_->self;
#line 755 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _data7_->machine;
#line 755 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = boxes_collection_item_get_name ((BoxesCollectionItem*) _tmp0_);
#line 755 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = _tmp1_;
#line 755 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_debug ("app.vala:755: %s suspended", _tmp2_);
#line 756 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _data6_->waiting_counter;
#line 756 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data6_->waiting_counter = _tmp3_ - 1;
#line 5705 "app.c"
}


static void _______lambda208__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self) {
#line 754 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	______lambda208_ (self);
#line 754 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	block7_data_unref (self);
#line 5714 "app.c"
}


void boxes_app_suspend_machines (BoxesApp* self) {
	Block6Data* _data6_;
	BoxesCollection* _tmp0_;
	GMainContext* _tmp1_;
	GMainContext* context;
	GMainContext* _tmp2_;
	BoxesCollection* _tmp3_;
	GPtrArray* _tmp4_;
	gpointer* _tmp5_;
	gint _tmp5__length1;
	GMainContext* _tmp14_;
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data6_ = g_slice_new0 (Block6Data);
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data6_->_ref_count_ = 1;
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data6_->self = g_object_ref (self);
#line 736 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->collection;
#line 736 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp0_ == NULL) {
#line 737 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block6_data_unref (_data6_);
#line 737 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data6_ = NULL;
#line 737 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return;
#line 5747 "app.c"
	}
#line 739 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_debug ("app.vala:739: Suspending running boxes");
#line 741 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data6_->waiting_counter = 0;
#line 745 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = g_main_context_new ();
#line 745 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	context = _tmp1_;
#line 747 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = context;
#line 747 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_main_context_push_thread_default (_tmp2_);
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = self->collection;
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = _tmp3_->items;
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = _tmp4_->pdata;
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5__length1 = (gint) _tmp4_->len;
#line 5769 "app.c"
	{
		BoxesCollectionItem** item_collection = NULL;
		gint item_collection_length1 = 0;
		gint _item_collection_size_ = 0;
		gint item_it = 0;
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		item_collection = _tmp5_;
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		item_collection_length1 = _tmp5__length1;
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		for (item_it = 0; item_it < _tmp5__length1; item_it = item_it + 1) {
#line 5781 "app.c"
			BoxesCollectionItem* _tmp6_;
			BoxesCollectionItem* item = NULL;
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp6_ = _g_object_ref0 (item_collection[item_it]);
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			item = _tmp6_;
#line 5788 "app.c"
			{
				BoxesCollectionItem* _tmp7_;
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp7_ = item;
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp7_, BOXES_TYPE_LIBVIRT_MACHINE)) {
#line 5795 "app.c"
					Block7Data* _data7_;
					BoxesCollectionItem* _tmp8_;
					BoxesLibvirtMachine* _tmp9_;
					BoxesLibvirtMachine* _tmp10_;
					gboolean _tmp11_;
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data7_ = g_slice_new0 (Block7Data);
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data7_->_ref_count_ = 1;
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data7_->_data6_ = block6_data_ref (_data6_);
#line 750 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp8_ = item;
#line 750 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp9_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp8_, BOXES_TYPE_LIBVIRT_MACHINE) ? ((BoxesLibvirtMachine*) _tmp8_) : NULL);
#line 750 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data7_->machine = _tmp9_;
#line 752 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp10_ = _data7_->machine;
#line 752 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp11_ = ((BoxesMachine*) _tmp10_)->suspend_at_exit;
#line 752 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					if (_tmp11_) {
#line 5819 "app.c"
						gint _tmp12_;
						BoxesLibvirtMachine* _tmp13_;
#line 753 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						_tmp12_ = _data6_->waiting_counter;
#line 753 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						_data6_->waiting_counter = _tmp12_ + 1;
#line 754 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						_tmp13_ = _data7_->machine;
#line 754 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
						boxes_libvirt_machine_suspend (_tmp13_, _______lambda208__gasync_ready_callback, block7_data_ref (_data7_));
#line 5830 "app.c"
					}
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					block7_data_unref (_data7_);
#line 749 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_data7_ = NULL;
#line 5836 "app.c"
				}
#line 748 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (item);
#line 5840 "app.c"
			}
		}
	}
#line 761 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp14_ = context;
#line 761 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_main_context_pop_thread_default (_tmp14_);
#line 764 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	while (TRUE) {
#line 5850 "app.c"
		gint _tmp15_;
		GMainContext* _tmp16_;
#line 764 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp15_ = _data6_->waiting_counter;
#line 764 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (!(_tmp15_ > 0)) {
#line 764 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			break;
#line 5859 "app.c"
		}
#line 765 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp16_ = context;
#line 765 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_main_context_iteration (_tmp16_, TRUE);
#line 5865 "app.c"
	}
#line 767 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_debug ("app.vala:767: Running boxes suspended");
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_main_context_unref0 (context);
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	block6_data_unref (_data6_);
#line 733 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data6_ = NULL;
#line 5875 "app.c"
}


gboolean boxes_app_quit (BoxesApp* self) {
	gboolean result = FALSE;
	BoxesNotificationbar* _tmp0_;
	BoxesWizard* _tmp1_;
	GtkApplicationWindow* _tmp2_;
#line 770 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 771 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->notificationbar;
#line 771 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_notificationbar_cancel (_tmp0_);
#line 772 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_save_window_geometry (self);
#line 773 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = self->wizard;
#line 773 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_wizard_cleanup (_tmp1_);
#line 774 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = self->window;
#line 774 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	gtk_widget_destroy ((GtkWidget*) _tmp2_);
#line 776 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = FALSE;
#line 776 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 5904 "app.c"
}


static void _g_list_free__g_object_unref0_ (GList* self) {
#line 792 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_list_foreach (self, (GFunc) _g_object_unref0_, NULL);
#line 792 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_list_free (self);
#line 5913 "app.c"
}


void boxes_app_show_properties (BoxesApp* self) {
	BoxesCollectionView* _tmp0_;
	GList* _tmp1_ = NULL;
	GList* selected_items;
#line 792 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 793 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 793 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = boxes_collection_view_get_selected_items (_tmp0_);
#line 793 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	selected_items = _tmp1_;
#line 795 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_set_selection_mode (self, FALSE);
#line 5931 "app.c"
	{
		GList* item_collection = NULL;
		GList* item_it = NULL;
#line 798 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		item_collection = selected_items;
#line 798 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		for (item_it = item_collection; item_it != NULL; item_it = item_it->next) {
#line 5939 "app.c"
			BoxesCollectionItem* _tmp2_;
			BoxesCollectionItem* item = NULL;
#line 798 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp2_ = _g_object_ref0 ((BoxesCollectionItem*) item_it->data);
#line 798 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			item = _tmp2_;
#line 5946 "app.c"
			{
				BoxesCollectionItem* _tmp3_;
				BoxesCollectionItem* _tmp4_;
#line 799 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp3_ = item;
#line 799 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp4_ = _g_object_ref0 (_tmp3_);
#line 799 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (self->current_item);
#line 799 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				self->current_item = _tmp4_;
#line 800 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_ui_set_ui_state ((BoxesUI*) self, BOXES_UI_STATE_PROPERTIES);
#line 801 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (item);
#line 801 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				break;
#line 5964 "app.c"
			}
		}
	}
#line 792 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__g_list_free__g_object_unref0_0 (selected_items);
#line 5970 "app.c"
}


static Block8Data* block8_data_ref (Block8Data* _data8_) {
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_atomic_int_inc (&_data8_->_ref_count_);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return _data8_;
#line 5979 "app.c"
}


static void block8_data_unref (void * _userdata_) {
	Block8Data* _data8_;
	_data8_ = (Block8Data*) _userdata_;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (g_atomic_int_dec_and_test (&_data8_->_ref_count_)) {
#line 5988 "app.c"
		BoxesApp * self;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self = _data8_->self;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		__g_list_free__g_object_unref0_0 (_data8_->selected_items);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (self);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_slice_free (Block8Data, _data8_);
#line 5998 "app.c"
	}
}


static void __lambda158_ (Block8Data* _data8_) {
	BoxesApp * self;
	GList* _tmp0_;
#line 818 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = _data8_->self;
#line 819 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_debug ("app.vala:819: Box deletion cancelled by user, re-adding to view");
#line 820 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _data8_->selected_items;
#line 6012 "app.c"
	{
		GList* selected_collection = NULL;
		GList* selected_it = NULL;
#line 820 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		selected_collection = _tmp0_;
#line 820 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		for (selected_it = selected_collection; selected_it != NULL; selected_it = selected_it->next) {
#line 6020 "app.c"
			BoxesCollectionItem* _tmp1_;
			BoxesCollectionItem* selected = NULL;
#line 820 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp1_ = _g_object_ref0 ((BoxesCollectionItem*) selected_it->data);
#line 820 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			selected = _tmp1_;
#line 6027 "app.c"
			{
				BoxesCollection* _tmp2_;
				BoxesCollectionItem* _tmp3_;
#line 821 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp2_ = self->collection;
#line 821 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp3_ = selected;
#line 821 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_collection_add_item (_tmp2_, _tmp3_);
#line 820 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (selected);
#line 6039 "app.c"
			}
		}
	}
}


static void ___lambda158__boxes_notificationbar_ok_func (gpointer self) {
#line 818 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda158_ (self);
#line 6049 "app.c"
}


static void __lambda159_ (Block8Data* _data8_) {
	BoxesApp * self;
	GList* _tmp0_;
#line 825 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = _data8_->self;
#line 826 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_debug ("app.vala:826: Box deletion, deleting now");
#line 827 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = _data8_->selected_items;
#line 6062 "app.c"
	{
		GList* selected_collection = NULL;
		GList* selected_it = NULL;
#line 827 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		selected_collection = _tmp0_;
#line 827 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		for (selected_it = selected_collection; selected_it != NULL; selected_it = selected_it->next) {
#line 6070 "app.c"
			BoxesCollectionItem* _tmp1_;
			BoxesCollectionItem* selected = NULL;
#line 827 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp1_ = _g_object_ref0 ((BoxesCollectionItem*) selected_it->data);
#line 827 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			selected = _tmp1_;
#line 6077 "app.c"
			{
				BoxesCollectionItem* _tmp2_;
				BoxesMachine* _tmp3_;
				BoxesMachine* machine;
				BoxesMachine* _tmp4_;
#line 828 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp2_ = selected;
#line 828 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp3_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp2_, BOXES_TYPE_MACHINE) ? ((BoxesMachine*) _tmp2_) : NULL);
#line 828 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				machine = _tmp3_;
#line 830 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp4_ = machine;
#line 830 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (_tmp4_ != NULL) {
#line 6093 "app.c"
					BoxesMachine* _tmp5_;
#line 831 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp5_ = machine;
#line 831 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					boxes_machine_delete (_tmp5_, TRUE);
#line 6099 "app.c"
				}
#line 827 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (machine);
#line 827 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (selected);
#line 6105 "app.c"
			}
		}
	}
}


static void ___lambda159__boxes_notificationbar_cancel_func (gpointer self) {
#line 825 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda159_ (self);
#line 6115 "app.c"
}


static gint g_ptr_array_get_length (GPtrArray* self) {
	gint result;
	guint _tmp0_;
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_return_val_if_fail (self != NULL, 0);
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	_tmp0_ = self->len;
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	result = (gint) _tmp0_;
#line 4080 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	return result;
#line 6130 "app.c"
}


static void g_ptr_array_set_length (GPtrArray* self, gint value) {
	gint _tmp0_;
#line 4081 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_return_if_fail (self != NULL);
#line 4081 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	_tmp0_ = value;
#line 4081 "/home/zeenix/jhbuild/share/vala-0.20/vapi/glib-2.0.vapi"
	g_ptr_array_set_size (self, _tmp0_);
#line 6142 "app.c"
}


void boxes_app_remove_selected_items (BoxesApp* self) {
	Block8Data* _data8_;
	BoxesCollectionView* _tmp0_;
	GList* _tmp1_ = NULL;
	GList* _tmp2_;
	guint _tmp3_ = 0U;
	guint num_selected;
	guint _tmp4_;
	gchar* _tmp5_ = NULL;
	guint _tmp6_;
	const gchar* _tmp17_;
	gchar* _tmp18_;
	gchar* message;
	GList* _tmp19_;
	BoxesNotificationbarOKFunc undo;
	void* undo_target;
	GDestroyNotify undo_target_destroy_notify;
	BoxesNotificationbarCancelFunc really_remove;
	void* really_remove_target;
	GDestroyNotify really_remove_target_destroy_notify;
	BoxesNotificationbar* _tmp23_;
	const gchar* _tmp24_;
	BoxesNotificationbarOKFunc _tmp25_;
	void* _tmp25__target;
	GDestroyNotify _tmp25__target_destroy_notify;
	BoxesNotificationbarCancelFunc _tmp26_;
	void* _tmp26__target;
	GDestroyNotify _tmp26__target_destroy_notify;
	GdNotification* _tmp27_ = NULL;
	GdNotification* _tmp28_;
	BoxesApp* _tmp29_;
	BoxesCollection* _tmp30_;
	GPtrArray* _tmp31_;
	gint _tmp32_;
	gint _tmp33_;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data8_ = g_slice_new0 (Block8Data);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data8_->_ref_count_ = 1;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data8_->self = g_object_ref (self);
#line 806 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 806 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = boxes_collection_view_get_selected_items (_tmp0_);
#line 806 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data8_->selected_items = _tmp1_;
#line 807 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = _data8_->selected_items;
#line 807 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = g_list_length (_tmp2_);
#line 807 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	num_selected = _tmp3_;
#line 808 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = num_selected;
#line 808 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp4_ == ((guint) 0)) {
#line 809 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		block8_data_unref (_data8_);
#line 809 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_data8_ = NULL;
#line 809 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return;
#line 6211 "app.c"
	}
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = num_selected;
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp6_ == ((guint) 1)) {
#line 6217 "app.c"
		const gchar* _tmp7_ = NULL;
		GList* _tmp8_;
		gconstpointer _tmp9_;
		const gchar* _tmp10_;
		const gchar* _tmp11_;
		gchar* _tmp12_ = NULL;
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp7_ = _ ("Box '%s' has been deleted");
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp8_ = _data8_->selected_items;
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp9_ = _tmp8_->data;
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp10_ = boxes_collection_item_get_name ((BoxesCollectionItem*) _tmp9_);
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp11_ = _tmp10_;
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp12_ = g_strdup_printf (_tmp7_, _tmp11_);
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_tmp5_);
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp5_ = _tmp12_;
#line 6240 "app.c"
	} else {
		guint _tmp13_;
		const gchar* _tmp14_ = NULL;
		guint _tmp15_;
		gchar* _tmp16_ = NULL;
#line 812 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp13_ = num_selected;
#line 812 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp14_ = ngettext ("%u box has been deleted", "%u boxes have been deleted", (gulong) _tmp13_);
#line 812 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp15_ = num_selected;
#line 812 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp16_ = g_strdup_printf (_tmp14_, _tmp15_);
#line 812 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_tmp5_);
#line 812 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp5_ = _tmp16_;
#line 6258 "app.c"
	}
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp17_ = _tmp5_;
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp18_ = g_strdup (_tmp17_);
#line 811 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	message = _tmp18_;
#line 815 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp19_ = _data8_->selected_items;
#line 6268 "app.c"
	{
		GList* item_collection = NULL;
		GList* item_it = NULL;
#line 815 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		item_collection = _tmp19_;
#line 815 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		for (item_it = item_collection; item_it != NULL; item_it = item_it->next) {
#line 6276 "app.c"
			BoxesCollectionItem* _tmp20_;
			BoxesCollectionItem* item = NULL;
#line 815 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp20_ = _g_object_ref0 ((BoxesCollectionItem*) item_it->data);
#line 815 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			item = _tmp20_;
#line 6283 "app.c"
			{
				BoxesCollection* _tmp21_;
				BoxesCollectionItem* _tmp22_;
#line 816 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp21_ = self->collection;
#line 816 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp22_ = item;
#line 816 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_collection_remove_item (_tmp21_, _tmp22_);
#line 815 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_g_object_unref0 (item);
#line 6295 "app.c"
			}
		}
	}
#line 818 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	undo = ___lambda158__boxes_notificationbar_ok_func;
#line 818 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	undo_target = block8_data_ref (_data8_);
#line 818 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	undo_target_destroy_notify = block8_data_unref;
#line 825 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	really_remove = ___lambda159__boxes_notificationbar_cancel_func;
#line 825 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	really_remove_target = block8_data_ref (_data8_);
#line 825 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	really_remove_target_destroy_notify = block8_data_unref;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp23_ = self->notificationbar;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp24_ = message;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp25_ = undo;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp25__target = undo_target;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp25__target_destroy_notify = undo_target_destroy_notify;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	undo_target_destroy_notify = NULL;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp26_ = really_remove;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp26__target = really_remove_target;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp26__target_destroy_notify = really_remove_target_destroy_notify;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	really_remove_target_destroy_notify = NULL;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp27_ = boxes_notificationbar_display_for_action (_tmp23_, _tmp24_, GTK_STOCK_UNDO, _tmp25_, _tmp25__target, _tmp25__target_destroy_notify, _tmp26_, _tmp26__target, _tmp26__target_destroy_notify, BOXES_NOTIFICATIONBAR_DEFAULT_TIMEOUT);
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp28_ = _tmp27_;
#line 835 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_tmp28_);
#line 838 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp29_ = boxes_app_app;
#line 838 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp30_ = _tmp29_->collection;
#line 838 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp31_ = _tmp30_->items;
#line 838 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp32_ = g_ptr_array_get_length (_tmp31_);
#line 838 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp33_ = _tmp32_;
#line 838 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp33_ == 0) {
#line 6349 "app.c"
		BoxesApp* _tmp34_;
#line 839 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp34_ = boxes_app_app;
#line 839 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_set_selection_mode (_tmp34_, FALSE);
#line 6355 "app.c"
	}
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	(really_remove_target_destroy_notify == NULL) ? NULL : (really_remove_target_destroy_notify (really_remove_target), NULL);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	really_remove = NULL;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	really_remove_target = NULL;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	really_remove_target_destroy_notify = NULL;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	(undo_target_destroy_notify == NULL) ? NULL : (undo_target_destroy_notify (undo_target), NULL);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	undo = NULL;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	undo_target = NULL;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	undo_target_destroy_notify = NULL;
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (message);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (_tmp5_);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	block8_data_unref (_data8_);
#line 805 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data8_ = NULL;
#line 6381 "app.c"
}


static gboolean boxes_app_on_key_pressed (BoxesApp* self, GtkWidget* widget, GdkEventKey* event) {
	gboolean result = FALSE;
	GdkEventKey _tmp0_;
	guint _tmp1_;
#line 843 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 843 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (widget != NULL, FALSE);
#line 843 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (event != NULL, FALSE);
#line 844 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = *event;
#line 844 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _tmp0_.keyval;
#line 844 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp1_ == ((guint) GDK_KEY_F11)) {
#line 6401 "app.c"
		gboolean _tmp2_;
		gboolean _tmp3_;
#line 845 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp2_ = boxes_app_get_fullscreen (self);
#line 845 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp3_ = _tmp2_;
#line 845 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_set_fullscreen (self, !_tmp3_);
#line 846 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		result = TRUE;
#line 846 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return result;
#line 6414 "app.c"
	} else {
		GdkEventKey _tmp4_;
		guint _tmp5_;
#line 847 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp4_ = *event;
#line 847 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp5_ = _tmp4_.keyval;
#line 847 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_tmp5_ == ((guint) GDK_KEY_Escape)) {
#line 6424 "app.c"
			gboolean _tmp6_ = FALSE;
			gboolean _tmp7_;
			gboolean _tmp8_;
			gboolean _tmp11_;
#line 848 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp7_ = boxes_app_get_selection_mode (self);
#line 848 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp8_ = _tmp7_;
#line 848 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_tmp8_) {
#line 6435 "app.c"
				BoxesUIState _tmp9_;
				BoxesUIState _tmp10_;
#line 848 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp9_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 848 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp10_ = _tmp9_;
#line 848 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp6_ = _tmp10_ == BOXES_UI_STATE_COLLECTION;
#line 6444 "app.c"
			} else {
#line 848 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp6_ = FALSE;
#line 6448 "app.c"
			}
#line 848 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp11_ = _tmp6_;
#line 848 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_tmp11_) {
#line 849 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_app_set_selection_mode (self, FALSE);
#line 6456 "app.c"
			}
		} else {
			gboolean _tmp12_ = FALSE;
			GdkEventKey _tmp13_;
			guint _tmp14_;
			gboolean _tmp17_;
#line 850 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp13_ = *event;
#line 850 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp14_ = _tmp13_.keyval;
#line 850 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_tmp14_ == ((guint) GDK_KEY_q)) {
#line 6469 "app.c"
				GdkEventKey _tmp15_;
				GdkModifierType _tmp16_;
#line 851 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp15_ = *event;
#line 851 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp16_ = _tmp15_.state;
#line 851 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp12_ = (_tmp16_ & GDK_MODIFIER_MASK) == GDK_CONTROL_MASK;
#line 6478 "app.c"
			} else {
#line 850 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp12_ = FALSE;
#line 6482 "app.c"
			}
#line 850 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp17_ = _tmp12_;
#line 850 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_tmp17_) {
#line 852 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_app_quit (self);
#line 853 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				result = TRUE;
#line 853 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				return result;
#line 6494 "app.c"
			} else {
				gboolean _tmp18_ = FALSE;
				GdkEventKey _tmp19_;
				guint _tmp20_;
				gboolean _tmp23_;
#line 854 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp19_ = *event;
#line 854 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp20_ = _tmp19_.keyval;
#line 854 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (_tmp20_ == ((guint) GDK_KEY_a)) {
#line 6506 "app.c"
					GdkEventKey _tmp21_;
					GdkModifierType _tmp22_;
#line 855 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp21_ = *event;
#line 855 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp22_ = _tmp21_.state;
#line 855 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp18_ = (_tmp22_ & GDK_MODIFIER_MASK) == GDK_MOD1_MASK;
#line 6515 "app.c"
				} else {
#line 854 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					_tmp18_ = FALSE;
#line 6519 "app.c"
				}
#line 854 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				_tmp23_ = _tmp18_;
#line 854 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				if (_tmp23_) {
#line 856 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					boxes_app_quit (self);
#line 857 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					result = TRUE;
#line 857 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
					return result;
#line 6531 "app.c"
				}
			}
		}
	}
#line 860 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = FALSE;
#line 860 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 6540 "app.c"
}


static Block9Data* block9_data_ref (Block9Data* _data9_) {
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_atomic_int_inc (&_data9_->_ref_count_);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return _data9_;
#line 6549 "app.c"
}


static void block9_data_unref (void * _userdata_) {
	Block9Data* _data9_;
	_data9_ = (Block9Data*) _userdata_;
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (g_atomic_int_dec_and_test (&_data9_->_ref_count_)) {
#line 6558 "app.c"
		BoxesApp * self;
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self = _data9_->self;
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (_data9_->machine);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (self);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_slice_free (Block9Data, _data9_);
#line 6568 "app.c"
	}
}


static void __lambda133_ (BoxesApp* self, const gchar* message) {
	BoxesApp* _tmp0_;
	BoxesNotificationbar* _tmp1_;
	const gchar* _tmp2_;
	GdNotification* _tmp3_ = NULL;
	GdNotification* _tmp4_;
#line 883 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (message != NULL);
#line 884 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = boxes_app_app;
#line 884 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _tmp0_->notificationbar;
#line 884 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = message;
#line 884 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = boxes_notificationbar_display_error (_tmp1_, _tmp2_, BOXES_NOTIFICATIONBAR_DEFAULT_TIMEOUT);
#line 884 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = _tmp3_;
#line 884 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_tmp4_);
#line 6593 "app.c"
}


static void ___lambda133__boxes_machine_got_error (BoxesMachine* _sender, const gchar* message, gpointer self) {
#line 883 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda133_ (self, message);
#line 6600 "app.c"
}


static void __lambda134_ (Block9Data* _data9_, GObject* obj, GAsyncResult* res) {
	BoxesApp * self;
	GError * _inner_error_ = NULL;
#line 891 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = _data9_->self;
#line 891 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (res != NULL);
#line 6611 "app.c"
	{
		BoxesMachine* _tmp0_;
		GAsyncResult* _tmp1_;
#line 893 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp0_ = _data9_->machine;
#line 893 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp1_ = res;
#line 893 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_machine_connect_display_finish (_tmp0_, _tmp1_, &_inner_error_);
#line 893 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_inner_error_ != NULL) {
#line 6623 "app.c"
			goto __catch6_g_error;
		}
	}
	goto __finally6;
	__catch6_g_error:
	{
		GError* e = NULL;
		BoxesApp* _tmp2_;
		BoxesNotificationbar* _tmp3_;
		const gchar* _tmp4_ = NULL;
		BoxesMachine* _tmp5_;
		const gchar* _tmp6_;
		const gchar* _tmp7_;
		gchar* _tmp8_ = NULL;
		gchar* _tmp9_;
		GdNotification* _tmp10_ = NULL;
		GdNotification* _tmp11_;
		GError* _tmp12_;
		const gchar* _tmp13_;
#line 892 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		e = _inner_error_;
#line 892 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_inner_error_ = NULL;
#line 895 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_ui_set_ui_state ((BoxesUI*) self, BOXES_UI_STATE_COLLECTION);
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp2_ = boxes_app_app;
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp3_ = _tmp2_->notificationbar;
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp4_ = _ ("Connection to '%s' failed");
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp5_ = _data9_->machine;
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp6_ = boxes_collection_item_get_name ((BoxesCollectionItem*) _tmp5_);
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp7_ = _tmp6_;
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp8_ = g_strdup_printf (_tmp4_, _tmp7_);
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp9_ = _tmp8_;
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp10_ = boxes_notificationbar_display_error (_tmp3_, _tmp9_, BOXES_NOTIFICATIONBAR_DEFAULT_TIMEOUT);
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp11_ = _tmp10_;
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (_tmp11_);
#line 896 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_free0 (_tmp9_);
#line 897 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp12_ = e;
#line 897 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp13_ = _tmp12_->message;
#line 897 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_debug ("app.vala:897: connect display failed: %s", _tmp13_);
#line 892 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_error_free0 (e);
#line 6681 "app.c"
	}
	__finally6:
#line 892 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_inner_error_ != NULL) {
#line 892 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_critical ("file %s: line %d: uncaught error: %s (%s, %d)", __FILE__, __LINE__, _inner_error_->message, g_quark_to_string (_inner_error_->domain), _inner_error_->code);
#line 892 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_clear_error (&_inner_error_);
#line 892 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		return;
#line 6692 "app.c"
	}
}


static void ___lambda134__gasync_ready_callback (GObject* source_object, GAsyncResult* res, gpointer self) {
#line 891 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	__lambda134_ (self, source_object, res);
#line 891 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	block9_data_unref (self);
#line 6702 "app.c"
}


void boxes_app_connect_to (BoxesApp* self, BoxesMachine* machine, gfloat x, gfloat y) {
	Block9Data* _data9_;
	BoxesMachine* _tmp0_;
	BoxesMachine* _tmp1_;
	BoxesMachine* _tmp2_;
	BoxesCollectionItem* _tmp3_;
	BoxesMachine* _tmp4_;
	ClutterActor* _tmp5_;
	ClutterActor* _tmp6_;
	ClutterActor* _tmp7_;
	ClutterActor* actor;
	ClutterActor* _tmp8_;
	ClutterActor* _tmp9_ = NULL;
	ClutterActor* _tmp16_;
	ClutterActor* _tmp17_;
	ClutterActor* _tmp18_;
	ClutterActor* _tmp19_;
	ClutterActor* _tmp20_;
	ClutterActor* _tmp21_;
	BoxesApp* _tmp22_;
	guint _tmp23_;
	BoxesMachine* _tmp24_;
	BoxesTopbar* _tmp25_;
	GBinding* _tmp26_ = NULL;
	GBinding* _tmp27_;
	BoxesMachine* _tmp28_;
	gulong _tmp29_ = 0UL;
	BoxesMachine* _tmp30_;
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (machine != NULL);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data9_ = g_slice_new0 (Block9Data);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data9_->_ref_count_ = 1;
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data9_->self = g_object_ref (self);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = machine;
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _g_object_ref0 (_tmp0_);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_data9_->machine);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data9_->machine = _tmp1_;
#line 864 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = _data9_->machine;
#line 864 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _g_object_ref0 ((BoxesCollectionItem*) _tmp2_);
#line 864 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->current_item);
#line 864 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->current_item = _tmp3_;
#line 867 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp4_ = _data9_->machine;
#line 867 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = boxes_ui_get_actor ((BoxesUI*) _tmp4_);
#line 867 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp6_ = _tmp5_;
#line 867 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp7_ = _g_object_ref0 (_tmp6_);
#line 867 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	actor = _tmp7_;
#line 868 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp8_ = actor;
#line 868 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp9_ = clutter_actor_get_parent (_tmp8_);
#line 868 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp9_ == NULL) {
#line 6776 "app.c"
		BoxesApp* _tmp10_;
		ClutterActor* _tmp11_;
		ClutterActor* _tmp12_;
		ClutterActor* _tmp13_;
		gfloat _tmp14_;
		gfloat _tmp15_;
#line 869 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp10_ = boxes_app_app;
#line 869 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp11_ = _tmp10_->overlay_bin_actor;
#line 869 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp12_ = actor;
#line 869 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		clutter_actor_add_child (_tmp11_, _tmp12_);
#line 870 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp13_ = actor;
#line 870 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp14_ = x;
#line 870 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp15_ = y;
#line 870 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_allocate_actor_no_animation (_tmp13_, _tmp14_, _tmp15_, (gfloat) BOXES_MACHINE_SCREENSHOT_WIDTH, (gfloat) (BOXES_MACHINE_SCREENSHOT_HEIGHT * 2));
#line 6799 "app.c"
	}
#line 874 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp16_ = actor;
#line 874 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_show (_tmp16_);
#line 875 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp17_ = actor;
#line 875 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_easing_mode (_tmp17_, CLUTTER_LINEAR);
#line 876 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp18_ = actor;
#line 876 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp19_ = actor;
#line 876 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_set (_tmp19_, "natural-width", (gfloat) (BOXES_MACHINE_SCREENSHOT_WIDTH * 2), NULL);
#line 876 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_set (_tmp18_, "min-width", (gfloat) (BOXES_MACHINE_SCREENSHOT_WIDTH * 2), NULL);
#line 877 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp20_ = actor;
#line 877 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_fixed_position_set (_tmp20_, FALSE);
#line 878 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp21_ = actor;
#line 878 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp22_ = boxes_app_app;
#line 878 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp23_ = _tmp22_->duration;
#line 878 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	clutter_actor_set_easing_duration (_tmp21_, _tmp23_);
#line 881 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp24_ = _data9_->machine;
#line 881 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp25_ = self->topbar;
#line 881 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp26_ = g_object_bind_property_with_closures ((GObject*) _tmp24_, "status", (GObject*) _tmp25_, "status", G_BINDING_SYNC_CREATE, (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)), (GClosure*) ((NULL == NULL) ? NULL : g_cclosure_new ((GCallback) NULL, NULL, NULL)));
#line 881 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp27_ = _g_object_ref0 (_tmp26_);
#line 881 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->priv->status_bind);
#line 881 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->status_bind = _tmp27_;
#line 883 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp28_ = _data9_->machine;
#line 883 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp29_ = g_signal_connect_object (_tmp28_, "got-error", (GCallback) ___lambda133__boxes_machine_got_error, self, 0);
#line 883 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->got_error_id = _tmp29_;
#line 888 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_ui_set_ui_state ((BoxesUI*) self, BOXES_UI_STATE_CREDS);
#line 891 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp30_ = _data9_->machine;
#line 891 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_machine_connect_display (_tmp30_, ___lambda134__gasync_ready_callback, block9_data_ref (_data9_));
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (actor);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	block9_data_unref (_data9_);
#line 863 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_data9_ = NULL;
#line 6859 "app.c"
}


void boxes_app_select_item (BoxesApp* self, BoxesCollectionItem* item) {
	gboolean _tmp0_ = FALSE;
	BoxesUIState _tmp1_;
	BoxesUIState _tmp2_;
	gboolean _tmp5_;
#line 903 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 903 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (item != NULL);
#line 904 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 904 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = _tmp1_;
#line 904 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp2_ == BOXES_UI_STATE_COLLECTION) {
#line 6878 "app.c"
		gboolean _tmp3_;
		gboolean _tmp4_;
#line 904 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp3_ = boxes_app_get_selection_mode (self);
#line 904 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp4_ = _tmp3_;
#line 904 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp0_ = !_tmp4_;
#line 6887 "app.c"
	} else {
#line 904 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp0_ = FALSE;
#line 6891 "app.c"
	}
#line 904 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp5_ = _tmp0_;
#line 904 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp5_) {
#line 6897 "app.c"
		BoxesCollectionItem* _tmp6_;
		BoxesCollectionItem* _tmp7_;
		BoxesCollectionItem* _tmp8_;
		BoxesCollectionItem* _tmp20_;
#line 905 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp6_ = item;
#line 905 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp7_ = _g_object_ref0 (_tmp6_);
#line 905 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_g_object_unref0 (self->current_item);
#line 905 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		self->current_item = _tmp7_;
#line 907 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp8_ = self->current_item;
#line 907 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (G_TYPE_CHECK_INSTANCE_TYPE (_tmp8_, BOXES_TYPE_MACHINE)) {
#line 6914 "app.c"
			BoxesCollectionItem* _tmp9_;
			BoxesMachine* _tmp10_;
			BoxesMachine* machine;
			gfloat item_x = 0.0F;
			gfloat item_y = 0.0F;
			BoxesCollectionView* _tmp11_;
			BoxesCollectionItem* _tmp12_;
			gfloat _tmp13_ = 0.0F;
			gfloat _tmp14_ = 0.0F;
			BoxesMachine* _tmp15_;
			gfloat _tmp16_;
			gfloat _tmp17_;
			gboolean _tmp18_;
			gboolean _tmp19_;
#line 908 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp9_ = self->current_item;
#line 908 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp10_ = _g_object_ref0 (G_TYPE_CHECK_INSTANCE_TYPE (_tmp9_, BOXES_TYPE_MACHINE) ? ((BoxesMachine*) _tmp9_) : NULL);
#line 908 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			machine = _tmp10_;
#line 911 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp11_ = self->view;
#line 911 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp12_ = item;
#line 911 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			boxes_collection_view_get_item_pos (_tmp11_, _tmp12_, &_tmp13_, &_tmp14_);
#line 911 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			item_x = _tmp13_;
#line 911 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			item_y = _tmp14_;
#line 913 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp15_ = machine;
#line 913 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp16_ = item_x;
#line 913 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp17_ = item_y;
#line 913 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			boxes_app_connect_to (self, _tmp15_, _tmp16_, _tmp17_);
#line 915 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp18_ = boxes_app_get_maximized (self);
#line 915 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp19_ = _tmp18_;
#line 915 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			if (_tmp19_) {
#line 916 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
				boxes_app_set_fullscreen (self, TRUE);
#line 6961 "app.c"
			}
#line 907 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_object_unref0 (machine);
#line 6965 "app.c"
		} else {
#line 918 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			g_warning ("app.vala:918: unknown item, fix your code");
#line 6969 "app.c"
		}
#line 920 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp20_ = item;
#line 920 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_signal_emit_by_name (self, "item-selected", _tmp20_);
#line 6975 "app.c"
	} else {
		BoxesUIState _tmp21_;
		BoxesUIState _tmp22_;
#line 921 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp21_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 921 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp22_ = _tmp21_;
#line 921 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		if (_tmp22_ == BOXES_UI_STATE_WIZARD) {
#line 6985 "app.c"
			BoxesCollectionItem* _tmp23_;
			BoxesCollectionItem* _tmp24_;
#line 922 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp23_ = item;
#line 922 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_tmp24_ = _g_object_ref0 (_tmp23_);
#line 922 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			_g_object_unref0 (self->current_item);
#line 922 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			self->current_item = _tmp24_;
#line 924 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
			boxes_ui_set_ui_state ((BoxesUI*) self, BOXES_UI_STATE_PROPERTIES);
#line 6998 "app.c"
		}
	}
}


static ClutterActor* boxes_app_real_get_actor (BoxesUI* base) {
	ClutterActor* result;
	BoxesApp* self;
	ClutterStage* _tmp0_;
#line 40 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = (BoxesApp*) base;
#line 40 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->stage;
#line 40 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = (ClutterActor*) _tmp0_;
#line 40 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 7016 "app.c"
}


gboolean boxes_app_get_fullscreen (BoxesApp* self) {
	gboolean result;
	GtkApplicationWindow* _tmp0_;
	GdkWindow* _tmp1_ = NULL;
	GdkWindowState _tmp2_ = 0;
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->window;
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = gtk_widget_get_window ((GtkWidget*) _tmp0_);
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = gdk_window_get_state (_tmp1_);
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = (_tmp2_ & GDK_WINDOW_STATE_FULLSCREEN) == GDK_WINDOW_STATE_FULLSCREEN;
#line 44 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 7037 "app.c"
}


void boxes_app_set_fullscreen (BoxesApp* self, gboolean value) {
	gboolean _tmp0_;
#line 45 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 46 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = value;
#line 46 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	if (_tmp0_) {
#line 7049 "app.c"
		GtkApplicationWindow* _tmp1_;
#line 47 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp1_ = self->window;
#line 47 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		gtk_window_fullscreen ((GtkWindow*) _tmp1_);
#line 7055 "app.c"
	} else {
		GtkApplicationWindow* _tmp2_;
#line 49 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		_tmp2_ = self->window;
#line 49 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		gtk_window_unfullscreen ((GtkWindow*) _tmp2_);
#line 7062 "app.c"
	}
}


static gboolean boxes_app_get_maximized (BoxesApp* self) {
	gboolean result;
	GtkApplicationWindow* _tmp0_;
	GdkWindow* _tmp1_ = NULL;
	GdkWindowState _tmp2_ = 0;
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->window;
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = gtk_widget_get_window ((GtkWidget*) _tmp0_);
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = gdk_window_get_state (_tmp1_);
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = (_tmp2_ & GDK_WINDOW_STATE_MAXIMIZED) == GDK_WINDOW_STATE_MAXIMIZED;
#line 53 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 7084 "app.c"
}


const gchar* boxes_app_get_uri (BoxesApp* self) {
	const gchar* result;
	const gchar* _tmp0_;
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->priv->_uri;
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = _tmp0_;
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 7099 "app.c"
}


void boxes_app_set_uri (BoxesApp* self, const gchar* value) {
	const gchar* _tmp0_;
	gchar* _tmp1_;
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = value;
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = g_strdup (_tmp0_);
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (self->priv->_uri);
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->_uri = _tmp1_;
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_notify ((GObject *) self, "uri");
#line 7118 "app.c"
}


GVirConnection* boxes_app_get_default_connection (BoxesApp* self) {
	GVirConnection* result;
	BoxesLibvirtBroker* _tmp0_ = NULL;
	BoxesLibvirtBroker* _tmp1_;
	GVirConnection* _tmp2_ = NULL;
	GVirConnection* _tmp3_;
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = boxes_libvirt_broker_get_default ();
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _tmp0_;
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = boxes_libvirt_broker_get_connection (_tmp1_, "QEMU Session");
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp3_ = _tmp2_;
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (_tmp1_);
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = _tmp3_;
#line 86 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 7144 "app.c"
}


BoxesCollectionSource* boxes_app_get_default_source (BoxesApp* self) {
	BoxesCollectionSource* result;
	GHashTable* _tmp0_;
	gconstpointer _tmp1_ = NULL;
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->priv->sources;
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = g_hash_table_lookup (_tmp0_, "QEMU Session");
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = (BoxesCollectionSource*) _tmp1_;
#line 87 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 7162 "app.c"
}


gboolean boxes_app_get_selection_mode (BoxesApp* self) {
	gboolean result;
	gboolean _tmp0_;
#line 780 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, FALSE);
#line 780 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->priv->_selection_mode;
#line 780 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = _tmp0_;
#line 780 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 7177 "app.c"
}


void boxes_app_set_selection_mode (BoxesApp* self, gboolean value) {
	BoxesUIState _tmp0_;
	BoxesUIState _tmp1_;
	gboolean _tmp2_;
#line 781 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (self != NULL);
#line 782 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = boxes_ui_get_ui_state ((BoxesUI*) self);
#line 782 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = _tmp0_;
#line 782 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_if_fail (_tmp1_ == BOXES_UI_STATE_COLLECTION);
#line 784 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp2_ = value;
#line 784 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv->_selection_mode = _tmp2_;
#line 781 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_notify ((GObject *) self, "selection-mode");
#line 7199 "app.c"
}


GList* boxes_app_get_selected_items (BoxesApp* self) {
	GList* result;
	BoxesCollectionView* _tmp0_;
	GList* _tmp1_ = NULL;
#line 789 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 789 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp0_ = self->view;
#line 789 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_tmp1_ = boxes_collection_view_get_selected_items (_tmp0_);
#line 789 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	result = _tmp1_;
#line 789 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	return result;
#line 7217 "app.c"
}


static void boxes_app_class_init (BoxesAppClass * klass) {
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	boxes_app_parent_class = g_type_class_peek_parent (klass);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_type_class_add_private (klass, sizeof (BoxesAppPrivate));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	BOXES_UI_CLASS (klass)->ui_state_changed = boxes_app_real_ui_state_changed;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	BOXES_UI_CLASS (klass)->get_actor = boxes_app_real_get_actor;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	G_OBJECT_CLASS (klass)->get_property = _vala_boxes_app_get_property;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	G_OBJECT_CLASS (klass)->set_property = _vala_boxes_app_set_property;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	G_OBJECT_CLASS (klass)->finalize = boxes_app_finalize;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_APP_ACTOR, g_param_spec_object ("actor", "actor", "actor", clutter_actor_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_APP_FULLSCREEN, g_param_spec_boolean ("fullscreen", "fullscreen", "fullscreen", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_APP_URI, g_param_spec_string ("uri", "uri", "uri", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_APP_DEFAULT_CONNECTION, g_param_spec_object ("default-connection", "default-connection", "default-connection", gvir_connection_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_APP_DEFAULT_SOURCE, g_param_spec_object ("default-source", "default-source", "default-source", BOXES_TYPE_COLLECTION_SOURCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_APP_SELECTION_MODE, g_param_spec_boolean ("selection-mode", "selection-mode", "selection-mode", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_object_class_install_property (G_OBJECT_CLASS (klass), BOXES_APP_SELECTED_ITEMS, g_param_spec_pointer ("selected-items", "selected-items", "selected-items", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_new ("ready", BOXES_TYPE_APP, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	g_signal_new ("item_selected", BOXES_TYPE_APP, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, BOXES_TYPE_COLLECTION_ITEM);
#line 7254 "app.c"
}


static void boxes_app_instance_init (BoxesApp * self) {
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self->priv = BOXES_APP_GET_PRIVATE (self);
#line 7261 "app.c"
}


static void boxes_app_finalize (GObject* obj) {
	BoxesApp * self;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, BOXES_TYPE_APP, BoxesApp);
#line 41 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->window);
#line 54 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->notebook);
#line 55 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->embed);
#line 56 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->stage);
#line 57 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->stage_bin);
#line 58 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->overlay_bin_actor);
#line 59 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->overlay_bin);
#line 60 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->current_item);
#line 61 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->searchbar);
#line 62 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->topbar);
#line 63 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->notificationbar);
#line 64 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->sidebar_revealer);
#line 65 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->searchbar_revealer);
#line 66 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->sidebar);
#line 67 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->selectionbar);
#line 69 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->settings);
#line 70 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->wizard);
#line 71 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->properties);
#line 72 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->display_page);
#line 73 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->empty_boxes);
#line 74 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_free0 (self->priv->_uri);
#line 75 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->collection);
#line 76 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->filter);
#line 81 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->priv->application);
#line 82 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->view);
#line 84 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_hash_table_unref0 (self->priv->brokers);
#line 85 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_hash_table_unref0 (self->priv->sources);
#line 90 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	_g_object_unref0 (self->priv->status_bind);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	G_OBJECT_CLASS (boxes_app_parent_class)->finalize (obj);
#line 7327 "app.c"
}


GType boxes_app_get_type (void) {
	static volatile gsize boxes_app_type_id__volatile = 0;
	if (g_once_init_enter (&boxes_app_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (BoxesAppClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) boxes_app_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (BoxesApp), 0, (GInstanceInitFunc) boxes_app_instance_init, NULL };
		GType boxes_app_type_id;
		boxes_app_type_id = g_type_register_static (BOXES_TYPE_UI, "BoxesApp", &g_define_type_info, 0);
		g_once_init_leave (&boxes_app_type_id__volatile, boxes_app_type_id);
	}
	return boxes_app_type_id__volatile;
}


static void _vala_boxes_app_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	BoxesApp * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_APP, BoxesApp);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	switch (property_id) {
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_ACTOR:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_value_set_object (value, boxes_ui_get_actor ((BoxesUI*) self));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_FULLSCREEN:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_value_set_boolean (value, boxes_app_get_fullscreen (self));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_URI:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_value_set_string (value, boxes_app_get_uri (self));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_DEFAULT_CONNECTION:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_value_take_object (value, boxes_app_get_default_connection (self));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_DEFAULT_SOURCE:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_value_set_object (value, boxes_app_get_default_source (self));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_SELECTION_MODE:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_value_set_boolean (value, boxes_app_get_selection_mode (self));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_SELECTED_ITEMS:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		g_value_set_pointer (value, boxes_app_get_selected_items (self));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 7390 "app.c"
		default:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 7396 "app.c"
	}
}


static void _vala_boxes_app_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	BoxesApp * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, BOXES_TYPE_APP, BoxesApp);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
	switch (property_id) {
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_FULLSCREEN:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_set_fullscreen (self, g_value_get_boolean (value));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_URI:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_set_uri (self, g_value_get_string (value));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		case BOXES_APP_SELECTION_MODE:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		boxes_app_set_selection_mode (self, g_value_get_boolean (value));
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 7424 "app.c"
		default:
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
#line 38 "/extra-data/checkout/gnome/gnome-boxes/src/app.vala"
		break;
#line 7430 "app.c"
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



